<?php
namespace Topalis\ActiveSyncFilter\Config\Form;

use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Mapping\ClassMetadata;
use Topalis\ActiveSyncFilter\Config\GeneralConfig;
use Topalis\ActiveSyncFilter\FilterServer;
use Topalis\ActiveSyncFilter\Plugin\PluginManager;
use Topalis\ConfigGroup\Entity\ConfigGroup;

/**
 * Configuration for active sync filter form
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class FormConfig
{
    /**
     * @var ConfigGroup
     */
    protected $config;

    /**
     * @var FilterServer
     */
    protected $filterServer;

    /**
     * @var GeneralConfig
     */
    protected $generalConfig;

    /**
     * @var \Topalis\ActiveSyncFilter\Config\FileRemoveConfig
     */
    protected $fileRemoveConfig;

    /**
     * @var PluginManager
     */
    protected $pluginManager;

    protected $errorBehavior;

    /**
     * array with mime types to exclude or to remove
     * @var array
     */
    protected $mimeTypes;

    protected $removeUnfiltered;

    /**
     * @var array[FormPluginConfig]
     */
    protected $plugins;

    /**
     *
     * @param FilterServer $filterServer
     * @param ConfigGroup $config The group of the configuration to access.
     *     (Doesn't affect global settings like the ones of the apache module.)
     */
    public function __construct(FilterServer $filterServer, ConfigGroup $config)
    {
        $this->filterServer = $filterServer;
        $this->generalConfig = $filterServer->getGeneralConfig($config);
        $this->fileRemoveConfig = $filterServer->getFileRemoveConfig($config);
        $this->pluginManager = $filterServer->getPluginManager();
        $this->config = $config;

        $this->init();
    }

    /**
     * Initialize Config with different sources
     */
    protected function init()
    {
        $this
            ->setErrorBehavior($this->generalConfig->getErrorBehavior())
            ->setMimeTypes($this->fileRemoveConfig->getRemoveSettings())
            ->setRemoveUnfiltered($this->fileRemoveConfig->isRemoveUnfiltered())
        ;

        $plugins = array();
        foreach ($this->pluginManager->getPlugins() as $plugin) {
            $plugins[] = new FormPluginConfig($plugin, $this->config);
        }

        $this->setPlugins($plugins);
    }

    /**
     * Write changes back to configs
     */
    public function save()
    {
        $this->generalConfig->setErrorBehavior($this->getErrorBehavior());
        // for now we do the filtering independent of the roaming state
        $this->generalConfig->setHandleNonRoaming(true);
        $this->fileRemoveConfig->setRemoveSettings($this->getMimeTypes());
        $this->fileRemoveConfig->setRemoveUnfiltered($this->getRemoveUnfiltered());

        $pluginStatus = array();
        foreach ($this->getPlugins() as $plugin) {
            $pluginStatus[$plugin->getKey()] = $plugin->isEnabled();
            $plugin->save();
        }

        $this->pluginManager->setPluginsStatus($this->config, $pluginStatus);
        $this->filterServer->getCacheManager()->clear($this->config);
    }

    /**
     * Validation Vodoo
     * @param \Symfony\Component\Validator\Mapping\ClassMetadata $metadata
     */
    public static function loadValidatorMetadata(ClassMetadata $metadata)
    {
        $metadata
            ->addPropertyConstraint("errorBehavior", new Assert\NotBlank())
            ->addPropertyConstraint("mimeTypes", new \Topalis\ActiveSyncFilter\Validator\MimeType())
        ;
    }

    public function setErrorBehavior($errorBehavior)
    {
        $this->errorBehavior = $errorBehavior;

        return $this;
    }

    public function getErrorBehavior()
    {
        return $this->errorBehavior;
    }

    /**
     * @param array $mimeTypes
     *
     * @return $this
     */
    public function setMimeTypes(array $mimeTypes)
    {
        $this->mimeTypes = $mimeTypes;

        return $this;
    }

    /**
     * @return array
     */
    public function getMimeTypes()
    {
        return $this->mimeTypes;
    }

    public function setRemoveUnfiltered($removeUnfiltered)
    {
        $this->removeUnfiltered = $removeUnfiltered;

        return $this;
    }

    public function getRemoveUnfiltered()
    {
        return $this->removeUnfiltered;
    }

    /**
     * @param array $plugins
     *
     * @return $this
     */
    public function setPlugins(array $plugins)
    {
        $this->plugins = $plugins;

        return $this;
    }

    /**
     * @return FormPluginConfig[]
     */
    public function getPlugins()
    {
        return $this->plugins;
    }
}
