<?php
namespace Topalis\ActiveSyncFilter\Log;

/**
 * Class to access the log of an ActiveSync Filter Server.
 *
 * @author Dominic Tubach
 */
class ActiveSyncFilterLog
{
    /**
     * @var \Topalis\Database\MySqlDb
     */
    private $db;
    private $table;

    /**
     * Constructor.
     *
     * @param MySqlDb $db the database containing the log.
     * @param string $tablePrefix The table prefix used in the Filter Server.
     */
    public function __construct(\Topalis\Database\MySqlDb $db, $tablePrefix)
    {
        $this->db = $db;
        $this->table = $tablePrefix . "log";
    }

    /**
     * @return \Topalis\Database\MySqlDb
     */
    protected function getDB()
    {
        return $this->db;
    }

    protected function getTable()
    {
        return $this->table;
    }

    /**
     * Returns log entries in the specified period.
     *
     * @param int $from UNIX timestamp defining the begin of the period.
     * @param int $to UNIX timestamp defining the end of the period.
     * @param string $order The ordering of the entries by id meaning the time
     *     when the rows where inserted into the database (ASC or DESC).
     * @param int $limit maximum number of entries; selects the last entries
     *     in the specified period.
     * @return LogData[] Array of LogData objects containing the log entries.
     */
    public function getEntries($from, $to, $order = "DESC", $limit = null)
    {
        $sql = "SELECT * FROM " . $this->getTable() . " WHERE starttime >= :from AND starttime <= :to ORDER BY id DESC";
        if (is_int($limit)) {
            $sql .= " LIMIT " . $limit;
        }
        $rows = $this->getDB()->queryObjects(
            $sql,
            array("from" => $from, "to" => $to)
        );

        return $this->rowsToLogData($rows, $order);
    }

    /**
     * Returns the sum of the sizes of the handled ActiveSync messages in the
     * specified period.
     *
     * @param int $from UNIX timestamp defining the begin of the period.
     * @param int $to UNIX timestamp defining the end of the period.
     * @return int The sum of the sizes.
     */
    public function getInSize($from = null, $to = null)
    {
        $sql = "SELECT SUM(insize) FROM " . $this->getTable();
        $where = "";
        $params = array();
        if ($from !== null) {
            $where = "starttime >= :from";
            $params["from"] = $from;
        }
        if ($to !== null) {
            if (!empty($where)) {
                $where .= " AND ";
            }
            $where .= "starttime <= :to";
            $params["to"] = $to;
        }
        if (!empty($where)) {
            $sql .= " WHERE " . $where;
        }

        return intval($this->getDB()->queryValue($sql, $params));
    }

    /**
     * Returns the specified number of latest log entries.
     *
     * @param int $limit the number of log entries to return.
     * @param string $order The ordering of the entries by id meaning the time
     *     when the rows where inserted into the database (ASC or DESC).
     * @return LogData[] Array of LogData objects containing the log entries.
     */
    public function getLastEntries($limit, $order = "DESC")
    {
        $sql = "SELECT * FROM " . $this->getTable() . " ORDER BY id DESC LIMIT " . intval($limit);
        $rows = $this->getDB()->queryObjects($sql);
        return $this->rowsToLogData($rows, $order);
    }

    /**
     * Returns the sum of the sizes of the filtered ActiveSync messages in the
     * specified period.
     *
     * @param int $from UNIX timestamp defining the begin of the period.
     * @param int $to UNIX timestamp defining the end of the period.
     * @return int The sum of the sizes.
     */
    public function getOutSize($from = null, $to = null)
    {
        $sql = "SELECT SUM(outsize) FROM " . $this->getTable();
        $where = "";
        $params = array();
        if ($from !== null) {
            $where = "starttime >= :from";
            $params["from"] = $from;
        }
        if ($to !== null) {
            if (!empty($where)) {
                $where .= " AND ";
            }
            $where .= "starttime <= :to";
            $params["to"] = $to;
        }
        if (!empty($where)) {
            $sql .= " WHERE " . $where;
        }

        return intval($this->getDB()->queryValue($sql, $params));
    }

    /**
     * Returns the date where the first request was handled in milliseconds
     * since 1970-01-01.
     *
     * @return number The date of the first handled request in milliseconds
     *     since 1970-01-01.
     */
    public function getStartDate()
    {
        $sql = "SELECT MIN(starttime) FROM ".$this->getTable();
        $value = $this->getDB()->queryValue($sql);

        return (float) $value; // int is max. 2147483647
    }

    /**
     * Converts the specified rows of the log table to an array of LogData
     *     objects.
     * @param \stdClass[] $rows Array of rows of the log table.
     * @param string $order The order of the resulting array (ASC or DESC).
     * @return LogData[] The resulting array of LogData objects.
     */
    protected function rowsToLogData(array $rows, $order)
    {
        $result = array();
        if (strtoupper($order) == "ASC") {
            for ($i = count($rows) - 1; $i >= 0; $i--) {
                $result[] = $this->rowToLogData($rows[$i]);
            }
        } else {
            foreach ($rows as $row) {
                $result[] = $this->rowToLogData($row);
            }
        }

        return $result;
    }

    /**
     * Converts the specified row of the log table to a LogData object.
     *
     * @param \stdClass $row log table row.
     * @return LogData the resulting LogData object.
     */
    protected function rowToLogData(\stdClass $row)
    {
        $logData = new LogData($row->id);
        $logData->setCommand($row->command);
        $logData->setEndTime($row->endtime);
        $logData->setInSize($row->insize);
        $logData->setOutSize($row->outsize);
        $logData->setStartTime($row->starttime);
        $logData->setUser($row->user);
        return $logData;
    }
}
