<?php
namespace Topalis\ActiveSyncFilter\Plugin;

use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\Configuration\DatabaseConfiguration;

/**
 * This class allows access to the configuration entries of a plugin.
 *
 * @author Dominic Tubach
 */
class PluginConfig
{
    /**
     * @var \Topalis\Configuration\Configuration
     */
    private $configuration;

    /**
     * @var PluginConfigMetaInterface
     */
    private $meta;

    /**
     * @var array containing PluginConfigMetaInterface objects
     */
    private $entries = array();

    /**
     * Constructor.
     *
     * @param PluginConfigMetaInterface $configMeta The meta object of this
     *     plugin config.
     * @param ConfigGroup $config The configuration to access.
     */
    public function __construct(PluginConfigMetaInterface $configMeta, ConfigGroup $config)
    {
        $this->meta = $configMeta;

        $plugin = $this->getPlugin();
        $filterserver = $plugin->getPluginManager()->getFilterServer();
        $db = $filterserver->getDatabase();
        $dbConfig = new DatabaseConfiguration(
            $db,
            $filterserver->getServerConfig()->getJdbcTablePrefix()
            . "conf_plugin",
            "key",
            "value"
        );
        $dbConfig->addRequiredValue("configid", $config->getId());
        $dbConfig->addRequiredValue("pluginkey", $this->getPlugin()->getKey());
        $this->configuration = $dbConfig;

        foreach ($this->getMeta()->getEntries() as $key => $entryMeta) {
            $this->entries[$key] = new PluginConfigEntry($entryMeta, $dbConfig);
        }
    }

    /**
     * Clears the configuration.
     */
    public function clear()
    {
        $this->configuration->clear();
    }

    /**
     * @param string $key Key of a configuration entry.
     * @return boolean true if a configuration entry with the specified key
     *   exists, false otherwise.
     */
    public function containsEntry($key)
    {
        return $this->getMeta()->containsEntry($key);
    }

    /**
     * @return PluginConfigEntry[] All configuration entries.
     */
    public function getEntries()
    {
        return $this->entries;
    }

    /**
     * @param string $key The configuration key to get the configuration entry
     *   for.
     * @return PluginConfigEntry|NULL The configuration entry of the
     * specified key, or null if such an entry does not exist.
     */
    public function getEntry($key)
    {
        return $this->containsEntry($key) ? $this->entries[$key] : null;
    }

    /**
     * @return \Topalis\ActiveSyncFilter\Plugin\Plugin The plugin this config
     *     belongs to.
     */
    public function getPlugin()
    {
        return $this->getMeta()->getPlugin();
    }

    /**
     * Reverts the configuration by setting all entries to their default value.
     */
    public function revert()
    {
        foreach ($this->getEntries() as $entry) {
            $entry->setValue($entry->getDefault());
        }
    }

    /**
     * @return \Topalis\ActiveSyncFilter\Plugin\PluginConfigMetaInterface
     */
    protected function getMeta()
    {
        return $this->meta;
    }
}
