<?php
namespace Topalis\ActiveSyncFilter\Plugin;

use Topalis\Configuration\Configuration;

/**
 * This class allows access to a plugin configuration entry.
 *
 * @author Dominic Tubach
 */
class PluginConfigEntry
{

    /**
     * @var Configuration
     */
    private $configuration;

    /**
     * @var PluginConfigEntryMetaInterface
     */
    private $meta;

    /**
     * Constructor.
     *
     * @param PluginConfigEntryMetaInterface $configEntryMeta The meta object
     *         belonging to this plugin config entry.
     * @param Configuration $configuration The configuration to access the the
     *         configuration value.
     */
    public function __construct(PluginConfigEntryMetaInterface $configEntryMeta, Configuration $configuration)
    {
        $this->meta = $configEntryMeta;
        $this->configuration = $configuration;
    }

    /**
     * @param boolean $localized Specifies if the returned name shall be
     *   localized if possible.
     * @return string The name of the configuration entry.
     */
    public function getName($localized = true)
    {
        return $this->getMeta()->getName($localized);
    }

    /**
     * @return string The key of the configuration entry.
     */
    public function getKey()
    {
        return $this->getMeta()->getKey();
    }

    /**
     * @return string The default value.
     */
    public function getDefault()
    {
        return $this->getMeta()->getDefault();
    }

    /**
     * @return string The type of the configuration entry.
     */
    public function getType()
    {
        return $this->getMeta()->getType();
    }

    /**
     * Returns the localized options for entries of type select.
     *
     * @return string[string] An associative array mapping options to values.
     */
    public function getOptions()
    {
        return $this->getMeta()->getOptions();
    }

    /**
     * @return integer Returns the maximal allowed value for number entries.
     *         For all other types of entries it returns the maximal length of
     *         allowed values. If no value is set PHP_INT_MAX is returned
     *         for number entries and {@link MAX_INT} for all other entries.
     */
    public function getMax()
    {
        return $this->getMeta()->getMax();
    }

    /**
     * @return integer Returns the minimal allowed value for number entries.
     *         For all other types of entries it returns the minimal length of
     *         allowed values. If no value is set (PHP_INT_MAX * -1)-1 is
     *         returned for numbers and 0 for all other entries.
     */
    public function getMin()
    {
        return $this->getMeta()->getMin();
    }

    /**
     * @return integer The maximal string length of allowed values. Returns the
     *         same value as {@link getMax()} for non-number entries. For number
     *         entries the maximal length is calculated based on the values
     *         returned by {@link getMax()} and {@link getMin()}.
     */
    public function getMaxLength()
    {
        return $this->getMeta()->getMaxLength();
    }

    /**
     * @return integer The minimal string length of allowed values. Returns the
     *         same value as {@link getMin()} for non-number entries. For number
     *         entries the minimal length is calculated based on the values
     *         returned by {@link getMax()} and {@link getMin()}.
     */
    public function getMinLength()
    {
        return $this->getMeta()->getMinLength();
    }

    /**
     * @param boolean $localized Specifies if the returned description shall be
     *   localized if possible.
     * @return string The description of the configuration entry.
     */
    public function getDesc($localized = true)
    {
        return $this->getMeta()->getDesc($localized);
    }

    /**
     * @return string[] Array of keys of entries that have to be disabled if
     *         this entry is enabled. This is only relevant for entries of type
     *         bool.
     */
    public function getDisablesKeys()
    {
        return $this->getMeta()->getDisablesKeys();
    }

    /**
     * @return string[] Array of keys of entries that require this entry to be
     *         enabled. This is only relevant for entries of type boolean.
     */
    public function getRequiredByKeys()
    {
        return $this->getMeta()->getRequiredByKeys();
    }

    /**
     * @return Plugin The plugin the configuration entry belongs to.
     */
    public function getPlugin()
    {
        return $this->getMeta()->getPlugin();
    }

    /**
     * @return string The value of the configuration entry.
     */
    public function getValue()
    {
        return $this->getConfiguration()->getProperty($this->getKey(), $this->getDefault());
    }

    /**
     * Sets the value of the configuration entry.
     *
     * @param string $value The value to be set. For entries of type integer the
     *         value might be an integer. For entries of type boolean the value
     *         might be a boolean.
     * @throws \InvalidArgumentException if the specified value is not a valid
     *     value for the configuration entry.
     */
    public function setValue($value)
    {
        if (!$this->validateValue($value)) {
            if ($this->isBool()) {
                $msg = "Expected boolean";
            } else if ($this->isString()) {
                $msg = sprintf(
                    "Expected string between %d and %d characters long",
                    $this->getMinLength(),
                    $this->getMaxLength()
                );
            } else if ($this->isInt()) {
                $msg = sprintf("Expected integer between %d and %d", $this->getMin(), $this->getMax());
            } else if ($this->isSelect()) {
                $msg = "Expected one of the following values: "
                    . implode(", ", $this->getOptions()) . " '" . $value . "' given";
            }
            throw new \InvalidArgumentException($msg);
        }
        $this->getConfiguration()->setProperty($this->getKey(), $this->toPropertyStr($value));
    }

    /**
     * @return boolean true if this entry stores a boolean, false otherwise.
     */
    public function isBool()
    {
        return $this->getMeta()->isBool();
    }

    /**
     * @return boolean true if this entry stores an integer, false otherwise.
     */
    public function isInt()
    {
        return $this->getMeta()->isInt();
    }

    /**
     * @return boolean true if this entry is a select (e.g. only the predefined
     *         values are allowed), false otherwise.
     * @see getOptions()
     */
    public function isSelect()
    {
        return $this->getMeta()->isSelect();
    }

    /**
     * @return boolean true if this entry stores a string, false otherwise.
     */
    public function isString()
    {
        return $this->getMeta()->isString();
    }

    /**
     * Checks if the specified value is a valid value for the configuration
     * entry.
     *
     * @param string $value The value to be checked. For entries of type integer
     *         the value might be an integer. For entries of type boolean the
     *         value might be a boolean.
     * @return boolean true if the value is valid, false otherwise.
     */
    public function validateValue($value)
    {
        return $this->getMeta()->validateValue($value);
    }

    /**
     * Converts the specified value to a string that can be stored.
     *
     * @param string|integer|boolean $value The value to be converted.
     * @return string The converted value.
     */
    public function toPropertyStr($value)
    {
        return $this->getMeta()->toPropertyStr($value);
    }

    /**
     * @return \Topalis\Configuration\Configuration A Configuration object to
     *     access the configuration.
     */
    protected function getConfiguration()
    {
        return $this->configuration;
    }

    /**
     * @return \Topalis\ActiveSyncFilter\Plugin\PluginConfigEntryMetaInterface
     */
    protected function getMeta()
    {
        return $this->meta;
    }
}
