<?php

namespace Topalis\Admin\Entity;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Security\Core\Role;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Topalis\Apache\Apache;
use Topalis\Util\PasswordUtil;

/**
 * Admin
 * @ORM\Table(name="admins")
 * @ORM\Entity
 * @UniqueEntity(fields={"login"}, message="admin.usernameTaken")
 */
class Admin implements UserInterface, \Serializable
{
    const LEVEL_ADMIN = 0;
    const LEVEL_MONITOR = 1;
    const LEVEL_SUPPORTER = 2;

    /**
     * @var integer
     * @ORM\Column(name="id", type="integer", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    private $id;

    /**
     * @var integer
     * @ORM\Column(name="level", type="integer", nullable=false, options={"unsigned"=true})
     */
    private $level;

    /**
     * @var string
     * @ORM\Column(name="login", type="string", length=50, nullable=false, unique=true,
     *     options={"comment"="The unique login name"})
     */
    private $login;

    /**
     * @var string
     * @ORM\Column(name="pwhash", type="string", length=80, nullable=true,
     *     options={"comment"="Password hash for GUI"})
     */
    private $pwhash;

    /**
     * @var string
     * @ORM\Column(name="davhash", type="string", length=80, nullable=true,
     *     options={"comment"="HTTP password hash for WebDAV access"})
     */
    private $davhash;

    /**
     * @var string
     * @ORM\Column(name="realname", type="string", length=50, nullable=true,
     *     options={"comment"="The real name"})
     */
    private $realname;

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set level
     *
     * @param integer $level
     *
     * @return Admin
     */
    public function setLevel($level)
    {
        $this->level = $level;

        return $this;
    }

    /**
     * Get level
     *
     * @return integer
     */
    public function getLevel()
    {
        return $this->level;
    }

    /**
     * Set login
     *
     * @param string $login
     *
     * @return Admin
     */
    public function setLogin($login)
    {
        $this->login = $login;

        return $this;
    }

    /**
     * Get login
     *
     * @return string
     */
    public function getLogin()
    {
        return $this->login;
    }

    /**
     * Set pwhash
     *
     * @param string $pwhash
     *
     * @return Admin
     */
    public function setPwhash($pwhash)
    {
        $this->pwhash = $pwhash;

        return $this;
    }

    /**
     * Get pwhash
     *
     * @return string
     */
    public function getPwhash()
    {
        return $this->pwhash;
    }

    /**
     * Set davhash
     *
     * @param string $davhash
     *
     * @return Admin
     */
    public function setDavhash($davhash)
    {
        $this->davhash = $davhash;

        return $this;
    }

    /**
     * Get davhash
     *
     * @return string
     */
    public function getDavhash()
    {
        return $this->davhash;
    }

    /**
     * Set realname
     *
     * @param string $realname
     *
     * @return Admin
     */
    public function setRealname($realname)
    {
        $this->realname = $realname;

        return $this;
    }

    /**
     * Get realname
     *
     * @return string
     */
    public function getRealname()
    {
        return $this->realname;
    }

    /**
     * @return boolean always false.
     */
    public function hasPlainPassword()
    {
        return false;
    }

    /**
     * @param string $plainPassword
     *
     * @return self
     */
    public function setPlainPassword($plainPassword)
    {
        if (null !== $plainPassword) {
            $this->setPwhash(PasswordUtil::createSaltedPassword($plainPassword));
            $this->setDavHash(Apache::genHtpasswdHash($plainPassword));
        }

        return $this;
    }

    /**
     * Returns the roles granted to the user.
     * <code>
     * public function getRoles()
     * {
     *     return array('ROLE_USER');
     * }
     * </code>
     * Alternatively, the roles might be stored on a ``roles`` property,
     * and populated in any number of different ways when the user object
     * is created.
     * @return Role[] The user roles
     */
    public function getRoles()
    {
        switch ($this->getLevel()) {
            case self::LEVEL_ADMIN:
                return array("ROLE_ADMIN");
            case self::LEVEL_MONITOR:
                return array("ROLE_MONITOR");
            case self::LEVEL_SUPPORTER:
                return array("ROLE_SUPPORTER");
            default:
                return array();
        }
    }

    /**
     * Returns the password used to authenticate the user.
     * This should be the encoded password. On authentication, a plain-text
     * password will be salted, encoded, and then compared to this value.
     * @return string The password
     */
    public function getPassword()
    {
        $a = explode(':', $this->pwhash);
        return isset($a[1]) ? $a[1] : null;
    }

    /**
     * Returns the salt that was originally used to encode the password.
     * This can return null if the password was not encoded using a salt.
     * @return string The salt
     */
    public function getSalt()
    {
        $a = explode(':', $this->pwhash);

        return $a[0];
    }

    /**
     * Returns the username used to authenticate the user.
     * @return string The username
     */
    public function getUsername()
    {
        return $this->getLogin();
    }

    /**
     * Removes sensitive data from the user.
     * This is important if, at any given point, sensitive information like
     * the plain-text password is stored on this object.
     * @return void
     */
    public function eraseCredentials()
    {
        // we have not plain password to erase
    }

    /**
     * @see \Serializable::serialize()
     */
    public function serialize()
    {
        return serialize(array(
            $this->id,
        ));
    }

    /**
     * @param string $serialized The value returned by serialize().
     * @see \Serializable::unserialize()
     */
    public function unserialize($serialized)
    {
        list ($this->id) = unserialize($serialized);
    }

    public function __toString()
    {
        return get_class($this) . "::" . $this->getLogin() . " (id:" . $this->getId().")";
    }
}
