<?php
namespace Topalis\Apache\Command;

use Topalis\Apache\Apache;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * This command gets or sets values in the apache config. Returns 64 if a value
 * on set is unchanged. Returns 65 if a value on get doesn't exist.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ApacheAccessConfigCommand extends ContainerAwareCommand
{
    const EX_UNCHANGED = 64;
    const EX_NO_SUCH_VALUE = 65;

    /**
     * @return \Topalis\Apache\Apache
     */
    private function getApache()
    {
        return $this->getContainer()->get("apache");
    }

    protected function configure()
    {
        $this
        ->setName("apache:config")
        ->setDescription("Get or set values in the Apache HTTPD config.")
        ->setHelp(
            "Get or set values in the Apache HTTPD config. If no value is defined the current value is returned.")
        ->addArgument("name", InputArgument::REQUIRED, "The variable name to access")
        ->addArgument("value", InputArgument::OPTIONAL, "The value to set")
        ->addOption("restart", null, InputOption::VALUE_NONE,
            "Apache HTTPD gets restarted if a value is changed, otherwise it gets reloaded")
        ->addOption("no-reload", null, InputOption::VALUE_NONE, "Apache HTTPD is not reload if a value is canged")
        ;
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $name = $input->getArgument("name");
        $value = $input->getArgument("value");

        $apache = $this->getApache();

        if (strlen($value) > 0) {
            $restart = $input->getOption("restart");
            $noReload = $input->getOption("no-reload");
            if ($restart && $noReload) {
                $output->writeln("--restart and --no-reload may not be specified at the same time.");
                return 1;
            }
            if ($restart) {
                $action = Apache::ACTION_RESTART;
            } else if ($noReload) {
                $action = Apache::ACTION_NONE;
            } else {
                $action = Apache::ACTION_RELOAD;
            }
            $changed = $apache->setDefine($name, $value, $action);
            if ($changed) {
                switch ($action) {
                    case Apache::ACTION_RELOAD:
                        $output->writeln("Configuration changed. Reload Apache HTTPD.");
                        break;
                    case Apache::ACTION_RESTART:
                        $output->writeln("Configuration changed. Restart Apache HTTPD.");
                        break;
                    default:
                        $output->writeln("Configuration changed.");
                        break;
                }
                return 0;
            } else {
                $output->writeln("Configuration unchanged.");
                return self::EX_UNCHANGED;
            }
        } else {
            $value = $apache->getDefine($name);
            if ($value !== null) {
                $output->writeln($value);
            } else {
                $output->writeln("No such value exists.");
                return self::EX_NO_SUCH_VALUE;
            }
        }
    }
}
