<?php
namespace Topalis\Apns;

use Topalis\MDM\Entity\MdmCommand;
use Topalis\Device\Entity\Device;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * This class extends the push server and adds auralis specific functionality.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class AuralisApnsPushServer extends ApnsPushServer implements ContainerAwareInterface
{
    private $apnsCert;
    private $apnsCertFilename;
    private $apnsCertParsed;

    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    /**
     * @var \Psr\Log\LoggerInterface $logger
     */
    private $psrLogger;

    /**
     * @param boolean $useDevelopmentApns
     * @throws ApnsCertificateLoadException
     */
    public function __construct($useDevelopmentApns)
    {
        $environment = $useDevelopmentApns ?
            \ApnsPHP_Abstract::ENVIRONMENT_SANDBOX : \ApnsPHP_Abstract::ENVIRONMENT_PRODUCTION;
        parent::__construct($environment, $this->getApnsCertFilename());
    }

    /**
     * Sets the Container.
     *
     * @param ContainerInterface|null $container A ContainerInterface instance,
     *     or null
     */
    public function setContainer(\Symfony\Component\DependencyInjection\ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    protected function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    protected function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     *
     * @return integer
     */
    protected function getDeviceOs()
    {
        static $deviceOs = null;
        if ($deviceOs === null) {
            $deviceTypeFactory = $this->container->get("device.type.factory");
            $deviceOs = $deviceTypeFactory->getType("ios")->getId();
        }

        return $deviceOs;
    }

    /**
     * @return \Symfony\Bridge\Doctrine\RegistryInterface
     */
    protected function getDoctrine()
    {
        return $this->container->get("doctrine");
    }

    /**
     * @return \Topalis\MDM\Entity\MdmCommandEntityManager
     */
    public function getMdmCommandEntityManager()
    {
        return $this->container->get("mdm.entity.manager.mdm_command");
    }

    /**
     * @return \Topalis\MDM\MdmQueue
     */
    protected function getMdmQueue()
    {
        return $this->container->get("mdm.queue");
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    public function getPsrLogger()
    {
        return $this->psrLogger;
    }

    /**
     * @param \Psr\Log\LoggerInterface $psrLogger
     */
    public function setPsrLogger(\Psr\Log\LoggerInterface $psrLogger)
    {
        $this->psrLogger = $psrLogger;
        $this->setLogger(new ApnsPsrLogger($psrLogger));
    }

    public function onConnected()
    {
        parent::onConnected();
        if (!$this->getConfig()->isApnsdActive()) {
            $this->getConfig()->setApnsdActive(true);
        }
    }

    public function onDisconnected()
    {
        parent::onDisconnected();
        if (!$this->isConnected()) {
            $this->getConfig()->setApnsdActive(false);
        }
    }

    /**
     * Adds a for the specified device to the inter-process message queue.
     *
     * @param Device $device The device to send the message to.
     * @see ApnsPHP_Push_Server::add()
     */
    public function pushMessage(Device $device)
    {
        $message = new AuralisMdmMessage($device);
        $this->getPsrLogger()->debug("Send message for device " . $device->getId());
        $this->add($message);
    }

    /**
     * Push messages to all devices that require a message.
     */
    public function pushMessages()
    {
        $this->getDeviceEntityManager()->disableDeleteFilter();
        // clear the entity manager otherwise we will get old entity states
        $this->getDeviceEntityManager()->clear();
        // add new commands to queue
        /** @var $newMdmCommands MdmCommand[] */
        $newMdmCommands = $this->getMdmCommandEntityManager()->findByStatusAndOs(
            MdmCommand::STATUS_NEW, $this->getDeviceOs());
        foreach ($newMdmCommands as $mdmCommand) {
            $device = $mdmCommand->getDevice();
            if ($device->getMdmToken() === null || $device->getMdmPushMagic() === null) {
                $this->getPsrLogger()->debug("Device '". $device->getId() . "' has no MDM token or no MDM push magic");
                continue;
            }
            $this->getMdmQueue()->addCommand($mdmCommand);
            $mdmCommand->setStatus(MdmCommand::STATUS_QUEUED);
            $this->getMdmCommandEntityManager()->save($mdmCommand);
        }

        // send push where required
        $mdmQueue = $this->getMdmQueue();
        $devices = $mdmQueue->getPushRequired();
        $messageSentCount = 0;
        foreach ($devices as $device) {
            if ($messageSentCount >= 10) {
                // don't send too many messages, otherwise APNS blocks
                sleep(10);
                $messageSentCount = 0;
            }
            $this->pushMessage($device);
            $mdmQueue->setWaiting($device, true);
            $mdmQueue->setPushTime($device, time());
            $messageSentCount++;
        }

        // check the error queue
        $aErrorQueue = $this->getErrors();
        if (!empty($aErrorQueue)) {
            $this->getPsrLogger()->info(count($aErrorQueue) . " messages could not be sent");
            foreach ($aErrorQueue as $k => $aMessage) {
                $deviceId = $aMessage['MESSAGE']->getDeviceId();
                // see ApnsPHP_Push for status codes
                if ($aMessage['ERRORS'][0]['statusCode'] == 1 || $aMessage['ERRORS'][0]['statusCode'] > 8) {
                    // set to not waiting such that we will send a new message
                    // but only if message has been correct (at least according to the first error)
                    $logMsg = "Sending push message $k to device $deviceId failed ("
                        . $aMessage['ERRORS'][0]['statusMessage'] . "). Retry...";
                    $this->getPsrLogger()->warning($logMsg);
                    $device = $this->getDeviceEntityManager()->find($deviceId);
                    $mdmQueue->setWaiting($device, false);
                } else {
                    $logMsg = "Error sending push message $k to device $deviceId: "
                        . $aMessage['ERRORS'][0]['statusMessage'];
                    $this->getPsrLogger()->error($logMsg);
                }
            }
            sleep(60);
        }
    }

    /**
     * @throws ApnsCertificateLoadException
     */
    public function init()
    {
        $this->getConfig()->setApnsdActive(false);
        $this->loadApnsCert();
        // Set the number of concurrent processes
        $this->setProcesses(1);
    }

    protected function getApnsCertFilename()
    {
        if ($this->apnsCertFilename === null) {
            $home = getenv("HOME");
            $dataDir = "$home/.local/share/data/auralis/apnsd";
            $this->apnsCertFilename = $dataDir . "/cert.pem";
        }
        return $this->apnsCertFilename;
    }

    /**
     * Loads the certificate for the APNS.
     * @throws ApnsCertificateLoadException
     */
    protected function loadApnsCert()
    {
        $config = $this->getConfig();
        $this->apnsCert = $config->getApnsCert();
        if (strlen($this->apnsCert) == 0) {
            throw new ApnsCertificateLoadException("No APNS certificate defined");
        }
        $apnsKey = $config->getApnsPrivKey();
        $apnsCertWithKey = $this->apnsCert . "\n" . $apnsKey;
        $home = getenv("HOME");
        $dataDir = "$home/.local/share/data/auralis/apnsd";
        if (!is_dir($dataDir)) {
            if (!mkdir($dataDir, 755, true)) {
                throw new ApnsCertificateLoadException("Could not create directory $dataDir");
            }
        }
        if (!file_put_contents($this->getApnsCertFilename(), $apnsCertWithKey)) {
            throw new ApnsCertificateLoadException(
                "Could not write APNS certificate into " . $this->getApnsCertFilename());
        }

        $this->apnsCertParsed = openssl_x509_parse($this->apnsCert);
    }

    /**
     * Ensures that the APNS certificate is valid. If that's not the
     * case wait until a valid non expired certificate is available.
     * Then start the server forking all processes and return immediately.
     *
     * Every forked process is connected to Apple Push Notification Service on
     * start and enter on the main loop.
     *
     * @throws ApnsCertificateLoadException
     */
    public function start()
    {
        if ($this->apnsCertParsed['validTo_time_t'] < time()) {
            while ($this->apnsCertParsed['validTo_time_t'] < time()) {
                // if APNS cert is invalid wait a minute and reload it
                sleep(60);
                $this->loadApnsCert();
                $this->apnsCertParsed = openssl_x509_parse($this->apnsCert);
            }
        }

        /*
         * we have to disconnect all database connections before forking,
         * otherwise they will be closed when a child exits.
         */
        $connections = array();
        foreach ($this->getDoctrine()->getConnections() as $connection) {
            if ($connection->isConnected()) {
                $connections[] = $connection;
                $connection->close();
            }
        }

        parent::start();

        foreach ($connections as $connection) {
            $connection->connect();
        }
    }
}
