<?php
namespace Topalis\App\Android;

use Buzz\Browser;
use Symfony\Component\DependencyInjection\ContainerAware;
use Symfony\Component\DomCrawler\Crawler;
use Topalis\App\AppStore;
use Topalis\App\Entity\AndroidApp;
use Topalis\App\Entity\App;
use Topalis\Configuration\Client\AuralisCurl;
use Topalis\Util\StringUtil;
use Topalis\App\Util\CrawlerUtil;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class GooglePlayStore extends ContainerAware implements AppStore
{
    /**
     * @var Browser
     */
    protected $browser;

    protected $lookupUrl = "https://play.google.com/store/apps/details?hl={lang}&";

    protected $searchUrl = "https://play.google.com/store/search?q=%s&c=apps&hl={lang}";

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    private function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * @param string $identifier
     *
     * @return AndroidApp
     */
    public function lookupByIdentifier($identifier)
    {
        try {
            $html = $this->getStoreHtml($identifier);
            if (null === $html) {
                return null;
            }

            return $this->parseAppDetails($html);
        } catch (\Exception $e) {
            $this->getLogger()->error(
                sprintf("failed to get details for android app %s: %s", $identifier, $e->getMessage()),
                array(
                    "exception" => $e,
                    "identifier" => $identifier
                )
            );

            return null;
        }
    }

    /**
     * @param AndroidApp[] $apps
     */
    public function refreshApps(array $apps)
    {
        foreach ($apps as $app) {
            if ($app->getType() == App::TYPE_UPLOADED) {
                continue;
            }
            $storeApp = $this->lookupByIdentifier($app->getIdentifier());
            if (null === $storeApp) {
                continue;
            }
            $app->setType(App::TYPE_STORE);
            $app->merge($storeApp);
        }

    }

    public function search($term)
    {
        $lang = $this->getConfig()->getGooglePlayStoreLang();
        $url = str_replace("{lang}", $lang, sprintf($this->searchUrl, $term));
        $response = $this->getBrowser()->get($url);
        $result = $response->getContent();

        $crawler = new Crawler();
        $crawler->addHtmlContent($result);
        $apps = $crawler->filter(".card-list > .card")->each(function (Crawler $node) {
            $app = new AndroidApp();
            $app->setType(App::TYPE_STORE);

            $identifier = $node->attr("data-docid");
            if (!$identifier) {
                throw new \InvalidArgumentException("App identifier not found");
            }
            $app->setIdentifier($identifier);

            $name = CrawlerUtil::getFromCrawler($node, "a.title", "attr", "title");
            if (!$name) {
                throw new \InvalidArgumentException("App name not found");
            }
            $app->setName($name);

            $vendor = CrawlerUtil::getFromCrawler($node, "a.subtitle", "attr", "title");
            $app->setVendor($vendor);

            $price = self::splitPrice(CrawlerUtil::getFromCrawler($node, ".price > span"));
            $app->setPrice($price['price']);
            $app->setCurrency($price['currency']);

            $icon = CrawlerUtil::getFromCrawler($node, "img.cover-image", "attr", "src");
            $largeIcon = preg_replace("=w[0-9]+(-rw)?$=", "w300\\1", $icon);
            $app->setIconLarge($largeIcon);
            $smallIcon = preg_replace("=w[0-9]+(-rw)?$=", "w60\\1", $icon);
            $app->setIcon($smallIcon);

            $app->setSystem($this->isSystemApp($app));

            return $app;
        });

        return $apps;
    }

    /**
     * determine if this app is part of the android os
     *
     * @param AndroidApp $app
     *
     * @return bool
     */
    public function isSystemApp(AndroidApp $app)
    {
        return StringUtil::startsWith($app->getIdentifier(), "com.android.") || "android" == $app->getIdentifier();
    }

    protected function getDatePublished($identifier)
    {
        try {
            // lang must be en to get a date parseable by strtotime()
            $html = $this->getStoreHtml($identifier, "en");
            if (null === $html) {
                return null;
            }

            $crawler = new Crawler();
            $crawler->addHtmlContent($html);
            $datePublished = CrawlerUtil::getFromCrawler($crawler, "div[itemprop='datePublished']");
            $timePublished = strtotime($datePublished);
            if (false === $timePublished) {
                $this->getLogger()->info("Couldn't parse '" . $datePublished . "' into a UNIX timestamp",
                    array("identifier" => $identifier));
                return null;
            }

            return $timePublished;
        } catch (\Exception $e) {
            $this->getLogger()->error(
                sprintf("Failed to get publish date for android app " . $identifier . ": " . $e->getMessage()),
                array(
                    "exception" => $e,
                    "identifier" => $identifier
                )
            );

            return null;
        }
    }

    protected function getStoreHtml($identifier, $lang = null)
    {
        if (null === $lang) {
            $lang = $this->getConfig()->getGooglePlayStoreLang();
        }
        $url = str_replace("{lang}", $lang, $this->lookupUrl . "id=" . $identifier);
        $response = $this->getBrowser()->get($url);
        if ($response->getStatusCode() === 404) {
            return null;
        }

        return $response->getContent();
    }

    /**
     * @param string $html
     *
     * @return AndroidApp
     */
    protected function parseAppDetails($html)
    {
        $crawler = new Crawler();
        $crawler->addHtmlContent($html);

        $app = new AndroidApp();
        $app->setType(App::TYPE_STORE);

        $identifier = CrawlerUtil::getFromCrawler($crawler, "div.details-wrapper", "attr", "data-docid");
        if (!$identifier) {
            $url = CrawlerUtil::getFromCrawler($crawler, "link[rel='canonical']", "attr", "href");
            $query = parse_url($url, PHP_URL_QUERY);
            $queryArr = array();
            parse_str($query, $queryArr);
            if (isset($queryArr['id'])) {
                $identifier = $queryArr['id'];
            }
            if (!$identifier) {
                throw new \InvalidArgumentException("App identifier not found");
            }
        }
        $app->setIdentifier($identifier);

        $details = $crawler->filter(".details-info");
        $name = CrawlerUtil::getFromCrawler($details, ".document-title > div");
        if (!$name) {
            throw new \InvalidArgumentException("App name not found");
        }
        $app->setName($name);

        $vendor = CrawlerUtil::getFromCrawler($details, "div[itemprop='author'] [itemprop='name']");
        $app->setVendor($vendor);

        $description = CrawlerUtil::getFromCrawler($crawler,
            "div[itemprop='description'] > div.id-app-orig-desc", "html");
        $app->setDescription($description);

        $releaseNotes = $crawler->filter("div.recent-change")->each(function (Crawler $node) {
            return $node->text();
        });
        $app->setReleaseNotes($releaseNotes ? implode("<br/>", $releaseNotes) : null);

        $operatingSystem = trim(CrawlerUtil::getFromCrawler($crawler, "[itemprop='operatingSystems']"));
        $operatingSystemSplit = explode(" ", $operatingSystem);
        $minOperatingSystem = reset($operatingSystemSplit);
        if (!is_numeric($minOperatingSystem)) {
            // may be we have a right-to-left language
            $minOperatingSystem = end($operatingSystemSplit);
        }
        if (is_numeric($minOperatingSystem)) {
            $app->setSupports(array("minOSVersion" => $minOperatingSystem));
        }

        $price = self::splitPrice(CrawlerUtil::getFromCrawler($crawler, "[itemprop='price']", "attr", "content"));
        $app->setPrice($price['price']);
        $app->setCurrency($price['currency']);

        $version = trim(CrawlerUtil::getFromCrawler($crawler, "div[itemprop='softwareVersion']"));
        if ($version) {
            $app->setVersion($version);
        } else {
            // version is not set use publish date instead
            $datePublished = $this->getDatePublished($identifier);
            if ($datePublished) {
                $app->setVersion(date("Y-m-d", $datePublished));
            }
        }

        $largeIcon = trim(CrawlerUtil::getFromCrawler($crawler, "[itemprop='image']", "attr", "src"));
        $app->setIconLarge($largeIcon);

        $smallIcon = preg_replace("=w300(-rw)?$=", "w60\\1", $largeIcon);
        $app->setIcon($smallIcon);

        $app->setSystem($this->isSystemApp($app));

        return $app;
    }

    protected static function splitPrice($price)
    {
        $split = preg_split("/(\s+)|(\xc2\xa0)/", $price);
        if (!$split) {
            return array("price" => 0, "currency" => null);
        }

        $rv = array(
            "price" => str_replace(",", ".", $split[0]),
            "currency" => null
        );
        if (isset($split[1])) {
            $rv['currency'] = $split[1];
        }

        return $rv;
    }

    /**
     * @return \Symfony\Bridge\Monolog\Logger
     */
    protected function getLogger()
    {
        return $this->container->get("logger");
    }

    /**
     * @return \Buzz\Browser
     */
    protected function getBrowser()
    {
        if (null !== $this->browser) {
            return $this->browser;
        }

        return $this->browser = new Browser(new AuralisCurl($this->container->get("config.auralis")));
    }
}
