<?php
namespace Topalis\App\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Config\Definition\Exception\Exception;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DomCrawler\Crawler;
use Symfony\Component\Process\ProcessBuilder;
use Topalis\App\Entity\AndroidApp;
use Topalis\Util\FileSystemUtil;

/**
 * Decompiles .apk with apktool
 *
 * @see https://code.google.com/p/android-apktool/
 * @see https://developer.android.com/tools/projects/index.html
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class RefreshUploadedAndroidAppsCommand extends ContainerAwareCommand
{
    protected $iconSizes = array(
        "ldpi", "mdpi", "tvdpi", "hdpi", "xhdpi", "xxhdpi", "xxxhdpi"
    );

    protected function configure()
    {
        $this
            ->setDefinition(array())
            ->setDescription("refresh android apks")
            ->setName("app:android:refresh_uploaded");
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $input; // unused

        /** @var AndroidApp[] $apps */
        $apps = $this->getAppEntityManager()->findBy(array(
            "type"         => AndroidApp::TYPE_UPLOADED,
            "needsUpdate" => 1,
        ));
        foreach ($apps as $app) {
            if (null === $app->getApkPath()) {
                continue;
            }
            $tmpDir = sys_get_temp_dir() . "/aur-android-app-" . $app->getId();
            $this->getLogger()->info("updating android app " . $app->getIdentifier() ." (id: " . $app->getId().")");
            try {
                $processBuilder = new ProcessBuilder();
                $processBuilder
                    ->setPrefix("java")
                    ->setArguments(array(
                        "-jar",
                        __DIR__ . "/../Resources/java/apktool.jar",
                        "d", // decode
                        "-f",
                        "-s", // don't decode classes
                        $app->getApkPath(),
                        $tmpDir
                    ));
                $process = $processBuilder->getProcess();
                $this->getLogger()->info("extracting android app " . $app->getId() . ": " . $process->getCommandLine());
                $process->run();
                if (!$process->isSuccessful()) {
                    $output->writeln($process->getOutput());
                    $output->writeln($process->getErrorOutput());
                    FileSystemUtil::rmdirRecursive($tmpDir);

                    return;
                }
                if ($output->getVerbosity() == OutputInterface::VERBOSITY_DEBUG) {
                    $output->writeln($process->getOutput());
                }
                $manifest = new Crawler(file_get_contents($tmpDir . "/AndroidManifest.xml"));
                $label = $manifest->filterXPath("//manifest/application")->attr("android:label");
                $label = $this->getString($tmpDir, $label);
                $app->setName($label);

                // get icons
                $iconId = $manifest->filterXPath("//manifest/application")->attr("android:icon");
                $icon = $this->getDrawable($tmpDir, $iconId, "mdpi");
                if (null !== $icon) {
                    $path = $this->getUploadDir() . $app->getId() . "/icon.png";
                    @mkdir(dirname($this->getWebDir() . $path), 0755, true);
                    copy($icon, $this->getWebDir() . $path);
                    $app->setIcon($path);
                }
                $largeIcon = $this->getDrawable($tmpDir, $iconId, "xxhdpi");
                if (null !== $largeIcon) {
                    $path = $this->getUploadDir() . $app->getId() . "/large.png";
                    @mkdir(dirname($this->getWebDir() . $path), 0755, true);
                    copy($largeIcon, $this->getWebDir() . $path);
                    $app->setIconLarge($path);
                }

                $app->setNeedsUpdate(false);

                $this->getAppEntityManager()->save($app);
            } catch (Exception $e) {
                $this->getLogger()->error("error while updating android app" . $app->getId() . ":" . $e->getMessage());
            }
            FileSystemUtil::rmdirRecursive($tmpDir);
        }
    }

    /**
     * Get string from android string file in res/values/string.xml
     *
     * @param string $dir
     * @param string $id e.g. @string/app_name
     *
     * @return string
     */
    protected function getString($dir, $id)
    {
        list($file, $key) = explode("/", $id, 2);
        if ("@string" !== $file) {
            return $id;
        }

        $path = $dir . "/res/values/strings.xml";
        if (!file_exists($path)) {
            return $id;
        }
        $content = new Crawler(file_get_contents($path));
        $resource = $content->filterXPath("//resources/string[@name='" . $key . "']");
        if (count($resource) > 0) {
            return $resource->html();
        }

        return $id;
    }

    /**
     * @param string $dir
     * @param string $id
     * @param string $size {@see $iconSizes}
     *
     * @return string|null
     */
    protected function getDrawable($dir, $id, $size)
    {
        list($type, $key) = explode("/", $id, 2);
        if ("@drawable" !== $type) {
            return null;
        }

        $path = $dir . "/res/drawable-" . $size . "/" . $key . ".png";
        if (!file_exists($path)) {
            // get smaller icon
            $sizeIndex = array_search($size, $this->iconSizes);
            if ($sizeIndex == 0) {
                return null;
            }

            return $this->getDrawable($dir, $id, $this->iconSizes[$sizeIndex - 1]);
        }

        return $path;
    }

    /**
     * @return \Topalis\App\Entity\AppEntityManager
     */
    protected function getAppEntityManager()
    {
        return $this->getContainer()->get("app.entity.manager.android_app");
    }

    protected function getWebDir()
    {
        return $this->getContainer()->getParameter("kernel.root_dir") . "/../web";
    }

    /**
     * @return string
     */
    protected function getUploadDir()
    {
        return "/uploads/android_app_icons/";
    }

    /**
     * @return \Symfony\Bridge\Monolog\Logger
     */
    protected function getLogger()
    {
        return $this->getContainer()->get("logger");
    }
}
