<?php
namespace Topalis\App\Controller\Section;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Form\FormError;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Topalis\App\AppStore;
use Topalis\App\Entity\App;
use Topalis\App\Entity\AppState;
use Topalis\App\Form\Handler\AppEditFormHandler;
use Topalis\App\Form\Type\AppSearchForm;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\Device\Entity\Device;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
abstract class GenericAppSectionController extends Controller
{

    /**
     * @param ConfigGroup $group
     * @param bool        $embed
     *
     * @return array
     */
    protected function doManagedAction(ConfigGroup $group = null, $embed = false)
    {
        // get an array with groups and the ids of all devices within this group
        $groupsList = array();
        /** @var ConfigGroup[] $groups */
        $groups = $this->getConfigGroupEntityManager()->createQueryBuilder("c")
            ->select("partial c.{id, name}, partial d.{id}")
            ->join("c.devices", "d")
            ->where("d.os = :os")
            ->setParameter("os", $this->getDeviceOs())
            ->getQuery()
            ->getResult();
        foreach ($groups as $tmpGroup) {
            $devices = array();
            foreach ($tmpGroup->getDevices() as $device) {
                $devices[] = $device->getId();
            }
            $groupsList[$tmpGroup->getId()] = array(
                "id"      => $tmpGroup->getId(),
                "devices" => $devices
            );
        }

        $apps = $this->getAppEntityManager()->createQueryBuilder("a")
            ->select("
                partial a.{id, name, identifier, icon} as app,
                partial c.{id, name}
            ")
            ->leftJoin("a.configGroups", "c")
            ->where("a.managed = 1")
            ->orderBy("a.name")
            ->getQuery()
            ->getResult();

        foreach ($apps as &$data) {
            /** @var $app App */
            $app = $data['app'];
            $devices = array();
            foreach ($app->getConfigGroups() as $tmpGroup) {
                if (!isset($groupsList[$tmpGroup->getId()])) {
                    continue;
                }
                $devices = array_merge($devices, $groupsList[$tmpGroup->getId()]['devices']);
            }

            $installCount = $this->getAppStateEntityManager()->createQueryBuilder("s")
                ->select("COUNT(s)")
                ->where("s.app = :app AND s.state IN (:states) AND s.device IN (:devices)")
                ->setParameter("app", $app)
                ->setParameter("states", AppState::getInstalledStates())
                ->setParameter("devices", $devices)
                ->getQuery()
                ->getSingleScalarResult();
            $data['install_count'] = $installCount;
            $data['devices'] = count($devices);

            unset($data);
        }

        // clear partial loaded objects from cache. To avoid being reused later
        $this->getAppEntityManager()->clear();

        return array(
            "group"          => $group,
            "embed"          => $embed,
            "apps"           => $apps,
            "sections"       => $this->container->getParameter("app.sections"),
            "active_section" => $this->getOsName(),
            "sub_section"    => "managed",
        );
    }

    /**
     * @return array
     */
    protected function doInventoryAction()
    {
        $apps = $this
            ->getInventoryQuery()
            ->getQuery()
            ->getResult()
        ;

        return array(
            "apps"           => $apps,
            "device_count"   => $this->getDeciveCount(),
            "sections"       => $this->container->getParameter("app.sections"),
            "active_section" => $this->getOsName(),
            "sub_section"    => "inventory",
        );
    }

    protected function getDeciveCount()
    {
        return $this->get("device.entity.manager.device")->count(array(
            "os" => $this->getDeviceOs()
        ));
    }

    protected function getInventoryQuery()
    {
        // the device filter does not apply here, so we have to add
        // "d.deleted = 0 OR d.wipestatus = :wipePending"
        // TODO we should should remove the app states for completely removed
        // devices so we can remove the having clause
        return $this->getAppEntityManager()->createQueryBuilder("a")
            ->select("partial a.{id, name, identifier, icon, type, managed} as app,
                COUNTIF((d.deleted = 0 OR d.wipestatus = :wipePending) AND s.state IN (:states)) AS install_count")
            ->leftJoin("a.states", "s")
            ->leftJoin("s.device", "d")
            ->orderBy("a.name")
            ->groupBy("a.id")
            ->having("install_count > 0")
            ->setParameter("wipePending", Device::WIPE_PENDING)
            ->setParameter("states", AppState::getInstalledStates())
        ;
    }

    /**
     * @param Request $request
     * @param App     $app
     *
     * @return array
     */
    protected function doEditAction(Request $request, App $app)
    {
        $formHandler = $this->getEditFormHandler();
        $form = $formHandler->createForm($app);
        try {
            if ($formHandler->process($request, $form, $app)) {
                return new Response("OK");
            } else if ($form->isSubmitted() && $app->getType() == App::TYPE_UPLOADED) {
                // required to have the old package file name
                $oldForm = $form;
                $form = $formHandler->createForm($app);
                foreach ($oldForm->getErrors() as $error) {
                    $form->addError($error);
                }
                foreach ($oldForm as $name => $oldChildForm) {
                    $childForm = $form->get($name);
                    foreach ($oldChildForm->getErrors() as $error) {
                        $childForm->addError($error);
                    }
                }
            } else if (!$form->isSubmitted() && $app->getId() !== null && $app->getVersion() === null) {
                // app is installed on a device, but the metadata wasn't fetched
                // from the store, yet (cron job). so we do it here.
                $this->getAppStore()->refreshApps(array($app));
                $this->getAppEntityManager()->save($app);
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("Error saving app '" . $app->getIdentifier() . "'",
                array("exception" => $e, "id" => $app->getId()));
            $msg = $this->getTranslator()->trans(
                "admin.app.error.save",
                array("%errmsg%" => $e->getMessage())
            );
            $form->addError(new FormError($msg));
        }

        if (null !== $app->getId()) {
            /** @var AppState[] $states */
            $states = $this->getAppStateEntityManager()->createQueryBuilder("s")
                ->select("s, d, u, c")
                ->join("s.device", "d")
                ->join("d.user", "u")
                ->join("d.configGroup", "c")
                ->where("s.app = :app")
                ->setParameter("app", $app)
                ->getQuery()
                ->getResult();
        } else {
            $states = array();
        }

        return array(
            "active_section" => $this->getOsName(),
            "ios_app"    => $app,
            "states"     => $states,
            "form"       => $form->createView()
        );
    }

    /**
     * @param Request $request
     *
     * @return array
     */
    protected function doSearchAction(Request $request)
    {
        $form = $this->createForm(new AppSearchForm());
        $form->handleRequest($request);
        $apps = array();
        if ($form->isValid()) {
            // get ids of apps already in the db with identifiers as keys.
            $managedAppIds = $this->getAppEntityManager()->getAppIds(array("managed" => 1));
            // unmanaged uploaded apps may be replaced by store apps
            $unmanagedAppIds = $this->getAppEntityManager()->getAppIds(array("managed" => 0),
                array("type" => App::TYPE_UPLOADED));
            try {
                $apps = $this->getAppStore()->search($form->get("name")->getData());
                foreach ($apps as $app) {
                    // set id of found app to already existing app
                    // allow showing edit modal instead of add modal for these
                    if (isset($managedAppIds[$app->getIdentifier()])) {
                        $app->setId($managedAppIds[$app->getIdentifier()]);
                        $app->setManaged(true);
                    } else if (isset($unmanagedAppIds[$app->getIdentifier()])) {
                        $app->setId($unmanagedAppIds[$app->getIdentifier()]);
                    }
                }
            } catch (\Exception $e) {
                $form->addError(new FormError(
                    $this->getTranslator()->trans("admin.app.error.appStoreConnection", array(
                        "%errmsg%" => $e->getMessage()
                    ))
                ));
            }
        }

        return array(
            "sections"       => $this->container->getParameter("app.sections"),
            "active_section" => $this->getOsName(),
            "form"           => $form->createView(),
            "apps"           => $apps,
            "sub_section"    => "search",
        );
    }

    /**
     * @param AppState $state
     *
     * @return array
     */
    protected function doRetryInstallAction(AppState $state)
    {
        $flashMessages = array("success" => array(), "error" => array());
        try {
            $removeBeforeInstall = false;
            if ($state->getRejectionReason() == "AppAlreadyQueued") {
                /*
                 * If we want to retry the installation and the last one was
                 * rejected with the reason AppAlreadyQueued we have to send a
                 * remove command first. A remove command must be send in that
                 * case if the installation failed because of network problems,
                 * anyway #AUR-661
                 */
                $removeBeforeInstall = true;
            }
            $this->getMdmAdmin()->installApp($state->getDevice(), $state->getApp(), true, $removeBeforeInstall);
            $flashMessages['success'][] = $this->getTranslator()->trans("admin.app.retryInstallation.success");
        } catch (\Exception $e) {
            $this->getLogger()->error("Error requesting another installation of app " . $state->getApp()->getId()
                . " on device " . $state->getDevice()->getId() . "(state: " . $state->getId() . ")",
                array("exception" => $e));
            $msg = $this->getTranslator()->trans(
                "admin.app.error.retryInstallation",
                array(
                    "%errmsg%" => $e->getMessage()
                )
            );
            $flashMessages['error'][] = $msg;
        }

        return array(
            "flashMessages" => $flashMessages
        );
    }

    /**
     * @param AppState $state
     *
     * @return array
     */
    protected function doRemoveAction(AppState $state)
    {
        $flashMessages = array("success" => array(), "error" => array());
        try {
            $this->getMdmAdmin()->removeApp($state);
            $flashMessages['success'][] = $this->getTranslator()->trans("admin.app.remove.success");
        } catch (\Exception $e) {
            $this->getLogger()->error("Error requesting removal of app " . $state->getApp()->getId()
                . " on device " . $state->getDevice()->getId() . "(state: " . $state->getId() . ")",
                array("exception" => $e));
            $msg = $this->getTranslator()->trans(
                "admin.app.error.remove",
                array(
                    "%errmsg%" => $e->getMessage()
                )
            );
            $flashMessages['error'][] = $msg;
        }

        return array(
            "flashMessages" => $flashMessages
        );
    }

    /**
     * @param ConfigGroup $group
     * @param App         $app
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    protected function doEnableAction(ConfigGroup $group, App $app)
    {
        $app->setManaged(true);
        $app->addConfigGroup($group);
        $this->getAppEntityManager()->save($app);

        return $this->redirect($this->generateUrl("admin_group_section_apps", array(
            "group"      => $group->getId(),
            "subSection" => $this->getOsName(),
        )));
    }

    /**
     * @param ConfigGroup $group
     * @param App          $app
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function doDisableAction(ConfigGroup $group, App $app)
    {
        $app->removeConfigGroup($group);
        $this->getAppEntityManager()->save($app);

        return $this->redirect($this->generateUrl("admin_group_section_apps", array(
            "group"      => $group->getId(),
            "subSection" => $this->getOsName(),
        )));
    }

    protected function getDeviceOs()
    {
        $deviceTypeFactory = $this->container->get("device.type.factory");

        return $deviceTypeFactory->getType($this->getOsName())->getId();
    }

    /**
     * @return \Topalis\ConfigGroup\Entity\ConfigGroupEntityManager
     */
    protected function getConfigGroupEntityManager()
    {
        return $this->get("config_group.entity.manager.config_group");
    }

    /**
     * @return \Topalis\MDM\MdmAdmin
     */
    protected function getMdmAdmin()
    {
        return $this->get("mdm.admin");
    }

    /**
     * @return \Symfony\Bridge\Monolog\Logger
     */
    protected function getLogger()
    {
        return $this->get("logger");
    }

    /**
     * @return \Symfony\Bundle\FrameworkBundle\Translation\Translator
     */
    protected function getTranslator()
    {
        return $this->get("translator");
    }

    /**
     * @return string
     */
    abstract protected function getOsName();

    /**
     * @return \Topalis\App\Entity\AppEntityManager
     */
    abstract protected function getAppEntityManager();

    /**
     * @return \Topalis\App\Entity\AppStateEntityManager
     */
    abstract protected function getAppStateEntityManager();

    /**
     * @return AppStore
     */
    abstract protected function getAppStore();

    /**
     * @return AppEditFormHandler
     */
    abstract protected function getEditFormHandler();
}
