<?php
namespace Topalis\App\Controller\Section;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Topalis\Kernel\Controller;
use Topalis\App\Form\Type\WinPhone\WinPhoneAppPolicyType;
use Topalis\App\Form\Handler\WinPhoneAppPolicyFormHandler;
use Topalis\App\Entity\WinPhoneAppPolicy;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\App\Form\Type\WinPhone\WinPhoneAppPolicyForm;
use Symfony\Component\HttpFoundation\Response;

/**
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 * @Route("/admin/app/winphone", service="app.controller.section.winphone")
 */
class WinPhoneAppSectionController extends Controller
{
    /**
     * @return \Topalis\Database\EntityManager
     */
    private function getWinPhoneAppPolicyEntityManager()
    {
        return $this->container->get("app.entity.manager.winphone_app_policy");
    }

    /**
     * @Route("/lookupName/{id}", name="admin_app_winphone_lookup_name", options={"expose" = true})
     *
     * @param string $id The identifier of the app to lookup
     * @return Response Contains a string with "name (vendor)" on success.
     */
    public function lookupNameAction($id)
    {
        try {
            $app = $this->container->get("app.winphone.store")->lookup($id);
            if ($app === null) {
                $response = new Response(Response::$statusTexts[404], 404);
            } else {
                $response = new Response($app->getName() . " (" . $app->getVendor() . ")");
            }
        } catch (\InvalidArgumentException $e) {
            $response = new Response($e->getMessage(), 400);
        } catch (\Exception $e) {
            $response = new Response($e->getMessage(), 500);
        }

        $response->headers->set("Content-Type", "plain/text; charset=UTF-8");
        return $response;
    }

    /**
     * @Route("/policy/add", name="admin_app_winphone_policy_add", options={"expose" = true})
     * @Template()
     *
     * @return string|Response
     */
    public function policyAddAction()
    {
        $appPolicy = new WinPhoneAppPolicy();

        return $this->forward("app.controller.section.winphone:policyEditAction", array("appPolicy" => $appPolicy));
    }

    /**
     * @Route("/policy", name="admin_app_winphone_policy")
     * @Template()
     *
     * @param ConfigGroup $group
     * @param boolean $embed
     *
     * @return array
     */
    public function policyAction(ConfigGroup $group = null, $embed = false)
    {
        $appPolicies = $this->getWinPhoneAppPolicyEntityManager()->findAll();

        return array(
            "appPolicies" => $appPolicies,
            "embed" => $embed,
            "group"          => $group,
            "active_section" => $this->getSectionName(),
            "subSection" => "policy",
            "sections"       => $this->container->getParameter("app.sections"),
        );
    }

    /**
     * @Route("/policy/{appPolicy}/edit", name="admin_app_winphone_policy_edit")
     * @Template()
     *
     * @param WinPhoneAppPolicy $appPolicy
     *
     * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function policyEditAction(WinPhoneAppPolicy $appPolicy)
    {
        $form = $this->createForm(new WinPhoneAppPolicyForm($this->container->get("app.winphone.store")));
        $handler = new WinPhoneAppPolicyFormHandler($this->getRequest(), $appPolicy);

        if ($handler->process($form)) {
            try {
                $this->getWinPhoneAppPolicyEntityManager()->save($appPolicy);
                $this->addSuccess($this->tr("admin.app.winphone.policy.success.save"));
            } catch (\Exception $e) {
                $this->getLogger()->error("Error saving Windows Phone app policy", array("exception" => $e));
                $msg = $this->tr("admin.app.winphone.policy.error.save", array("%errmsg%" => $e->getMessage()));
                $this->addError($msg);
            }
            return $this->redirect(
                $this->generateUrl("admin_app_winphone_policy")
            );
        }

        return array(
            "form" => $form->createView(),
            "appPolicy" => $appPolicy,
            "active_section" => $this->getSectionName(),
            "subSection" => "policy",
            "sections" => $this->container->getParameter("app.sections"),
        );
    }

    /**
     * @Route("/policy/{appPolicy}/delete", name="admin_app_winphone_policy_delete")
     *
     * @param WinPhoneAppPolicy $appPolicy
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function policyDeleteAction(WinPhoneAppPolicy $appPolicy)
    {
        try {
            $this->getWinPhoneAppPolicyEntityManager()->delete($appPolicy);
            $this->addSuccess($this->tr("admin.app.winphone.policy.success.delete"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error deleting Windows Phone app policy with id " . $appPolicy->getId(),
                array("exception" => $e));
            $msg = $this->tr(
                "admin.app.winphone.policy.error.delete",
                array("%name%" => $appPolicy->getName(), "%errmsg%" => $e->getMessage())
            );
            $this->addError($msg);
        }

        return $this->redirect($this->generateUrl("admin_app_winphone_policy"));
    }

    /**
     * @Route("/policy/{appPolicy}/enable/{group}", name="admin_app_winphone_policy_enable")
     *
     * @param ConfigGroup $group
     * @param WinPhoneAppPolicy $appPolicy
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function policyEnableAction(ConfigGroup $group, WinPhoneAppPolicy $appPolicy)
    {
        $group->setWinPhoneAppPolicy($appPolicy);
        $this->container->get("config_group.entity.manager.config_group")->save($group);

        return $this->redirect($this->generateUrl("admin_group_section_apps", array(
            "group"      => $group->getId(),
            "subSection" => "winphone"
        )));
    }

    /**
     * @Route("/policy/{appPolicy}/disable/{group}", name="admin_app_winphone_policy_disable")
     *
     * @param ConfigGroup $group
     * @param WinPhoneAppPolicy $appPolicy
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function policyDisableAction(ConfigGroup $group, WinPhoneAppPolicy $appPolicy)
    {
        $appPolicy; // unused

        $group->setWinPhoneAppPolicy(null);
        $this->container->get("config_group.entity.manager.config_group")->save($group);

        return $this->redirect($this->generateUrl("admin_group_section_apps", array(
            "group"      => $group->getId(),
            "subSection" => "winphone"
        )));
    }

    protected function getSectionName()
    {
        return "winphone";
    }
}
