<?php
namespace Topalis\App\Entity;

use Doctrine\ORM\Mapping as ORM;
use Topalis\Device\Entity\Device;

/**
 * @ORM\MappedSuperclass
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
abstract class AppState
{
    const INSTALLED = "INSTALLED";
    const INSTALLATION_FAILED = "INSTALLATION_FAILED";
    const INSTALLATION_REJECTED = "INSTALLATION_REJECTED";
    const INSTALLATION_REQUESTED = "INSTALLATION_REQUESTED";
    const REMOVAL_REQUESTED = "REMOVAL_REQUESTED";
    const REMOVED = "REMOVED";
    const UNSUPPORTED = "UNSUPPORTED";
    const UPDATE_FAILED = "UPDATE_FAILED";
    const UPDATE_REJECTED = "UPDATE_REJECTED";
    const UPDATE_REQUESTED = "UPDATE_REQUESTED";
    const UPDATE_UNSUPPORTED = "UPDATE_UNSUPPORTED";

    /**
     * A request is pending or was not rejected.
     * @var int
     */
    const REJECTED_NONE = 0;

    /**
     * A request was rejected by the user.
     * @var int
     */
    const REJECTED_USER = 1;

    /**
     * A request was rejected by the device.
     * @var int
     */
    const REJECTED_DEVICE = 2;

    /**
     * A request was rejected by the system i.e. auralis
     * @var int
     */
    const REJECTED_SYSTEM = 3;

    /**
     * Array with states which indicate that the a version of the app is
     * installed.
     * @var array
     */
    private static $installedStates = array(
        self::INSTALLED,
        self::REMOVAL_REQUESTED,
        self::UPDATE_FAILED,
        self::UPDATE_REJECTED,
        self::UPDATE_REJECTED,
        self::UPDATE_REQUESTED,
        self::UPDATE_UNSUPPORTED,
    );

    /**
     * @var integer
     * @ORM\Column(type="integer", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    protected $id;

    /**
     * Association has to be defined in sub classes.
     * @var App
     */
    protected $app;

    /**
     * @var Device
     * @ORM\ManyToOne(targetEntity="Topalis\Device\Entity\Device")
     */
    protected $device;

    /**
     * @var string
     * @ORM\Column(type="string", nullable=true)
     */
    protected $version;

    /**
     * @var string
     * @ORM\Column(type="string")
     */
    protected $state;

    /**
     * State reported by mdm
     *
     * @var string
     * @ORM\Column(type="string", nullable=true)
     */
    protected $mdmState;

    /**
     * @var \DateTime
     * @ORM\Column(type="datetime", nullable=true)
     */
    protected $stateChangeTime;

    /**
     * @var \DateTime
     * @ORM\Column(type="datetime", nullable=true)
     */
    protected $mdmStateTime;

    /**
     * @var boolean
     * @ORM\Column(type="boolean")
     */
    protected $managed = false;

    /**
     * @var int
     * @ORM\Column(type="smallint")
     */
    protected $rejected = false;

    /**
     * @var string
     * @ORM\Column(type="string", nullable = true)
     */
    protected $rejectionReason;

    /**
     * size of the app in bytes
     *
     * @var int
     * @ORM\Column(type="integer")
     */
    protected $appSize = 0;

    /**
     * size of the app data in bytes
     *
     * @var int
     * @ORM\Column(type="integer")
     */
    protected $dataSize = 0;

    /**
     * @return array Array with states which indicate that the a version of the
     *     app is installed.
     */
    public static function getInstalledStates()
    {
        return self::$installedStates;
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return App
     */
    public function getApp()
    {
        return $this->app;
    }

    /**
     * @param App $app
     *
     * @return $this
     */
    public function setApp(App $app)
    {
        if ($this->app !== $app) {
            if ($this->app !== null) {
                $this->app->removeState($this);
            }
            $this->app = $app;
            $app->addState($this);
        }

        return $this;
    }

    /**
     * @return Device
     */
    public function getDevice()
    {
        return $this->device;
    }

    /**
     * @param Device $device
     *
     * @return $this
     */
    public function setDevice(Device $device)
    {
        $this->device = $device;

        return $this;
    }

    /**
     * @return string
     */
    public function getVersion()
    {
        return $this->version;
    }

    /**
     * @param string $version
     *
     * @return $this
     */
    public function setVersion($version)
    {
        $this->version = $version;

        return $this;
    }

    /**
     * @return string
     */
    public function getState()
    {
        return $this->state;
    }

    /**
     * @param string $state
     *
     * @return $this
     */
    public function setState($state)
    {
        if ($state !== $this->state) {
            $this->setStateChangeTime(new \DateTime());
        }
        $this->state = $state;

        return $this;
    }

    public function isInstalled()
    {
        return in_array($this->state, self::$installedStates);
    }

    /**
     *
     * @return AppState
     */
    public function setFailed()
    {
        $this->state = $this->state == self::UPDATE_REQUESTED ? self::UPDATE_FAILED : self::INSTALLATION_FAILED;

        return $this;
    }

    /**
     * @return boolean
     */
    public function isFailed()
    {
        return $this->state == self::INSTALLATION_FAILED || $this->state == self::UPDATE_FAILED;
    }

    /**
     * @return string
     */
    public function getMdmState()
    {
        return $this->mdmState;
    }

    /**
     * @param string $mdmState
     *
     * @return $this
     */
    public function setMdmState($mdmState)
    {
        if ($mdmState !== $this->mdmState) {
            $this->setMdmStateTime(new \DateTime());
        }
        $this->mdmState = $mdmState;

        return $this;
    }

    /**
     * @return \DateTime
     */
    public function getStateChangeTime()
    {
        return $this->stateChangeTime;
    }

    /**
     * @param \DateTime $stateChangeTime
     *
     * @return $this
     */
    public function setStateChangeTime(\DateTime $stateChangeTime)
    {
        $this->stateChangeTime = $stateChangeTime;

        return $this;
    }

    /**
     * @return \DateTime
     */
    public function getMdmStateTime()
    {
        return $this->mdmStateTime;
    }

    /**
     * @param \DateTime $mdmStateTime
     *
     * @return self
     */
    public function setMdmStateTime(\DateTime $mdmStateTime)
    {
        $this->mdmStateTime = $mdmStateTime;

        return $this;
    }

    /**
     * @return boolean
     */
    public function isManaged()
    {
        return $this->managed;
    }

    /**
     * @param boolean $managed
     *
     * @return $this
     */
    public function setManaged($managed)
    {
        $this->managed = $managed;

        return $this;
    }

    /**
     * @return int One of the class constants starting with REJECTED.
     */
    public function getRejected()
    {
        return $this->rejected;
    }

    /**
     * @param int $rejected One of the class constants starting with REJECTED.
     *
     * @return AppState
     * @throws \InvalidArgumentException If the specified value matches none of
     *     the class constants starting with REJECTED.
     */
    public function setRejected($rejected)
    {
        $rejected = intval($rejected);
        switch ($rejected) {
            case self::REJECTED_NONE:
                /* fall through */
            case self::REJECTED_DEVICE:
                /* fall through */
            case self::REJECTED_USER:
                /* fall through */
            case self::REJECTED_SYSTEM:
                break;
            default:
                throw new \InvalidArgumentException("'". $rejected . "' is not a valid value");
        }

        $this->rejected = $rejected;

        return $this;
    }

    /**
     * @return string
     */
    public function getRejectionReason()
    {
        return $this->rejectionReason;
    }

    /**
     * @param string $rejectionReason
     *
     * @return AppState self
     */
    public function setRejectionReason($rejectionReason)
    {
        $this->rejectionReason = $rejectionReason;

        return $this;
    }

    /**
     * @return int
     */
    public function getAppSize()
    {
        return $this->appSize;
    }

    /**
     * @param int $appSize
     *
     * @return self
     */
    public function setAppSize($appSize)
    {
        $this->appSize = $appSize;

        return $this;
    }

    /**
     * @return int
     */
    public function getDataSize()
    {
        return $this->dataSize;
    }

    /**
     * @param int $dataSize
     *
     * @return self
     */
    public function setDataSize($dataSize)
    {
        $this->dataSize = $dataSize;

        return $this;
    }
}
