<?php
namespace Topalis\App\EventListener\MDM;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Topalis\App\Entity\AppEntityManager;
use Topalis\App\Entity\AppState;
use Topalis\App\Entity\AppStateEntityManager;
use Topalis\App\Entity\MdmAppState;
use Topalis\Device\Entity\Device;
use Topalis\MDM\Entity\MdmCommand;
use Topalis\MDM\MdmAdmin;
use Topalis\MDM\MdmEvent;
use Topalis\MDM\MdmEvents;
use Topalis\App\Entity\App;
use Topalis\MDM\MdmRequest;
use Psr\Log\LoggerInterface;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
abstract class AppMdmListener implements EventSubscriberInterface
{

    /**
     * @var \Topalis\App\Entity\AppEntityManager
     */
    protected $appManager;

    /**
     * @var \Topalis\App\Entity\AppStateEntityManager
     */
    protected $appStateEntityManager;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    /**
     * @var MdmAdmin
     */
    protected $mdmAdmin;

    public function __construct(
        MdmAdmin $mdmAdmin,
        AppEntityManager $appManager,
        AppStateEntityManager $appStateEntityManager,
        LoggerInterface $logger)
    {
        $this->appManager = $appManager;
        $this->appStateEntityManager = $appStateEntityManager;
        $this->mdmAdmin = $mdmAdmin;
        $this->logger = $logger;
    }

    public static function getSubscribedEvents()
    {
        return array(
            MdmEvents::INSTALLED_APPLICATION_LIST => "handleInstalledApplications",
            MdmEvents::INSTALL_APPLICATION        => "handleInstallApplication",
            MdmEvents::REMOVE_APPLICATION         => "handleRemoveApplication",
            MdmEvents::MANAGED_APPLICATION_LIST   => "handleManagedApplicationList",
        );
    }

    public function handleInstalledApplications(MdmEvent $event)
    {
        if (!$event->isAcknowledged()) {
            return;
        }

        if (!$this->supports($event->getDevice())) {
            return;
        }

        $this->handleData(
            $event->getDevice(),
            $event->getContext()->getMdmRequest()->get("InstalledApplicationList")
        );
        $this->mdmAdmin->installAppsForDevice($event->getDevice());
    }

    public function handleInstallApplication(MdmEvent $event)
    {
        $context = $event->getContext();
        $request = $context->getMdmRequest();

        if (!$this->supports($event->getDevice())) {
            return;
        }

        /** @var MdmAppState $state */
        $state = $this->appStateEntityManager->findOneBy(array("installCommand" => $context->getMdmCommand()));
        if (null === $state) {
            // this should never happen...
            $this->logger->error("App state for install command "
                . $context->getMdmCommand()->getId() . " has been deleted for unknown reason");
            return;
        }

        if ($event->isAcknowledged()) {
            $mdmState = $request->get("State");
            $state->setMdmState($mdmState);
            // if ("NeedsRedemption" == $mdmState) {
            // TODO VPP
            // }
        } else if ($event->isError()) {
            if ($state->getState() == AppState::UPDATE_REQUESTED) {
                $state->setState(AppState::UPDATE_REJECTED);
            } else {
                $state->setState(AppState::INSTALLATION_REJECTED);
            }
            $state->setRejected(AppState::REJECTED_DEVICE);
            $rejectionReason = $request->get("RejectionReason");
            if ($rejectionReason === null) {
                $rejectionReason = "CommandError";
            } else {
                switch ($rejectionReason) {
                    case "AppAlreadyInstalled":
                        $state->setState(AppState::INSTALLED);
                        break;
                }
            }
            $state->setRejectionReason($rejectionReason);
        }
        $this->appStateEntityManager->save($state);
    }

    public function handleRemoveApplication(MdmEvent $event)
    {
        $context = $event->getContext();

        if (!$this->supports($event->getDevice())) {
            return;
        }

        /** @var MdmAppState $state */
        $state = $this->appStateEntityManager->findOneBy(array("removeCommand" => $context->getMdmCommand()));
        if (null === $state) {
            return;
        }
        if ($event->isAcknowledged()) {
            $state->setState(AppState::REMOVED);
            $state->setRejected(AppState::REJECTED_NONE);
        } else if ($event->isError()) {
            $state->setRejected(AppState::REJECTED_DEVICE);
            $errors = $event->getContext()->getMdmRequest()->getErrorChain();
            $error = reset($errors);
            if ($error) {
                $state->setRejectionReason(MdmRequest::getErrorDescription($error));
            }
            foreach ($errors as $error) {
                // App not managed
                if ("MCMDMErrorDomain" === $error['ErrorDomain'] && 12029 === $error['ErrorCode']) {
                    $state->setState(AppState::REMOVED);
                    $state->setRejected(AppState::REJECTED_NONE);
                    break;
                }
            }
        }
        if ($state->getInstallCommand()
            && ($state->getInstallCommand()->getStatus() == MdmCommand::STATUS_NEW
            || $state->getInstallCommand()->getStatus() == MdmCommand::STATUS_QUEUED)
        ) {
            // app will be reinstalled
            $state
                ->setState(AppState::INSTALLATION_REQUESTED)
                ->setRejected(AppState::REJECTED_NONE);
            $this->appStateEntityManager->save($state);
        } else if ($state->getState() == AppState::REMOVED) {
            $app = $state->getApp();
            $device = $state->getDevice();
            if ($app->isManaged() && $app->getConfigGroups()->contains($device->getConfigGroup())) {
                /*
                 * this may happen if an old version of an app has been removed
                 * and the new one is unsupported. nevertheless lets try again
                 * to get the correct state.
                 */
                $state->setInstallAttempts(0);
                $this->appStateEntityManager->save($state);
                $this->mdmAdmin->installApp($state->getDevice(), $state->getApp(), true);
            } else {
                $this->appStateEntityManager->delete($state);
            }
        }
    }

    /**
     * @param MdmEvent $event
     *
     * @return AppState[]
     */
    public function handleManagedApplicationList(MdmEvent $event)
    {
        $context = $event->getContext();
        $request = $context->getMdmRequest();

        if (!$this->supports($event->getDevice())) {
            return;
        }

        if (!$event->isAcknowledged()) {
            return;
        }
        $stateList = array();
        $states = (array) $request->get("ManagedApplicationList");
        foreach ($states as $identifier => $appState) {
            $app = $this->appManager->findOneBy(array("identifier" => $identifier));
            /** @var AppState $state */
            $state = $this->appStateEntityManager->findOneBy(array("app" => $app, "device" => $context->getDevice()));
            if (null === $state) {
                continue;
            }
            $mdmState = $appState['Status'];
            $state->setMdmState($mdmState);
            switch ($mdmState) {
                case "Failed":
                    $state
                        ->setFailed()
                        ->setRejected(AppState::REJECTED_DEVICE)
                        ->setRejectionReason($mdmState);
                    break;
                case "UserRejected":
                    $state
                        ->setState(AppState::INSTALLATION_REJECTED)
                        ->setRejected(AppState::REJECTED_USER)
                        ->setRejectionReason($mdmState);
                    break;
                case "UpdateRejected":
                    $state
                        ->setState(AppState::UPDATE_REJECTED)
                        ->setRejected(AppState::REJECTED_USER)
                        ->setRejectionReason($mdmState);
                    break;
                case "UserInstalledApp":
                    $state
                        ->setState(AppState::INSTALLED)
                        ->setManaged(false)
                        ->setRejected(AppState::REJECTED_DEVICE)
                        ->setRejectionReason($mdmState);
                    break;
                case "ManagedButUninstalled":
                    $state
                        ->setState(AppState::REMOVED)
                        ->setManaged(true);
                    break;
                case "Managed":
                    if ($app->getType() != App::TYPE_UPLOADED
                        || $app->getVersion() == $state->getVersion()
                        || ($state->getState() == AppState::UPDATE_REQUESTED
                        && $state->getRejected() === AppState::REJECTED_NONE)) {
                        // if the last condition (state is UPDATE_REQUESTED) matched
                        // the update was accepted, but the new version is not yet updated,
                        // which will be done when handling InstalledApplications.
                        $state->setState(AppState::INSTALLED);
                        $state->setRejected(AppState::REJECTED_NONE);
                        $state->setRejectionReason(null);
                    }
                    $state->setManaged(true);

                    // app is managed but shouldn't. This can happen if the config group is added and removed again
                    // before the first managed app query happened #AUR-665
                    if (!$app->getConfigGroups()->contains($event->getDevice()->getConfigGroup())) {
                        $this->appStateEntityManager->save($state);
                        $this->mdmAdmin->removeApp($state);
                    }
                    break;
            }
            $this->appStateEntityManager->save($state);
            $stateList[] = $state;
        }

        return $stateList;
    }

    protected function handleData(Device $device, array $data)
    {
        /** @var AppState[] $states */
        $states = $this->appStateEntityManager->createQueryBuilder("s")
            ->select("a, s")
            ->leftJoin("s.app", "a")
            ->where("s.device = :device")
            ->setParameter("device", $device)
            ->getQuery()
            ->getResult();
        $stateIndex = array();
        foreach ($states as $state) {
            $stateIndex[$state->getApp()->getIdentifier()] = $state;
        }
        $installed = array();

        foreach ($data as $app) {
            if (!isset($app['Identifier'])) {
                // if there's no Identifier there's something really wrong
                continue;
            }
            if (empty($app['Name'])) {
                // use Identifier as Name if there's none
                // e.g. "Space Borders: Alien Encounter" returned no Name (iTunes Store ID 490625226)
                $app['Name'] = $app['Identifier'];
            }
            $app += array(
                "ShortVersion" => null,
                "Version"      => null,
                "BundleSize"   => 0,
                "DynamicSize"  => 0,
            );
            $version = $app['ShortVersion'];
            if (null === $version) {
                $version = $app['Version'];
            }
            $installed[] = $app['Identifier'];

            if (!isset($stateIndex[$app['Identifier']])) {
                // was not installed previously
                $appEntity = $this->appManager->getOrCreate($app['Identifier'], $app['Name']);

                /** @var AppState $state */
                $state = $this->appStateEntityManager->create();
                $state
                    ->setApp($appEntity)
                    ->setDevice($device)
                    ->setState(AppState::INSTALLED)
                ;
            } else {
                $state = $stateIndex[$app['Identifier']];
                $appEntity = $state->getApp();
                if ($appEntity->getType() != App::TYPE_UPLOADED || $appEntity->getVersion() == $version) {
                    // for uploaded apps the version must match
                    $state->setState(AppState::INSTALLED);
                }
            }
            $state
                ->setVersion($version)
                ->setAppSize($app['BundleSize'])
                ->setDataSize($app['DynamicSize'])
            ;
            $this->appStateEntityManager->save($state, false);
        }

        $this->appStateEntityManager->flush();

        // detect deleted
        $deleted = array_diff(array_keys($stateIndex), $installed);
        foreach ($deleted as $identifier) {
            $state = $stateIndex[$identifier];
            if ($state->isManaged()) {
                continue;
            }

            // don't remove $state if an installation or update is pending
            switch ($state->getState()) {
                case AppState::INSTALLATION_REQUESTED:
                    /* fall through */
                case AppState::UPDATE_REQUESTED:
                    continue;
            }

            // flush must be done immediately #AUR-541
            $this->appStateEntityManager->delete($state);
        }
    }

    abstract protected function supports(Device $device);
}
