<?php
namespace Topalis\App\EventListener\ORM;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Symfony\Component\HttpFoundation\File\File;
use Topalis\App\Entity\IosApp;
use Topalis\App\Ios\IpaFile;
use Topalis\Util\FileSystemUtil;
use Doctrine\ORM\Event\PostFlushEventArgs;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class IosUploadEventListener
{
    private $inPostFlush = false;
    private $preRemovedAppIds = array();
    private $postRemovedApps = array();

    private $webDir;

    private $uploadDir = "/uploads/ios_app_icons/";

    public function __construct($webDir)
    {
        $this->webDir = $webDir;
    }

    public function prePersist(LifecycleEventArgs $eventArgs)
    {
        $app = $eventArgs->getEntity();
        if (!$app instanceof IosApp) {
            return;
        }
        $this->handleUpload($app);
    }

    public function postPersist(LifecycleEventArgs $eventArgs)
    {
        $app = $eventArgs->getEntity();
        if (!$app instanceof IosApp) {
            return;
        }
        if (null !== $app->getIpaPath()) {
            $app->setIpa(new IpaFile($app->getIpaPath()));
            $this->updateIcons($app, $eventArgs);
        }
    }

    public function preUpdate(LifecycleEventArgs $eventArgs)
    {
        $app = $eventArgs->getEntity();
        if (!$app instanceof IosApp) {
            return;
        }
        $this->handleUpload($app);
    }

    public function postUpdate(LifecycleEventArgs $eventArgs)
    {
        $app = $eventArgs->getEntity();
        if (!$app instanceof IosApp) {
            return;
        }

        $uow = $eventArgs->getEntityManager()->getUnitOfWork();
        $changeSet = $uow->getEntityChangeSet($app);

        if (isset($changeSet['ipaPath'])) {
            if ($changeSet['ipaPath'][0]) {
                // remove previous IPA
                FileSystemUtil::tryUnlinkFile($changeSet['ipaPath'][0]);
            }
            if (!$changeSet['ipaPath'][1]) {
                $this->removeAppDir($app->getId());
            } else {
                $app->setIpa(new IpaFile($changeSet['ipaPath'][1]));
                $this->updateIcons($app, $eventArgs);
            }
        }
    }

    private function removeAppDir($id)
    {
        if ($id <= 0) {
            throw new \InvalidArgumentException("id must be a non negative integer");
        }
        if (!is_int($id)) {
            $intId = (int) $id;
            if ((string) $intId != $id) {
                throw new \InvalidArgumentException("id must be a non negative integer");
            }
        }

        $appDir = $this->webDir . $this->uploadDir . $id;
        if (is_dir($appDir)) {
            FileSystemUtil::rmdirRecursive($appDir);
        }
    }

    private function updateIcons(IosApp $app, LifecycleEventArgs $eventArgs)
    {
        $uow = $eventArgs->getEntityManager()->getUnitOfWork();
        $changeSet = $uow->getEntityChangeSet($app);
        if (!isset($changeSet['ipaPath'])) {
            return;
        }

        $icon = $app->getIpa()->getIcon();
        if (null !== $icon) {
            $app->setIcon(
                $this->uploadIcon($icon['file'], $app->getId() . "/icon.png")
            );
        }
        $largeIcon = $app->getIpa()->getLargeIcon();
        if (null !== $largeIcon) {
            $app->setIconLarge(
                $this->uploadIcon($largeIcon['file'], $app->getId() . "/icon-large.png")
            );
        }
        $eventArgs->getEntityManager()->persist($app);
        $eventArgs->getEntityManager()->flush($app);
    }

    private function uploadIcon($icon, $name)
    {
        $path = $this->webDir . $this->uploadDir . $name;
        @mkdir(dirname($path), 0755, true);
        copy($icon, $path);
        FileSystemUtil::tryUnlinkFile($icon);

        return $this->uploadDir . $name;
    }

    public function preRemove(LifecycleEventArgs $eventArgs)
    {
        $app = $eventArgs->getEntity();
        if (!$app instanceof IosApp) {
            return;
        }

        // ids are not available in postRemove, so we use both events to remove
        // the app dir in postFlush if necessary
        $this->preRemovedAppIds[$app->getIdentifier()] = $app->getId();
    }

    public function postRemove(LifecycleEventArgs $eventArgs)
    {
        $app = $eventArgs->getEntity();
        if (!$app instanceof IosApp) {
            return;
        }

        // ids are not available in postRemove, so we use both events to remove
        // the app dir in postFlush if necessary
        $this->postRemovedApps[$app->getIdentifier()] = $app;
    }

    public function postLoad(LifecycleEventArgs $eventArgs)
    {
        $app = $eventArgs->getEntity();
        if (!$app instanceof IosApp) {
            return;
        }
        if (null !== $app->getIpaPath()) {
            $app->setIpa(new IpaFile($app->getIpaPath()));
        }
    }

    public function postFlush(PostFlushEventArgs $eventArgs)
    {
        $eventArgs; // unused

        if ($this->inPostFlush) {
            return;
        }

        $this->inPostFlush = true;

        foreach ($this->preRemovedAppIds as $identifier => $id) {
            if (isset($this->postRemovedApps[$identifier])) {
                $app = $this->postRemovedApps[$identifier];
                if ($app->getIpaPath()) {
                    FileSystemUtil::tryUnlinkFile($app->getIpaPath());
                }

                $this->removeAppDir($id);
                unset($this->postRemovedApps[$identifier]);
            }
            unset($this->preRemovedAppIds[$identifier]);
        }

        $this->inPostFlush = false;
    }

    private function handleUpload(IosApp $app)
    {
        if (null === $app->getIpaPath()) {
            return;
        }
        if (!$app->getIpaPath() instanceof File) {
            return;
        }
        $file = $app->getIpaPath();
        $ipa = new IpaFile($file->getPathname());

        $app->setIpaPath($file->getPathname());
        $app->setIpa($ipa);
    }
}
