<?php
namespace Topalis\App\Export;

use PHPExcel;
use PHPExcel_IOFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpFoundation\StreamedResponse;
use Symfony\Component\Translation\TranslatorInterface;
use Topalis\App\Entity\AndroidApp;
use Topalis\App\Entity\App;
use Topalis\App\Entity\AppState;
use Topalis\Device\Entity\Device;
use Topalis\Device\Entity\DeviceEntityManager;
use Topalis\Device\Type\DeviceTypeFactory;
use Topalis\Device\Type\DeviceTypeInterface;

/**
 * export apps and devices using this apps as pivot table
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class AppExporter
{
    /**
     * @var ContainerInterface
     */
    private $container;

    /**
     * @var DeviceTypeFactory
     */
    private $deviceTypeFactory;
    /**
     * @var TranslatorInterface
     */
    private $translator;
    /**
     * @var \Topalis\Device\Entity\DeviceEntityManager
     */
    protected $deviceEntityManager;

    /**
     * @param ContainerInterface  $container
     * @param TranslatorInterface $translator
     * @param DeviceTypeFactory   $deviceTypeFactory
     * @param DeviceEntityManager $deviceEntityManager
     */
    public function __construct(
        ContainerInterface $container,
        TranslatorInterface $translator,
        DeviceTypeFactory $deviceTypeFactory,
        DeviceEntityManager $deviceEntityManager
    )
    {
        $this->container = $container;
        $this->translator = $translator;
        $this->deviceTypeFactory = $deviceTypeFactory;
        $this->deviceEntityManager = $deviceEntityManager;
    }

    public function export()
    {
        $excel = new PHPExcel();
        $first = true;
        foreach ($this->deviceTypeFactory->getTypes() as $type) {
            if (!$type->hasSupport("apps", null)) {
                continue;
            }
            // phpexcel creates always one sheet
            if ($first) {
                $sheet = $excel->getActiveSheet();
                $first = false;
            } else {
                $sheet = $excel->createSheet();
            }
            $name = $this->translator->trans($type->getTranslation());
            $name = str_replace(array("*", ":", "/", "\\", "?", "[", "]"), "-", $name);
            $sheet->setTitle($name);
            $this->createSheet($sheet, $type);
        }
        // set first sheet as active sheet
        $excel->setActiveSheetIndex(0);

        $writer = PHPExcel_IOFactory::createWriter($excel, "Excel2007");
        $response = new StreamedResponse();
        $response->setStatusCode(200);
        $response->headers->set("Content-Type", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        $disposition = $response->headers->makeDisposition(
            ResponseHeaderBag::DISPOSITION_ATTACHMENT,
            "auralis-apps.xlsx"
        );
        $response->headers->set("Content-Disposition", $disposition);
        $response->setCallback(function () use ($writer) {
            $writer->save("php://output");
        });

        return $response;
    }

    /**
     * @param \PHPExcel_Worksheet $sheet
     * @param DeviceTypeInterface $deviceType
     *
     * @throws \PHPExcel_Exception
     */
    private function createSheet(\PHPExcel_Worksheet $sheet, DeviceTypeInterface $deviceType)
    {
        /** @var AppState[] $states */
        $states = $this->container->get($deviceType->getAppStateEntityManagerId())
            ->createQueryBuilder("s")
            ->select("s, a, d")
            ->orderBy("a.name")
            ->join("s.app", "a")
            ->join("s.device", "d")
            ->where("s.state IN (:states)")
            ->setParameter("states", AppState::getInstalledStates())
            ->getQuery()
            ->getResult()
        ;

        /** @var Device[] $devices */
        $devices = $this->deviceEntityManager->createQueryBuilder("d")
            ->select("d")
            ->where("d.os = :os")
            ->setParameter("os", $deviceType->getId())
            ->orderBy("d.name")
            ->getQuery()
            ->getResult()
        ;

        $row = 2;
        $deviceIndex = array();
        // devices in column A
        foreach ($devices as $device) {
            if (!$device->isActive() || $device->isRollout()) {
                continue;
            }
            if (null === $device->getName()) {
                continue;
            }

            $sheet
                ->getCellByColumnAndRow(0, $row)
                ->setValue($device->getName() . " (" . $device->getUser()->getLogin() . ")")
            ;
            $deviceIndex[$device->getId()] = $row;
            $row++;
        }
        $sheet->getColumnDimension('A')->setWidth(40);

        $column = 1;
        $appIndex = array();
        foreach ($states as $state) {
            $app = $state->getApp();
            if ($app instanceof AndroidApp && $app->isSystem()) {
                continue;
            }
            $id = strtolower($app->getName() . " (" . $state->getVersion() . " )");
            if (!isset($appIndex[$id])) {
                $appIndex[$id] = array("app" => $app, "version" => $state->getVersion(), "devices" => array());
            }
            $appIndex[$id]['devices'][] = $state->getDevice();
        }
        ksort($appIndex);
        foreach ($appIndex as $id => $data) {
            /** @var App $app */
            $app = $data['app'];
            /** @var Device[] $devices */
            $devices = $data['devices'];

            $sheet
                ->getCellByColumnAndRow($column, 1)
                ->setValue($app->getName() . " (" . $data['version'] . ")")
            ;
            $sheet
                ->getColumnDimension(\PHPExcel_Cell::stringFromColumnIndex($column))
                ->setWidth(3.5)
            ;
            foreach ($devices as $device) {
                if (!isset($deviceIndex[$device->getId()])) {
                    continue;
                }

                $row = $deviceIndex[$device->getId()];
                // insert version
                $sheet
                    ->getCellByColumnAndRow($column, $row)
                    ->setValue("x")
                ;
                // colorize cell
                $sheet
                    ->getStyleByColumnAndRow($column, $row)->applyFromArray(array(
                        "fill" => array(
                            "type"       => \PHPExcel_Style_Fill::FILL_SOLID,
                            "startcolor" => array("rgb" => "88a451"),
                        )
                    ))
                    ->getAlignment()->setHorizontal(\PHPExcel_Style_Alignment::HORIZONTAL_CENTER)
                ;
            }

            $column++;
        }
        // rotate app names
        $sheet
            ->getStyle("B1:" . $sheet->getHighestColumn() . "1")
            ->getAlignment()->setTextRotation(90)
        ;
        // fix table headers
        $sheet->freezePane("B2");

        // auralis logo in A1
        $drawing = new \PHPExcel_Worksheet_Drawing();
        $drawing
            ->setPath("bundles/topalisapp/img/logo_auralis.png")
            ->setOffsetX(20)
            ->setOffsetY(20)
            ->setCoordinates("A1")
            ->setWorksheet($sheet)
        ;
    }
}
