<?php
namespace Topalis\App\Form\Handler;

use Symfony\Component\Form\FormInterface;
use Topalis\App\Android\ApkFile;
use Topalis\App\Entity\AndroidApp;
use Topalis\App\Entity\App;
use Topalis\App\Form\Type\AndroidAppForm;
use Symfony\Component\Form\FormError;
use Topalis\Util\FileSystemUtil;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class AndroidAppEditFormHandler extends AppEditFormHandler
{
    private $oldApkPath;

    /**
     * @param App $app
     *
     * @return FormInterface
     */
    public function createForm(App $app)
    {
        /** @var AndroidApp $app */
        $builder = $this->formFactory->createBuilder(new AndroidAppForm(), $app);
        if (AndroidApp::TYPE_UPLOADED !== $app->getType()) {
            $builder->remove("apkPath");
        }

        return $builder->getForm();
    }

    protected function handleUpload(FormInterface $form, App &$app)
    {
        /** @var AndroidApp $app */
        if (AndroidApp::TYPE_UPLOADED === $app->getType()) {
            $apk = new ApkFile($app->getApkPath());
            if ($app->getIdentifier() !== null && $app->getIdentifier() != $apk->getIdentifier()) {
                $form->addError(new FormError("app.identifierChanged", null, array(
                    "%oldIdentifier%" => $app->getIdentifier(),
                    "%newIdentifier%" => $apk->getIdentifier()
                )));
                // remove new APK
                FileSystemUtil::tryUnlinkFile($app->getApkPath());
                // reset path to APK
                $app->setApkPath($this->oldApkPath);
                return;
            }

            // TODO we could use additional device requirements
            // https://developer.android.com/google/play/filters.html
            $app
                ->setSupports(array("minOSVersion" => $apk->getMinOSVersion()))
                ->setNeedsUpdate(true)
                ->setIdentifier($apk->getIdentifier())
                ->setName($app->getIdentifier()) // name will be updated by RefreshUploadedAndroidAppsCommand
                ->setVersion($apk->getManifest()->getVersionName())
            ;
        }

        // find app with same identifier and replace it
        if (null !== $app->getId()) {
            return;
        }

        /** @var AndroidApp $oldApp */
        $oldApp = $this->appEntityManager->findOneBy(array(
            "identifier" => $app->getIdentifier(),
        ));
        if (null === $oldApp) {
            return;
        }

        if (AndroidApp::TYPE_UPLOADED !== $app->getType() && AndroidApp::TYPE_UPLOADED !== $oldApp->getType()) {
            // either the new app or the old app have to be uploaded, otherwise
            // we let the add fail because of duplicate identifiers
            return;
        }

        // copy attributes from new app to already persisted app
        $oldApp->merge($app, true);
        $oldApp->setManaged(true);
        $oldApp->setConfigGroups($app->getConfigGroups()->toArray());
        $oldApp->setForceInstall($app->isForceInstall());
        $oldApp->setType($app->getType());
        $oldApp->setApkPath($app->getApkPath());
        $oldApp->setNeedsUpdate($app->isNeedsUpdate());
        // $app is a reference
        $app = $oldApp;
    }

    protected function beforeHandleRequest(App $app)
    {
        $this->oldApkPath = $app->getApkPath();
    }

    protected function handleRequestFailed(App $app)
    {
        if ($this->oldApkPath != $app->getApkPath()) {
            FileSystemUtil::tryUnlinkFile($app->getApkPath());
        }
    }
}
