<?php
namespace Topalis\App\Form\Handler;

use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\Validator\ValidatorInterface;
use Topalis\App\Entity\App;
use Topalis\App\Entity\AppEntityManager;
use Topalis\App\Entity\IosApp;
use Topalis\App\Form\Type\IosAppForm;
use Topalis\App\Ios\IpaFile;
use Topalis\Configuration\AuralisConfig;
use Symfony\Component\Form\FormError;
use Symfony\Component\Form\FormInterface;
use Topalis\Util\FileSystemUtil;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class IosAppEditFormHandler extends AppEditFormHandler
{
    /**
     * @var AuralisConfig
     */
    private $config;

    /**
     * @var string
     */
    private $oldIpaPath;

    public function __construct(
        AppEntityManager $appEntityManager,
        ValidatorInterface $validator,
        FormFactoryInterface $formFactory,
        AuralisConfig $config)
    {
        parent::__construct($appEntityManager, $validator, $formFactory);

        $this->config = $config;
    }

    public function createForm(App $app)
    {
        /** @var IosApp $app */
        $builder = $this->formFactory->createBuilder(new IosAppForm($this->config), $app);
        if (IosApp::TYPE_UPLOADED !== $app->getType()) {
            $builder->remove("ipaPath");
        }

        return $builder->getForm();
    }

    protected function handleUpload(FormInterface $form, App &$app)
    {
        /** @var IosApp $app */
        if (IosApp::TYPE_UPLOADED === $app->getType()) {
            $ipa = ($app->getIpa() ?: new IpaFile($app->getIpaPath()));
            if ($app->getIdentifier() !== null && $app->getIdentifier() != $ipa->getIdentifier()) {
                $form->addError(new FormError("app.identifierChanged", null, array(
                    "%oldIdentifier%" => $app->getIdentifier(),
                    "%newIdentifier%" => $ipa->getIdentifier()
                )));
                // remove new IPA
                FileSystemUtil::tryUnlinkFile($app->getIpaPath());
                // reset path to IPA
                $app->setIpaPath($this->oldIpaPath);
                return;
            }

            $app
                ->setIdentifier($ipa->getIdentifier())
                ->setName($ipa->getName())
                ->setVersion($ipa->getVersion())
                ->setSupports(array(
                    "supportedDevices" => $ipa->getSupportedDevices(),
                    "minOSVersion" => $ipa->getMinOSVersion()
                ))
            ;
        }
        // find app with same identifier and replace it
        if (null !== $app->getId()) {
            return;
        }

        /** @var IosApp $oldApp */
        $oldApp = $this->appEntityManager->findOneBy(array(
            "identifier" => $app->getIdentifier()
        ));
        if (null === $oldApp) {
            return;
        }

        if (IosApp::TYPE_UPLOADED !== $app->getType() && IosApp::TYPE_UPLOADED !== $oldApp->getType()) {
            // either the new app or the old app have to be uploaded, otherwise
            // we let the add fail because of duplicate identifiers
            return;
        }

        // copy attributes from new app to already persisted app
        $oldApp->merge($app, true);
        $oldApp->setManaged(true);
        $oldApp->setConfigGroups($app->getConfigGroups()->toArray());
        $oldApp->setForceInstall($app->isForceInstall());
        $oldApp->setType($app->getType());
        if (null !== $app->getIpa()) {
            $oldApp->setIpa($app->getIpa());
        }
        $oldApp->setIpaPath($app->getIpaPath());
        $oldApp->setPreventBackup($app->isPreventBackup());
        $oldApp->setRemoveWheneMdmIsRemoved($app->isRemoveWheneMdmIsRemoved());
        // $app is a reference
        $app = $oldApp;
    }

    protected function beforeHandleRequest(App $app)
    {
        $this->oldIpaPath = $app->getIpaPath();
    }

    protected function handleRequestFailed(App $app)
    {
        if ($app->getIpaPath() != $this->oldIpaPath) {
            FileSystemUtil::tryUnlinkFile($app->getIpaPath());
        }
    }
}
