<?php
namespace Topalis\App\WinPhone;

use Symfony\Component\DependencyInjection\ContainerAware;
use Symfony\Component\DomCrawler\Crawler;
use Topalis\App\Entity\App;
use Topalis\App\Entity\WinPhoneApp;
use Topalis\Configuration\Client\AuralisCurl;
use Buzz\Browser;
use Topalis\Configuration\AuralisConfig;
use Topalis\App\AppStore;
use Topalis\App\Form\Type\WinPhone\WinPhoneAppProductIdType;
use Topalis\App\Util\CrawlerUtil;

class WinPhoneStore extends ContainerAware implements AppStore
{
    /**
     * @var \Buzz\Browser
     */
    private $browser;

    private $lookupUrl = "http://windowsphone.com/s?appid={id}";

    private $searchUrl = "http://www.windowsphone.com/{locale}/store/search?q={term}";

    /**
     * @param string $id The id of the app to lookup.
     * @return \Topalis\App\Entity\WinPhone|NULL
     */
    public function lookup($id)
    {
        if (!preg_match(WinPhoneAppProductIdType::PATTERN, $id)) {
            throw new \InvalidArgumentException("The id '" . $id . "' is not a valid app store id");
        }

        $url = str_replace("{id}", $id, $this->lookupUrl);
        $headers = array(
            "Host" => "www.windowsphone.com",
            "User-Agent" => "Mozilla/5.0 (X11; Linux x86_64; rv:33.0) Gecko/20100101 Firefox/33.0",
            "Accept" => "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
            "Accept-Language" => $this->container->get("config.auralis")->getWinPhoneStoreLocale()
        );

        $response = $this->getBrowser()->get($url, $headers);
        if (!$response->isSuccessful()) {
            return null;
        }
        $result = $response->getContent();

        $app = null;
        try {
            $app = $this->parseAppDetails($result);
        } catch (\Exception $e) {
            $this->getLogger()->error(
                sprintf("failed to get details for Windows Phone app %s: %s", $id, $e->getMessage()),
                array(
                    "exception" => $e,
                    "identifier" => $id
                ));
        }

        return $app;
    }

    public function refreshApps(array $apps)
    {
        foreach ($apps as $app) {
            if ($app->getType() == App::TYPE_UPLOADED) {
                continue;
            }
            $storeApp = $this->lookup($app->getIdentifier());
            if (null === $storeApp) {
                continue;
            }
            $app->setType(App::TYPE_STORE);
            $app->merge($storeApp);
        }
    }

    public function search($term)
    {
        $url = str_replace("{term}", $term, $this->searchUrl);

        $locales = array($this->container->get("config.auralis")->getWinPhoneStoreLocale());
        if (!in_array("en-us", $locales)) {
            // fallback
            $locales[] = "en-us";
        }

        $apps = array();
        foreach ($locales as $locale) {
            $headers = array(
                "Host" => "www.windowsphone.com",
                "User-Agent" => "Mozilla/5.0 (X11; Linux x86_64; rv:33.0) Gecko/20100101 Firefox/33.0",
                "Accept" => "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
                "Accept-Language" => $this->container->get("config.auralis")->getWinPhoneStoreLocale()
            );

            $response = $this->getBrowser()->get(str_replace("{locale}", $locale, $url), $headers);
            if (!$response->isSuccessful()) {
                continue;
            }

            $apps = $this->parseSearchResult($response->getContent());
        }

        return $apps;
    }

    private function parseAppDetails($html)
    {
        $crawler = new Crawler();
        $crawler->addHtmlContent($html);

        $app = new WinPhoneApp();
        $app->setType(App::TYPE_STORE);

        $url = CrawlerUtil::getFromCrawler($crawler, "[rel='canonical']", "attr", "href");
        $path = parse_url($url, PHP_URL_PATH);
        $identifier = pathinfo($path, PATHINFO_FILENAME);
        if (!$identifier) {
            throw new \InvalidArgumentException("App identifier not found");
        }
        $app->setIdentifier($identifier);

        $name = CrawlerUtil::getFromCrawler($crawler, "meta[property='og:title']", "attr", "content");
        if (!$name) {
            throw new \InvalidArgumentException("App name not found");
        }
        $app->setName($name);

        $vendor = trim(CrawlerUtil::getFromCrawler($crawler, "[itemprop='publisher']"));
        $app->setVendor($vendor);

        $description = CrawlerUtil::getFromCrawler($crawler, "meta[property='og:description']", "attr", "content");
        $app->setDescription($description);

        $supportedOSVersions = array();
        $operatingSystems = CrawlerUtil::getAllFromCrawler($crawler, "[itemprop='operatingSystems']");
        foreach ($operatingSystems as $operatingSystem) {
            $version = trim(str_replace("Windows Phone", "", $operatingSystem));
            if (preg_match("/[0-9]+(\.[0-9]+)*/", $version)) {
                $supportedOSVersions[] = $version;
            }
        }
        if ($supportedOSVersions) {
            $app->setSupports(array("osVersions" => $supportedOSVersions));
        }

        $price = self::splitPrice(trim(CrawlerUtil::getFromCrawler($crawler, "[itemprop='price']")));
        $app->setPrice($price['price']);
        $app->setCurrency($price['currency']);

        $version = trim(CrawlerUtil::getFromCrawler($crawler, "[itemprop='softwareVersion']"));
        $app->setVersion($version);

        $largeIcon = CrawlerUtil::getFromCrawler($crawler, "meta[property='og:image']", "attr", "content");
        $app->setIconLarge($largeIcon);

        $smallIcon = str_replace("ws_icon_large", "ws_icon_small", $largeIcon);
        $app->setIcon($smallIcon);

        return $app;
    }

    private function parseSearchResult($html)
    {
        $crawler = new Crawler();
        $crawler->addHtmlContent($html);

        $apps = array();
        foreach ($crawler->filter("#storeResults .appList") as $node) {
            $apps[] = $this->parseSingleSearchResult(new Crawler($node));
        }
        return $apps;
    }

    private function parseSingleSearchResult(Crawler $node)
    {
        $app = new WinPhoneApp();
        $app->setType(App::TYPE_STORE);

        $url = CrawlerUtil::getFromCrawler($node, "a.title", "attr", "href");
        $path = parse_url($url, PHP_URL_PATH);
        $identifier = pathinfo($path, PATHINFO_FILENAME);
        if (!$identifier) {
            throw new \InvalidArgumentException("App identifier not found");
        }
        $app->setIdentifier($identifier);

        $name = trim(CrawlerUtil::getFromCrawler($node, "a.title"));
        if (!$name) {
            throw new \InvalidArgumentException("App name not found");
        }
        $app->setName($name);

        $smallIcon = CrawlerUtil::getFromCrawler($node, "a.appImage img", "attr", "src");
        $app->setIcon($smallIcon);

        $largeIcon = str_replace("ws_icon_small", "ws_icon_large", $smallIcon);
        $app->setIconLarge($largeIcon);

        $price = self::splitPrice(trim(CrawlerUtil::getFromCrawler($node, "div.cost")));
        $app->setPrice($price['price']);
        $app->setCurrency($price['currency']);

        return $app;
    }

    protected static function splitPrice($price)
    {
        $matches = array();
        preg_match("/([^\d]*)(\d+(\.|,)\d{2})([^\d]*)/", $price, $matches);
        if (count($matches) !== 5) {
            return array("price" => "0", "currency" => null);
        }

        $price = str_replace(",", ".", $matches[2]);
        // currency is either at the beginning or at the end
        $currency = trim($matches[1]);
        if (!$currency) {
            $currency = trim($matches[4]);
        }

        return array(
            "price" => $price,
            "currency" => $currency
        );
    }

    /**
     * @return \Symfony\Bridge\Monolog\Logger
     */
    protected function getLogger()
    {
        return $this->container->get("logger");
    }

    /**
     * @return \Buzz\Browser
     */
    protected function getBrowser()
    {
        if (null !== $this->browser) {
            return $this->browser;
        }

        return $this->browser = new Browser(new AuralisCurl($this->container->get("config.auralis")));
    }
}
