<?php
namespace Topalis\ConfigGroup\Controller;

use Topalis\Kernel\Controller;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\ConfigGroup\Form\Type\ConfigGroupForm;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Doctrine\ORM\Query\Expr\Join;
use Topalis\ConfigGroup\Form\Type\ConfigGroupRemoveForm;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Form\FormError;

/**
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @Route("/admin/group", service="config_group.controller")
 */
class ConfigGroupController extends Controller
{
    /**
     * @Route("/", name="admin_group")
     * @Template()
     */
    public function indexAction()
    {
        $groups = $this->getConfigGroupEntityManager()->createQueryBuilder("c")
            ->select("c AS config_group", "COUNT(DISTINCT d.id) as device_count", "COUNT(DISTINCT u.id) as user_count")
            ->leftJoin("c.devices", "d", Join::WITH, "d.deleted = 0")
            ->leftJoin("c.users", "u")
            ->groupBy("c.id")
            ->getQuery()
            ->getResult()
        ;

        return array(
            "groups" => $groups
        );
    }

    /**
     * @Route("/add/{noExchange}", name="admin_group_add", defaults={"noExchange" = ""})
     * @Template()
     *
     * @param string $noExchange If defined a config group without email
     *                           accounts will be generated.
     *
     * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function addAction($noExchange)
    {
        $group = new ConfigGroup();
        $group->setNoExchange($noExchange ? true : false);
        $request = $this->getRequest();

        $form = $this->createForm(new ConfigGroupForm(), $group);

        if ($request->isMethod("POST")) {
            $form->submit($request);
            if ($form->isValid()) {
                $this->container->get("config_group.entity.manager.config_group")->save($group);
                $this->addSuccess($this->tr("admin.group.saveSuccess"));

                return $this->redirect($this->generateUrl("admin_group_edit", array("id" => $group->getId())));
            }
        }

        return array(
            "form" => $form->createView()
        );
    }

    /**
     * @param ConfigGroup $group
     *
     * @return array
     * @Route("/{id}/edit", name="admin_group_edit")
     */
    public function editAction(ConfigGroup $group)
    {
        $sectionManager = $this->container->get("config_group.section.manager");
        $section = current($sectionManager->getAllSections());

        return $this->redirect(
            $this->generateUrl($section->getRoute(), array("group" => $group->getId()))
        );
    }

    /**
     * @Route("/{id}/delete", name="admin_group_delete", options={"expose" = true})
     * @Template()
     *
     * @param ConfigGroup $group
     *
     * @return array|JsonResponse
     */
    public function deleteAction(ConfigGroup $group)
    {
        $lastGroup = !$this->getConfigGroupEntityManager()
            ->exists("entity != :group", array("group" => $group));
        if ($lastGroup) {
            $lastDeviceGroup = true;
        } else {
            $lastDeviceGroup = !$this->getConfigGroupEntityManager()
                ->exists("entity != :group AND entity.noExchange = :noExchange",
                    array("group" => $group, "noExchange" => $group->isNoExchange()));
        }

        $form = $this->createForm(
            new ConfigGroupRemoveForm($group, $lastDeviceGroup, $lastGroup, $this->getConfigGroupEntityManager()));
        try {
            if ($this->getConfigGroupRemoveFormHandler()->process($form, $group)) {
                $this->addFlash("success", $this->tr("admin.group.deleteSuccess"));
                return new JsonResponse(array("status" => "OK"));
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("Error removing config group '" . $group->getId(). "'",
                array("exception" => $e));
            $msg = $this->tr("admin.group.error.delete", array("%errmsg%" => $e->getMessage()));
            $form->addError(new FormError($msg));
        }

        return array(
            "lastDeviceGroup" => $lastDeviceGroup,
            "lastGroup" => $lastGroup,
            "form" => $form->createView(),
            "group" => $group,
        );
    }

    /**
     * @return \Topalis\ConfigGroup\Entity\ConfigGroupEntityManager
     */
    private function getConfigGroupEntityManager()
    {
        return $this->container->get("config_group.entity.manager.config_group");
    }

    /**
     * @return \Topalis\ConfigGroup\Form\Handler\ConfigGroupRemoveFormHandler
     */
    private function getConfigGroupRemoveFormHandler()
    {
        return $this->container->get("config_group.form.remove.handler");
    }
}
