<?php
namespace Topalis\ConfigGroup\Entity;

use Doctrine\ORM\Mapping as ORM;
use Topalis\Device\Entity\Device;
use Topalis\ReverseProxy\Entity\ReverseProxyUrl;
use Doctrine\Common\Collections\ArrayCollection;
use Symfony\Component\Validator\Constraints as Assert;
use Doctrine\ORM\PersistentCollection;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Topalis\App\Entity\WinPhoneAppPolicy;
use Topalis\WebClip\Entity\WebClip;

/**
 * This entity represents a configuration group.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 *
 * @ORM\Table(name="configgroups")
 * @ORM\Entity
 * @UniqueEntity(fields={"name"}, message="config_group.nameTaken")
 */
class ConfigGroup
{
    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    protected $id;

    /**
     * @var string
     *
     * @ORM\Column(name="comment", type="string", length=255, nullable=true)
     */
    protected $comment;

    /**
     * @var integer
     *
     * @ORM\Column(type="integer", options={"unsigned"=true})
     */
    protected $deviceTimeout = 0;

    /**
     * @var boolean
     *
     * @ORM\Column(name="noexchange", type="boolean", nullable=false)
     */
    protected $noExchange = false;

    /**
     * @var boolean
     * @ORM\Column(type="boolean")
     */
    protected $allowProfileRemoval = false;

    /**
     * @var integer
     * @ORM\Column(type="integer")
     */
    protected $syncDays = 14;
    /**
     * @var boolean
     * @ORM\Column(type="boolean")
     */
    protected $allowMove = false;

    /**
     * @var boolean
     * @ORM\Column(type="boolean")
     */
    protected $fromAnyApp = true;

    /**
     * ArrayCollection with \Topalis\Device\Entity\Device objects.
     * @var ArrayCollection
     *
     * @ORM\OneToMany(targetEntity="Topalis\Device\Entity\Device", mappedBy="configGroup", cascade={"persist"})
     */
    protected $devices;

    /**
     * @var string
     * @Assert\NotBlank
     *
     * @ORM\Column(name="name", type="string", length=255, nullable=false)
     */
    protected $name;

    /**
     * @var string
     *
     * @ORM\Column(name="profile", type="text", length=65532, nullable=true)
     */
    protected $profile;

    /**
     * @var boolean
     * @ORM\Column(type="boolean", nullable=true)
     */
    protected $customProfile;

    /**
     * @var string
     * @ORM\Column(type="string", nullable=true)
     */
    protected $customProfileName;

    /**
     * @var string
     * @ORM\Column(type="text", length=65532, nullable=true)
     */
    protected $androidProfile;

    /**
     * @var \Topalis\App\Entity\WinPhoneAppPolicy
     * @ORM\ManyToOne(targetEntity="Topalis\App\Entity\WinPhoneAppPolicy",
     *     inversedBy="configGroups", cascade={"persist"})
     */
    protected $winPhoneAppPolicy;

    /**
     * @var array
     * @ORM\Column(type="json_array_string", length=511, nullable=true)
     */
    protected $winPhoneExchangeConfig;

    /**
     * @var array
     * @ORM\Column(type="json_array_string", length=511, nullable=true)
     */
    protected $winPhonePassCode;

    /**
     * @var array
     * @ORM\Column(type="json_array", length=65535, nullable=true)
     */
    protected $winPhoneRestrictions;

    /**
     * ArrayCollection with \Topalis\WinPhone\Entity\MSWlanConfig objects.
     * @var ArrayCollection
     *
     * @ORM\OneToMany(targetEntity="Topalis\WinPhone\Entity\MSWlanConfig", mappedBy="configGroup", cascade={"all"},
     *     orphanRemoval=true)
     */
    protected $msWlanConfigs;

    /**
     * ArrayCollection containing \Topalis\ReverseProxy\Entity\ReverseProxyUrl objects
     *
     * @var \Doctrine\Common\Collections\ArrayCollection
     *
     * @ORM\ManyToMany(targetEntity="Topalis\ReverseProxy\Entity\ReverseProxyUrl",
     *     mappedBy="configGroups", cascade={"persist"})
     */
    protected $reverseProxyUrls;

    /**
     * @var \Doctrine\Common\Collections\ArrayCollection
     * @ORM\ManyToMany(targetEntity="Topalis\Wifi\Entity\WifiConfig", mappedBy="configGroups")
     */
    protected $wifiConfigs;

    /**
     * ArrayCollection containing \Topalis\User\Entity\User objects
     *
     * @var \Doctrine\Common\Collections\ArrayCollection
     *
     * @ORM\OneToMany(targetEntity="Topalis\User\Entity\User", mappedBy="configGroup", cascade={"persist"})
     */
    protected $users;

    /**
     * @var ArrayCollection
     *
     * @ORM\ManyToMany(targetEntity="Topalis\WebClip\Entity\WebClip", mappedBy="configGroups", cascade={"persist"})
     */
    protected $webClips;

    public function __construct()
    {
        $this->devices = new ArrayCollection();
        $this->msWlanConfigs = new ArrayCollection();
        $this->reverseProxyUrls = new ArrayCollection();
        $this->wifiConfigs = new ArrayCollection();
        $this->users = new ArrayCollection();
        $this->webClips = new ArrayCollection();
    }

    /**
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param integer $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @return string
     */
    public function getComment()
    {
        return $this->comment;
    }

    /**
     * @param string $comment
     */
    public function setComment($comment)
    {
        $this->comment = $comment;
    }

    /**
     * @return integer The device timeout in seconds. 0 if timeout is disabled.
     */
    public function getDeviceTimeout()
    {
        return $this->deviceTimeout;
    }

    /**
     * @param integer $deviceTimeout The device timeout in seconds to set. 0 to
     *     disable timeout.
     */
    public function setDeviceTimeout($deviceTimeout)
    {
        $this->deviceTimeout = $deviceTimeout;
    }

    /**
     *
     * @return boolean
     */
    public function isNoExchange()
    {
        return $this->noExchange;
    }

    /**
     *
     * @param boolean $noExchange
     */
    public function setNoExchange($noExchange)
    {
        $this->noExchange = $noExchange;
    }

    /**
     * @param \Topalis\Device\Entity\Device $device
     */
    public function addDevice(\Topalis\Device\Entity\Device $device)
    {
        $device->setConfigGroup($this);

        if ($this->devices instanceof PersistentCollection && !$this->devices->isInitialized()) {
            return;
        }

        if ($this->devices->contains($device)) {
            return;
        }

        $this->devices->add($device);
    }

    /**
     * @return Device[]
     */
    public function getDevices()
    {
        return $this->devices;
    }

    /**
     * @param \Topalis\Device\Entity\Device $device
     */
    public function removeDevice(\Topalis\Device\Entity\Device $device)
    {
        if ($this->devices instanceof PersistentCollection && !$this->devices->isInitialized()) {
            return;
        }

        $this->devices->removeElement($device);
    }

    /**
     * @param ArrayCollection $devices
     */
    public function setDevices(ArrayCollection $devices)
    {
        $this->devices = $devices;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return string
     */
    public function getProfile()
    {
        return $this->profile;
    }

    /**
     * @param string $profile
     */
    public function setProfile($profile)
    {
        $this->profile = $profile;
    }

    /**
     * @return ReverseProxyUrl[]
     */
    public function getReverseProxyUrls()
    {
        return $this->reverseProxyUrls;
    }

    /**
     * @param ReverseProxyUrl $reverseProxyUrl
     * @return $this
     */
    public function addReverseProxyUrl(ReverseProxyUrl $reverseProxyUrl)
    {
        if ($this->reverseProxyUrls->contains($reverseProxyUrl)) {
            return $this;
        }

        $this->reverseProxyUrls->add($reverseProxyUrl);
        $reverseProxyUrl->addConfigGroup($this);

        return $this;
    }

    /**
     * @param ReverseProxyUrl $reverseProxyUrl
     * @return $this
     */
    public function removeReverseProxyUrl(ReverseProxyUrl $reverseProxyUrl)
    {
        if (!$this->reverseProxyUrls->contains($reverseProxyUrl)) {
            return $this;
        }
        $this->reverseProxyUrls->removeElement($reverseProxyUrl);
        $reverseProxyUrl->removeConfigGroup($this);

        return $this;
    }

    /**
     * @param ReverseProxyUrl[] $reverseProxyUrls
     * @return $this
     */
    public function setReverseProxyUrls(\Traversable $reverseProxyUrls)
    {
        $this->reverseProxyUrls->clear();
        foreach ($reverseProxyUrls as $reverseProxyUrl) {
            $this->addReverseProxyUrl($reverseProxyUrl);
        }

        return $this;
    }

    /**
     * @param \Topalis\User\Entity\User $user
     */
    public function addUser(\Topalis\User\Entity\User $user)
    {
        $user->setConfigGroup($this);

        if ($this->users instanceof PersistentCollection && !$this->users->isInitialized()) {
            return;
        }

        if (!$this->users->contains($user)) {
            return;
        }

        $this->users->add($user);
    }

    /**
     * @return \Doctrine\Common\Collections\ArrayCollection
     */
    public function getUsers()
    {
        return $this->users;
    }

    /**
     * @param \Topalis\User\Entity\User $user
     */
    public function removeUser(\Topalis\User\Entity\User $user)
    {
        if ($this->users instanceof PersistentCollection && !$this->users->isInitialized()) {
            return;
        }

        $this->users->removeElement($user);
    }

    /**
     * @param ArrayCollection $users
     */
    public function setUsers(ArrayCollection $users)
    {
        $this->users = $users;
    }

    /**
     * @param boolean $allowMove
     *
     * @return self
     */
    public function setAllowMove($allowMove)
    {
        $this->allowMove = $allowMove;

        return $this;
    }

    /**
     * @return boolean
     */
    public function isAllowMove()
    {
        return $this->allowMove;
    }

    /**
     * @param boolean $allowProfileRemoval
     *
     * @return self
     */
    public function setAllowProfileRemoval($allowProfileRemoval)
    {
        $this->allowProfileRemoval = $allowProfileRemoval;

        return $this;
    }

    /**
     * @return boolean
     */
    public function isAllowProfileRemoval()
    {
        return $this->allowProfileRemoval;
    }

    /**
     * @param boolean $fromAnyApp
     *
     * @return self
     */
    public function setFromAnyApp($fromAnyApp)
    {
        $this->fromAnyApp = $fromAnyApp;

        return $this;
    }

    /**
     * @return boolean
     */
    public function isFromAnyApp()
    {
        return $this->fromAnyApp;
    }

    /**
     * @param int $syncDays
     *
     * @return self
     */
    public function setSyncDays($syncDays)
    {
        $this->syncDays = $syncDays;

        return $this;
    }

    /**
     * @return int
     */
    public function getSyncDays()
    {
        return $this->syncDays;
    }

    /**
     * @param boolean $customProfile
     *
     * @return self
     */
    public function setCustomProfile($customProfile)
    {
        $this->customProfile = $customProfile;

        return $this;
    }

    /**
     * @return boolean
     */
    public function isCustomProfile()
    {
        return $this->customProfile;
    }

    /**
     * @return string
     */
    public function getCustomProfileName()
    {
        return $this->customProfileName;
    }

    /**
     * @param string $customProfileName
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup self
     */
    public function setCustomProfileName($customProfileName)
    {
        $this->customProfileName = $customProfileName;

        return $this;
    }

    /**
     * @param string $androidProfile
     *
     * @return self
     */
    public function setAndroidProfile($androidProfile)
    {
        $this->androidProfile = $androidProfile;

        return $this;
    }

    /**
     * @return string
     */
    public function getAndroidProfile()
    {
        return $this->androidProfile;
    }

    /**
     * @return string|NULL
     */
    public function getWinPhoneAppPolicy()
    {
        return $this->winPhoneAppPolicy;
    }

    /**
     * @param string|NULL $winPhoneAppPolicy
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup self
     */
    public function setWinPhoneAppPolicy(WinPhoneAppPolicy $winPhoneAppPolicy = null)
    {
        $this->winPhoneAppPolicy = $winPhoneAppPolicy;
        if ($this->winPhoneAppPolicy !== null) {
            $this->winPhoneAppPolicy->addConfigGroup($this);
        }

        return $this;
    }

    /**
     * @return array|NULL
     */
    public function getWinPhoneExchangeConfig()
    {
        return $this->winPhoneExchangeConfig;
    }

    /**
     * @param array|NULL $winPhoneExchangeConfig
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup self
     */
    public function setWinPhoneExchangeConfig(array $winPhoneExchangeConfig = null)
    {
        $this->winPhoneExchangeConfig = $winPhoneExchangeConfig;

        return $this;
    }

    /**
     * @return array|NULL
     */
    public function getWinPhonePassCode()
    {
        return $this->winPhonePassCode;
    }

    /**
     * @param array|NULL $winPhonePassCode
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup self
     */
    public function setWinPhonePassCode(array $winPhonePassCode = null)
    {
        $this->winPhonePassCode = $winPhonePassCode;

        return $this;
    }

    /**
     * @return array|NULL
     */
    public function getWinPhoneRestrictions()
    {
        return $this->winPhoneRestrictions;
    }

    /**
     * @param array|NULL $winPhoneRestrictions
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup self
     */
    public function setWinPhoneRestrictions(array $winPhoneRestrictions = null)
    {
        $this->winPhoneRestrictions = $winPhoneRestrictions;

        return $this;
    }

    /**
     *
     * @return \Doctrine\Common\Collections\ArrayCollection
     */
    public function getMSWlanConfigs()
    {
        return $this->msWlanConfigs;
    }

    public function addMSWlanConfig(\Topalis\WinPhone\Entity\MSWlanConfig $msWlanConfig)
    {
        if ($this->msWlanConfigs->contains($msWlanConfig)) {
            return;
        }

        $this->msWlanConfigs->add($msWlanConfig);
        $msWlanConfig->setConfigGroup($this);
    }

    public function removeMSWlanConfig(\Topalis\WinPhone\Entity\MSWlanConfig $msWlanConfig)
    {
        $this->msWlanConfigs->removeElement($msWlanConfig);
    }

    /**
     *
     * @param \Topalis\Wifi\Entity\WifiConfig $wifiConfig
     */
    public function addWifiConfig(\Topalis\Wifi\Entity\WifiConfig $wifiConfig)
    {
        if (!$this->wifiConfigs->contains($wifiConfig)) {
            $this->wifiConfigs[] = $wifiConfig;
            $wifiConfig->addConfigGroup($this);
        }
    }

    /**
     *
     * @return \Doctrine\Common\Collections\ArrayCollection
     */
    public function getWifiConfigs()
    {
        return $this->wifiConfigs;
    }

    /**
     *
     * @param \Topalis\Wifi\Entity\WifiConfig $wifiConfig
     */
    public function removeWifiConfig(\Topalis\Wifi\Entity\WifiConfig $wifiConfig)
    {
        if ($this->wifiConfigs->removeElement($wifiConfig)) {
            $wifiConfig->removeConfigGroup($this);
        }
    }

    /**
     * @return WebClip[]
     */
    public function getWebClips()
    {
        return $this->webClips;
    }

    /**
     * @param WebClip $webClip
     *
     * @return $this
     */
    public function addWebClip(WebClip $webClip)
    {
        if ($this->webClips->contains($webClip)) {
            return $this;
        }

        $this->webClips->add($webClip);
        $webClip->addConfigGroup($this);

        return $this;
    }

    /**
     * @param WebClip $webClip
     *
     * @return $this
     */
    public function removeWebClip(WebClip $webClip)
    {
        if ($this->webClips->removeElement($webClip)) {
            $webClip->removeConfigGroup($this);
        }

        return $this;
    }

    /**
     * @param WebClip[] $webClips
     *
     * @return $this
     */
    public function setWebClips(\Traversable $webClips)
    {
        $this->webClips->clear();
        foreach ($webClips as $webClip) {
            $this->addWebClip($webClip);
        }

        return $this;
    }
}
