<?php
namespace Topalis\Configuration;

/**
 * This class represents a line in an Apache HTTPD configuration. Lines defining
 * a block (e.g. <Location />) are excluded.
 *
 * @author Dominic Tubach
 */
class ApacheConfigLine
{
    private $name = "";
    private $options = array();

    /**
     * @return string The name of the configuration. This is the first string in
     *   the line.
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $key Key of a configuration option.
     * @return string|NULL The value defined for the specified key; an empty
     *   string if no value is set, but the key exists; null if the key does not
     *   exist.
     */
    public function getOption($key)
    {
        return $this->hasOption($key) ? $this->options[$key] : null;
    }

    /**
     * @return string[string] Associative array containing the defined
     *   configuration options with the key of configuration options as keys.
     */
    public function getOptions()
    {
        return $this->options;
    }

    /**
     * @param string $key Key of a configuration option.
     * @return boolean true if the specified key exists, false otherwise.
     */
    public function hasOption($key)
    {
        return array_key_exists($key, $this->options);
    }

    /**
     * Sets the name of the configuration line. This is the first string in the
     * line.
     *
     * @param string $name The name to be set.
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * Sets the configuration option with the specified key to the specified
     * value.
     *
     * @param string $key The key of the configuration option to be set.
     * @param string $value The value to be set.
     */
    public function setOption($key, $value)
    {
        $this->options[$key] = $value;
    }

    /**
     * Sets the configuration options.
     * @param string[string] $options The options to be set; an associative
     *   array with the key of configuration options as keys.
     */
    public function setOptions(array $options)
    {
        $this->options = $options;
    }

    /**
     * @return string A string representing the configuration in the form it is
     *   written in an Apache HTTPD configuration file.
     */
    public function __toString()
    {
        $line = $this->name;
        foreach ($this->options as $key => $value) {
            $line .= " " . $key;
            if (!empty($value)) {
                $line .= "=" . $value;
            }
        }
        return $line;
    }

    /**
     * Parses the specified line and returns the ApacheConfigLine object
     * representing the specified line.
     *
     * @param string $line The line to pass.
     * @return ApacheConfigLine The ApacheConfigLine object representing the
     *   specified line.
     */
    public static function parse($line)
    {
        $rv = new ApacheConfigLine();
        $lineSplit = explode(" ", $line);
        $rv->setName($lineSplit[0]);
        $lineSplitCount = count($lineSplit);
        for ($i = 1; $i < $lineSplitCount; $i++) {
            $option = explode("=", $lineSplit[$i]);
            if (count($option) == 1) {
                $rv->setOption($option[0], "");
            } else {
                $rv->setOption($option[0], $option[1]);
            }
        }
        return $rv;
    }
}
