<?php
namespace Topalis\Configuration;

use Topalis\Util\Process;

/**
 * Class to access the auralis sysconfig.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class AuralisSysConfig
{
    const DEFAULT_FILENAME = "/etc/auralis/conf";

    private $config;
    private $filename;

    public function __construct($filename = self::DEFAULT_FILENAME)
    {
        $this->setFilename($filename);
    }

    /**
     * Returns the configuration option with the specified key.
     *
     * @param string $key     The configuration key.
     * @param string $default The value returned if no option with the specified
     *                        key is set.
     *
     * @throws \Topalis\Util\Exception\Process\ExecException
     * @return string The configuration option of the specified key.
     */
    protected function getValue($key, $default = null)
    {
        $cfg =& $this->config;
        if (!isset($cfg[$key])) {
            if (file_exists($this->filename)) {
                $cmd = "$key=$default; . $this->filename >/dev/null 2>&1; echo \$$key";
                $out = "";
                Process::execThrow($cmd, $out);
                $cfg[$key] = $out;
            } else {
                $cfg[$key] = $default;
            }
        }
        return $cfg[$key];
    }

    /**
     * Fetches the database configuration from the global configuration file and
     * returns it as associative arrays with the following keys:
     * <ul>
     * <li>host (default: localhost)</li>
     * <li>port (3306)</li>
     * <li>database (auralis)</li>
     * <li>user (auralis)</li>
     * <li>password</li>
     * </ul>
     *
     * @return array Associative array with database configuration.
     */
    public function getDbConfig()
    {
        return array(
            "host" => $this->getValue("DBHOST", "localhost"),
            "port" => $this->getValue("DBPORT", "3306"),
            "database" => $this->getValue("DBNAME", "auralis"),
            "user" => $this->getValue("DBUSER", "auralis"),
            "password" => $this->getValue("DBPASS")
        );
    }

    /**
     * @return string The hosts which are allowed to use the maintenance tunnel.
     *     Default: fw.topalis.com,rz-str.topalis.com,jumphost.topalis.com
     */
    public function getMaintenanceSources()
    {
        return $this->getValue("MAINTENANCE_SOURCES", "fw.topalis.com,rz-str.topalis.com,jumphost.topalis.com");
    }

    /**
     * @return string The web root directory. (Default: /var/www/auralis)
     */
    public function getWebRoot()
    {
        return $this->getValue("WEBROOT", "/var/www/auralis");
    }

    /**
     * Sets the file name of the sysconfig file. By default it set to DEFAULT_FILENAME.
     *
     * @param string $filename The file name to set.
     * @see DEFAULT_FILENAME
     */
    public function setFilename($filename)
    {
        if ($this->filename != $filename) {
            $this->filename = $filename;
            $this->config = array();
        }
    }
}
