<?php
namespace Topalis\Configuration\Controller;

use Symfony\Component\Filesystem\Exception\IOException;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\File\File;
use Topalis\Configuration\Exception\VersionMismatchException;
use Symfony\Component\Form\FormError;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Topalis\Configuration\Form\BackupForm;
use Topalis\Configuration\Form\RestoreForm;
use Topalis\Kernel\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Topalis\Util\CryptUtil;

/**
 *
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/admin/backup", service="config.backup.controller")
 */
class BackupController extends Controller
{
    private $tmpFiles = array();

    /**
     * @return \Topalis\Configuration\BackupManager
     */
    private function getBackupManager()
    {
        return $this->container->get("config.backup.manager");
    }

    private function getFilesystem()
    {
        return $this->container->get("filesystem");
    }

    /**
     * @Route("", name="admin_backup", methods="GET")
     * @Template()
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function indexAction()
    {
        return array(
            "backup" => $this->createForm(new BackupForm())->createView(),
            "restore" => $this->createForm(new RestoreForm())->createView()
        );
    }

    /**
     * @Route("", name="admin_backup_create", methods="POST")
     * @return Response
     * @throws \Exception
     */
    public function createBackupAction()
    {
        $form = $this->createForm(new BackupForm());
        $form->submit($this->getRequest());
        if (!$form->isValid()) {
            return $this->render("TopalisConfigurationBundle:Backup:index.html.twig", array(
                    "backup" => $form->createView(),
                    "restore" => $this->createForm(new RestoreForm())->createView()
            ));
        }

        try {
            $backupFile = $this->getBackupManager()->createDump();
        } catch (\Exception $e) {
            $this->getLogger()->error("Error creating backup", array("exception" => $e));
            $msg = $this->tr("admin.backup.error.createConfigBackup", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
            return $this->redirect($this->generateUrl("admin_backup"));
        }

        try {
            $tmpEncrypted = CryptUtil::encryptInFilesystem($backupFile, $form->get("password")->getData());
            $this->tmpFiles[] = $tmpEncrypted;
            $this->getFilesystem()->remove($backupFile);

            $response = new BinaryFileResponse($tmpEncrypted);
            $response->headers->set("Content-Type", "application/vnd.auralis.backup");
            $disposition = $response->headers->makeDisposition(
                ResponseHeaderBag::DISPOSITION_ATTACHMENT,
                "auralis-config-" . $this->getAuralisVersion() . "_" . $this->getTimeProvider()->getDate("Ymd")
                . ".backup"
            );
            $response->headers->set("Content-Disposition", $disposition);

            return $response;
        } catch (\Exception $e) {
            $msg = $this->tr("admin.backup.error.cantEncrypt");
            $this->addError($msg);
            return $this->redirect($this->generateUrl("admin_backup"));
        }
    }

    /**
     * @Route("/restore", name="admin_backup_restore", methods={"POST", "PUT"})
     * @Template("TopalisConfigurationBundle:Backup:index.html.twig")
     *
     * @return Response
     * @throws \Exception
     */
    public function restoreBackupAction()
    {
        $form = $this->createForm(new RestoreForm());
        $form->submit($this->getRequest());

        if ($form->isValid()) {
            /** @var $file UploadedFile */
            $file = $form->get("file")->getData();
            if (0 === filesize($file->getRealPath())) {
                $form->get("file")->addError(new FormError("backup.emptyConfig"));
            } else {
                $tmpDecrypted = null;
                try {
                    // decrypt uploaded file
                    $tmpDecrypted = CryptUtil::decryptInFilsystem($file->getRealPath(),
                        $form->get("password")->getData());
                    $this->tmpFiles[] = $tmpDecrypted;
                } catch (\Exception $e) {
                    $form->get("password")->addError(new FormError("backup.cantDecrypt"));
                }
            }
        }

        if (!$form->isValid()) {
            return array(
                "backup" => $this->createForm(new BackupForm())->createView(),
                "restore" => $form->createView()
            );
        }

        try {
            $this->getBackupManager()->restoreDump(new File($tmpDecrypted));
            $this->addFlash("success", $this->tr("admin.backup.restoreSuccess"));
        } catch (VersionMismatchException $e) {
            $this->getLogger()->info($e->getMessage());
            $backupVersion = $e->getBackupVersion();
            if ($backupVersion === null) {
                $backupVersion = $this->tr("unknown");
            }
            $currentVersion = $e->getCurrentVersion();
            $msg = $this->tr(
                "admin.backup.error.restoreVersionMismatch",
                array("%backupVersion%" => $backupVersion, "%currentVersion%" => $currentVersion)
            );
            $this->addError($msg);
        } catch (\Exception $e) {
            $this->getLogger()->error("Error restoring backup", array("exception" => $e));
            $msg = $this->tr("admin.backup.error.restoreConfig", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }

        return $this->redirect($this->generateUrl("admin_backup"));
    }

    public function __destruct()
    {
        foreach ($this->tmpFiles as $file) {
            try {
                $this->getFilesystem()->remove($file);
            } catch (IOException $e) {
                $this->getLogger()->error("Error deleting file " . $file, array("exception" => $e));
            }
        }
    }
}
