<?php
namespace Topalis\Configuration;

use Topalis\Util\StringUtil;

/**
 * Class for accessing configuration stored in a database.
 *
 * @author Dominic Tubach
 */
class DatabaseConfiguration implements Configuration
{
    private $db;
    private $table;
    private $keyColumn;
    private $valueColumn;
    private $requiredValues;

    /**
     * Constructor.
     *
     * @param MySqlDb $db
     * @param string $table
     * @param string $keyColumn
     * @param string $valueColumn
     * @param string[string] $requiredValues additional values used in INSERT and WHERE clause.
     */
    public function __construct(\Topalis\Database\MySqlDb $db, $table, $keyColumn, $valueColumn,
        array $requiredValues = array())
    {
        $this->db = $db;
        $this->table = $table;
        $this->keyColumn = $keyColumn;
        $this->valueColumn = $valueColumn;
        $this->requiredValues = $requiredValues;
    }

    /**
     * Add required value used in INSERT and WHERE clause.
     * @param string $key
     * @param string $value
     */
    public function addRequiredValue($key, $value)
    {
        $this->requiredValues[$key] = $value;
    }

    public function clear()
    {
        return $this->db->removeRows($this->table, $this->requiredValues);
    }

    public function containsProperty($key)
    {
        return $this->getProperty($key) !== null;
    }

    public function getProperties()
    {
        return $this->db->fetchRows(
            $this->table,
            array($this->getQuotedValueColumn()),
            $this->requiredValues,
            array($this->getQuotedKeyColumn()),
            $this->keyColumn
        );
    }

    public function getProperty($key, $default = null)
    {
        $whereArr = $this->toRequiredValues($key);
        $value = $this->db->fetchValue($this->table, $this->getQuotedValueColumn(), $whereArr);
        return $value === null ? $default : $value;
    }

    public function removeProperty($key)
    {
        $whereArr = $this->toRequiredValues($key);
        return $this->db->removeRows($this->table, $whereArr) > 0;
    }

    /**
     * Sets the property with the specified key to the specified value.
     *
     * @param string $key The key to set the property for.
     * @param string $value The value to be set.
     * @return boolean true on success, false on error.
     */
    public function setProperty($key, $value)
    {
        if (is_bool($value)) {
            $value = StringUtil::booltostr($value);
        }

        $whereArr = $this->toRequiredValues($key);
        if ($this->db->exists($this->table, $whereArr)) {
            $this->db->updateValue($this->table, $this->valueColumn, $value, $whereArr);
        } else {
            $values = $whereArr;
            $values[$this->valueColumn] = $value;
            $this->db->insert($this->table, $values);
        }
        return true;
    }

    /**
     *
     * @param string $key
     * @return string[string] column value pairs
     */
    protected function toRequiredValues($key)
    {
        $requiredValues = array();
        foreach ($this->requiredValues as $rkey => $rvalue) {
            $requiredValues[$rkey] = $rvalue;
        }

        $requiredValues[$this->keyColumn] = $key;
        return $requiredValues;
    }

    protected function getQuotedValueColumn()
    {
        return "`" . $this->valueColumn . "`";
    }

    protected function getQuotedKeyColumn()
    {
        return "`" . $this->keyColumn . "`";
    }
}
