<?php
namespace Topalis\Dashboard\Controller;

use Topalis\Util\Process;
use Topalis\Util\DateUtil;
use Symfony\Component\HttpFoundation\JsonResponse;
use Topalis\Kernel\Controller;
use Topalis\Dashboard\System;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Controller for dashboard
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/admin", service="dashboard.controller")
 */
class DashboardController extends Controller
{
    /**
     * @return \Topalis\Util\DateUtil
     */
    private function getDateUtil()
    {
        static $dateUtil = null;
        if ($dateUtil === null) {
            $dateUtil = new DateUtil($this->getTranslator());
        }
        return $dateUtil;
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Topalis\Device\DeviceManager
     */
    private function getDeviceManager()
    {
        return $this->container->get("device.manager");
    }

    /**
     * @return \Topalis\ActiveSyncFilter\FilterServer
     */
    private function getFilterServer()
    {
        return $this->container->get("asf.filter_server");
    }

    /**
     * @return \Topalis\Maintenance\Tunnel
     */
    private function getMaintenanceTunnel()
    {
        return $this->container->get("maintenance.tunnel");
    }

    /**
     * @return \Maxmind\Bundle\GeoipBundle\Service\GeoipManager
     */
    private function getGeoIP()
    {
        return $this->container->get("maxmind.geoip");
    }

    /**
     * @return \Symfony\Component\Templating\Helper\CoreAssetsHelper
     */
    private function getAssetHelper()
    {
        return $this->container->get("templating.helper.assets");
    }

    /**
     * @Route("/", name="admin_dashboard")
     * @Template()
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function indexAction()
    {
        $system = new System();

        $trafficStats = $this->getAccessStat();
        $trafficEndDate = $this->getTimeProvider()->getTime();
        // last 30 days
        $trafficStartDate = $trafficEndDate - 2592000;

        return array(
            "system"           => $system,
            "data"             => $this->jsonAction(false),
            "deviceCount"      => $this->getDeviceTypeCount(),
            "deviceOwner"      => $this->getDeviceOwner(),
            "deviceTraffic"    => $this->getDeviceTraffic(),
            "topApps"          => $this->getTopApps(),
            "trafficLast24h"   => $this->getTrafficLast24h(),
            "trafficStats"     => $trafficStats,
            "trafficSeries"    => $this->getTrafficSeries($trafficStartDate, $trafficEndDate),
            "trafficStartDate" => $trafficStartDate,
            "trafficEndDate"   => $trafficEndDate,
            "trafficMinDate"   => $this->getDeviceManager()->getLogStartDate()->format("U"),
            "mimeTypeData"     => $this->getMimeTypeData($trafficStartDate, $trafficEndDate),
            "networkConnectivity" => $this->getNetworkConnectivity(),
        );
    }

    /**
     * @Route("/index.php", name="admin_dashboard_old")
     * @deprecated This method is used to have a route for /index.php after
     *     an update from version 2.0
     */
    public function oldIndexAction()
    {
        return new RedirectResponse($this->generateUrl("admin_dashboard"));
    }

    /**
     * @Route("/dashboard/data", name="admin_dashboard_json", defaults={"_format"="json"})
     * @param bool $sleep
     *
     * @return \Symfony\Component\HttpFoundation\JsonResponse
     */
    public function jsonAction($sleep = true)
    {
        $system = new System($sleep);
        $output = array();
        $memory = $system->getMemory();
        $output['memory'] = array(
            "percentage" => $memory->getPercentage(),
            "used"       => $memory->getUsed(),
            "total"      => $memory->getTotal(),
            "sawp"       => $memory->getSwapUsed(),
        );

        $cpu = $system->getCPU();
        $output['cpu'] = array(
            "user" => number_format($cpu->getUser(), 2, ".", ""),
            "system" => number_format($cpu->getSystem(), 2, ".", ""),
            "ioWait" => number_format($cpu->getIoWait(), 2, ".", ""),
            "nice" => number_format($cpu->getNice(), 2, ".", ""),
            "total" => number_format($cpu->getLoad(), 2, ".", ""),
            "percentage" => $cpu->getPercentage()
        );

        $disks = $system->getFilesystem()->getDisks();
        foreach ($disks as $disk) {
            $output['disks'][] = array(
                "mountPoint" => $disk->getMountPoint(),
                "used" => $disk->getUsed(),
                "size" => $disk->getSize(),
                "percentage" => ($disk->getSize() > 0) ? $disk->getUsed() / $disk->getSize() : 1,
            );
        }

        $interfaces = $system->getNetwork()->getInterfaces();
        foreach ($interfaces as $interface) {
            $output['interfaces'][] = array(
                "name" => $interface->getName(),
                "received" => $interface->getReceivedSpeed(),
                "send" => $interface->getSendSpeed()
            );
        }

        $output['devices'] = $this->getLastActiveDevices();

        return new JsonResponse($output);
    }

    /**
     * @Route("/dashboard/traffic/data", name="admin_dashboard_traffic_data",
     *   defaults={"_format"="json", "expose" = true})
     *
     * @return \Symfony\Component\HttpFoundation\JsonResponse
     */
    public function trafficDataAction()
    {
        $start = $this->getRequest()->get("start");
        if ($start !== null) {
            $start = intval($start);
        }

        $end = $this->getRequest()->get("end");
        if ($end !== null) {
            $end = intval($end);
        }

        return new JsonResponse($this->getTrafficSeries($start, $end));
    }

    /**
     * @Route("/dashboard/mimetype/data", name="admin_dashboard_mimetype_data",
     *   defaults={"_format"="json", "expose" = true})
     *
     * @return \Symfony\Component\HttpFoundation\JsonResponse
     */
    public function mimeTypeDataAction()
    {
        $start = $this->getRequest()->get("start");
        if ($start !== null) {
            $start = intval($start);
        }

        $end = $this->getRequest()->get("end");
        if ($end !== null) {
            $end = intval($end);
        }

        return new JsonResponse($this->getMimeTypeData($start, $end));
    }

    private function getDeviceTypeCount()
    {
        $typeFactory = $this->container->get("device.type.factory");
        $rv = array();
        foreach ($this->getDeviceManager()->getDeviceTypeCount() as $os => $count) {
            $rv[] = array(
                "name" => $this->tr($typeFactory->getTypeById($os)->getTranslation()),
                "count" => $count
            );
        }

        return $rv;
    }

    /**
     * @return array Array with key owner and value number of devices.
     */
    private function getDeviceOwner()
    {
        $rv = array(
            "company" => 0,
            "user" => 0
        );
        $owners = $this->getDeviceEntityManager()->createQueryBuilder("d")
            ->select("d.owner, COUNT(d) AS number")
            ->groupBy("d.owner")
            ->getQuery()->getResult();
        foreach ($owners as $owner) {
            $rv[$owner['owner']] = $owner['number'];
        }
        return $rv;
    }

    /**
     * @return array
     */
    private function getLastActiveDevices()
    {
        $db = $this->getDB();
        $sql = "SELECT devices.deviceid, devices.cert_id, devices.name, devices.comment, devices.os,
            users.id AS userId, users.login, users.realname,
            a.ip AS ip, a.tstamp AS tstamp
            FROM devices
            JOIN accesslog a
            ON devices.deleted = 0 AND a.id = (SELECT MAX(id) FROM accesslog al WHERE al.device_id = devices.id)
            JOIN users ON users.id = devices.user_id
            ORDER BY a.id DESC LIMIT 0,5";
        $devices = $db->queryObjects($sql);

        $deviceList = array();
        foreach ($devices as $device) {
            $location = null;
            if ($device->ip) {
                $ip = $this->getGeoIP()->lookup($device->ip);
                if ($ip) {
                    $location = strtolower($ip->getCountryCode());
                }
            }
            if (strlen($location) > 0) {
                $icon = $this->getAssetHelper()->getUrl("bundles/topaliskernel/img/countries/" . $location . ".png");
            } else {
                $location = "-";
                $icon = null;
            }

            $timestamp = strtotime($device->tstamp);
            $data = array(
                "deviceid" => $device->deviceid,
                "login" => $device->login,
                "name" => $device->name,
                "comment" => $device->comment,
                "os" => $device->os,
                "location" => $location,
                "locationIcon" => $icon,
                "timestamp" => $timestamp,
                "date" => $this->getDateUtil()->since($this->getTimeProvider()->getTime() - $timestamp, true),
                "dateTime" => date(DATE_W3C, $timestamp)
            );

            $deviceList[] = $data;
        }

        return $deviceList;
    }

    private function getAccessStat()
    {
        $statTotal = false;
        $statLastMonth = false;
        $statCurrentMonth = false;
        $deviceManagement = $this->getDeviceManager();
        $asfTrafficStat = $this->getFilterServer()->getTrafficStat();
        try {
            // in rare cases we have increased data by some bytes #AUR-242, so we say we have saved 0 traffic
            $statTotal = $deviceManagement->getStatTotal();
            $statTotal->traffic_saved = max(array($asfTrafficStat->getTotalCompression(), 0));
            $statLastMonth = $deviceManagement->getStatLastMonth();
            $statCurrentMonth = $deviceManagement->getStatThisMonth();
            $statToday = $deviceManagement->getStatToday();
            $asfSavedToday = $asfTrafficStat->getTrafficDayLast(1);
            $statToday->traffic_saved = max(array($asfSavedToday[0]->diff, 0));
            $asfSaved = $asfTrafficStat->getTrafficMonthEach("-1 month", "now");
            $statLastMonth->traffic_saved = isset($asfSaved[1]) ? max(array($asfSaved[1]->diff, 0)) : 0;
            $statCurrentMonth->traffic_saved = isset($asfSaved[0]) ? max(array($asfSaved[0]->diff, 0)) : 0;
        } catch (\Exception $e) {
            $this->getLogger()->error("Error loading device stats", array("exception" => $e));
            $msg = $this->tr("admin.devices.error.loadStats", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
            throw $e;
        }

        return array(
            "total"     => $statTotal,
            "lastMonth" => $statLastMonth,
            "thisMonth" => $statCurrentMonth,
            "today"     => $statToday,
        );
    }

    /**
     *
     * @return \stdClass[]
     */
    private function getDeviceTraffic()
    {
        $db = $this->getDB();
        $sql =
            "SELECT devices.os, users.login, devices.name, devices.comment,
                SUM(access.traffic) AS traffic
            FROM access_cert_stat AS access
            JOIN devices ON devices.deleted = 0 AND access.device_id = devices.id
            JOIN users ON users.id = devices.user_id
            GROUP BY devices.id
            ORDER BY traffic DESC LIMIT 0, 5";
        $devices = $db->queryObjects($sql);

        return $devices;
    }

    /**
     * @param integer|string $start The date to start from as timestamp or as
     *     string PHP can interpret as date.
     * @param integer|string $end The date to end as timestamp or as string PHP
     *     can interpret as date.
     * @return array Pairs of timestamp and traffic in bytes.
     */
    private function getTrafficStats($start = null, $end = null)
    {
        return $this->getDeviceManager()->getStatDayArray($start, $end);
    }

    /**
     * @param integer|string $start The date to start from as timestamp or as
     *     string PHP can interpret as date.
     * @param integer|string $end The date to end as timestamp or as string PHP
     *     can interpret as date.
     * @return array Pairs of timestamp and traffic in bytes.
     */
    private function getASFTraffic($start = null, $end = null)
    {
        return $this->getFilterServer()->getTrafficStat()->getTrafficDayArray($start, $end);
    }

    private function getMimeTypeData($start = null, $end = null)
    {
        $mimeTypes = array();
        foreach ($this->getMimeTypeStats($start, $end) as $mediaTypeStats) {
            foreach ($mediaTypeStats as $subTypeStat) {
                $mimeType = array();
                $mimeType['name'] = $subTypeStat->getMimeType();
                $mimeType['total'] = $subTypeStat->getTotal();
                $mimeType['changed'] = $subTypeStat->getChanged();
                $mimeType['unchanged'] = $subTypeStat->getUnchanged();
                $mimeType['removed'] = $subTypeStat->getRemoved();
                $mimeType['failed'] = $subTypeStat->getFailed();
                $mimeType['inSize'] = $subTypeStat->getInSize();
                $mimeType['outSize'] = $subTypeStat->getOutSize();
                $mimeType['diff'] = $subTypeStat->getDiff();
                $mimeType['lastSeen'] = $subTypeStat->getLastChange();
                $mimeType['lastSeenFormatted'] = date($this->tr("Y-m-d H:i:s"), $mimeType['lastSeen'] / 1000);
                $mimeTypes[] = $mimeType;
            }
        }

        return $mimeTypes;
    }

    private function getMimeTypeStats($start = null, $end = null)
    {
        return $this->getFilterServer()->getMimeTypeStat()->getAll($start, $end);
    }

    /**
     * @return array Array with the following keys: saved, real, theoretical.
     *     The value is a number of bytes.
     */
    private function getTrafficLast24h()
    {
        $trafficLast24h = array(
            "real" => $this->getDeviceManager()->getTrafficLast24h(),
            "saved" => $this->getFilterServer()->getTrafficStat()->getCompressionLast24h()
        );
        if ($trafficLast24h['saved'] < 0) {
            // in rare cases we have increased data by some bytes #AUR-242
            $trafficLast24h['saved'] = 0;
        }
        $trafficLast24h['theoretical'] = $trafficLast24h['real'] + $trafficLast24h['saved'];
        return $trafficLast24h;
    }

    private function getTrafficSeries($start = null, $end = null)
    {
        $traffic = array();
        $theoreticalTraffic = array();
        $savedTraffic = array();
        try {
            if ($end === null) {
                $end = $this->getTimeProvider()->getTime();
            } else {
                $end = min(array($end, $this->getTimeProvider()->getTime()));
            }
            if ($start === null) {
                $start = $end - 30 * 86400;
            }

            if ($start > $end) {
                return array();
            }

            // the last value should be not more than 10 seconds before $end
            $lastExpectedTstamp = $end - 10;

            $realTraffic = $this->getTrafficStats($start, $end);
            $asfTraffic = $this->getASFTraffic($start, $end);

            if (count($realTraffic) == 0) {
                // no data, yet
                $realTraffic[$end] = 0;
                $asfTraffic[$end] = 0;
            }

            if (!isset($realTraffic[$start])) {
                // obviously there was no traffic at choosen start
                reset($realTraffic);
                $firstTime = key($realTraffic);
                // no traffic between $start and $firstTime - 1
                $realTraffic = array($start => 0, $firstTime - 1 => 0) + $realTraffic;
                $asfTraffic = array($start => 0, $firstTime - 1 => 0) + $asfTraffic;
            }

            end($realTraffic);
            if (key($realTraffic) < $lastExpectedTstamp) {
                $tmpTime = min(array(key($realTraffic) + 10, $lastExpectedTstamp));
                $realTraffic[$tmpTime] = $asfTraffic[$tmpTime] = 0;
                $realTraffic[$lastExpectedTstamp] = $asfTraffic[$lastExpectedTstamp] = 0;
            }

            foreach ($realTraffic as $time => $real) {
                if (!isset($asfTraffic[$time])) {
                    $diff = 0;
                } else {
                    // in rare cases we have increased data by some bytes #AUR-242
                    $diff = $asfTraffic[$time] < 0 ? 0 : $asfTraffic[$time];
                }

                $traffic[] = array("x" => $time, "y" => $real);
                $theoretical = $real + $diff;
                $theoreticalTraffic[] = array("x" => $time, "y" => $theoretical);
                $savedTraffic[] = array("x" => $time, "y" => $diff);
            }
        } catch (\Exception $e) {
            throw $e;
        }

        $output = array(
            array(
                "key" => "theoretical",
                "name" => $this->tr("admin.dashboard.traffic.theoretical"),
                "color" => "rgba(192,132,255,0.3)",
                "stroke" => "rgba(0,0,0,0.15)",
                "data" => $theoreticalTraffic
            ),
            array(
                "key" => "real",
                "name" => $this->tr("admin.dashboard.traffic.real"),
                "color" => "rgba(96,170,255,0.5)",
                "stroke" => "rgba(0,0,0,0.15)",
                "data" => $traffic
            ),
            array(
                "key" => "saved",
                "name" => $this->tr("admin.dashboard.traffic.saved"),
                "color" => "rgba(63,154,6,0.5)",
                "stroke" => "rgba(0,0,0,0.15)",
                "data" => $savedTraffic,
                "disabled" => true
            ),
        );

        return $output;
    }

    /**
     * @return array
     */
    private function getTopApps()
    {
        $androidTopApps = $this->container->get("app.entity.manager.android_app")->getTopApps(10);
        $iosTopApps = $this->container->get("app.entity.manager.ios_app")->getTopApps(10);
        return array(
            "android" => $androidTopApps,
            "ios" => $iosTopApps
        );
    }

    /**
     * @return boolean true if the APNSD is running and connected, false
     *     otherwise
     */
    private function isApnsdActive()
    {
        return $this->getConfig()->isApnsdActive()
        && Process::exec("pgrep -F /var/run/auralis/apnsd.pid") === 0;
    }

    /**
     * @param string|NULL $errorMessage Contains the last error message if the
     *     status is "ERROR".
     * @return string DISABLED, OK, ERROR, INVALID_API_KEY
     */
    private function getGcmdStatus(&$errorMessage = null)
    {
        if (Process::exec("pgrep -F /var/run/auralis/gcmd.pid") === 0) {
            $status = $this->getConfig()->getGcmStatus();
        } else {
            // GCMD not running, check if it should
            $gcmConfig = $this->getConfig()->getGcmConfig();
            if ($gcmConfig->getApiKey() && $gcmConfig->getProjectId()) {
                $status = "ERROR";
            } else {
                $status = "DISABLED";
            }
        }

        if ($status == "ERROR" && $errorMessage !== null) {
            $errorMessage = $this->getConfig()->getGcmErrorMessage();
        }
        return $status;
    }

    /**
     * @return array
     */
    private function getNetworkConnectivity()
    {
        // apple push certificate
        $apnsCertMissing = false;
        $apnsCert = $this->getConfig()->getApnsCert();
        if (null == $apnsCert) {
            $apnsCertMissing = true;
        }
        $apnsdStatus = $this->isApnsdActive();
        $apnsCertRemainingDays = $this->getLicense()->getRemainingApnsCertDays();
        $apnsMessage = null;
        $apnsWarning = false;
        if ($apnsCertMissing) {
            $apnsMessage = $this->tr("admin.dashboard.push.cert.missing", array(
                "%url%" => $this->generateUrl("admin_config_push")
            ));
        } else if ($apnsCertRemainingDays == 0) {
            $apnsMessage = $this->tr("admin.dashboard.push.cert.invalid", array(
                "%url%" => $this->generateUrl("admin_config_push")
            ));
        } else if (!$apnsdStatus) {
            $apnsWarning = true;
            $apnsMessage = $this->tr("admin.dashboard.apnsd.inactive.warning");
        } else if ($apnsCertRemainingDays <= 90) {
            $apnsMessage = $this->tr("admin.dashboard.push.cert.becomesInvalid", array(
                "%remainingDays%" => $apnsCertRemainingDays,
                "%url%" => $this->generateUrl("admin_config_push")
            ));
        }

        // GCM
        $gcmErrorMessage = "";
        $gcmdStatus = $this->getGcmdStatus($gcmErrorMessage);
        $gcmMessage = null;
        $gcmWarning = false;
        if ($gcmdStatus == "ERROR") {
            $gcmMessage = $this->tr("admin.dashboard.gcmd.error", array("%errMsg%" => $gcmErrorMessage));
            $gcmWarning = true;
        } else if ($gcmdStatus == "INVALID_API_KEY") {
            $gcmMessage = $this->tr("admin.dashboard.gcmd.invalidApiKey", array(
                "%url%" => $this->generateUrl("admin_config_push_gcm")
            ));
        } else if ($gcmdStatus == "DISABLED") {
            $gcmMessage = $this->tr("admin.dashboard.gcmd.disabled", array(
                "%url%" => $this->generateUrl("admin_config_push_gcm")
            ));
        }

        // compose network connectivity
        $networkConnectivity = array(
            "apns" => array(
                "name" => $this->tr("admin.dashboard.apnsd.name"),
                "status" => $this->isApnsdActive(),
                "message" => $apnsMessage,
                "warning" => $apnsWarning,
                "certMissing" => $apnsCertMissing,
                "certRemainingDays" => $apnsCertRemainingDays
            ),
            "gcm" => array(
                "name" => $this->tr("admin.dashboard.gcmd.name"),
                "status" => $gcmdStatus == "OK",
                "message" => $gcmMessage,
                "warning" => $gcmWarning
            ),
        );

        // maintenance tunnel. first check if the maintenance tunnel is supported
        if ($this->getMaintenanceTunnel()->isSupportedSystem()) {
            $tunnelMsg = null;
            try {
                $tunnelStatus = $this->getMaintenanceTunnel()->status();
            } catch (\Exception $e) {
                $tunnelStatus = null;
                $this->getLogger()->error("Error getting tunnel status", array("exception" => $e));
                $tunnelMsg = $this->tr("admin.maintenance.error.status", array("%errmsg%" => $e->getMessage()));
                $this->addError($tunnelMsg);
            }
            $networkConnectivity['tunnel'] = array(
                "name" => $this->tr("admin.dashboard.tunnel.name"),
                "status" => $tunnelStatus,
                "message" => $tunnelMsg,
                "warning" => $tunnelStatus === null
            );
        }

        // update. first check if updates are possible in general
        if ($this->container->has("update.manager")) {
            $updateMessage = null;
            $updateWarning = false;
            if ($this->getLicense()->isExpired()) {
                $updateStatus = false;
                $updateMessage = $this->tr("admin.update.error.licenseExpired");
            } else if ($this->getLicense()->isTestLicense() && !$this->getLicense()->existsApnsCertificate()) {
                $updateStatus = false;
                $updateMessage = $this->tr("admin.update.error.testApnsCertRequired");
            } else {
                $updateErrorMessage = "";
                $updateStatus = $this->container->get("update.manager")->checkConnection($updateErrorMessage);
                if (!$updateStatus) {
                    $updateMessage = $this->tr("admin.dashboard.update.error", array("%errMsg%" => $updateErrorMessage));
                    $updateWarning = true;
                }
            }

            $networkConnectivity['update'] = array(
                "name" =>  $this->tr("admin.dashboard.update.name"),
                "status" => $updateStatus,
                "message" => $updateMessage,
                "warning" => $updateWarning
            );
        }

        return $networkConnectivity;
    }
}
