<?php
namespace Topalis\Database;

use Doctrine\ORM\QueryBuilder;
use InvalidArgumentException;
use Doctrine\Common\Collections\Criteria;

/**
 * Entity manager for common tasks
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@to.com>
 *
 */
class EntityManager
{
    /**
     * @var \Doctrine\ORM\EntityManager
     */
    protected $em;
    protected $class;
    protected $repository;

    public function __construct(\Doctrine\ORM\EntityManager $em, $class)
    {
        $this->em = $em;
        $this->class = $class;
    }

    public function beginTransaction()
    {
        $this->em->beginTransaction();
    }

    public function commit()
    {
        $this->em->commit();
    }

    public function rollback()
    {
        $this->em->rollback();
    }

    public function getClass()
    {
        return $this->class;
    }

    public function setClass($class)
    {
        $this->class = $class;

        return $this;
    }

    public function clear()
    {
        $this->em->clear();
    }

    public function create()
    {
        return new $this->class();
    }

    /**
     * @return \Doctrine\ORM\EntityRepository
     */
    public function getRepository()
    {
        return $this->em->getRepository($this->class);
    }

    /**
     * @param string $alias
     *
     * @return QueryBuilder
     */
    public function createQueryBuilder($alias)
    {
        return $this->getRepository()->createQueryBuilder($alias);
    }

    public function save($entity, $flush = true)
    {
        if (!$entity instanceof $this->class) {
            throw new InvalidArgumentException(
                sprintf("entity must be instance of %s. %s given", $this->class, get_class($entity)));
        }
        $this->em->persist($entity);
        if ($flush) {
            $this->em->flush();
        }
    }

    public function contains($entity)
    {
        return $this->em->contains($entity);
    }

    /**
     * @param string|array $predicates Use "entity" as alias in the predicates
     *     to specify entity attributes. If an array is given it must contain
     *     pairs of field name and value; $parameters is ignored in that case.
     * @param \Doctrine\Common\Collections\ArrayCollection|array $parameters
     * @return int The number of entities matching the predicates.
     */
    public function count($predicates = null, $parameters = null)
    {
        $qb = $this->createQueryBuilder("entity")->select("COUNT(entity)");
        if (is_array($predicates)) {
            $i = 0;
            foreach ($predicates as $field => $value) {
                $qb->andWhere("entity." . $field . " = ?$i");
                $qb->setParameter($i, $value);
                $i++;
            }
        } else {
            if ($predicates !== null) {
                $qb->where($predicates);
            }

            if ($parameters !== null) {
                $qb->setParameters($parameters);
            }
        }

        return intval($qb->getQuery()->getSingleScalarResult());
    }

    public function delete($entity, $flush = true)
    {
        if (!$entity instanceof $this->class) {
            throw new InvalidArgumentException(
                sprintf("entity must be instance of %s. %s given", $this->class, get_class($entity)));
        }
        $this->em->remove($entity);
        if ($flush) {
            $this->em->flush();
        }
    }

    /**
     * @param string|array $predicates Use "entity" as alias in the predicates
     *     to specify entity attributes. If an array is given it must contain
     *     pairs of field name and value; $parameters is ignored in that case.
     * @param \Doctrine\Common\Collections\ArrayCollection|array $parameters
     * @return boolean true if at least one entity matching the predicates
     *     exist, false otherwise.
     */
    public function exists($predicates = null, $parameters = null)
    {
        $qb = $this->createQueryBuilder("entity")->select("1")->setMaxResults(1);
        if (is_array($predicates)) {
            $i = 0;
            foreach ($predicates as $field => $value) {
                $qb->andWhere("entity." . $field . " = ?$i");
                $qb->setParameter($i, $value);
                $i++;
            }
        } else {
            if ($predicates !== null) {
                $qb->where($predicates);
            }

            if ($parameters !== null) {
                $qb->setParameters($parameters);
            }
        }
        $query = $qb->getQuery();
        return count($query->execute()) == 1;
    }

    public function flush()
    {
        $this->em->flush();
    }

    public function findOneBy(array $criteria, array $orderBy = null)
    {
        return $this->getRepository()->findOneBy($criteria, $orderBy);
    }

    /**
     * @param integer $id
     *
     * @return object
     */
    public function find($id)
    {
        return $this->getRepository()->find($id);
    }

    public function findById($id)
    {
        return $this->find($id);
    }

    public function findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
    {
        return $this->getRepository()->findBy($criteria, $orderBy, $limit, $offset);
    }

    public function findAll()
    {
        return $this->getRepository()->findAll();
    }

    public function deleteBy(array $criteria)
    {
        $entities = $this->getRepository()->findBy($criteria);
        foreach ($entities as $entity) {
            $this->delete($entity, false);
        }
        $this->flush();
    }

    public function detatch($entity)
    {
        $this->em->detach($entity);
    }

    public function merge($entity)
    {
        return $this->em->merge($entity);
    }

    public function refresh($entity)
    {
        $this->em->refresh($entity);
    }

    /**
     * Select all elements from a selectable that match the expression and
     * return a new collection containing these elements.
     *
     * @param \Doctrine\Common\Collections\Criteria $criteria
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function matching(Criteria $criteria)
    {
        return $this->em->getRepository()->matching($criteria);
    }
}
