<?php
namespace Topalis\Database\Migrations;

use Doctrine\ORM\Query;
use Doctrine\DBAL\Schema\Schema;

/**
 * Class for the database migration to auralis 2.1.0
 * It contains code to fill new tables based on data in old tables.
 * Extend this class in the class created by doctrine:migrations:diff and call
 * the up and down methods. Note the comments on these mhetods.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
abstract class AuralisMigration020100 extends AbstractAuralisMigration
{
    /**
     * Must be called after tables have been created and new columns to existing
     * ones have been updated, but before tables are dropped. This implies that
     * the new ActiveSync Filter Server has been started before.
     *
     * Note: The column profile_id in devices MUST NOT be dropped when calling
     * this method!
     *
     * @param Schema $schema
     */
    public function up(Schema $schema)
    {
        $schema; // unused

        $this->disableEventListeners();
        $this->disableFilters();

        $this->getConfig()->removeConfigValue("caserialnegative");
        $this->getConfig()->removeConfigValue("httproxy");
        $this->getConfig()->removeConfigValue("apnspubkey");

        $this->createGroups();
        $this->updateDeviceVersions();

        if ($this->connection->getHost() == $this->getFilterServer()->getDatabase()->getHost()
            && $this->connection->getDatabase() == $this->getFilterServer()->getDatabase()->getName()) {
            // update ASF config if it runs on the same database
            $this->createASFGroupConfig("conf");
            $this->createASFGroupConfig("conf_remove");
            $this->createASFGroupConfig("conf_plugins");

            $this->addASFForeigenKeyConfigId("conf");
            $this->addASFForeigenKeyConfigId("conf_remove");
            $this->addASFForeigenKeyConfigId("conf_plugins");
            $this->addASFForeigenKeyConfigId("conf_plugin");

            foreach (array_keys($this->getFilterServer()->getPluginManager()->getPlugins()) as $pluginKey) {
                $this->convertPluginConfig($pluginKey);
            }
        }

        // Disallow removal of WebClips TID #3393
        $this->addSql("UPDATE reverseproxy_urls SET removable = 0");
    }

    public function down(Schema $schema)
    {
        $schema; // unused

        throw new \Exception("down is not implemented");
    }

    private function addASFForeigenKeyConfigId($table)
    {
        // This is not possible as long as the ASF tables are of type MyISAM
        // @codingStandardsIgnoreStart
        /*
        $prefixedTable = $this->getFilterServer()->getServerConfig()->getJdbcTablePrefix() . $table;
        $schema = $this->getFilterServer()->getDatabase()->getDatabaseName();
        $this->addSql(sprintf("ALTER TABLE %1$s.%2$s ADD CONSTRAINT fk_%2$s_configgroup_id
            FOREIGN KEY (configid) REFERENCES configgroups (id)"), $schema, $prefixedTable);
        */
        // @codingStandardsIgnoreEnd
    }

    private function dropASFForeignKeyConfigId($table)
    {
        // This is not possible aslong as the ASF tables are of type MyISAM
        // @codingStandardsIgnoreStart
        /*
        $prefixedTable = $this->getFilterServer()->getServerConfig()->getJdbcTablePrefix() . $table;
        $schema = $this->getFilterServer()->getDatabase()->getDatabaseName();
        $this->addSql(sprintf("ALTER TABLE %1$s.%2$s DROP FOREIGN KEY fk_%2$s_configgroup_id"),
            $schema, $prefixedTable);
        */
        // @codingStandardsIgnoreEnd
    }

    private function createGroups()
    {
        $config = $this->getConfig();

        $stmt = $this->connection->executeQuery("SELECT * FROM profiles");
        $stmt->setFetchMode(Query::HYDRATE_ARRAY);
        foreach ($stmt as $profile) {
            $groupId = $profile['id'];
            $params = array(
                "id" => $groupId, "name" => $profile['name'], "comment" => $profile['comment'],
                "profile" => base64_decode($profile['content']), "allowmove" => $config->getConfigValue("allowmove", 0),
                "fromanyapp" => $config->getConfigValue("fromanyapp", 0),
                "syncdays" => $config->getConfigValue("syncdays", 3)
            );
            $this->addSql("INSERT INTO configgroups (id, name, comment, profile, allowmove, fromanyapp, syncdays)
            VALUES (:id, :name, :comment, :profile, :allowmove, :fromanyapp, :syncdays)", $params);

            $this->addSql("INSERT INTO configgroups_reverseproxyurls
                SELECT :groupid, id FROM reverseproxy_urls", array("groupid" => $groupId));
        }

        $config->removeConfigValue("allowmove");
        $config->removeConfigValue("allowprofileremoval");
        $config->removeConfigValue("fromanyapp");
        $config->removeConfigValue("syncdays");

        $this->addSql("UPDATE users u SET u.configgroup_id = (
            SELECT d.configgroup_id FROM devices d WHERE
                d.user_id = u.id AND d.id = (
                    SELECT MIN(d2.id) FROM devices d2 WHERE d2.user_id = u.id
                )
            )");
    }

    private function createASFGroupConfig($table)
    {
        $prefixedTable = $this->getFilterServer()->getServerConfig()->getJdbcTablePrefix() . $table;
        $asfConnection = $this->getFilterServer()->getDatabase()->getConnection();
        $columns = array_keys($asfConnection->getSchemaManager()->listTableColumns($prefixedTable));

        $this->addSql("UPDATE " . $prefixedTable . " SET configid = 0");

        $sql = "INSERT INTO " . $prefixedTable . " (" . implode(",", $columns) . ")";
        unset($columns[array_search("configid", $columns)]);
        $sql .= " SELECT " . implode(",", $columns) . ", cg.id FROM " . $prefixedTable;
        $sql .= " JOIN " . $this->connection->getDatabase() . ".configgroups cg";
        $this->addSql($sql);

        $this->addSql("DELETE FROM " . $prefixedTable . " WHERE configid = 0");
    }

    private function convertPluginConfig($pluginKey)
    {
        $asfTablePrefix = $this->getFilterServer()->getServerConfig()->getJdbcTablePrefix();
        $pluginTable = $asfTablePrefix . "plugin_" . $pluginKey;
        $confTable = $asfTablePrefix . "conf_plugin";

        $sql = "INSERT INTO " . $confTable . " (`key`, `value`, configid, pluginkey)";
        $sql .= " SELECT `key`, `value`, cg.id, :pluginKey FROM " . $pluginTable;
        $sql .= " JOIN " . $this->connection->getDatabase() . ".configgroups cg";
        $this->addSql($sql, array(":pluginKey" => $pluginKey));

        $this->addSql("DROP TABLE " . $pluginTable);
    }

    private function updateDeviceVersions()
    {
        $stmt = $this->connection
            ->executeQuery("SELECT id, deviceinformation FROM devices WHERE deviceinformation IS NOT NULL");
        $stmt->setFetchMode(Query::HYDRATE_ARRAY);
        foreach ($stmt as $device) {
            $value = json_decode($device['deviceinformation'], true);
            if (isset($value['OSVersion'])) {
                $this->addSql("UPDATE devices SET version = :version WHERE id = :id",
                    array("version" => $value['OSVersion'], "id" => $device['id']));
            }
        }
    }
}
