<?php
namespace Topalis\Database;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Statement;
use Doctrine\ORM\Query;
use Symfony\Bridge\Doctrine\RegistryInterface;
use Topalis\Kernel\Exception\Database\DatabaseException;
use Topalis\Util\TypeConverter;
use Topalis\Util\TypeCheck;
use Topalis\Util\StringUtil;

/**
 * A simple wrapper for accessing a MySQL database.
 *
 * @copyright Copyright (C) Thinking Objects GmbH
 * @author Frederick Roth <frederick.roth@topalis.com>
 * @author Fritz Elfert <fritz.elfert@topalis.com>
 * @author Holger Widmann <holger.widmann@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class MySqlDb
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * The PDO object.
     * @var Connection
     */
    protected $dbh;

    /**
     * @var boolean $ping
     * Flag: Use ping to recover a lost connection
     * before any query.
     */
    protected $ping = false;

    /**
     * Assoc array to cache table columns
     * @var array
     */
    protected $columns = array();

    /**
     * @var \Symfony\Bridge\Doctrine\RegistryInterface
     */
    private $doctrine;

    /**
     * Database object constructor.
     *
     * @param \Psr\Log\LoggerInterface                   $logger
     * @param \Symfony\Bridge\Doctrine\RegistryInterface $doctrine
     * @param string                                     $connectionName The connection name.
     */
    public function __construct(\Psr\Log\LoggerInterface $logger, RegistryInterface $doctrine,
        $connectionName = "default")
    {
        $this->logger = $logger;
        $this->doctrine = $doctrine;
        $this->dbh = $doctrine->getConnection($connectionName);
        $this->dbh->getPassword();
    }

    /**
     * @return \Symfony\Bridge\Doctrine\RegistryInterface
     */
    public function getDoctrine()
    {
        return $this->doctrine;
    }

    /**
     * @return \Doctrine\DBAL\Connection
     */
    public function getConnection()
    {
        return $this->dbh;
    }

    /**
     *
     * @return \Doctrine\DBAL\Connection
     */
    protected function getDbh()
    {
        return $this->dbh;
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    public function getLogger()
    {
        return $this->logger;
    }

    /**
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function setLogger(\Psr\Log\LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    /**
     * @return string The db configuration as string to use with the mysql
     *     command line utility.
     */
    public function getConfigAsCommandLine()
    {
        $host = $this->getHost();
        $port = $this->getDbh()->getPort();
        if ($port === null) {
            $port = 3306;
        }
        $database = $this->getName();
        $user = $this->getDbh()->getUsername();
        $password = $this->getDbh()->getPassword();
        $params = "--host=" . $host . " --port=". $port . " -u " . $user
            . " --password=\"" . $password . "\" " . $database;

        return $params;
    }

    /**
     * @return string The database host.
     */
    public function getHost()
    {
        $host = $this->getDbh()->getHost();
        if ($host === null) {
            $host = "localhost";
        }

        return $host;
    }

    /**
     * @return string The database name.
     */
    public function getName()
    {
        return $this->getDbh()->getDatabase();
    }

    /**
     * Sets the auto-reconnect option
     *
     * @param boolean $enabled true, to enable auto-reconnect, false otherwise.
     */
    public function setAutoReconnect($enabled)
    {
        $this->ping = $enabled;
    }

    /**
     * Quote an identifier for MySQL.
     * @param string $s The unquoted name.
     * @return string The quoted name.
     */
    protected function qid($s)
    {
        return '`' . $s . '`';
    }

    /**
     * Auto-reconnect, if the database connection was lost.
     * Since this is expensive, it is optional (default: off)
     *
     * @throws DatabaseException If connection fails.
     */
    protected function checkAndReconnect()
    {
        if (!$this->ping) {
            return;
        }

        // check
        try {
            $this->getDbh()->query('select 1;')->fetch();
        } catch (\Exception $e) {
            $this->getDbh()->close();
            $this->getDbh()->connect();
        }
    }

    /**
     * Get a database escaped string
     *
     * @param string $string The string to be escaped
     * @return string The escaped string.
     */
    public function getEscaped($string)
    {
        $ret = $this->getDbh()->quote($string);
        // remove quotes
        $ret = substr($ret, 1, -1);
        return $ret;
    }

    /**
     * Begin a transaction.
     *
     * @throws DatabaseException If transaction begin fails.
     */
    public function beginTransaction()
    {
        $this->getDbh()->beginTransaction();
    }

    /**
     * Commit a transaction.
     *
     * @throws DatabaseException If commit fails.
     */
    public function commitTransaction()
    {
        $this->getDbh()->commit();
    }

    /**
     * Checks if a constraint with the specified name exists.
     *
     * @param string $name The constraint name.
     * @return boolean true if a constraint with the specified name exists, false otherwise.
     * @throws DatabaseException On database error.
     */
    public function constraintExists($name)
    {
        $whereArr = array("CONSTRAINT_SCHEMA" => $this->getName(), "CONSTRAINT_NAME" => $name);
        return $this->exists("information_schema.TABLE_CONSTRAINTS", $whereArr);
    }

    /**
     * Counts the rows in the specified table that meet the specified condition.
     *
     * @param string $table The table to count the rows.
     * @param string[string] $whereArr column value pairs defining the WHERE
     *   condition.
     * @return int The number of rows matching the specified condition.
     */
    public function count($table, array $whereArr = null)
    {
        return intval($this->fetchValue($table, "count(*)", $whereArr));
    }

    /**
     * Execute an sql query.
     * @param string $sql    The SQL query to execute.
     * @param array  $params The parameters to bind to parameter markers in the
     *                        statement. Might be a simple array or an associative array. The
     *                        simple array is required for markers with question mark, the
     *                        assocative array is required for named markers. Then the keys must
     *                        be named as the markers; the leading ":" can be omitted.
     * @param int    $offset The offset for this query.
     * @param int    $limit  The limit for this query.
     *
     * @return int The number of rows that were modified or deleted by the SQL statement.
     * @throws DatabaseException On database error.
     */
    public function exec($sql, array $params = null, $offset = 0, $limit = 0)
    {
        $stmt = null;
        try {
            $stm = $this->query($sql, $params, $offset, $limit);
            $rowCount = $stm->rowCount();
            $this->closeStatement($stmt);
            return $rowCount;
        } catch (\Exception $e) {
            $this->closeStatement($stmt);
            $this->handleException($e);
        }
    }

    /**
     * Execute an sql query.
     *
     * @param string $sql    The SQL query to execute.
     * @param array  $params The parameters to bind to parameter markers in the
     *                        statement. Might be a simple array or an associative array. The
     *                        simple array is required for markers with question mark, the
     *                        assocative array is required for named markers. Then the keys must
     *                        be named as the markers; the leading ":" can be omitted.
     * @param int    $offset The offset for this query.
     * @param int    $limit  The limit for this query.
     *
     * @return Statement The query result.
     */
    public function query($sql, array $params = null, $offset = 0, $limit = 0)
    {
        $stmt = null;
        try {
            if ($limit > 0 || $offset > 0) {
                $sql .= ' LIMIT '.$offset.', '.$limit;
            }

            $this->checkAndReconnect();

            $execParams = array();
            if (TypeCheck::isAssocArray($params)) {
                foreach ($params as $column => $value) {
                    if (!StringUtil::startsWith($column, ":")) {
                        $column = ":$column";
                    }
                    $execParams[$column] = $value;
                }
            } else {
                $execParams = $params;
            }
            $stmt = $this->getDbh()->prepare($sql);
            $stmt->execute($execParams);
            return $stmt;
        } catch (\Exception $e) {
            $this->closeStatement($stmt);
            if ($e instanceof \PDOException) {
                $this->handleException($e, $sql, $params);
            }
            $this->handleException($e);
        }
    }

    /**
     * Checks if at least one row with the specified condition exists.
     *
     * @param string   $table    [string] $whereArr
     * @param string[] $whereArr column value pairs defining the WHERE
     *                           condition.
     *
     * @return boolean true if at lease one row exists that meets the specified
     *     condition, false otherwise.
     * @throws DatabaseException On database error.
     */
    public function exists($table, array $whereArr = null)
    {
        return $this->fetchValue($table, "1", $whereArr) == "1";
    }

    /**
     * @param string $table
     * @param array  $columns
     * @param array  $whereArr
     * @param array  $sortColumns
     * @param string $key
     *
     * @return Statement
     */
    private function prepareAndExec($table, array $columns = null, array $whereArr = null, array $sortColumns = null,
        $key = null)
    {
        static $fmtQuery = "SELECT %s FROM %s";
        static $fmtWhereQuery = "SELECT %s FROM %s WHERE %s";

        try {
            if (empty($columns)) {
                $columns[] = "*";
            }
            $colsStr = "";
            foreach ($columns as $column) {
                if ($column == "*") {
                    $colsStr .= "*, ";
                } else {
                    $colsStr .= $this->getEscaped($column) . ", ";
                }
            }
            $keyInSelect = $key === null ? true : in_array($key, $columns);
            if ($keyInSelect) {
                // remove trailing ", "
                $colsStr = substr($colsStr, 0, -2);
            } else {
                $colsStr .= $this->getEscaped($key);
            }

            if (!empty($whereArr)) {
                $whereStr = $this->toWhereStr($whereArr);
            }
            if (empty($whereStr)) {
                $query = sprintf($fmtQuery, $colsStr, $this->getEscaped($table));
            } else {
                $query = sprintf($fmtWhereQuery, $colsStr, $this->getEscaped($table), $whereStr);
            }
            if (!empty($sortColumns)) {
                $query .= " ORDER BY ";
                $query .= $this->getEscaped(implode(", ", $sortColumns));
            }
        } catch (\Exception $e) {
            $this->handleException($e);
        }

        return $this->query($query, $this->toParams($whereArr));
    }

    /**
     * Fetches and returns a single value.
     *
     * @param string   $table       The table to fetch the value from.
     * @param string   $valueColumn The column containing the value.
     * @param string[] $whereArr    column value pairs defining the WHERE condition.
     * @param int      $offset      The offset for this query.
     *
     * @return string The value in the first row matching the conditions,
     *                null on not found.
     */
    public function fetchValue($table, $valueColumn, array $whereArr = null, $offset = 0)
    {
        static $fmtQuery = "SELECT %s FROM %s";
        static $fmtWhereQuery = "SELECT %s FROM %s WHERE %s";
        $stmt = null;
        try {
            $ret = null;
            if ($whereArr !== null) {
                $whereStr = $this->toWhereStr($whereArr);
            }
            if (empty($whereStr)) {
                $query = sprintf($fmtQuery, $this->getEscaped($valueColumn), $this->getEscaped($table));
            } else {
                $whereStr = $this->toWhereStr($whereArr);
                $query = sprintf($fmtWhereQuery, $this->getEscaped($valueColumn), $this->getEscaped($table), $whereStr);
            }
            $stmt = $this->query($query, $this->toParams($whereArr), $offset, 1);
            if ($row = $stmt->fetch(\PDO::FETCH_NUM)) {
                $ret = $row[0];
            }
            $this->closeStatement($stmt);
            return $ret;
        } catch (\Exception $e) {
            $this->closeStatement($stmt);
            $this->handleException($e);
        }
    }

    /**
     * Fetch multiple columns into an object.
     *
     * @param string     $query  The SQL query to execute.
     * @param array|null $params sarameters to bind to parameter markers in the
     *                           statement. Might be a simple array or an associative array. The
     *                           simple array is required for markers with question mark, the
     *                           assocative array is required for named markers. Then the keys must
     *                           be named as the markers; the leading ":" can be omitted.
     * @param int        $offset The offset for this query.
     *
     * @return \stdClass The first row of the result or null on not found.
     * @throws DatabaseException On database error.
     */
    public function queryObject($query, array $params = null, $offset = 0)
    {
        $stmt = $this->query($query, $params, $offset, 1);
        try {
            $ret = null;
            if ($obj = $stmt->fetch(Query::HYDRATE_SIMPLEOBJECT)) {
                $ret = $obj;
            }
            $this->closeStatement($stmt);
            return $ret;
        } catch (\Exception $e) {
            $this->closeStatement($stmt);
            $this->handleException($e);
        }
    }

    /**
     * Fetches rows from the database based on the specified parameters and
     * returns the first as stdClass object.
     *
     * @param string $table   The table to fetch the rows from.
     * @param array  $columns The columns to fetch.
     * @param string[string] $whereArr Associative array with coloumn value
     *                        pairs defining the where condition.
     *
     * @return \stdClass The first row as stdClass object, or null if no matching
     *     row exists.
     * @throws DatabaseException On database error.
     */
    public function fetchObject($table, array $columns, array $whereArr)
    {
        $objs = $this->fetchObjects($table, $columns, $whereArr);
        if (!empty($objs)) {
            return $objs[0];
        }
        return null;
    }

    /**
     * Fetch multiple rows of multiple columns into an object array.
     *
     * @param string $query  The SQL query to execute.
     * @param array  $params The parameters to bind to parameter markers in the
     *                       statement. Might be a simple array or an associative array. The
     *                       simple array is required for markers with question mark, the
     *                       assocative array is required for named markers. Then the keys must
     *                       be named as the markers; the leading ":" can be omitted.
     * @param string $key    An optional column name to be used as key for the resulting array.
     *
     * @return \stdClass[] | \stdClass[string] Array containing the rows as
     *     stdClass objects. If a key column is specified the array keys contain
     *     the respective value of the key column.
     * @throws DatabaseException On database error.
     */
    public function queryObjects($query, array $params = null, $key = '')
    {
        $stmt = $this->query($query, $params);
        try {
            $ret = array();
            if ($key) {
                while ($row = $stmt->fetch(Query::HYDRATE_SIMPLEOBJECT)) {
                    $ret[$row->$key] = $row;
                }
            } else {
                while ($row = $stmt->fetch(Query::HYDRATE_SIMPLEOBJECT)) {
                    $ret[] = $row;
                }
            }
            $this->closeStatement($stmt);
            return $ret;
        } catch (\Exception $e) {
            $this->closeStatement($stmt);
            $this->handleException($e);
        }
    }

    /**
     * Executes the specified query and returns the value in the first column
     * in the first resulting row.
     *
     * @param string $query  The SQL query to execute.
     * @param array  $params The parameters to bind to parameter markers in the
     *                       statement. Might be a simple array or an associative array. The
     *                       simple array is required for markers with question mark, the
     *                       assocative array is required for named markers. Then the keys must
     *                       be named as the markers; the leading ":" can be omitted.
     *
     * @return string The value in the first row matching the conditions, or
     *     null on not found.
     * @throws DatabaseException On database error.
     */
    public function queryValue($query, array $params = null)
    {
        $stmt = $this->query($query, $params);
        try {
            $ret = null;
            if ($row = $stmt->fetch(\PDO::FETCH_NUM)) {
                $ret = $row[0];
            }
            $this->closeStatement($stmt);
            return $ret;
        } catch (\Exception $e) {
            $this->closeStatement($stmt);
            $this->handleException($e);
        }
    }

    /**
     * Fetches rows from the database based on the specified parameters.
     *
     * @param string   $table       The table to fetch the rows from.
     * @param array    $columns     The columns to fetch.
     * @param string[] $whereArr    Associative array with coloumn value
     *                              pairs defining the where condition.
     * @param string[] $sortColumns The columns to sort ascending. May be null
     *                              or empty.
     * @param string   $key         The key column.
     *
     * @return row[string] | string[string] Associative array with the value of
     *   the key column as keys. If no key column is specified an ordinary array
     *   is returned. The values are associative arrays with pairs of column
     *   name and value, or if $columns only contains one column the column
     *   values as strings.
     * @throws DatabaseException On database error.
     */
    public function fetchRows($table, array $columns = null, array $whereArr = null, array $sortColumns = null,
        $key = null)
    {
        $oneColumn = count($columns) == 1 && $columns[0] != "*";
        $stmt = $this->prepareAndExec($table, $columns, $whereArr, $sortColumns, $key);
        try {
            $ret = array();
            while ($row = $stmt->fetch()) {
                if ($oneColumn) {
                    $value = reset($row);
                } else {
                    $value = $row;
                }
                if ($key) {
                    $ret[$row[$key]] = $value;
                } else {
                    $ret[] = $value;
                }
            }
            $this->closeStatement($stmt);
            return $ret;
        } catch (\Exception $e) {
            $this->closeStatement($stmt);
            $this->handleException($e);
        }
    }

    /**
     * Fetches rows from the database based on the specified parameters and
     * returns them as stdClass objects.
     *
     * @param string   $table       The table to fetch the rows from.
     * @param array    $columns     The columns to fetch.
     * @param string[] $whereArr    Associative array with coloumn value
     *                              pairs defining the where condition.
     * @param string[] $sortColumns The columns to sort. May be null or empty.
     *                              The column names may be followed by ASC/DESC to define the direction.
     * @param string   $key         The key column.
     *
     * @return \stdClass[] | \stdClass[string] Array containing the rows as
     *     stdClass objects. If a key column is specified the array keys contain
     *     the respective value of the key column.
     * @throws DatabaseException On database error.
     */
    public function fetchObjects($table, array $columns = null, array $whereArr = null,
        array $sortColumns = null, $key = null)
    {
        $stmt = $this->prepareAndExec($table, $columns, $whereArr, $sortColumns, $key);
        try {
            $ret = array();
            if ($key) {
                while ($row = $stmt->fetch(Query::HYDRATE_SIMPLEOBJECT)) {
                    $ret[$row->$key] = $row;
                }
            } else {
                while ($row = $stmt->fetch(Query::HYDRATE_SIMPLEOBJECT)) {
                    $ret[] = $row;
                }
            }
            $this->closeStatement($stmt);
            return $ret;
        } catch (\Exception $e) {
            $this->closeStatement($stmt);
            $this->handleException($e);
        }
    }

    /**
     * Checks if an index with the specified name exists.
     *
     * @param string $name The index name.
     *
     * @return boolean true if an index with the specified name exists, false otherwise.
     * @throws DatabaseException On database error.
     */
    public function indexExists($name)
    {
        $whereArr = array("TABLE_SCHEMA" => $this->getName(), "INDEX_NAME" => $name);

        return $this->exists("INFORMATION_SCHEMA.STATISTICS", $whereArr);
    }

    /**
     * Checks if a trigger with the specified name exists.
     *
     * @param string $name The trigger name.
     *
     * @return boolean true if a trigger with the specified name exists, false otherwise.
     * @throws DatabaseException On database error.
     */
    public function triggerExists($name)
    {
        $whereArr = array("TRIGGER_SCHEMA" => $this->getName(), "TRIGGER_NAME" => $name);

        return $this->exists("INFORMATION_SCHEMA.TRIGGERS", $whereArr);
    }

    /**
     * Inserts a row in the specified table with the specified values.
     *
     * @param string          $table   The table to insert the row.
     * @param string[]|object $data   Associative array with column value
     *                                 pairs containing the values to insert, or an object whose properties
     *                                 match table fields.
     * @param string          $keyName The name of the primary key. If provided the
     *                                 array or object property is updated.
     *
     * @throws DatabaseException On database error.
     */
    public function insert($table, &$data, $keyName = null)
    {
        static $fmtQuery = "INSERT INTO %s (%s) VALUES (%s)";
        try {
            if (is_object($data)) {
                $values = TypeConverter::objectToArray($data);
            } else {
                $values = $data;
            }

            $columns = array();
            $markers = array();
            $columnNames = array_keys($values);
            foreach ($columnNames as $column) {
                $columns[] = $this->qid($this->getEscaped($column));
                $markers[] = ":" . $column;
            }

            $query = sprintf($fmtQuery, $this->getEscaped($table), implode(", ", $columns), implode(", ", $markers));
            $this->exec($query, $values);
            if (strlen($keyName) > 0) {
                $id = $this->getDbh()->lastInsertId($keyName);
                if (is_object($data)) {
                    $data->$keyName = $id;
                } else {
                    $data[$keyName] = $id;
                }
            }
        } catch (\Exception $e) {
            $this->handleException($e);
        }
    }

    /**
     * Removes rows from the specified table matching the specified conditions.
     *
     * @param string   $table    The table to remove the rows from.
     * @param string[] $whereArr Associative array with coloumn value
     *                           pairs defining the where condition.
     *
     * @return int The number of rows removed.
     * @throws DatabaseException On error.
     */
    public function removeRows($table, array $whereArr)
    {
        static $fmtQuery = "DELETE FROM %s WHERE %s";
        $whereStr = $this->toWhereStr($whereArr);
        if (empty($whereStr)) {
            // remove all
            $whereStr = "1";
        }
        $query = sprintf($fmtQuery, $this->getEscaped($table), $whereStr);

        return $this->exec($query, $this->toParams($whereArr));
    }

    /**
     * Rollback a transaction.
     */
    public function rollbackTransaction()
    {
        $this->getDbh()->rollBack();
    }

    /**
     * Inserts or updates a table row based on an associative array or on object
     * properties. If a row with the specified key does not exist or no value
     * for the key column is specified, a new row is inserted, otherwise an
     * existing row is updated. If a new row is inserted and no key was specified
     * its value is set in the specified array/object.
     *
     * @param string          $table       The name of the table to update.
     * @param string[]|object $object     Associative array with column value
     *                                      pairs containing the values to merge, or an object whose properties
     *                                      match table fields.
     * @param string          $keyName     The name of the primary key.
     * @param boolean         $updateNulls If true null properties (if an object is given)
     *                                      are updated in the table.
     *
     * @return boolean true if a row is inserted or changed.
     * @throws DatabaseException On error.
     */
    public function merge($table, &$object, $keyName, $updateNulls = false)
    {
        try {
            $whereArr = array();
            if (is_object($object)) {
                $values = TypeConverter::objectToArray($object, $updateNulls);
            } else {
                $values = $object;
            }

            if (!array_key_exists($keyName, $values) || $values[$keyName] === null) {
                $keyExists = false;
            } else {
                $keyExists = true;
                $whereArr[$keyName] = $values[$keyName];
            }

            if (!$keyExists || !$this->exists($table, $whereArr)) {
                $this->insert($table, $values, $keyName);
                if (!$keyExists) {
                    // set key in parameter
                    if (is_object($object)) {
                        $object->$keyName = $values[$keyName];
                    } else {
                        $object[$keyName] = $values[$keyName];
                    }
                }
                return true;
            } else {
                // no need to update key column
                unset($values[$keyName]);
                return $this->updateValues($table, $values, $whereArr) == 1;
            }
        } catch (\Exception $e) {
            $this->handleException($e);
        }
    }

    /**
     * Updates a table row based on an associative array or on object properties.
     *
     * @param string          $table       The name of the table to update.
     * @param string[]|object $data        Associative array with column value
     *                                     pairs containing the values to update, or an object whose properties
     *                                     match table fields.
     * @param string          $keyName     The name of the primary key.
     * @param boolean         $updateNulls If true null properties (if an object is given)
     *                                     are updated in the table.
     *
     * @return boolean true, if a row is changed, false otherwise.
     * @throws DatabaseException On error.
     */
    public function update($table, $data, $keyName, $updateNulls = false)
    {
        try {
            if (is_object($data)) {
                $values = TypeConverter::objectToArray($data, $updateNulls);
            } else {
                $values = $data;
            }

            if (!array_key_exists($keyName, $values) || $values[$keyName] === null) {
                throw new DatabaseException("The key (column ". $keyName . ") must not be empty");
            }

            $whereArr = array($keyName => $values[$keyName]);
            // no need to update key column
            unset($values[$keyName]);
        } catch (\Exception $e) {
            $this->handleException($e);
        }
        return $this->updateValues($table, $values, $whereArr) > 0;
    }

    /**
     * Sets the values in the specified column of the specified table to the
     * specified value in rows matching the specified condition.
     *
     * @param string   $table       The table to update.
     * @param string   $valueColumn The column to set the value.
     * @param string   $value       The value to set.
     * @param string[] $whereArr    An associative array with coloumn value
     *                              pairs defining the where condition.
     *
     * @return int The number of affected rows.
     * @throws DatabaseException On error.
     */
    public function updateValue($table, $valueColumn, $value, array $whereArr)
    {
        return $this->updateValues($table, array($valueColumn => $value), $whereArr);
    }

    /**
     * Sets the values in the specified columns of the specified table to the
     * specified values in rows matching the specified condition.
     *
     * @param string   $table    The table to update.
     * @param string[] $values   An associative array mapping column names
     *                           to values.
     * @param string[] $whereArr An associative array with coloumn value
     *                           pairs defining the where condition.
     *
     * @return int The number of affected rows.
     * @throws DatabaseException On error.
     */
    public function updateValues($table, array $values, array $whereArr)
    {
        static $fmtQuery = "UPDATE %s SET %s WHERE %s";
        try {
            $tmp = array();
            $params = $this->toParams($whereArr);
            foreach ($values as $column => $value) {
                $tmp[] = $this->qid($this->getEscaped($column)) . "= :_" . $column . "_";
                $params["_" . $column . "_"] = $value;
            }

            $whereStr = $this->toWhereStr($whereArr);
            if (empty($whereStr)) {
                // update all
                $whereStr = "1";
            }
            $query = sprintf($fmtQuery, $this->getEscaped($table), implode(", ", $tmp), $whereStr);
        } catch (\Exception $e) {
            $this->handleException($e);
        }
        return $this->exec($query, $params);
    }

    /**
     * Closes the statement cursor if the specified statement is not null.
     * @param \PDOStatement|NULL $statement
     */
    protected function closeStatement($statement)
    {
        if ($statement !== null) {
            $statement->closeCursor();
        }
    }

    /**
     * Converts the specified exception to a DatabasException and throws it. If
     * the specified exception is already a DatabaseException, it is thrown
     * unchanged.
     *
     * @param \Exception $e
     * @param string     $sql    The sql query that lead to the exception.
     * @param array      $params Array of parameters used in the query.
     *
     * @throws DatabaseException The converted exception.
     */
    protected function handleException(\Exception $e, $sql = null, array $params = null)
    {
        if ($e instanceof DatabaseException) {
            throw $e;
        }

        if ($sql !== null) {
            if ($params !== null) {
                $msg = sprintf(
                    'Database error when executing "%s" with parameters %s: %s',
                    $sql,
                    json_encode($params),
                    $e->getMessage()
                );
            } else {
                $msg = sprintf('Database error when executing "%s": %s', $sql, $e->getMessage());
            }
        } else {
            $msg = $e->getMessage();
        }
        $this->getLogger()->error($e->getFile() . ":" . $e->getLine() . " " . $msg);
        throw new DatabaseException($e->getMessage(), $e->getCode(), $e);
    }

    /**
     * @param array|NULL $whereArr An associative array with
     *                        coloumn value pairs defining the where condition. If the value is an
     *                        array it consists of two values: the operator and the value itself.
     *
     * @return string A string containing the WHERE clause corresponding to the
     *   specified condition.
     */
    protected function toWhereStr($whereArr)
    {
        $whereStr = "";
        foreach ($whereArr as $column => $value) {
            if (!empty($whereStr)) {
                $whereStr .= " AND ";
            }
            $whereStr .= $this->qid($this->getEscaped($column));
            if (is_array($value)) {
                $operator = $value[0];
            } else {
                $operator = "=";
            }
            $whereStr .= " " . $operator . " :" . $column;
        }

        return $whereStr;
    }

    /**
     * @param array|NULL $whereArr An associative array with
     *                        coloumn value pairs defining the where condition. If the value is an
     *                        array it consists of two values: the operator and the value itself.
     * @return array An associative array with column value pairs.
     *     Possible operators in the specified array are removed.
     */
    protected function toParams($whereArr)
    {
        if (empty($whereArr)) {
            return array();
        }

        $params = array();
        foreach ($whereArr as $column => $value) {
            if (is_array($value)) {
                $params[$column] = $value[1];
            } else {
                $params[$column] = $value;
            }
        }
        return $params;
    }

    /**
     * return all column names of a table
     *
     * @param string $table Name of the table
     *
     * @return array
     */
    public function getColumns($table)
    {
        if (!empty($this->columns[$table])) {
            return $this->columns[$table];
        }
        $result = $this->queryObjects("SHOW COLUMNS FROM " . $table . "");

        $columns = array();
        foreach ($result as $field) {
            $columns[] = $field->Field;
        }

        return $this->columns[$table] = $columns;
    }
}
