<?php
namespace Topalis\Device\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Topalis\Device\Entity\DeviceEntityManager;
use Topalis\Device\Entity\AccessLogEntityManager;
use Topalis\Device\Entity\Device;
use Topalis\Device\Entity\AccessLog;

/**
 * Checks if a device has exceeded the timeout. If true a mail is sent to the
 * admin.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class DeviceTimeoutCheckCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        $this
        ->setName("device:timeout:check")
        ->setDescription("Checks if a device has exceeded the timeout. If true a mail is sent to the admin.")
        ;
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $input; // unused
        $output; // unused

        $accessLogEntityManager = $this->getAccessLogEntityManager();
        $deviceEntityManager = $this->getDeviceEntityManager();
        foreach ($deviceEntityManager->findBy(array("timedOut" => false)) as $device) {
            if ($device->getConfigGroup() === null || $device->getConfigGroup()->getDeviceTimeout() <= 0) {
                // no timeout defined
                continue;
            }

            $timeout = $device->getConfigGroup()->getDeviceTimeout();
            $lastLog = $accessLogEntityManager->getLastLogForDevice($device);
            if ($lastLog === null) {
                // no log entry, yet
                continue;
            }

            if ($this->getTimeProvider()->getTime() - $lastLog->getTstamp()->format("U") <= $timeout) {
                // timeout not exceeded
                continue;
            }

            $this->getLogger()->debug("Device " . $device->getId() . " has exceeded the timeout of "
                . $timeout . "seconds");
            if ($this->getMailer()->isConfigured()) {
                $this->sendTimeoutMail($device, $lastLog);
            }

            $device->setTimedOut(true);
            $deviceEntityManager->save($device);
        }
    }

    private function sendTimeoutMail(Device $device, AccessLog $lastLog)
    {
        $translator = $this->getTranslator();
        $timeout = $device->getConfigGroup()->getDeviceTimeout();
        $timeoutDays = floor($timeout / 86400);
        $lastAccessDate = $lastLog->getTstamp()->format($translator->trans("Y-m-d H:i:s"));

        $subject = $translator->trans("admin.device.timeout.exceeded.mail.subject", array(
            "%deviceName%" => $device->getName(),
            "%user%" => $device->getUser()->getLogin()
        ));
        $text = $translator->trans("admin.device.timeout.exceeded.mail.text", array(
            "%deviceName%" => $device->getName(),
            "%deviceid%" => $device->getDeviceid(),
            "%user%" => $device->getUser()->getLogin(),
            "%timeoutDays%" => $timeoutDays,
            "%lastAccessDate%" => $lastAccessDate
        ));
        $this->getMailer()->sendAdminMessage($subject, $text);
    }

    /**
     * @return \Topalis\Device\Entity\AccessLogEntityManager
     */
    private function getAccessLogEntityManager()
    {
        return $this->getContainer()->get("device.entity.manager.access_log");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->getContainer()->get("device.entity.manager.device");
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    private function getLogger()
    {
        return $this->getContainer()->get("logger");
    }

    /**
     * @return \Topalis\Mail\Mailer
     */
    private function getMailer()
    {
        return $this->getContainer()->get("mailer");
    }

    /**
     * @return \Topalis\Kernel\TimeProviderInterface
     */
    private function getTimeProvider()
    {
        return $this->getContainer()->get("kernel.time_provider");
    }

    /**
     * @return \Symfony\Component\Translation\TranslatorInterface
     */
    private function getTranslator()
    {
        return $this->getContainer()->get("translator");
    }
}
