<?php
namespace Topalis\Device\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Topalis\Device\Entity\AccessLog;

/**
 * This command logs the access of a device in the database.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class LogDeviceAccessCommand extends ContainerAwareCommand
{
    /**
     * @return \Topalis\Device\Entity\AccessLogEntityManager
     */
    private function getAccessLogEntityManager()
    {
        return $this->getContainer()->get("device.entity.manager.access_log");
    }

    /**
     * @return \Topalis\Certificate\Entity\ClientCertificateEntityManager
     */
    private function getCertificateEntityManager()
    {
        return $this->getContainer()->get("certificate.entity.manager.certificate");
    }

    /**
     * @return \Symfony\Bridge\Doctrine\RegistryInterface
     */
    private function getDoctrine()
    {
        return $this->getContainer()->get("doctrine");
    }

    protected function configure()
    {
        $this
        ->setName("device:log_access")
        ->setDescription("Logs a device access with data read from STDIN.")
        ->setHelp("Logs a device access with data read from STDIN.");
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $input; // unused

        try {
            openlog("auralis-logger", LOG_PID, LOG_DAEMON);
            return $this->logAccess();
        } catch (\Exception $e) {
            if ($output instanceof \Symfony\Component\Console\Output\ConsoleOutput) {
                $output = $output->getErrorOutput();
            }
            $output->writeln("Error logging device access:");
            $output->writeln((string) $e);
            return -1;
        }
    }

    /**
     * Checks if a DB connection is established. If not, a new connection will
     * be opened.
     */
    private function ensureDBConnection()
    {
        try {
            $connection = $this->getDoctrine()->getConnection();
            $connection->query("select 1;")->fetch();
        } catch (\Exception $e) {
            $connection->close();
            $connection->connect();
        }
    }

    private function logAccess()
    {
        while (($line = fgets(STDIN)) !== false) {
            $a = explode(" ", $line);
            if (5 === count($a)) {
                try {
                    $this->ensureDBConnection();
                    $accessLog = new AccessLog();
                    $dt = \DateTime::createFromFormat("U", intval($a[0]));
                    $dt->setTimezone($this->getContainer()->get("kernel.time_provider")->getTimezone());
                    $accessLog->setTstamp($dt);
                    $accessLog->setIp($a[1]);
                    $certId = hexdec($a[2]);
                    $cert = $this->getCertificateEntityManager()->find($certId);
                    if ($cert === null) {
                        throw new \Exception(
                            sprintf("Invalid certificate id '%s'", $certId));
                    }
                    $accessLog->setCert($cert);
                    $accessLog->setStatus(intval($a[3]));
                    $accessLog->setTraffic(intval($a[4]));
                    $accessLog->setDevice($cert->getDevice());
                    $this->getAccessLogEntityManager()->save($accessLog);
                } catch (\Exception $e) {
                    $this->logError("Could not store access data: " . $e->getMessage());
                }
            }
        }
        if (!feof(STDIN)) {
            $this->logError("Unexpected fgets() failure, exiting");
            return -1;
        }
    }

    private function logError($message)
    {
        syslog(LOG_ERR, $message);
    }

}
