<?php
namespace Topalis\Device\Controller;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Symfony\Component\Form\FormError;
use Topalis\Device\Export\ExcelDeviceExporter;
use Topalis\Device\Form\LockForm;
use Topalis\Device\Form\SetPasscodeForm;
use Topalis\Form\Validator\Constraint\AndroidPasscode;
use Topalis\HttpError;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpFoundation\Response;
use Topalis\Device\Entity\Device;
use Topalis\Device\Form\DeviceForm;
use Topalis\MDM\Entity\MdmCommand;
use Topalis\User\Entity\User;
use Topalis\Kernel\Controller;
use Topalis\Kernel\Exception\Http\NotFoundException;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Topalis\App\Entity\AppState;

/**
 * controller for device list
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/admin/device", service="device.controller")
 */

class DeviceController extends Controller
{
    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\Device\DeviceManager
     */
    private function getDeviceManager()
    {
        return $this->container->get("device.manager");
    }

    /**
     * @return \Topalis\MDM\MdmAdmin
     */
    private function getMdmAdmin()
    {
        return $this->container->get("mdm.admin");
    }

    /**
     * @return \Topalis\Device\MDM\MdmFacadeInterface
     */
    private function getMdmFacade()
    {
        return $this->container->get("device.mdm.facade");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Topalis\User\Entity\UserEntityManager
     */
    private function getUserEntityManager()
    {
        return $this->container->get("user.entity.manager.user");
    }

    /**
     * @return \Topalis\ConfigGroup\Entity\ConfigGroupEntityManager
     */
    private function getConfigGroupEntityManager()
    {
        return $this->container->get("config_group.entity.manager.config_group");
    }

    /**
     * @return \Topalis\Device\EventListener\SendUserMailEventListener
     */
    private function getSendUserMailEventListener()
    {
        return $this->container->get("device.listener.doctrine.send_user_mail");
    }

    /**
     * @Route("", name="admin_device")
     * @Route("/user/{user}", name="admin_device_user")
     * @Route("/group/{group}", name="admin_device_group")
     * @Template()
     * @param ConfigGroup $group
     * @param User $user
     *
     * @return Response
     */
    public function indexAction(ConfigGroup $group = null, User $user = null)
    {
        $pageTitle = $this->tr("admin.devices.title");

        $condition = array();
        $devices = array();
        $filtered = false;

        try {
            if (null !== $user) {
                $pageTitle = $this->tr("admin.devices.userTitle", array("%user%" => $user->getRealname()));
                $condition['u.id'] = $user->getId();
                $filtered = true;
            } else if (null !== $group) {
                $pageTitle = $this->tr("admin.devices.groupTitle", array("%group%" => $group->getName()));
                $condition['c.id'] = $group->getId();
                $filtered = true;
            }
            $devices = $this->getDeviceManager()->getDevices("d.deviceid", "ASC", $condition);
        } catch (\Exception $e) {
            $this->getLogger()->error("Error building device list", array("exception" => $e));
            $msg = $this->tr("admin.devices.error.buildDeviceList", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }

        $usersExist = $this->getUserEntityManager()->exists();

        $vars = array(
            "devices" => $devices,
            "title" => $pageTitle,
            "filter" => $filtered,
            "maxPasswordValidity" => $this->getConfig()->getPasswordValidity(),
            "group" => $group,
            "user" => $user,
            "usersExist" => $usersExist
        );

        return $vars;

    }

    /**
     * @Route("/{id}/delete", name="admin_device_delete")
     * @param integer $id Device Id
     *
     * @throws \Exception
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function deleteAction($id)
    {
        try {
            $this->getDeviceEntityManager()->disableDeleteFilter(); // throws exceptions otherwise in some cases
            $this->getDeviceManager()->deleteDevice($id);
            $this->addFlash("success", $this->tr("admin.devices.deleteSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error deleting device with id ".$id, array("exception" => $e));
            $msg = $this->tr("admin.devices.error.delete", array("%id%" => $id, "%errmsg%" => $e->getMessage()));
            $this->addError($msg);
            throw $e;
        }

        return $this->redirect($this->generateUrl("admin_device"));
    }

    /**
     * @Route("/{id}/enable", name="admin_device_enable")
     *
     * @param integer $id Device Id
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function enableAction($id)
    {
        try {
            $this->getDeviceManager()->setDeviceEnabled($id, true);
            $this->addFlash("success", $this->tr("admin.devices.enableSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error enabling device with id ".$id, array("exception" => $e));
            $msg = $this->tr("admin.devices.error.enable", array("%id%" => $id, "%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }

        return $this->redirect($this->generateUrl("admin_device"));
    }

    /**
     * @param integer $id Device Id
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     *
     * @Route("/{id}/disable", name="admin_device_disable")
     */
    public function disableAction($id)
    {
        try {
            $this->getDeviceManager()->setDeviceEnabled($id, false);
            $this->addFlash("success", $this->tr("admin.devices.disableSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error disabling device with id ".$id, array("exception" => $e));
            $msg = $this->tr("admin.devices.error.disable", array("%id%" => $id, "%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }
        return $this->redirect($this->generateUrl("admin_device"));
    }

    /**
     * Reset phone to factory defaults
     *
     * @param integer $id
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     *
     * @Route("/{id}/erase", name="admin_device_erase")
     */
    public function eraseAction($id)
    {
        try {
            $this->getDeviceManager()->disableDevice($id, "erase");
            $this->addFlash("success", $this->tr("admin.devices.eraseSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error erasing device with id ".$id, array("exception" => $e));
            $msg = $this->tr("admin.devices.error.erase", array("%id%" => $id, "%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }
        return $this->redirect($this->generateUrl("admin_device"));
    }

    /**
     * make enterprise wipe
     *
     * @param integer $id
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     *
     * @Route("/{id}/wipe", name="admin_device_wipe")
     */
    public function wipeAction($id)
    {
        try {
            $this->getDeviceManager()->disableDevice($id);
            $this->addFlash("success", $this->tr("admin.devices.wipeSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error wiping device with id ".$id, array("exception" => $e));
            $msg = $this->tr("admin.devices.error.wipe", array("%id%" => $id, "%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }
        return $this->redirect($this->generateUrl("admin_device"));
    }

    /**
     * @Route("/add", name="admin_device_add", options={"expose" = true})
     * @Template()
     * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
     * @return string|Response
     */
    public function addAction()
    {
        $device = new Device();
        $userId = $this->getRequest()->query->get("user");
        if (!empty($userId)) {
            $user = $this->getUserEntityManager()->find($userId);
            if (!$user) {
                throw new NotFoundHttpException();
            }
            $device->setUser($user);
        }
        $groupId = $this->getRequest()->query->get("group");
        if (!empty($groupId)) {
            $group = $this->getConfigGroupEntityManager()->find($groupId);
            if (!$group) {
                throw new NotFoundHttpException();
            }
            $device->setConfigGroup($group);
        }

        return $this->forward("device.controller:editAction", array("device" => $device));
    }

    /**
     * @Route("/{id}/edit", name="admin_device_edit", options={"expose" = true})
     * @Template()
     *
     * @param Device $device
     *
     * @throws \Topalis\Kernel\Exception\Http\NotFoundException
     * @throws \Exception
     * @return string|Response
     */
    public function editAction(Device $device)
    {
        if ($device === null) {
            throw new NotFoundException();
        }

        $request = $this->getRequest();

        try {
            $new = false;
            if ($device->getId() === null) {
                $new = true;
                $rollout = true;
            } else {
                $rollout = $device->isRollout();
            }

            $configGroup = null;
            if (null !== $device->getUser()) {
                $configGroup = $device->getUser()->getConfigGroup();
            }

            if ($device->getConfigGroup()) {
                $noExchange = $device->getConfigGroup()->isNoExchange();
            } else {
                $noExchange = null;
            }

            $formBuilder = $this->getFormFactory()->createBuilder(
                new DeviceForm($new, $rollout, $configGroup, null !== $device->getUser(), $noExchange));
            $form = $formBuilder->getForm();
            $form->setData($device);

            if ($request->isMethod("POST")) {
                $form->submit($request);

                $this->validateMailAccounts($form, $device);

                if ($form->isValid()) {
                    if ($new) {
                        $sendUserMail = $form->get("sendUserMail")->getData();
                        $this->getSendUserMailEventListener()->setEnabled($sendUserMail);
                    }

                    $this->getDeviceEntityManager()->save($device);

                    if ($device->isRollout() && !$rollout) {
                        // rerollout is not allowed. this condition won't be matched
                        // anyway. so just to ensure everything is ok
                        throw new \Exception("Re-rollout not possible");
                    }

                    $this->addFlash("success", $this->tr("admin.devices.saveSuccess"));

                    return new Response("OK");
                }
            } else if ($new) {
                $form->get("sendUserMail")->setData($this->getSendUserMailEventListener()->isEnabled());
            }
        } catch (\Exception $e) {
            if ($device->getId() === null) {
                $this->getLogger()->error("Error inserting device", array("exception" => $e));
                $msg = $this->tr(
                    "admin.devices.error.add",
                    array("%errmsg%" => $e->getMessage())
                );
            } else {
                $this->getLogger()->error("Error updating device", array("exception" => $e));
                $msg = $this->tr(
                    "admin.devices.error.change",
                    array("%id%" => $device->getId(), "%errmsg%" => $e->getMessage())
                );
            }
            if ($form) {
                $form->addError(new FormError($msg));
            } else {
                throw $e;
            }
        }

        return array(
            "form" => $form->createView(),
            "device" => $device,
        );
    }

    /**
     * @Route("/{id}/cert", name="admin_device_cert")
     * @param integer $id
     *
     * @return Response
     */
    public function downloadCertAction($id)
    {
        $device = $this->getDeviceManager()->getDevice($id);

        if ($device->isManaged()) {
            return HttpError::badRequestResponse("Certificate download not possible for managed devices");
        }

        $content = $this->getCertificateManager()->createClientP12($device->getCert(), 'changeit');

        /**
         * @see http://symfony.com/doc/master/components/http_foundation/introduction.html#downloading-files
         */
        $response = new Response($content);
        $disposition = $response->headers->makeDisposition(
            ResponseHeaderBag::DISPOSITION_ATTACHMENT,
            "client.p12"
        );

        $response->headers->set("Content-Disposition", $disposition);

        return $response;
    }

    /**
     * @Route("/{id}/requestInfo", name="admin_device_request_info")
     * @Template()
     * @param integer $id Device id
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function requestDeviceInfoAction($id)
    {
        $flashMessages = array(
            "error" => array(),
            "success" => array()
        );
        try {
            $device = $this->getDeviceEntityManager()->find($id);
            $this->getMdmFacade()->requestDeviceInfo($device);
            $flashMessages['success'][] = $this->tr("admin.devices.requestDeviceInfoSuccess");
        } catch (\Exception $e) {
            $this->getLogger()->error("Error requesting device information", array("exception" => $e));
            $flashMessages['error'][] = $this->tr("admin.devices.error.requestDeviceInfo",
                array("%errmsg%" => $e->getMessage()));
        }

        return array(
            "flashMessages" => $flashMessages
        );
    }

    /**
     * @Route("/requestInfo", name="admin_devices_request_info")
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function requestDevicesInfoAction()
    {
        try {
            $this->getMdmFacade()->requestDeviceInfos();
            $this->addFlash("success", $this->tr("admin.devices.requestDeviceInfoSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error requesting device information", array("exception" => $e));
            $msg = $this->tr("admin.devices.error.requestDeviceInfo", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }
        if (null !== $groupId = $this->getRequest()->query->get("group", null)) {
            return $this->redirect($this->generateUrl("admin_group_edit", array("id" => $groupId)));
        }

        return $this->redirect($this->generateUrl("admin_device"));
    }

    /**
     * @Route("/{device}/resendCommand/{commandId}", name="admin_resend_command", options={"expose" = true})
     *
     * @param Device $device
     * @param int $commandId
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function resendCommandAction(Device $device, $commandId)
    {
        try {
            $this->getMdmFacade()->resendCommand($device, $commandId);
            $this->addFlash("success", $this->tr("admin.device.mdm.resendCommand.success"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error resending command",
                array("exception" => $e, "device" => $device->getCombinedId(), "command" => $commandId));
            $msg = $this->tr("admin.device.mdm.resendCommand.error", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }

        // TODO we should reload the MDM commands table instead of doing a redirect
        return $this->redirect($this->generateUrl("admin_device"));
    }

    /**
     * @Route("/export/{format}", name="admin_device_export")
     *
     * @param string $format csv, xls, or xlsx
     * @return Response
     */
    public function exportDeviceInfoAction($format)
    {
        $devices = $this->getDeviceEntityManager()->findBy(array("deleted" => 0));

        switch ($format) {
            case 'xlsx':
                $exporter = new ExcelDeviceExporter();
                break;
            case 'xls':
                $exporter = new ExcelDeviceExporter("5");
                break;
            case 'csv':
            default:
                $exporter = new ExcelDeviceExporter("csv");
                break;
        }

        return $exporter->export($devices);
    }

    /**
     * @Route("/{id}/repush", name="admin_device_repush", options={"expose" = true})
     * @Template()
     * @param \Topalis\Device\Entity\Device $device
     *
     * @return Response
     */
    public function repushAction(Device $device)
    {
        $flashMessages = array(
            "error" => array(),
            "success" => array()
        );
        try {
            $this->getMdmAdmin()->repush($device);
            $flashMessages['success'][] = $this->tr("admin.devices.repushSuccess");
        } catch (\Exception $e) {
            $this->getLogger()->error("Error requesting a repush for device ".$device->getId(),
                array("exception" => $e));
            $msg = $this->tr("admin.devices.error.repush",
                array("%id%" => $device->getId(), "%errmsg%" => $e->getMessage()));
            $flashMessages['error'][] = $msg;
        }
        return array("flashMessages" => $flashMessages);
    }

    protected function validateMailAccounts(\Symfony\Component\Form\FormInterface $form, Device $device)
    {
        if ($device->getConfigGroup()->isNoExchange()) {
            if (!$device->getMailAccounts()->isEmpty()) {
                $form->addError(new FormError("devices.noExchange"));
            }
            return;
        }

        $accountList = array();
        foreach ($device->getMailAccounts() as $account) {
            if (in_array($account->getEmail(), $accountList)) {
                $form->addError(
                    new FormError("devices.duplicateEmail", null,
                        array("%email%" => $account->getEmail())
                    )
                );
                continue;
            }
            $accountList[] = $account->getEmail();
        }
    }

    /**
     * @Route("/{id}/info", name="admin_device_info", options={"expose" = true})
     * @Template()
     * @param \Topalis\Device\Entity\Device $device
     *
     * @return Response
     */
    public function viewDeviceInfoAction(Device $device)
    {
        $user = $device->getUser();
        $deviceType = $device->getType();
        if (!$deviceType->hasDeviceInfo($device)) {
            throw new NotFoundHttpException(sprintf("device %s has no device info", $device->getId()));
        }
        $deviceData = $device->getType()->getDeviceInfo($device);

        /** @var MdmCommand[] $mdmCommands */
        $mdmCommands = $this->getMdmFacade()->getCommands($device, 20, array("id" => "DESC"));

        $appStates = array();
        if ($deviceType->hasSupport("apps", $device)) {
            /** @var \Topalis\App\Entity\AppStateEntityManager $appStateEntityManager */
            $appStateEntityManager = $this->container->get($device->getType()->getAppStateEntityManagerId());
            $appStates = $appStateEntityManager->createQueryBuilder("s")
                ->select("s, a")
                ->join("s.app", "a")
                ->where("s.device = :device")
                ->andWhere("s.state in (:installedStates)")
                ->setParameter("device", $device)
                ->setParameter("installedStates", AppState::getInstalledStates())
                ->orderBy("a.name")
                ->getQuery()
                ->getResult();
        }
        return array(
            "data"                => $deviceData,
            "device"              => $device,
            "user"                => $user,
            "maxPasswordValidity" => $this->getConfig()->getPasswordValidity(),
            "access"              => $this->getDeviceManager()->getDeviceTraffic($device),
            "mdmCommands"         => $mdmCommands,
            "appStates"          => $appStates,
        );
    }


    /**
     * @Route("/{id}/clearPasscode", name="admin_device_clear_passcode")
     * @param \Topalis\Device\Entity\Device $device
     */
    public function clearPasscodeAction(Device $device)
    {
        try {
            $this->getMdmAdmin()->clearPasscode($device);
            $this->addFlash("success", $this->tr("admin.devices.clearPasscodeSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error while clearing device passcode", array("exception" => $e));
            $msg = $this->tr("admin.devices.error.clearPasscode", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }
        return $this->redirect($this->generateUrl("admin_device"));
    }

    /**
     * @Route("/{id}/lock", name="admin_device_lock")
     * @Template()
     *
     * @param \Symfony\Component\HttpFoundation\Request $request
     * @param \Topalis\Device\Entity\Device             $device
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function lockAction(Request $request, Device $device)
    {
        $form = $this->createForm(new LockForm());
        if (!$device->getType()->hasSupport("deviceLockMessage", $device)) {
            $form->remove("message");
            $form->remove("phone");
        } else if (strlen($device->getType()->getDeviceInfo($device)->getPhoneNumber()) == 0) {
            $form->remove("phone");
        }

        $form->handleRequest($request);
        if ($form->isValid()) {
            $message = null;
            if ($form->has("message")) {
                $message = $form->get("message")->getData();
            }
            $phoneNumber = null;
            if ($form->has("phone")) {
                $phoneNumber = $form->get("phone")->getData();
            }
            $this->getMdmFacade()->lockDevice($device, $message, $phoneNumber);

            $this->addFlash("success", $this->tr("admin.devices.lock.success"));
            return new Response("OK");
        }

        return array(
            "form" => $form->createView()
        );
    }

    /**
     * @Route("/{id}/setPasscode", name="admin_device_set_passcode")
     * @Template()
     *
     * @param \Symfony\Component\HttpFoundation\Request $request
     * @param Device                                    $device
     *
     * @throws \Symfony\Component\Security\Core\Exception\AccessDeniedException
     * @return array|\Symfony\Component\HttpFoundation\Response
     */
    public function setPasscodeAction(Request $request, Device $device)
    {
        if (!$device->getType()->hasSupport("setPasscode", $device)) {
            throw new AccessDeniedException("device doesn't support setPasscode");
        }
        $passcode = (array) $device->getType()->getDeviceInfo($device)->get("PasscodePolicy");
        $passcode += array(
            "Quality"      => AndroidPasscode::PASSWORD_QUALITY_COMPLEX,
            "MinLength"    => 6,
            "MinLetters"   => 1,
            "MinLowercase" => 1,
            "MinUppercase" => 1,
            "MinSymbols"   => 1,
            "MinNonLetter" => 1,
            "MinNumeric"   => 1,
        );
        $form = $this->createForm(new SetPasscodeForm(), null, array(
            "passcode" => array(
                "quality"      => $passcode['Quality'],
                "minLength"    => $passcode['MinLength'],
                "minLetters"   => $passcode['MinLetters'],
                "minLowercase" => $passcode['MinLowercase'],
                "minUppercase" => $passcode['MinUppercase'],
                "minSymbols"   => $passcode['MinSymbols'],
                "minNonLetter" => $passcode['MinNonLetter'],
                "minNumeric"   => $passcode['MinNumeric'],
            )
        ));
        $form->handleRequest($request);
        if ($form->isValid()) {
            $this->getMdmAdmin()->setPasscode($device, $form->get("passcode")->getData());
            $this->addFlash("success", $this->tr("admin.devices.setPasscode.success"));

            return new Response("OK");
        }

        return array(
            "form"     => $form->createView(),
            "passcode" => $passcode,
        );
    }
}
