<?php

namespace Topalis\Device\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Validator\Constraints as Assert;
use Topalis\Device\Type\DeviceTypeInterface;
use Topalis\MDM\Entity\MdmCommand;
use Topalis\MDM\Entity\MdmQueueEntry;
use Topalis\User\Entity\User;
use Topalis\Util\PasswordUtil;

/**
 * Device
 * @ORM\Table(name="devices", indexes={
 *     @ORM\Index(name="idx_devices_disabled", columns={"disabled"}),
 *     @ORM\Index(name="idx_devices_deleted", columns={"deleted"}),
 *     @ORM\Index(name="idx_devices_deviceid", columns={"deviceid"})})
 *
 * @ORM\Entity
 */
class Device implements UserInterface, \Serializable
{
    const WIPE_ERROR = -1;
    const WIPE_NONE = 0;
    const WIPE_PENDING = 1;
    const WIPE_EXECUTED = 2;
    const WIPE_UNKNOWN = 3;

    const OWNER_COMPANY = "company";
    const OWNER_USER = "user";

    /**
     * @var integer
     * @ORM\Column(name="id", type="integer", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    protected $id;

    /**
     * @var string
     * @ORM\Column(name="deviceid", type="string", length=50, nullable=true,
     *     options={
     *         "comment"="The unique ID of this device (iPhone: serial-number, or a hash value for other devices)"
     *     })
     */
    protected $deviceid;

    /**
     * @var string
     * @ORM\Column(name="password", type="string", length=80, nullable=true,
     *     options={"comment"="The one-time activation password"})
     */
    protected $password;

    /**
     * @var \DateTime
     * @ORM\Column(name="password_stamp", type="datetime", nullable=false,
     *     options={"version"=true, "comment"="Timestamp for password expiry"})
     */
    protected $passwordStamp;

    /**
     * @var string
     * @ORM\Column(name="comment", type="string", length=255, nullable=true,
     *     options={"comment"="A comment without functional meaning"})
     */
    protected $comment;

    /**
     * @var integer
     * @ORM\Column(name="os", type="integer", nullable=false,
     *     options={"unsigned"=true, "comment"="Operating system: 0=iOS, 1=Other"})
     */
    protected $os;

    /**
     * @var \Topalis\User\Entity\User
     * @ORM\ManyToOne(targetEntity="Topalis\User\Entity\User", inversedBy="devices", cascade={"persist"})
     * @ORM\JoinColumns({
     *     @ORM\JoinColumn(name="user_id", referencedColumnName="id", nullable=true)
     * })
     */
    protected $user;

    /**
     * @var string
     * @ORM\Column(type="string", length=255)
     */
    protected $owner;

    /**
     * @var string
     * @ORM\Column(name="challenge", type="string", length=255, nullable=true,
     *     options={"comment"="The challenge to authenticate the user/device during the rollout process"})
     */
    protected $challenge;

    /**
     *
     * @var string
     */
    protected $plainChallenge;

    /**
     * @var string
     * @ORM\Column(name="imei", type="string", length=19, nullable=true)
     */
    protected $imei;

    /**
     * @var string
     * @ORM\Column(name="product", type="string", length=64, nullable=true,
     *     options={"comment"="The devices product name"})
     */
    protected $product;

    /**
     * @var string
     * @ORM\Column(name="version", type="string", length=64, nullable=true)
     */
    protected $version;

    /**
     * @var boolean
     * @ORM\Column(name="blocked", type="boolean", nullable=false, options={"default"=0})
     */
    protected $blocked = false;

    /**
     * @var string
     * @ORM\Column(name="mdmtoken", type="binary", length=255, nullable=true)
     */
    protected $mdmtoken;

    /**
     * @var string
     * @ORM\Column(name="mdmpushmagic", type="string", length=255, nullable=true)
     */
    protected $mdmpushmagic;

    /**
     * @var string
     * @ORM\Column(name="mdmunlocktoken", type="blob", nullable=true)
     */
    protected $mdmunlocktoken;

    /**
     * @var array
     * @ORM\Column(name="deviceinformation", type="json_array", length=65532, nullable=true)
     */
    protected $deviceinformation;

    /**
     * @var integer
     * @ORM\Column(name="lastdeviceinformationupdate", type="integer", nullable=true, options={"unsigned"=true})
     */
    protected $lastdeviceinformationupdate;

    /**
     * @var integer
     * @ORM\Column(name="wipestatus", type="smallint", nullable=false, options={"default"=Device::WIPE_NONE})
     */
    protected $wipestatus = self::WIPE_NONE;

    /**
     * @var boolean
     * @ORM\Column(name="disabled", type="boolean", nullable=false, options={"default"=0})
     */
    protected $disabled = false;

    /**
     * @var boolean
     * @ORM\Column(name="deleted", type="boolean", nullable=false, options={"default"=0})
     */
    protected $deleted = false;

    /**
     * @var string
     * @ORM\Column(name="name", type="string", length=255, nullable=true,
     *     options={"comment"="The device name of an iPhone in the device information"})
     */
    protected $name;

    /**
     * The certificate created for the rollout (phase 2). For devices rolled out
     * without MDM it is the server side generated certificate.
     *
     * @var \Topalis\Certificate\Entity\ClientCertificate
     * @ORM\ManyToOne(targetEntity="Topalis\Certificate\Entity\ClientCertificate",
     *     cascade={"persist"})
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="cert_id", referencedColumnName="id")
     * })
     */
    protected $cert;

    /**
     * The certificate created for MDM (phase 3).
     *
     * @var \Topalis\Certificate\Entity\ClientCertificate
     * @ORM\ManyToOne(targetEntity="Topalis\Certificate\Entity\ClientCertificate",
     *     cascade={"persist"})
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="mdm_cert_id", referencedColumnName="id")
     * })
     */
    protected $mdmCert;

    /**
     * The config group the device belongs to.
     *
     * @var \Topalis\ConfigGroup\Entity\ConfigGroup
     * @Assert\NotBlank()
     * @ORM\ManyToOne(targetEntity="Topalis\ConfigGroup\Entity\ConfigGroup", inversedBy="devices", cascade={"persist"})
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="configgroup_id", referencedColumnName="id", nullable=true)
     * })
     */
    protected $configGroup;

    /**
     * @var \Doctrine\Common\Collections\ArrayCollection
     * @ORM\OneToMany(targetEntity="Topalis\Device\Entity\MailAccount", mappedBy="device", cascade={"all"},
     *     orphanRemoval=true)
     */
    protected $mailAccounts;

    /**
     * @var \Doctrine\Common\Collections\ArrayCollection
     * @ORM\OneToMany(targetEntity="Topalis\MDM\Entity\MdmQueueEntry", mappedBy="device", cascade={"all"})
     */
    protected $mdmQueue;

    /**
     * @var \Doctrine\Common\Collections\ArrayCollection
     * @ORM\OneToMany(targetEntity="Topalis\MDM\Entity\MdmCommand", mappedBy="device", cascade={"all"})
     */
    protected $mdmCommands;

    /**
     * @var string
     *
     * @ORM\Column(name="omaauthappsrv", type="string", length=255, nullable=true)
     */
    protected $omaAuthAppSrv;

    /**
     * @var string
     *
     * @ORM\Column(name="omaauthclient", type="string", length=255, nullable=true)
     */
    protected $omaAuthClient;

    /**
     * @var string
     *
     * @ORM\Column(name="omaauthnonce", type="string", length=255, nullable=true)
     */
    protected $omaAuthNonce;

    /**
     * @var \Doctrine\Common\Collections\ArrayCollection
     * @ORM\OneToMany(targetEntity="Topalis\WinPhone\Entity\SyncMLCommand", mappedBy="device", cascade={"all"})
     */
    protected $syncMLCommands;

    /**
     * @var boolean
     * @ORM\Column(name="timedout", type="boolean")
     */
    protected $timedOut = false;

    /**
     * Will be set by Topalis\Device\EventListener\DeviceTypeListener
     * @var DeviceTypeInterface
     */
    private $type;

    /**
     * @var string
     * Plain password for internal use, gets encrypted before saved
     */
    protected $plainPassword;

    public function __construct()
    {
        $this->mailAccounts = new ArrayCollection();
        $this->mdmQueue = new ArrayCollection();
        $this->mdmCommands = new ArrayCollection();
        $this->syncMLCommands = new ArrayCollection();
        $this->passwordStamp = new \DateTime();
        $this->blocked = false;
        $this->wipestatus = self::WIPE_NONE;
    }

    public function __toString()
    {
        return (string) $this->getId();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set deviceid
     *
     * @param string $deviceid
     *
     * @return Device
     */
    public function setDeviceid($deviceid)
    {
        $this->deviceid = $deviceid;

        return $this;
    }

    /**
     * Get deviceid
     *
     * @return string
     */
    public function getDeviceid()
    {
        return $this->deviceid;
    }

    /**
     * @return string Combination of database id and deviceid in the form
     *     "id (deviceid)"
     */
    public function getCombinedId()
    {
        return $this->getId() . " (" . $this->getDeviceid() . ")";
    }

    /**
     * Set password
     *
     * @param string $password
     *
     * @return Device
     */
    public function setPassword($password)
    {
        $this->password = $password;

        return $this;
    }

    /**
     * Get password
     *
     * @return string
     */
    public function getPassword()
    {
        return $this->password;
    }

    /**
     * Set passwordStamp
     *
     * @param \DateTime $passwordStamp
     *
     * @return Device
     */
    public function setPasswordStamp(\DateTime $passwordStamp)
    {
        $this->passwordStamp = $passwordStamp;

        return $this;
    }

    /**
     * Get passwordStamp
     *
     * @return \DateTime
     */
    public function getPasswordStamp()
    {
        return $this->passwordStamp;
    }

    /**
     * Set comment
     *
     * @param string $comment
     *
     * @return Device
     */
    public function setComment($comment)
    {
        $this->comment = $comment;

        return $this;
    }

    /**
     * Get comment
     *
     * @return string
     */
    public function getComment()
    {
        return $this->comment;
    }

    /**
     * Set os
     *
     * @param integer $os
     *
     * @return Device
     */
    public function setOs($os)
    {
        $this->os = $os;

        return $this;
    }

    /**
     * Get os
     *
     * @return integer
     */
    public function getOs()
    {
        return $this->os;
    }

    /**
     * Set user
     *
     * @param User|NULL $user
     *
     * @throws \Exception
     * @return Device
     */
    public function setUser($user)
    {
        if ($this->user !== null && $user !== null && $this->user !== $user) {
            throw new \Exception("Changing the user of a device is not possible");
        }

        $this->user = $user;
        if ($user !== null) {
            $user->addDevice($this);
        }

        return $this;
    }

    /**
     * Get user
     *
     * @return User
     */
    public function getUser()
    {
        return $this->user;
    }

    /**
     * Set owner
     *
     * @param string $owner
     * @return Device self
     */
    public function setOwner($owner)
    {
        $this->owner = $owner;

        return $this;
    }

    /**
     * Get owner
     *
     * @return string
     */
    public function getOwner()
    {
        return $this->owner;
    }

    /**
     * Set challenge
     *
     * @param string $challenge
     *
     * @return Device
     */
    public function setChallenge($challenge)
    {
        $this->challenge = $challenge;

        return $this;
    }

    /**
     * Get challenge
     *
     * @return string
     */
    public function getChallenge()
    {
        return $this->challenge;
    }

    /**
     * @return string The challenge for SCEP. It is not persited.
     */
    public function getPlainChallenge()
    {
        return $this->plainChallenge;
    }

    /**
     * Sets the challenge for SCEP. The specified challenge will be persisted as
     * SHA1 hash. The plain challenge will be stored in memory only.
     *
     * @param string $plainChallenge
     *
     * @return Device
     */
    public function setPlainChallenge($plainChallenge)
    {
        $this->plainChallenge = $plainChallenge;
        $this->setChallenge($plainChallenge === null ? null : sha1($plainChallenge));

        return $this;
    }

    /**
     * Set imei
     *
     * @param string $imei
     *
     * @return Device
     */
    public function setImei($imei)
    {
        $this->imei = $imei;

        return $this;
    }

    /**
     * Get imei
     *
     * @return string
     */
    public function getImei()
    {
        return $this->imei;
    }

    /**
     * Set product
     *
     * @param string $product
     *
     * @return Device
     */
    public function setProduct($product)
    {
        $this->product = $product;

        return $this;
    }

    /**
     * Get product
     *
     * @return string
     */
    public function getProduct()
    {
        return $this->product;
    }

    /**
     * Set version
     *
     * @param string $version
     *
     * @return Device
     */
    public function setVersion($version)
    {
        $this->version = $version;

        return $this;
    }

    /**
     * Get version
     *
     * @return string
     */
    public function getVersion()
    {
        return $this->version;
    }

    /**
     * Set blocked
     *
     * @param boolean $blocked
     *
     * @return Device
     */
    public function setBlocked($blocked)
    {
        $this->blocked = $blocked;

        return $this;
    }

    /**
     * Get blocked
     *
     * @return boolean
     */
    public function getBlocked()
    {
        return $this->blocked;
    }

    /**
     * Set mdmtoken
     *
     * @param string $mdmtoken
     *
     * @return Device
     */
    public function setMdmtoken($mdmtoken)
    {
        $this->mdmtoken = $this->getType()->parseMdmToken($this, $mdmtoken);

        return $this;
    }

    /**
     * Get mdmtoken
     *
     * @return string
     */
    public function getMdmtoken()
    {
        return $this->mdmtoken;
    }

    /**
     * Set mdmpushmagic
     *
     * @param string $mdmpushmagic
     *
     * @return Device
     */
    public function setMdmpushmagic($mdmpushmagic)
    {
        $this->mdmpushmagic = $mdmpushmagic;

        return $this;
    }

    /**
     * Get mdmpushmagic
     *
     * @return string
     */
    public function getMdmpushmagic()
    {
        return $this->mdmpushmagic;
    }

    /**
     * Set mdmunlocktoken
     *
     * @param string|resource $mdmunlocktoken
     *
     * @return Device
     */
    public function setMdmunlocktoken($mdmunlocktoken)
    {
        $this->mdmunlocktoken = $mdmunlocktoken;

        return $this;
    }

    /**
     * Get mdmunlocktoken
     *
     * @return string|resource
     */
    public function getMdmunlocktoken()
    {
        return $this->mdmunlocktoken;
    }

    /**
     * Set deviceinformation
     *
     * @param array $deviceinformation
     *
     * @return Device
     */
    public function setDeviceinformation(array $deviceinformation)
    {
        $this->deviceinformation = $deviceinformation;

        return $this;
    }

    /**
     * Get deviceinformation
     *
     * @return array
     */
    public function getDeviceinformation()
    {
        return $this->deviceinformation;
    }

    /**
     * Set lastdeviceinformationupdate
     *
     * @param integer $lastdeviceinformationupdate
     *
     * @return Device
     */
    public function setLastDeviceInformationUpdate($lastdeviceinformationupdate)
    {
        $this->lastdeviceinformationupdate = $lastdeviceinformationupdate;

        return $this;
    }

    /**
     * Get lastdeviceinformationupdate
     *
     * @return integer
     */
    public function getLastDeviceInformationUpdate()
    {
        return $this->lastdeviceinformationupdate;
    }

    /**
     * Set wipestatus
     *
     * @param integer $wipestatus
     *
     * @return Device
     */
    public function setWipestatus($wipestatus)
    {
        $this->wipestatus = $wipestatus;

        return $this;
    }

    /**
     * Get wipestatus
     *
     * @return integer
     */
    public function getWipestatus()
    {
        return $this->wipestatus;
    }

    /**
     * Set disabled
     *
     * @param boolean $disabled
     *
     * @return Device
     */
    public function setDisabled($disabled)
    {
        $this->disabled = $disabled;

        return $this;
    }

    /**
     * Get disabled
     *
     * @return boolean
     */
    public function isDisabled()
    {
        return $this->disabled;
    }

    /**
     * Set deleted
     *
     * @param boolean $deleted
     *
     * @return Device
     */
    public function setDeleted($deleted)
    {
        $this->deleted = $deleted;

        return $this;
    }

    /**
     * Get deleted
     *
     * @return boolean
     */
    public function isDeleted()
    {
        return $this->deleted;
    }

    /**
     * Set name
     *
     * @param string $name
     *
     * @return Device
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set cert
     *
     * @param \Topalis\Certificate\Entity\ClientCertificate $cert
     *
     * @return Device
     */
    public function setCert(\Topalis\Certificate\Entity\ClientCertificate $cert = null)
    {
        $this->cert = $cert;
        $cert->setDevice($this);

        return $this;
    }

    /**
     * Get cert
     *
     * @return \Topalis\Certificate\Entity\ClientCertificate
     */
    public function getCert()
    {
        return $this->cert;
    }

    /**
     * Set mdmCert
     *
     * @param \Topalis\Certificate\Entity\ClientCertificate $mdmCert
     *
     * @return Device
     */
    public function setMdmCert(\Topalis\Certificate\Entity\ClientCertificate $mdmCert = null)
    {
        $this->mdmCert = $mdmCert;
        $mdmCert->setDevice($this);

        return $this;
    }

    /**
     * Get mdmCert
     *
     * @return \Topalis\Certificate\Entity\ClientCertificate
     */
    public function getMdmCert()
    {
        return $this->mdmCert;
    }

    /**
     * Set config group
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup
     *
     * @return Device
     */
    public function setConfigGroup(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup = null)
    {
        if ($this->configGroup === $configGroup) {
            return $this;
        }

        if ($this->configGroup !== null) {
            $this->configGroup->removeDevice($this);
        }

        $this->configGroup = $configGroup;
        $configGroup->addDevice($this);

        return $this;
    }

    /**
     * Get profile
     *
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup
     */
    public function getConfigGroup()
    {
        return $this->configGroup;
    }

    /**
     * @param MailAccount[] $mailAccounts
     *
     * @return $this
     */
    public function setMailAccounts(array $mailAccounts)
    {
        $this->mailAccounts->clear();
        foreach ($mailAccounts as $account) {
            $this->addMailAccount($account);
        }

        return $this;
    }

    /**
     * @return \Doctrine\Common\Collections\ArrayCollection
     */
    public function getMailAccounts()
    {
        return $this->mailAccounts;
    }

    public function addMailAccount(MailAccount $account)
    {
        if ($this->mailAccounts->contains($account)) {
            return $this;
        }

        $this->mailAccounts->add($account);
        $account->setDevice($this);

        return $this;
    }

    public function removeMailAccount(MailAccount $account)
    {
        $this->mailAccounts->removeElement($account);

        return $this;
    }

    public function getAdditionalMailAccounts()
    {
        $additional = array();
        $accounts = $this->getMailAccounts();
        foreach ($accounts as $account) {
            if ($this->getUser()->getEmail() == $account->getEmail()) {
                continue;
            }
            $additional[] = $account;
        }

        return $additional;
    }

    public function setAdditionalMailAccounts(array $mailAccounts)
    {
        $primaryMailAccount = $this->getPrimaryMailAccount();
        $this->setMailAccounts($mailAccounts);
        $this->addMailAccount($primaryMailAccount);
    }

    public function addAdditionalMailAccount(MailAccount $account)
    {
        return $this->addMailAccount($account);
    }

    public function removeAdditionalMailAccount(MailAccount $account)
    {
        if ($account->getEmail() == $this->getUser()->getEmail()) {
            return false;
        }

        return $this->removeMailAccount($account);
    }

    /**
     * @return MailAccount
     * @throws \Exception
     */
    public function getPrimaryMailAccount()
    {
        foreach ($this->getMailAccounts() as $account) {
            if ($this->getUser()->getEmail() == $account->getEmail()) {
                return $account;
            }
        }

        throw new \Exception("no primary mail account");
    }

    public function isIPhone()
    {
        return $this->getType()->getName() == "ios";
    }

    /**
     * @return boolean true if the device is manged with MDM.
     */
    public function isManaged()
    {
        return $this->getType()->isManaged($this);
    }

    /**
     * @return boolean true, if the device is blocked, false otherwise.
     */
    public function isBlocked()
    {
        return (boolean) $this->blocked;
    }

    /**
     * @return boolean true if the device is active i.e. it is not deleted, not
     *     disabled, and no wipe is pending, false otherwise.
     */
    public function isActive()
    {
        return !$this->isDeleted() && !$this->isDisabled() && $this->getWipeStatus() == self::WIPE_NONE;
    }

    /**
     * @return boolean true if the device is in a state in which it can be
     *     rolled out.
     */
    public function isRollout()
    {
        return null === $this->getDeviceid() && null === $this->getChallenge()
        && null === $this->getOmaAuthAppSrv() && $this->getPassword() !== null
        && !$this->isDisabled();
    }

    /**
     * Sets the rollout status of the device.
     *
     * @param boolean $rollout true if rollout shall be possible, false
     *                         otherwise.
     */
    public function setRollout($rollout)
    {
        if ($rollout) {
            $this->setDeviceid(null);
            $this->setChallenge(null);
        } else {
            $this->setPlainPassword(null);
            $this->setPassword(null);
        }
    }

    /**
     * @param mixed $plainPassword
     *
     * @return self
     */
    public function setPlainPassword($plainPassword)
    {
        $this->plainPassword = $plainPassword;
        if (null !== $plainPassword) {
            $this->setPassword(PasswordUtil::createSaltedPassword($plainPassword));
            $this->setPasswordStamp(new \DateTime());
        }

        return $this;
    }

    /**
     * @return mixed
     */
    public function getPlainPassword()
    {
        return $this->plainPassword;
    }

    /**
     * @return \Doctrine\Common\Collections\ArrayCollection
     */
    public function getMdmQueue()
    {
        return $this->mdmQueue;
    }

    public function isExpired($maxPasswordValidity)
    {
        $now = new \DateTime();
        $diff = $now->diff($this->getPasswordStamp());
        if ($diff->d >= $maxPasswordValidity) {
            return true;
        }

        return false;
    }

    public function getStatus($maxPasswordValidity)
    {
        if ($this->isRollout()) {
            if ($this->isExpired($maxPasswordValidity)) {
                return "expired";
            }

            return "rolloutPending";
        }

        if ($this->getWipestatus() == self::WIPE_PENDING) {
            return "wipePending";
        }

        if ($this->isDisabled()) {
            return "disabled";
        }

        if (!$this->getType()->isRolloutFinished($this)) {
            return $this->getType()->isRolloutFailed($this) ? "rolloutFailed" : "rolloutUnfinished";
        }

        return "enabled";
    }

    /**
     * @param boolean $timedOut Specifies if the device timed out, i.e. haven't
     *     connected for a specified timeout.
     * @return Device self
     */
    public function setTimedOut($timedOut)
    {
        $this->timedOut = $timedOut;

        return $this;
    }

    /**
     * @return boolean true if the device timed out, i.e. haven't connected for
     *     a specified timeout, or false.
     */
    public function isTimedOut()
    {
        return $this->timedOut;
    }

    /**
     * @param \Topalis\Device\Type\DeviceTypeInterface $type
     *
     * @return self
     */
    public function setType(\Topalis\Device\Type\DeviceTypeInterface $type)
    {
        $this->type = $type;

        return $this;
    }

    /**
     * @return \Topalis\Device\Type\DeviceTypeInterface
     */
    public function getType()
    {
        return $this->type;
    }

    public function eraseCredentials()
    {
        $this->setPlainPassword("");
    }

    /**
     * @see \Serializable::serialize()
     */
    public function serialize()
    {
        return serialize(array(
            $this->id,
        ));
    }

    /**
     * @param string $serialized The value returned by serialize().
     * @see \Serializable::unserialize()
     */
    public function unserialize($serialized)
    {
        list ($this->id) = unserialize($serialized);
    }

    public function getRoles()
    {
        return array();
    }

    public function getSalt()
    {
        return null;
    }

    public function getUsername()
    {
        return null === $this->deviceid ? $this->getId() : $this->getDeviceid();
    }

    /**
     * @return string
     */
    public function getOmaAuthAppSrv()
    {
        return $this->omaAuthAppSrv;
    }

    /**
     * @param string $omaAuthAppSrv
     */
    public function setOmaAuthAppSrv($omaAuthAppSrv)
    {
        $this->omaAuthAppSrv = $omaAuthAppSrv;
        return $this;
    }

    /**
     * @return string
     */
    public function getOmaAuthClient()
    {
        return $this->omaAuthClient;
    }

    /**
     * @param string $omaAuthClient
     */
    public function setOmaAuthClient($omaAuthClient)
    {
        $this->omaAuthClient = $omaAuthClient;
        return $this;
    }

    /**
     * @return string
     */
    public function getOmaAuthNonce()
    {
        return $this->omaAuthNonce;
    }

    /**
     * @param string $omaAuthNonce
     */
    public function setOmaAuthNonce($omaAuthNonce)
    {
        $this->omaAuthNonce = $omaAuthNonce;
        return $this;
    }
}
