<?php
namespace Topalis\Device\EventListener;

use Symfony\Component\DependencyInjection\ContainerAware;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Topalis\Device\Entity\Device;
use Topalis\Device\Entity\AccessLog;

/**
 * This listener sends a mail to the administrator if a device of which the
 * last connection has exceeded the timeout connects again.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class DeviceReturnListener extends ContainerAware
{
    public function postPersist(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof AccessLog) {
            return;
        }

        $device = $entity->getDevice();
        if (!$device->isTimedOut()) {
            return;
        }

        /*
         * If there are multiple requests at the same time a mail may have
         * been sent by another process. We might have an outdated value, so
         * we make a refresh.
         */
        $this->getDeviceEntityManager()->refresh($device);
        if (!$device->isTimedOut()) {
            return;
        }

        $device->setTimedOut(false);
        $this->getDeviceEntityManager()->save($device);

        if ($this->getMailer()->isConfigured()) {
            $this->sendReturnedMail($entity);
        }
    }

    private function sendReturnedMail(AccessLog $accessLog)
    {
        $device = $accessLog->getDevice();
        $returnDate = $accessLog->getTstamp()->format("Y-m-d H:i:s");
        $subject = $this->tr("admin.device.timeout.returned.mail.subject", array(
            "%deviceName%" => $device->getName(),
            "%user%" => $device->getUser()->getLogin()
        ));
        $text = $this->tr("admin.device.timeout.returned.mail.text", array(
            "%deviceName%" => $device->getName(),
            "%deviceid%" => $device->getDeviceid(),
            "%user%" => $device->getUser()->getLogin(),
            "%returnDate%" => $returnDate
        ));
        $this->getMailer()->sendAdminMessage($subject, $text);
    }

    private function tr($id, array $parameters = array(), $domain = 'messages', $locale = null)
    {
        return $this->container->get("translator")->trans($id, $parameters, $domain, $locale);
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Topalis\Mail\Mailer
     */
    private function getMailer()
    {
        return $this->container->get("mailer");
    }
}
