<?php
namespace Topalis\Device\EventListener;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Event\PostFlushEventArgs;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Topalis\Device\Entity\MailAccount;

/**
 * Install/remove changed mail accounts to device
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class MailAccountMdmEventListener
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    /**
     * @var MailAccount[]
     */
    protected $toInstall = array();

    /**
     * @var MailAccount[]
     */
    protected $toRemove = array();

    private $inPostFlush = false;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * @return \Topalis\Certificate\Entity\ClientCertificateEntityManager
     */
    private function getClientCertificateEntityManager()
    {
        return $this->container->get("certificate.entity.manager.certificate");
    }

    /**
     * @return \Topalis\Device\MDM\MdmFacadeInterface
     */
    private function getMdmFacade()
    {
        return $this->container->get("device.mdm.facade");
    }

    public function postPersist(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof MailAccount) {
            return;
        }

        $device = $entity->getDevice();
        if (!$device->isManaged() || !$device->isActive()) {
            return;
        }

        // add mdm command on flush to maintain order
        $this->toInstall[] = $entity;
    }

    public function postUpdate(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof MailAccount) {
            return;
        }

        $device = $entity->getDevice();
        if (!$device->isManaged() || !$device->isActive()) {
            return;
        }

        $uow = $eventArgs->getEntityManager()->getUnitOfWork();
        $changeSet = $uow->getEntityChangeSet($entity);
        if (isset($changeSet['user']) || isset($changeSet['email'])) {
            $this->getMdmFacade()->installMailAccount($entity);
        }
    }

    public function preRemove(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof MailAccount) {
            return;
        }

        // remove after flush to avoid multiple removing when a flush is executed in preRemove
        $this->toRemove[] = clone $entity;
    }

    /**
     * Execute profile installation and removal. Remove old profiles before installing new ones
     *
     * @param PostFlushEventArgs $eventArgs
     */
    public function postFlush(PostFlushEventArgs $eventArgs)
    {
        $eventArgs; // unused

        if ($this->inPostFlush) {
            return;
        }

        $this->inPostFlush = true;

        foreach ($this->toRemove as $key => $entity) {
            // unset required because postFlush is called during the loop run is finished
            unset($this->toRemove[$key]);
            $cert = $entity->getCert();
            if ($cert !== null) {
                $cert->setUnused(true);
                $this->getClientCertificateEntityManager()->save($cert, false);
            }

            $device = $entity->getDevice();
            if ($device->isManaged() && $device->isActive()) {
                $this->getMdmFacade()->removeMailAccount($entity);
            }
        }

        foreach ($this->toInstall as $key => $entity) {
            // unset required because postFlush is called during the loop run is finished
            unset($this->toInstall[$key]);
            $this->getMdmFacade()->installMailAccount($entity);
        }

        $this->inPostFlush = false;
    }
}
