<?php
namespace Topalis\Device\Form;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Constraints\NotBlank;
use Topalis\Device\Entity\Device;

/**
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class DeviceForm extends AbstractType
{
    /**
     * whether device is new
     * @var boolean
     */
    protected $new;

    /**
     * @var \Topalis\ConfigGroup\Entity\ConfigGroup
     */
    protected $preferredGroup;

    /**
     * true if the device has not been rolled out.
     * @var boolean
     */
    protected $rollout;

    /**
     * @var boolean
     */
    protected $noExchange;

    /**
     * @var boolean
     */
    private $selectUser;

    /**
     * Constructor
     * @param boolean $new true if the device is new.
     * @param boolean $rollout true if the device has not been rolled out.
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $preferredGroup The
     *     preferred config group for the device.
     * @param boolean $selectUser false to show an input field to select a
     *     user. true to show disabled text fields for user name and email
     *     address.
     * @param boolean $noExchange true if the edited device belongs to a
     *     non-exchange group, or false otherwise, or null if the device doesn't
     *     belong to any group at all. This flag defines which groups are
     *     possible to choose (devices cannot be moved from a non-exchange group
     *     to an exchange group, and vice versa).
     */
    public function __construct($new, $rollout, \Topalis\ConfigGroup\Entity\ConfigGroup $preferredGroup = null,
        $selectUser = false, $noExchange = null)
    {
        $this->new = $new;
        $this->rollout = $rollout;
        $this->preferredGroup = $preferredGroup;
        $this->selectUser = $selectUser;
        $this->noExchange = $noExchange;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused
        if ($this->selectUser) {
            $builder
                ->add("user", "text", array(
                    "disabled" => true,
                    "property_path" => "user.login",
                    "label" => "admin.devices.user"
                ))
                ->add("email", "text", array(
                    "disabled" => true,
                    "property_path" => "user.email",
                    "label" => "admin.users.email"
                ))
            ;
        } else {
            $builder
                ->add("user", "user_select", array(
                    "label" => "admin.devices.user",
                    "constraints" => new NotBlank()
                ))
            ;
        }

        if ($this->new) {
            $builder->add("sendUserMail", "checkbox", array(
                "label" => "admin.devices.sendUserMail",
                "mapped" => false,
                "required" => false,
            ));
        }

        if ($this->rollout) {
            $builder
                ->add("plainPassword", "password", array(
                    "label" => "admin.devices.rolloutPassword",
                    "constraints" => ($this->new ? new Assert\NotBlank() : null),
                    "required" => $this->new,
                    "had_value" => !$this->new
                ));
        }

        // @codingStandardsIgnoreStart
        if ($this->noExchange === null) {
            $configGroupQueryBuilder = function($repository) {
                    return $repository->createQueryBuilder("cg")->orderBy("cg.name", "ASC");
            };
        } else {
            $noExchange = $this->noExchange;
            $configGroupQueryBuilder = function($repository) use ($noExchange) {
                    return $repository->createQueryBuilder("cg")
                        ->where("cg.noExchange = :noExchange")
                        ->orderBy("cg.name", "ASC")
                        ->setParameter("noExchange", $noExchange);
            };
        }
        // @codingStandardsIgnoreEnd

        $builder
            ->add("comment", "text", array(
                "label" => "Comment",
                "required" => false,
            ))
            ->add("os", "device_type", array(
                "label" => "admin.devices.os.title",
                "constraints" => new Assert\NotBlank(),
                "disabled" => !$this->rollout
            ))
            ->add("owner", "choice", array(
                "label" => "admin.devices.owner",
                "choices" => array(
                    Device::OWNER_COMPANY => "admin.device.owner.company",
                    Device::OWNER_USER => "admin.device.owner.user",
                )
            ))
            ->add("config_group", "entity", array(
                "label" => "admin.group.fieldName",
                "class" => "TopalisConfigGroupBundle:ConfigGroup",

                "query_builder" => $configGroupQueryBuilder,
                "property" => "name",
                "preferred_choices" => ($this->preferredGroup) ? array($this->preferredGroup) : array(),
                "constraints" => new Assert\NotBlank()
            ));
        if (!$this->noExchange) {
            $builder
                ->add("additionalMailAccounts", "collection", array(
                    "type" => new MailAccountForm(),
                    "allow_add" => true,
                    "by_reference" => false,
                    "allow_delete" => true,
                ))
            ;
        }
    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver
            ->setDefaults(array(
                "data_class" => "Topalis\Device\Entity\Device"
            ))
        ;
    }

    /**
     * Returns the name of this type.
     * @return string The name of this type
     */
    public function getName()
    {
        return "device";
    }
}
