<?php
namespace Topalis\Device\Type\Info;

/**
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class IosDeviceInfo implements DeviceInfo
{
    protected $data;
    /**
     * @var array
     * @see https://gist.github.com/Jaybles/1323251#comment-791121
     * @see http://theiphonewiki.com/wiki/IPhone
     * @see https://theiphonewiki.com/wiki/Models
     * @see https://support.apple.com/kb/ht5452
     * @see https://support.apple.com/kb/ht3939
     * @see https://support.apple.com/kb/ht1353
     */
    protected $models = array(
        "AppleTV2,1" => "Apple TV 2G",
        "AppleTV3,1" => "Apple TV 3G (Rev A)",
        "AppleTV3,2" => "Apple TV 3G (Rev B)",

        "iPhone1,1" => "iPhone 1G",
        "iPhone1,2" => "iPhone 3G",
        "iPhone2,1" => "iPhone 3GS",
        "iPhone3,1" => "iPhone 4",
        "iPhone3,2" => "iPhone 4",
        "iPhone3,3" => "Verizon iPhone 4",
        "iPhone4,1" => "iPhone 4S",
        "iPhone5,1" => "iPhone 5 (GSM)",
        "iPhone5,2" => "iPhone 5 (GSM+CDMA)",
        "iPhone5,3" => "iPhone 5c (GSM)",
        "iPhone5,4" => "iPhone 5c (GMS+CDMA)",
        "iPhone6,1" => "iPhone 5s (GSM)",
        "iPhone6,2" => "iPhone 5s (GSM+CDMA)",
        "iPhone7,1" => "iPhone 6 Plus",
        "iPhone7,2" => "iPhone 6",

        "iPod1,1"   => "iPod Touch 1G",
        "iPod2,1"   => "iPod Touch 2G",
        "iPod3,1"   => "iPod Touch 3G",
        "iPod4,1"   => "iPod Touch 4G",
        "iPod5,1"   => "iPod Touch 5G",

        "iPad1,1"   => "iPad",
        "iPad2,1"   => "iPad 2 (WiFi)",
        "iPad2,2"   => "iPad 2 (GSM)",
        "iPad2,3"   => "iPad 2 (CDMA)",
        "iPad2,4"   => "iPad 2 (WiFi)",
        "iPad2,5"   => "iPad mini (WiFi)",
        "iPad2,6"   => "iPad mini (GSM)",
        "iPad2,7"   => "iPad mini (GSM+CDMA)",
        "iPad3,1"   => "iPad 3 (WiFi)",
        "iPad3,2"   => "iPad 3 (GSM+CDMA)",
        "iPad3,3"   => "iPad 3 (GSM)",
        "iPad3,4"   => "iPad 4 (WiFi)",
        "iPad3,5"   => "iPad 4 (GSM)",
        "iPad3,6"   => "iPad 4 (GSM+CDMA)",
        "iPad4,1"   => "iPad Air (WiFi)",
        "iPad4,2"   => "iPad Air (LTE)",
        "iPad4,3"   => "iPad Air (TD-LTE)",
        "iPad4,4"   => "iPad mini 2 (WiFi)",
        "iPad4,5"   => "iPad mini 2 (LTE)",
        "iPad4,6"   => "iPad mini 2 (TD-LTE)",
        "iPad4,7"   => "iPad mini 3",
        // TODO is LTE/TD-LTE correct for iPad mini 3
        "iPad4,8"   => "iPad mini 3 (LTE)",
        "iPad4,9"   => "iPad mini 3 (TD-LTE)",
        "iPad5,3"   => "iPad Air 2",
        "iPad5,4"   => "iPad Air 2 (LTE)",

        "i386"      => "Simulator",
        "x86_64"    => "Simulator",
    );

    public function __construct(array $data)
    {
        $this->data = $data;
    }

    public function getVendor()
    {
        return "Apple";
    }

    public function getName()
    {
        return $this->getData('DeviceName');
    }

    public function getModelName()
    {
        $model = $this->getData('ProductName');
        if (isset($this->models[$model])) {
            return $this->models[$model];
        }

        return $model;
    }

    public function getOsVersion()
    {
        return "iOS " . $this->getData('OSVersion') . " (" . $this->getData('BuildVersion') . ")";
    }

    public function isRoaming()
    {
        return (boolean) $this->getData('IsRoaming');
    }

    public function getCarrier()
    {
        return $this->getData('SIMCarrierNetwork') ?: $this->getData('CurrentCarrierNetwork');
    }

    public function getPhoneNumber()
    {
        return $this->getData('PhoneNumber');
    }

    public function getUDID()
    {
        return $this->getData('UDID');
    }

    public function getIMEI()
    {
        return $this->getData('IMEI');
    }

    public function getWifiMac()
    {
        return $this->getData('WiFiMAC');
    }

    public function getBluetoothMac()
    {
        return $this->getData('BluetoothMAC');
    }

    public function getSerial()
    {
        return $this->getData('SerialNumber');
    }

    public function getAvailableCapacity()
    {
        return $this->getData('AvailableDeviceCapacity') * 1024 * 1024 * 1024;
    }

    public function getCapacity()
    {
        return $this->getData('DeviceCapacity') * 1024 * 1024 * 1024;
    }

    public function hasExternalStorage()
    {
        return false;
    }

    public function getAvailableExternalCapacity()
    {
        return null;
    }

    public function getExternalCapacity()
    {
        return null;
    }

    public function getBatteryLevel()
    {
        return $this->getData('BatteryLevel');
    }

    protected function getUsedKeys()
    {
        return array(
            "Vendor", "DeviceName", "ProductName", "ModelName", "OSVersion", "BuildVersion", "IsRoaming",
            "SIMCarrierNetwork", "PhoneNumber", "UDID", "IMEI", "WiFiMAC", "SerialNumber", "BluetoothMAC",
            "AvailableDeviceCapacity", "AvailableExternalCapacity", "DeviceCapacity", "ExternalCapacity",
            "Model", "BatteryLevel",
        );
    }

    public function getOther()
    {
        $rv = $this->data;
        $usedKeys = $this->getUsedKeys();
        foreach ($usedKeys as $key) {
            unset($rv[$key]);
        }
        ksort($rv);

        return $rv;
    }

    public function getIcon()
    {
        $path = "bundles/topalisdevice/images";
        if (preg_match("=^([A-Za-z]+)(\d),(\d)$=si", $this->getData('ProductName'), $matches)) {
            $model = $matches[1];
            $major = $matches[2];
            $minor = $matches[3];
            if ("iPad" == $model) {
                if ($major == 2 && $minor >= 5 && $minor <= 7) {
                    return $path . "/ipad-mini.png";
                }

                return $path . "/ipad.png";
            }

            if ("iPhone" == $model) {
                if ($major <= 4) {
                    return $path . "/iphone-4.png";
                }

                return $path . "/iphone-5.png";
            }
        }

        return $path . "/iphone-4.png";
    }

    public function get($key)
    {
        return $this->getData($key);
    }

    protected function getData($key)
    {
        return isset($this->data[$key]) ? $this->data[$key] : null;
    }
}
