<?php
namespace Topalis\Gcm;

use Buzz\Browser;
use Topalis\Configuration\AuralisConfig;
use Topalis\Configuration\Client\AuralisCurl;
use Topalis\Device\Entity\Device;
use Topalis\Gcm\Exception\GcmHttpException;
use Topalis\Gcm\Exception\GcmClientException;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 *
 * @see https://developer.android.com/google/gcm/http.html
 */
class Gcm
{
    /**
     * @var sting
     */
    protected $apiKey;

    /**
     * @var \Buzz\Browser
     */
    protected $browser;

    /**
     * @var \Topalis\Configuration\Client\AuralisCurl
     */
    protected $curl;

    /**
     * @var string
     */
    protected $apiUrl = 'https://android.googleapis.com/gcm/send';

    public function __construct(AuralisConfig $config)
    {
        $this->loadConfig($config);
    }

    public function getApiKey()
    {
        return $this->apiKey;
    }

    public function setApiKey($apiKey)
    {
        $this->apiKey = $apiKey;
    }

    /**
     * Loads API key and proxy configuration from the specified AuralisConfig.
     *
     * @param AuralisConfig $config The AuralisConfig to load.
     */
    public function loadConfig(AuralisConfig $config)
    {
        $this->setApiKey($config->getGcmConfig()->getApiKey());
        $this->curl = new AuralisCurl($config);
        $this->browser = new Browser($this->curl);
    }

    /**
     *
     * @param Device $device
     * @param array $data
     * @param array $options
     * @throws GcmClientException If a connection to the GCM server could not
     *     be established.
     * @throws GcmHttpException If the GCM returns a status code that doesn't
     *     match 2xx.
     * @return array The JSON decoded response.
     */
    public function send(Device $device, array $data, array $options = array())
    {
        $headers = array(
            "Authorization: key=" . $this->apiKey,
            "Content-Type: application/json"
        );

        $data = array_merge($options, array(
            "data" => $data,
            "registration_ids" => array($device->getMdmtoken())
        ));

        try {
            $response = $this->browser->post($this->apiUrl, $headers, json_encode($data));
        } catch (\Buzz\Exception\RuntimeException $e) {
            throw new GcmClientException($e->getMessage(), $e->getCode(), $e);
        }

        $status = $response->getStatusCode();
        if ($status < 200 || $status > 299) {
            throw new GcmHttpException($response);
        }

        return json_decode($response->getContent(), true);
    }

    /**
     *
     * @param string $apiKey If specified this API key is used instead of the
     *     one set in the configuration.
     * @throws GcmClientException If a connection to the GCM server could not
     *     be established.
     * @return \Buzz\Message\Response
     */
    public function getTestResponse($apiKey = null)
    {
        if ($apiKey === null) {
            $apiKey = $this->apiKey;
        }

        try {
            // reduce timeouts because we don't want to wait for the result too
            // long
            $this->curl->setConnectTimeout(5);
            $this->curl->setTimeout(10);

            $headers = array(
                "Authorization: key=" . $apiKey,
            );
            $response = $this->browser->post($this->apiUrl, $headers);
        } catch (\Buzz\Exception\RuntimeException $e) {
            $this->curl->resetTimeouts();
            throw new GcmClientException($e->getMessage(), $e->getCode(), $e);
        }

        $this->curl->resetTimeouts();

        return $response;
    }

    /**
     * Connects to the GCM service without data and returns the status code.
     *
     * @param string $apiKey If specified this API key is used instead of the
     *     one set in the configuration.
     * @return integer The status code.
     * @throws GcmClientException If a connection to the GCM server could not
     *     be established.
     * @see https://developer.android.com/google/gcm/http.html#response
     */
    public function getStatus($apiKey = null)
    {
        return $this->getTestResponse($apiKey)->getStatusCode();
    }
}
