<?php
namespace Topalis\Kernel;

use Symfony\Component\Debug\ExceptionHandler;
use Symfony\Component\Debug\Exception\FatalErrorException;
use Symfony\Component\Debug\Exception\ContextErrorException;
use Psr\Log\LoggerInterface;

/**
 * ErrorHandler. Used to convert errors into exceptions or to log them.
 * Based on Symfony\Component\Debug\ErrorHandler
 *
 * @author Fabien Potencier <fabien@symfony.com>
 * @author Konstantin Myakshin <koc-dp@yandex.ru>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class ErrorHandler
{
    private $levels = array(
        E_WARNING           => 'Warning',
        E_NOTICE            => 'Notice',
        E_USER_ERROR        => 'User Error',
        E_USER_WARNING      => 'User Warning',
        E_USER_NOTICE       => 'User Notice',
        E_STRICT            => 'Runtime Notice',
        E_RECOVERABLE_ERROR => 'Catchable Fatal Error',
        E_DEPRECATED        => 'Deprecated',
        E_USER_DEPRECATED   => 'User Deprecated',
        E_ERROR             => 'Error',
        E_CORE_ERROR        => 'Core Error',
        E_COMPILE_ERROR     => 'Compile Error',
        E_PARSE             => 'Parse',
    );

    private $level;

    private $reservedMemory;

    /**
     * @var LoggerInterface[] Loggers for channels
     */
    private static $loggers = array();

    /**
     * Registers the error handler.
     *
     * @param integer $level The level at which the conversion to Exception is
     *     done (null to use the error_reporting() value and 0 to disable)
     *
     * @return The registered error handler
     */
    public static function register($level = null)
    {
        $handler = new static();
        $handler->setLevel($level);

        set_error_handler(array($handler, 'handle'));
        register_shutdown_function(array($handler, 'handleFatal'));
        $handler->reservedMemory = str_repeat('x', 10240);

        return $handler;
    }

    public function setLevel($level)
    {
        $this->level = null === $level ? error_reporting() : $level;
    }

    public static function setLogger(LoggerInterface $logger = null, $channel = null)
    {
        if (null !== $channel) {
            self::$loggers[$channel] = $logger;
            return;
        }

        if (!isset(self::$loggers['deprecation'])) {
            self::$loggers['deprecation'] = $logger;
        }
        if (!isset(self::$loggers['error'])) {
            self::$loggers['error'] = $logger;
        }
        if (!isset(self::$loggers['emergency'])) {
            self::$loggers['emergency'] = $logger;
        }
    }

    /**
     * @param integer $level
     * @param string $message
     * @param string $file
     * @param integer $line
     * @param array $context
     * @throws ContextErrorException When error_reporting returns error
     */
    public function handle($level, $message, $file = 'unknown', $line = 0, array $context = array())
    {
        if (0 === $this->level || !($level & error_reporting())) {
            return false;
        }

        $type = isset($this->levels[$level]) ? $this->levels[$level] : $level;

        if ($level & (E_USER_DEPRECATED | E_DEPRECATED)) {
            if (isset(self::$loggers['deprecation'])) {
                $stack = $this->getBacktrace();
                self::$loggers['deprecation']->warning($message, array('type' => $type, 'stack' => $stack));
            }

            return false;
        }

        if ($this->level & $level) {
            $errMsg = sprintf('%s: %s in %s line %d', $type, $message, $file, $line);
            throw new ContextErrorException($errMsg, $level, 0, $file, $line, $context);
        } else {
            if (isset(self::$loggers['error'])) {
                $stack = $this->getBacktrace();
                self::$loggers['error']->error($message, array('type' => $type, 'stack' => $stack));
            }
        }

        return false;
    }

    public function handleFatal()
    {
        if (null === $error = error_get_last()) {
            return;
        }

        unset($this->reservedMemory);
        $type = $error['type'];
        if (0 === $this->level || !in_array($type, array(E_ERROR, E_CORE_ERROR, E_COMPILE_ERROR, E_PARSE))) {
            return;
        }

        if (isset(self::$loggers['emergency'])) {
            $fatal = array(
                'type' => $type,
                'file' => $error['file'],
                'line' => $error['line'],
            );

            self::$loggers['emergency']->emerg($error['message'], $fatal);
        }

        // get current exception handler
        // @codingStandardsIgnoreStart
        $exceptionHandler = set_exception_handler(function() {});
        // @codingStandardsIgnoreEnd
        restore_exception_handler();

        if (is_array($exceptionHandler) && $exceptionHandler[0] instanceof ExceptionHandler) {
            $level = isset($this->levels[$type]) ? $this->levels[$type] : $type;
            $message = sprintf('%s: %s in %s line %d', $level, $error['message'], $error['file'], $error['line']);
            $exception = new FatalErrorException($message, 0, $type, $error['file'], $error['line']);
            $exceptionHandler[0]->handle($exception);
        }
    }

    protected function getBacktrace()
    {
        if (version_compare(PHP_VERSION, '5.4', '<')) {
            $stack = array_map(
                function ($row) {
                    unset($row['args']);

                    return $row;
                },
                array_slice(debug_backtrace(false), 0, 10)
            );
        } else {
            $stack = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 10);
        }

        return $stack;
    }
}
