<?php
namespace Topalis\Kernel\Monolog;

use Topalis\Util\DebugUtil;

/**
 * This processor adds a backtrace to the record. If an exception is set in the
 * context with the index 'exception' it is used to get the trace. Otherwise
 * debug_backtrace() is used.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class BacktraceProcessor
{
    /**
     * @var boolean
     */
    private $hideStrings = false;

    /**
     * @var integer
     */
    private $minLevel = 300; // warning

    /**
     * @param array $record
     * @return array
     */
    public function __invoke(array $record)
    {
        if ($record['level'] >= $this->getMinLevel()) {
            if (isset($record['context']['exception']) && $record['context']['exception'] instanceof \Exception) {
                $e = $record['context']['exception'];
                if ($this->isHideStrings()) {
                    if (class_exists("Topalis\Util\DebugUtil")) {
                        // the class loader may not be loaded, yet.
                        $record['extra']['backtrace'] = implode("   \n",
                            DebugUtil::backtraceToString($e->getTrace(), 0, true));
                    }
                } else {
                    $record['extra']['backtrace'] = str_replace("\n", "   \n", $e->getTraceAsString());
                }
            } else if (class_exists("Topalis\Util\DebugUtil")) {
                // the class loader may not be loaded, yet.
                $record['extra']['backtrace'] = implode("   \n",
                    DebugUtil::backtraceToString(debug_backtrace(), 1, $this->isHideStrings()));
            }
        }
        return $record;
    }

    /**
     * @return int The minimal log level for which a backtrace is added.
     */
    public function getMinLevel()
    {
        return $this->minLevel;
    }

    /**
     * Sets the minimal log level for which a backtrace is added
     * @param int $minLevel The log level to set.
     */
    public function setMinLevel($minLevel)
    {
        $this->minLevel = $minLevel;
    }

    /**
     * @return boolean True if strings are removed in the backtrace. Default:
     *     false.
     */
    public function isHideStrings()
    {
        return $this->hideStrings;
    }

    /**
     * Enables or disables removing of strings in backtraces. This can be used
     * to hide sensible data such as passwords. Default: disabled.
     *
     * @param boolean $hideStrings True to enable removing of strings, false
     *     to show strings.
     */
    public function setHideStringsEnabled($hideStrings)
    {
        $this->hideStrings = $hideStrings;
    }
}
