<?php
namespace Topalis\Kernel\Security;

use Symfony\Component\Security\Core\Exception\UnsupportedUserException;
use Symfony\Component\Security\Core\Exception\UsernameNotFoundException;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Security\Core\User\UserProviderInterface;
use Topalis\Configuration\AuralisConfig;
use Topalis\Device\Entity\Device;
use Topalis\Device\Entity\DeviceEntityManager;
use Topalis\User\Entity\User;

/**
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class UserProvider implements UserProviderInterface
{
    /**
     * @var \Topalis\Configuration\AuralisConfig
     */
    private $config;

    /**
     * @var \Topalis\Device\Entity\DeviceEntityManager
     */
    private $entityManager;

    public function __construct(DeviceEntityManager $entityManager, \Topalis\Configuration\AuralisConfig $config)
    {
        $this->config = $config;
        $this->entityManager = $entityManager;
    }

    /**
     * Loads the user for the given username.
     * This method must throw UsernameNotFoundException if the user is not
     * found.
     *
     * @param string $username The username
     *
     * @return UserInterface
     * @see UsernameNotFoundException
     * @throws UsernameNotFoundException if the user is not found
     */
    public function loadUserByUsername($username)
    {
        $expire = new \DateTime();
        $expire->sub(new \DateInterval("P".$this->config->getPasswordValidity()."D"));

        $devices = $this->entityManager->createQueryBuilder("d")
            ->select("d")
            ->join("d.user", "u")
            ->where("u.login = :login")
            ->andWhere("d.deviceid IS NULL AND d.challenge is NULL AND d.disabled = 0 AND d.deleted = 0")
            ->andWhere("d.passwordStamp > :max_password_validity")
            ->andWhere("d.password IS NOT NULL")
            ->setParameters(array(
                "login" => $username,
                "max_password_validity" => $expire
            ))
            ->getQuery()
            ->getResult()
            ;

        if (empty($devices)) {
            throw new UsernameNotFoundException();
        }

        return $devices;
    }

    /**
     * Refreshes the user for the account interface.
     * It is up to the implementation to decide if the user data should be
     * totally reloaded (e.g. from the database), or if the UserInterface
     * object can just be merged into some internal array of users / identity
     * map.
     *
     * @param UserInterface $device
     * @return UserInterface
     * @throws UnsupportedUserException if the account is not supported
     */
    public function refreshUser(UserInterface $device)
    {
        if (!$device instanceof Device) {
            throw new UnsupportedUserException();
        }

        return $this->entityManager->find($device->getId());
    }

    /**
     * Whether this provider supports the given user class
     *
     * @param string $class
     *
     * @return boolean
     */
    public function supportsClass($class)
    {
        return $class == 'Topalis\Device\Entity\Device';
    }
}
