<?php
namespace Topalis\License\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;

class LicenseCheckCommand extends ContainerAwareCommand
{
    private static $APNS_CERT_WARN_DAYS = array(60, 30, 15, 7);
    private static $LICENSE_WARN_DAYS = array(60, 30, 15, 7);

    const TEST_LICENSE_EXPIRES_SUBJECT = "auralis test license expires soon";
    const TEST_LICENSE_EXPIRES_TEXT = <<<EOF
Your auralis test license will expire in %d days. Please consider purchasing a license.
EOF;

    const LICENSE_EXPIRES_SUBJECT = "auralis license expires soon";
    const LICENSE_EXPIRES_TEXT = <<<EOF
Your auralis license will expire in %d days. Please consider purchasing a new license.
EOF;

    const APNS_CERT_EXPIRES_SUBJECT = "APNS certificate expires soon";
    const APNS_CERT_EXPIRES_TEXT = <<<EOF
Your certificate for the Apple Push Notification Service (APNS) will expire in %d days.
You should consider requesting a new one.
EOF;

    /**
     * @return \Topalis\Apache\Apache
     */
    private function getApache()
    {
        return $this->getContainer()->get("apache");
    }

    /**
     * @return \Topalis\ActiveSyncFilter\Apache\ASFApacheMod
     */
    private function getASFApacheMod()
    {
        return $this->getContainer()->get("asf.apache_mod");
    }

    /**
     * @return \Topalis\License\License
     */
    private function &getLicense()
    {
        return $this->getLicenseManager()->getLicense();
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->getContainer()->get("certificate.manager");
    }

    /**
     * @return \Topalis\License\LicenseManager
     */
    private function getLicenseManager()
    {
        return $this->getContainer()->get("license.manager");
    }

    /**
     * @return \Topalis\Mail\Mailer
     */
    private function getMailer()
    {
        return $this->getContainer()->get("mailer");
    }

    protected function configure()
    {
        $this
        ->setName("license:check")
        ->setDescription("Checks the license validity.")
        ->setHelp("Checks the license validity.");
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $input; // unused

        try {
            $this->checkLicense();
        } catch (\Exception $e) {
            if ($output instanceof \Symfony\Component\Console\Output\ConsoleOutput) {
                $output = $output->getErrorOutput();
            }
            $output->writeln("License check failed:");
            $output->writeln((string) $e);
            return -1;
        }
    }

    private function checkLicense()
    {
        $license = $this->getLicense();
        if (!$license->isValid()) {
            // disable features
            $this->getASFApacheMod()->disable();
            $this->getApache()->setReverseProxyEnabled(false);
            if ($license->isTestLicense()) {
                // revoke all certificates
                $this->getCertificateManager()->doCreateRevocationList();
            }
        } else if ($license->isTestLicense()) {
            $remainingDays = $license->getRemainingDays();
            if (array_search($remainingDays, self::$LICENSE_WARN_DAYS) !== false) {
                $subject = sprintf(self::TEST_LICENSE_EXPIRES_SUBJECT, $remainingDays);
                $message = sprintf(self::TEST_LICENSE_EXPIRES_TEXT, $remainingDays);
                $this->sendAdminMail($subject, $message);
            }
        } else {
            $remainingDays = $license->getRemainingDays();
            if (array_search($remainingDays, self::$LICENSE_WARN_DAYS) !== false) {
                $subject = sprintf(self::LICENSE_EXPIRES_SUBJECT, $remainingDays);
                $message = sprintf(self::LICENSE_EXPIRES_TEXT, $remainingDays);
                $this->sendAdminMail($subject, $message);
            }
        }

        $remainingApnsDays = $license->getRemainingApnsCertDays();
        if (array_search($remainingApnsDays, self::$APNS_CERT_WARN_DAYS) !== false) {
            $subject = sprintf(self::APNS_CERT_EXPIRES_SUBJECT, $remainingApnsDays);
            $message = sprintf(self::APNS_CERT_EXPIRES_TEXT, $remainingApnsDays);
            $this->sendAdminMail($subject, $message);
        }
    }

    private function sendAdminMail($subject, $message)
    {
        $mailer = $this->getMailer();
        if ($mailer->isConfigured()) {
            return $mailer->sendAdminMessage($subject, $message);
        }
        return false;
    }
}
