<?php
namespace Topalis\License\Controller;

use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormError;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Topalis\Kernel\Controller;
use Topalis\License\Form\LicenseUploadForm;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;

/**
 * Show licence controller with auralis about
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/admin/license", service="license.controller")
 */
class LicenseController extends Controller
{
    /**
     * @return \Topalis\User\Entity\UserEntityManager
     */
    private function getUserEntityManager()
    {
        return $this->container->get("user.entity.manager.user");
    }

    /**
     * @Route("", name="admin_license", methods="GET")
     * @Template()
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function indexAction()
    {
        $license = $this->getLicense();
        $licenseData = array(
            "status"           => "",
            "users"            => 0,
            "currentUsers"     => $this->getDB()->count("users"),
            "validTo"          => 0,
            "validDiff"        => 0,
            "validClass"       => "",
        );

        $pushCert = $license->getApnsCertificate();
        $licenseData['validTo'] = $license->getValidTo();
        $remainingTime = $license->getRemainingTime();
        $licenseData['expired'] = $remainingTime < 0;
        $remainingDays = $license->getRemainingDays();
        $licenseData['validDiff'] = $this->getTranslator()->transChoice("date.diff.day", $remainingDays,
            array("%days%" => $remainingDays));
        $licenseData['users'] = $license->getAllowedUsers();
        $licenseData['test'] = $license->isTestLicense();
        $licenseData['existsApnsCert'] = $license->existsApnsCertificate();

        $licenseData['activeSyncFilter'] = $license->isASFAllowed();
        $licenseData['secureIntranet'] = $license->isReverseProxyAllowed();

        $licenseData['auralisId'] = $license->getAuralisId();

        $class = "success";
        if (!$license->isTestLicense()) {
            if ($remainingTime < 86400 * 30) {
                $class = "important";
            }
            if ($remainingTime < 86400 * 30 * 3) {
                $class = "warning";
            }
        } else {
            if ($remainingTime < 86400 * 5) {
                $class = "important";
            } else if ($remainingTime < 86400 * 15) {
                $class = "warning";
            }
        }

        if ($licenseData['expired']) {
            $licenseData['status'] = $this->tr("admin.license.inactive");
        } else if ($license->isTestLicense()) {
            $licenseData['status'] = $this->tr("admin.license.test");
        } else {
            $licenseData['status'] = $this->tr("admin.license.active");
        }

        $licenseData['validClass'] = $class;

        $licenseMessage = "";
        if (!$license->isApnsCertificateValid()) {
            if (!$license->existsApnsCertificate()) {
                $licenseMessage = $this->tr("admin.license.applePushCertNotPresent");
            } else if (!$license->isApnsPrivateKeyValid()) {
                $licenseMessage = $this->tr("admin.license.applePushKeyInvalid");
            } else {
                $licenseMessage = $this->tr("admin.license.applePushCertNotValid");
            }
        }

        return array(
            "form" => $this->createUploadForm()->createView(),
            "pushCert" => $pushCert,
            "licenceMessage" => $licenseMessage,
            "license" => $licenseData
        );
    }

    /**
     * @Route("/upload", name="admin_license_upload", methods="POST")
     * @Template("TopalisLicenseBundle:License:index.html.twig")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function uploadAction()
    {
        try {
            $form = $this->createUploadForm();
            $license = $this->getLicense();

            $form->submit($this->getRequest());
            if (!$form->isValid()) {
                // return index so the form error is printed
                return $this->indexAction();
            }

            /** @var $file UploadedFile */
            $file = $form->get("license")->getData();
            $content = file_get_contents($file->getRealPath());
            // replace CRLF with LF. When sending a license file by email the
            // normal line breaks may be converted into CRLF and so make the
            // signature invalid.
            $content = str_replace("\r\n", "\n", $content);
            $newLicense = $this->getLicenseManager()->createLicense($content);
            if (!$newLicense->isValid()) {
                if (!$newLicense->isSignatureValid()) {
                    $msgId = "license.invalidSignature";
                } else if ($newLicense->isExpired()) {
                    $msgId = "license.expired";
                } else if ($newLicense->getAuralisId() != $license->getAuralisId()) {
                    $msgId = "license.invalidAuralisId";
                } else {
                    $msgId = "license.invalid";
                }
                $form->addError(new FormError($msgId));
            } else if ($this->getUserEntityManager()->count() > $newLicense->getAllowedUsers()) {
                $formError = new FormError("license.tooManyUsers", null, array(
                        "%current%" => $this->getDB()->count("users"),
                        "%allowed%" => $license->getAllowedUsers()
                ));
                $form->addError($formError);
            } else {
                $this->getLicenseManager()->setLicense($newLicense);
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("Installing license failed", array("exception" => $e));
            $msg = $this->tr("admin.license.error.upload", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
            return $this->redirect($this->generateUrl("admin_license"));
        }

        if (!$form->isValid()) {
            // return index so the form error is printed
            return $this->indexAction();
        }

        return $this->redirect($this->generateUrl("admin_license"));
    }

    /**
     * @return \Symfony\Component\Form\FormInterface
     */
    private function createUploadForm()
    {
        static $form = null;
        if ($form === null) {
            $form = $this->createForm(new LicenseUploadForm());
        }
        return $form;
    }
}
