<?php
namespace Topalis\License;

use Topalis\Util\StringUtil;
use Topalis\Certificate\Util\CertUtil;

/**
 * @author Andreas Hügel <andreas.huegel@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class License
{
    const LICVALIDTO = 'validTo';
    const TESTPERIOD = '31 days';
    const LICSIGSEP = "\n;----- Auralis Signature ----";
    const LICPREFIX = "\n;";
    const LICAURALISID = 'auralisId';

    private $signatureValid;
    private $content;
    private $values;
    private $apnsCert;
    private $privKey;

    /**
     * @var \Topalis\Configuration\AuralisConfig
     */
    private $config;

    /**
     * @var \Topalis\Kernel\TimeProviderInterface
     */
    private $timeProvider;

    /**
     * @var \Topalis\Database\MySqlDb
     */
    private $db;

    public function __construct(\Topalis\License\LicenseManager $licenseManager, $content)
    {
        $this->config = $licenseManager->getConfig();
        $this->timeProvider = $licenseManager->getTimeProvider();
        $this->db = $licenseManager->getDB();

        $this->content = $content;
        if ($this->isSignatureValid()) {
            $this->values = parse_ini_string($this->content);
        }

    }

    public function save($filename)
    {
        file_put_contents($filename, $this->content);
    }

    /**
     * @param string $key
     * @return string|array|null
     */
    private function getValue($key)
    {
        return isset($this->values[$key]) ? $this->values[$key] : null;
    }

    /**
     * @return string the systems Apple Push Certificates public key
     */
    public function getApnsPrivKey()
    {
        if (!isset($this->privKey)) {
            $this->privKey = $this->config->getApnsPrivKey();
        }
        return $this->privKey;
    }

    /**
     * @return string the systems auralis-id as it is derived from the ApplePushCertificate
     */
    public function getAuralisIdFromApnsCert()
    {
        if (!$this->existsApnsCertificate()) {
            return null;
        }

        $ski = CertUtil::getSubjectKeyIdentifier($this->getApnsCertificate());
        $ski = str_replace(":", "", $ski);
        $skiArr = str_split($ski, 8);
        $ski = "";
        foreach ($skiArr as $skistring) {
            $ski = $ski."-".$skistring;
        }
        $ski = substr($ski, 1, 44);
        return $ski;
    }

    /**
     * @return bool true if the usage of the ActiveSyncFilter is allowed by the
     *   license, false otherwise.
     */
    public function isASFAllowed()
    {
        return $this->hasFeature("trafficControl");
    }

    /**
     * @return string The PEM encoded certificate.
     * @throws \Exception On error.
     */
    public function getApnsCertificate()
    {
        if ($this->apnsCert === null) {
            $this->apnsCert = $this->config->getApnsCert();
        }
        return $this->apnsCert;
    }

    /**
     * @return boolean, true if the APNS certificate can be verified against the
     *     CA chain of Apple, is not expired and the private key corresponds to
     *     the certificate, false otherwise.
     * @throws \Exception On error.
     */
    public function isApnsCertificateValid()
    {
        $APPLEAAICA = __DIR__ . '/Resources/cert/AppleAAICA.pem';
        if (!CertUtil::isCertValid($this->getApnsCertificate(), array(X509_PURPOSE_SSL_CLIENT),
            $APPLEAAICA, $this->timeProvider->getTime())
            || !$this->isApnsPrivateKeyValid()) {
            return false;
        }
        return true;
    }

    /**
     * @return boolean, true if the private key corresponds to the APNS
     *     certificate.
     * @throws \Exception On error.
     */
    public function isApnsPrivateKeyValid()
    {
        $cert = $this->getApnsCertificate();
        $key = $this->getApnsPrivKey();
        return CertUtil::isPrivateKeyValid($cert, $key);
    }

    /**
     * @return int UNIX timestamp of the end of test period
     * @throws \Exception On error.
     */
    public function getTestPeriodEnd()
    {
        if ($this->existsApnsCertificate()) {
            $validFrom = CertUtil::getValidFrom($this->getApnsCertificate());
            if ($validFrom !== null) {
                return strtotime(self::TESTPERIOD, $validFrom);
            }
        }
        // + 1 to ensure that it is not expired even if a second elapses
        // until the expiration check
        return $this->timeProvider->getTime() + 1;
    }

    /**
     * @return string The auralis-id stored in the license or the auralis id
     *     generated from the APNS certificate if it is a test license.
     */
    public function getAuralisId()
    {
        if ($this->isTestLicense()) {
            return $this->getAuralisIdFromApnsCert();
        }
        return $this->getValue(self::LICAURALISID);
    }

    /**
     * @return boolean, true if the license period is finished.
     * @throws \Exception On error.
     */
    public function isExpired()
    {
        return $this->getRemainingTime() < 0;
    }

    /**
     * @return boolean, true signature of license file is ok
     * @throws \Exception On error.
     */
    public function isSignatureValid()
    {
        if ($this->signatureValid === null) {
            $LICPUBKEY = <<<'EOD'
-----BEGIN PUBLIC KEY-----
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEApaKWxWdLixUdYBkg+Ip0
1wRKcHQ/mjJGTCuXDyVOqOqg94ylc48VbQtOp0PsVo+QExAWw+vbN37EdrLko9YC
BZhabFrJZADcEYnkYerixNqb0Tg7wOLtfoja0GcMwUIwrRWmwBi0vNQGD9xqY2sm
ocaC/LLjKazrD6z9I5ubxhlu2qQIVejBX57evoHSjU5zNtJcLB9kCYmK3Vto6Bc3
ctCegF6ISjrcIkfK5NHrAO0pjA6Okdzrs2/kt8BA52k41rm4H8rdirt5fvTu+7Qg
RttbG7/TUVXzTjBUa0KQVnJ2W4oN3BciRpNwxK0F0qwLt+nojfeTTAQDmcjDpN9R
9wIDAQAB
-----END PUBLIC KEY-----
EOD;
            $licarray = explode(self::LICSIGSEP, $this->content);
            if (isset($licarray[0]) && isset($licarray[1])) {
                $lic = $licarray[0];
                $signature = str_replace(self::LICPREFIX, "", $licarray[1]);
                $signature = base64_decode($signature);
                $pubKeyId = openssl_pkey_get_public($LICPUBKEY);
                $this->signatureValid = openssl_verify($lic, $signature, $pubKeyId) == 1;
            } else {
                $this->signatureValid = false;
            }
        }

        return $this->signatureValid;
    }

    /**
     * @return int The number of users allowed, -1 if unlimited.
     * @deprecated Use getAllowedUsers() instead.
     */
    public function getValidUsers()
    {
        return $this->getAllowedUsers();
    }

    /**
     * @return int The number of users allowed, -1 if unlimited.
     */
    public function getAllowedUsers()
    {
        if ($this->isTestLicense()) {
            return 5;
        }
        $users = $this->getValue("users");
        if ($users == "unlimited") {
            return -1;
        }
        return intval($users);
    }

    /**
     * @return int UNIX timestamp with the time until the license is valid.
     * @throws \Exception On error.
     */
    public function getValidTo()
    {
        if ($this->isTestLicense()) {
            return $this->getTestPeriodEnd();
        }
        $validTo = $this->getValue(self::LICVALIDTO);
        return $validTo === null ? null : strtotime($validTo);
    }

    /**
     * @return int The remaining days the APNS certificate is valid.
     */
    public function getRemainingApnsCertDays()
    {
        $validTo = CertUtil::getValidTo($this->getApnsCertificate());
        $remainingTime = $validTo - $this->timeProvider->getTime();
        return max(0, intval(floor($remainingTime / 86400)) + 1);
    }

    /**
     * @return int The remaining days the license is valid.
     */
    public function getRemainingDays()
    {
        return max(0, intval(floor($this->getRemainingTime() / 86400)) + 1);
    }

    /**
     * @return int The number of remaining seconds the license is valid.
     */
    public function getRemainingTime()
    {
        return $this->getValidTo() - $this->timeProvider->getTime();
    }

    /**
     * TODO IMHO this does not belong into the License class.
     * @return boolean true if the number of current users greater or equal the
     *     number of allowed users.
     */
    public function tooManyUsers()
    {
        $currusers = $this->db->count("users");
        $validUsers = $this->getAllowedUsers();
        if ($validUsers == -1) {
            // unlimited
            return false;
        }
        return $currusers >= $this->getAllowedUsers();
    }

    /**
     * @return boolean true if an APNS certificate is set, false otherwise.
     */
    public function existsApnsCertificate()
    {
        $cert = $this->config->getApnsCert();
        return !empty($cert);
    }

    /**
     * @return boolean true if the revocation of client certificates is required.
     */
    public function isCertRevokeRequired()
    {
        return $this->isTestLicense() && $this->isExpired();
    }

    public function isReverseProxyAllowed()
    {
        return $this->hasFeature("secureIntranet");
    }

    /**
     * @return boolean true if the license is a test license, false otherwise.
     */
    public function isTestLicense()
    {
        return $this->content === null;
    }

    /**
     * @return boolean true if production or test license is valid, false otherwise.
     */
    public function isValid()
    {
        if (!$this->existsApnsCertificate()) {
            return false;
        }

        if ($this->isExpired()) {
            return false;
        }

        return true;
    }

    /**
     * @return array Associative array defining which features are available
     *     (value true if available, false otherwise).
     */
    public function getFeatures()
    {
        $features = array(
                "secureIntranet" => false,
                "trafficControl" => false,
        );
        if ($this->isTestLicense() && !$this->isExpired()) {
            $features['trafficControl'] = true;
            $features['secureIntranet'] = true;
        } else if (!$this->isValid()) {
            return $features;
        } else {
            foreach (array_keys($features) as $key) {
                $features[$key] = StringUtil::strtobool($this->getValue($key));
            }
        }
        return $features;
    }

    /**
     * @param string $key Key of the feature.
     * @return boolean true if the specified feature is available in the license,
     *     false otherwise
     */
    public function hasFeature($key)
    {
        $features = $this->getFeatures();
        return isset($features[$key]) ? $features[$key] : false;
    }

}
