<?php
namespace Topalis\MDM;

use Topalis\App\Entity\App;
use Topalis\Device\Entity\Device;
use Topalis\Kernel\AuralisServiceInterface;

/**
 * This interface contains methods to manage MDM devices.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
interface MdmAdminInterface extends AuralisServiceInterface
{
    /**
     * Applies the specified redemption code for the device and the iOS app
     * specified by $iosManagedAppState.
     *
     * @param \Topalis\App\Entity\IosAppState $state          Contains the device and
     *                                                        iOS app to apply the redemption code.
     * @param string                          $redemptionCode The redemption code to apply.
     *
     * @throws \Topalis\MDM\Exception\MdmException If the iOS app on the device doesn't need a
     *     redemption code.
     */
    public function applyRedemptionCode(\Topalis\App\Entity\IosAppState $state, $redemptionCode);

    /**
     *
     * @return array Array of active (i.e. not disabled, not deleted, no
     *     wipe pending) managed devices.
     */
    public function getActiveManagedDevices();

    /**
     * Removes the profile with the specified identifier from the specified
     *     device.
     *
     * @param \Topalis\Device\Entity\Device $device The device.
     * @param string $identifier The profile identifier.
     */
    public function removeProfile(\Topalis\Device\Entity\Device $device, $identifier);

    /**
     * Removes the app from the device specified in the $state
     *
     * @param \Topalis\App\Entity\AppState $state Contains the app and the device to apply the removal.
     */
    public function removeApp(\Topalis\App\Entity\AppState $state);

    /**
     * Removes the specified app from all devices on which the app has been
     * installed through MDM.
     *
     * @param \Topalis\App\Entity\App $app The app to remove.
     */
    public function removeApps(\Topalis\App\Entity\App $app);

    /**
     * Removes the specified app from devices that are not part of a group the
     * app is assigned to.
     *
     * @param \Topalis\App\Entity\App $app the app to remove.
     */
    public function removeUnassignedApps(\Topalis\App\Entity\App $app);

    /**
     * Removes all managed apps from the specified device that are not assigned
     * to it (anymore). It also cancels outstanding install commands.
     *
     * @param Device $device The device to remove unassigned apps.
     */
    public function removeUnassignedAppsFromDevice(Device $device);

    /**
     * Removes the profile belonging to the specified mail account.
     *
     * @param \Topalis\Device\Entity\MailAccount $mailAccount
     */
    public function removeMailProfile(\Topalis\Device\Entity\MailAccount $mailAccount);

    /**
     * Creates an MDM command for every device to request its device information.
     */
    public function requestDeviceInfos();

    /**
     * Requests to send another push message to the APNS for the specified
     * device. This method does nothing for devices having no MDM command in the
     * queue.
     *
     * @param \Topalis\Device\Entity\Device $device The device to send another push.
     */
    public function repush(\Topalis\Device\Entity\Device $device);

    /**
     * Creates an MDM command to request device information.
     *
     * @param \Topalis\Device\Entity\Device $device The device to request the
     *     device information.
     */
    public function requestDeviceInfo(\Topalis\Device\Entity\Device $device);

    /**
     * Create MDM Command to request managed application list
     *
     * @param Device $device
     * @param App[]  $apps Optional array of apps to fetch
     * @return \Topalis\MDM\Entity\MdmCommand|NULL The created MDM command, or
     *     null if there's already a request managed application list command in
     *     the queue.
     */
    public function requestManagedApplicationList(\Topalis\Device\Entity\Device $device, array $apps = array());

    /**
     * Sends the specified MDM command again. A new command will be created so
     * that the old command stays in the history.
     *
     * @param \Topalis\Device\Entity\Device $device
     * @param \Topalis\MDM\Entity\MdmCommand $command
     */
    public function resendCommand(\Topalis\Device\Entity\Device $device, \Topalis\MDM\Entity\MdmCommand $command);

    /**
     * clears the current passcode of the device
     * @param Device $device
     */
    public function clearPasscode(Device $device);

    public function setPasscode(Device $device, $passcode);

    /**
     * locks the device immediately. Can show a message and a phone number on the device
     *
     * @param Device $device
     * @param string $message Message this message is displayed on the lock screen
     * @param string $phoneNumber this phone number is displayed on the lock screen
     */
    public function lockDevice(Device $device, $message = null, $phoneNumber = null);

    /**
     * Erases the device with the specified id. The device needs to be
     * reinitialized afterwards (full wipe).
     * The device attempts to send a response to the server, but unlike other
     * commands, the response cannot be resent if initial transmission fails.
     * Even if the acknowledgement did not make it to the server (due to network
     * conditions), the device will still be erased.
     *
     * @param \Topalis\Device\Entity\Device $device The device to erase.
     * @see wipeDevice()
     */
    public function eraseDevice(\Topalis\Device\Entity\Device $device);

    /**
     * Creates MDM commands for the initialization.
     *
     * @param \Topalis\Device\Entity\Device $device The device to initialize.
     */
    public function initDevice(\Topalis\Device\Entity\Device $device);

    /**
     * Installs the profile assigned to the device without any changes.
     *
     * @param \Topalis\Device\Entity\Device $device The device to install its
     *     profile.
     */
    public function installDeviceProfile(\Topalis\Device\Entity\Device $device);

    /**
     * Installs device profiles on every device in the specified config group.
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup The config
     *     group for which to install the device profiles.
     * @param string $type device type name or null for all types
     */
    public function installDeviceProfiles(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup, $type = null);

    /**
     * Installs the specified app on the specified device.
     *
     * @param Device                  $device The device to install the app on.
     * @param \Topalis\App\Entity\App $app    The app to install.
     * @param bool                    $force  If true a install request will be sent even though
     *                                        a previous one has been rejected. If false an install
     *                                        request will only be sent if there was no previous
     *                                        request rejected by the user or the device.
     * @param bool $removeBeforeInstall If true a remove command will be send
     *                                  before the install command.
     */
    public function installApp(\Topalis\Device\Entity\Device $device,
        \Topalis\App\Entity\App $app, $force = false, $removeBeforeInstall = false);

    /**
     * Installs the specified app on devices of the config group set in $app.
     *
     * @param \Topalis\App\Entity\App $app   The app toinstall.
     * @param bool                    $force If true a install request will be sent even though
     *                                       a previous one has been rejected. If false an install
     *                                       request will only be sent if there was no previous
     *                                       request rejected by the user or the device.
     */
    public function installApps(\Topalis\App\Entity\App $app, $force = false);

    /**
     * Install all applications for a device
     *
     * @param \Topalis\Device\Entity\Device $device
     */
    public function installAppsForDevice(\Topalis\Device\Entity\Device $device);

    /**
     * Installs a profile for the specified mail account on the device the mail
     * account belongs to.
     *
     * @param \Topalis\Device\Entity\MailAccount $mailAccount The mail account
     *     to install a profile for.
     */
    public function installMailProfile(\Topalis\Device\Entity\MailAccount $mailAccount);

    /**
     * Installs profiles for all mail accounts of the specified device that are
     * not installed, yet, i.e. they have no referenced certificate.
     *
     * @param \Topalis\Device\Entity\Device $device The device to install
     *     profiles for its mail accounts.
     * @param boolean $force If false the profile for each mail account is only
     *     installed if no certificate is related to it.
     */
    public function installMailProfiles(\Topalis\Device\Entity\Device $device, $force = false);

    /**
     * Installs mail profiles on every device in the specified config group.
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup
     */
    public function installMailProfilesForGroup(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup);

    /**
     * Installs the specified profile on the device with the specified id.
     *
     * @param \Topalis\Device\Entity\Device $device The device to install the
     *     profile on.
     * @param \Topalis\Profile\AbstractProfile $profile The profile to install.
     */
    public function installProfile(\Topalis\Device\Entity\Device $device, \Topalis\Profile\AbstractProfile $profile);

    /**
     * Installs the profile with the SecureIntranet CA and SCEP payload on the
     * specified device.
     *
     * @param Device $device The device to install the profile.
     */
    public function installSecureIntranetProfile(\Topalis\Device\Entity\Device $device);

    /**
     * Installs a profile with web clips for the specified \Topalis\Device\Entity\Device.
     *
     * @param \Topalis\Device\Entity\Device $device The device to install the
     *     profile with web clips.
     * @param boolean $remove If true a previous installed device profile will
     *     be removed first. Normally previous installed profiles are replaced
     *     by new ones with the same identifier. However if we don't remove the
     *     old one previous web clips might stay on the home screen.
     */
    public function installWebClipProfile(\Topalis\Device\Entity\Device $device, $remove = true);

    /**
     * Installs web clip profile on every device in the specified config group.
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup The config
     *     group for which to install the web clip profiles.
     */
    public function installWebClipProfiles(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup);

    /**
     * Installs the web clip profile on every device that is associated with the
     * specified url.
     *
     * @param \Topalis\WebClip\Entity\WebClipInterface $webClip
     */
    public function installWebClipProfilesForWebClip(\Topalis\WebClip\Entity\WebClipInterface $webClip);

    /**
     * Installs the profile containing all WiFi configurations associated with
     * the specified device.
     *
     * @param \Topalis\Device\Entity\Device $device The device to install the
     *     WiFi configurations.
     */
    public function installWifiProfile(\Topalis\Device\Entity\Device $device);

    /**
     * Installs WiFi profile on every device in the specified config group.
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup The config
     *     group for which to install the WiFi profiles.
     */
    public function installWifiProfiles(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup);

    /**
     * Invites the specified device to join the Volume Purchasing Program with
     * the specified URL.
     *
     * @param Device $device The device to invite.
     * @param string $invitationURL The invitation URL.
     */
    public function inviteToVolumePurchaseProgram(Device $device, $invitationURL);

    /**
     * Cancels the remaining commands for the device with the specified id.
     *
     * @param \Topalis\Device\Entity\Device $device The device to cancel the
     *     remaining commands.
     */
    public function cancelRemainingCommandsForDevice(\Topalis\Device\Entity\Device $device);

    /**
     * Makes an enterprise wipe by deleting the MDM profile.
     *
     * @param \Topalis\Device\Entity\Device $device The device to wipe.
     * @see eraseDevice()
     */
    public function wipeDevice(\Topalis\Device\Entity\Device $device);

    /**
     * @param Device $device
     *
     * @return \Topalis\App\Mdm\AppMdmAdminInterface
     */
    public function getAppAdmin(Device $device);

    /**
     * @param App $app
     *
     * @return \Topalis\App\Mdm\AppMdmAdminInterface
     */
    public function getAppAdminForApp(App $app);
}
