<?php
namespace Topalis\MDM;

use Topalis\Device\MDM\MdmFacadeInterface;
use Symfony\Component\DependencyInjection\ContainerAware;
use Topalis\Device\Entity\Device;
use Topalis\App\Entity\AppState;
use Topalis\Profile\ConfigurationProfile;

/**
 * MDM facade for Android and iOS devices.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class MdmFacade extends ContainerAware implements MdmFacadeInterface
{
    /**
     * @return \Topalis\MDM\MdmAdmin
     */
    private function getMdmAdmin()
    {
        return $this->container->get("mdm.admin");
    }

    /**
     * @return \Topalis\MDM\Entity\MdmCommandEntityManager
     */
    private function getCommandEntityManager()
    {
        return $this->container->get("mdm.entity.manager.mdm_command");
    }

    public function getCommands(\Topalis\Device\Entity\Device $device, $limit = 0, array $orderBy = null)
    {
        $qb = $this->getCommandEntityManager()->createQueryBuilder("c")
            ->where("c.device = :device")
            ->orderBy("c.id", "DESC")
            ->setParameter("device", $device);
        if ($limit > 0) {
            $qb->setMaxResults($limit);
        }
        foreach ($orderBy as $key => $value) {
            if (is_int($key)) {
                $qb->orderBy("c." . $value);
            } else {
                $qb->orderBy("c." . $key, $value);
            }
        }

        return $qb->getQuery()->getResult();
    }

    public function installMailAccount(\Topalis\Device\Entity\MailAccount $mailAccount)
    {
        $this->getMdmAdmin()->installMailProfile($mailAccount);
    }

    public function requestDeviceInfo(\Topalis\Device\Entity\Device $device)
    {
        $this->getMdmAdmin()->requestDeviceInfo($device);
    }

    public function requestDeviceInfos()
    {
        $this->getMdmAdmin()->requestDeviceInfos();
    }

    public function resendCommand(\Topalis\Device\Entity\Device $device, $commandId)
    {
        $command = $this->getCommandEntityManager()->find($commandId);
        if ($command === null) {
            throw new \InvalidArgumentException("No MDM command with the id " . $commandId . " found");
        }

        $this->getMdmAdmin()->resendCommand($device, $command);
    }

    public function lockDevice(\Topalis\Device\Entity\Device $device, $message = null, $phoneNumber = null)
    {
        $this->getMdmAdmin()->lockDevice($device, $message, $phoneNumber);
    }

    public function eraseDevice(\Topalis\Device\Entity\Device $device)
    {
        $this->getMdmAdmin()->eraseDevice($device);
    }

    public function wipeDevice(\Topalis\Device\Entity\Device $device)
    {
        if ($device->getMdmToken() === null) {
            /*
             * device has no MDM token or MDM push magic so it might have
             * the MDM profile, but nothing more, so it can safely be removed
             * directly.
             */
            $this->getMdmAdmin()->cancelRemainingCommandsForDevice($device);
        } else {
            $this->getMdmAdmin()->wipeDevice($device);
            $device->setWipeStatus(Device::WIPE_PENDING);
        }
    }

    public function updateConfigGroup(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup,
        \Doctrine\ORM\UnitOfWork $uow)
    {
        $changeSet = $uow->getEntityChangeSet($configGroup);
        if (isset($changeSet['profile'])) {
            $this->getMdmAdmin()->installDeviceProfiles($configGroup, "ios");

            // try to install apps again if store was disabled, but is now enabled
            $oldIosProfile = new ConfigurationProfile($changeSet['profile'][0]);
            $newIosProfile = new ConfigurationProfile($changeSet['profile'][1]);
            $oldRestrictions = $oldIosProfile->getRestrictions();
            $newRestrictions = $newIosProfile->getRestrictions();
            $appStoreWasAllowed = $oldRestrictions ? $oldRestrictions[0]->isAllowAppInstallation() : true;
            $appStoreIsAllowed = $newRestrictions ? $newRestrictions[0]->isAllowAppInstallation() : true;
            if ($appStoreIsAllowed && !$appStoreWasAllowed) {
                foreach ($configGroup->getDevices() as $device) {
                    if ($device->getType() == "ios") {
                        $this->getMdmAdmin()->installAppsForDevice($device);
                    }
                }
            }
        }
        if (isset($changeSet['androidProfile'])) {
            $this->getMdmAdmin()->installDeviceProfiles($configGroup, "android");
        }

        // reinstalling the exchange profile if syncDays has been changed is not
        // required.
        if (isset($changeSet['allowMove']) || isset($changeSet['fromAnyApp'])) {
            $this->getMdmAdmin()->installMailProfilesForGroup($configGroup);
        }
    }

    public function updateDevice(\Topalis\Device\Entity\Device $device, \Doctrine\ORM\UnitOfWork $uow)
    {
        $changeSet = $uow->getEntityChangeSet($device);
        if (isset($changeSet['configGroup'])) {
            /** @var \Topalis\ConfigGroup\Entity\ConfigGroup $oldConfigGroup */
            $oldConfigGroup = $changeSet['configGroup'][0];
            /** @var \Topalis\ConfigGroup\Entity\ConfigGroup $newConfigGroup */
            $newConfigGroup = $changeSet['configGroup'][1];
            if ($device->getType()->getName() == "ios") {
                if ($oldConfigGroup->getProfile() != $newConfigGroup->getProfile()) {
                    $this->getMdmAdmin()->installDeviceProfile($device);
                }
            } else if ($device->getType()->getName() == "android") {
                if ($oldConfigGroup->getAndroidProfile() != $newConfigGroup->getAndroidProfile()) {
                    $this->getMdmAdmin()->installDeviceProfile($device);
                }
            }

            // reinstalling the exchange profile if syncDays has been changed is
            // not required.
            if ($oldConfigGroup->isAllowMove() != $newConfigGroup->isAllowMove()
                || $oldConfigGroup->isFromAnyApp() != $newConfigGroup->isFromAnyApp()) {
                $this->getMdmAdmin()->installMailProfiles($device, true);
            }

            if ($oldConfigGroup->getReverseProxyUrls()->toArray()
                != $newConfigGroup->getReverseProxyUrls()->toArray()) {
                $this->getMdmAdmin()->installWebClipProfile($device);
            }

            if ($oldConfigGroup->getWifiConfigs()->toArray()
                != $newConfigGroup->getWifiConfigs()->toArray()) {
                $this->getMdmAdmin()->installWifiProfile($device);
            }
            
            $this->getMdmAdmin()->installAppsForDevice($device);
            $this->getMdmAdmin()->removeUnassignedAppsFromDevice($device);
        }

        if (isset($changeSet['version'])) {
            $appMdmAdmin = $this->getMdmAdmin()->getAppAdmin($device);
            foreach ($appMdmAdmin->getAppStates($device,
                array(AppState::UNSUPPORTED, AppState::UPDATE_UNSUPPORTED)) as $state) {
                // try to install again, maybe the app is supported now
                $appMdmAdmin->installApp($device, $state->getApp());
            }
        }
    }

    public function removeMailAccount(\Topalis\Device\Entity\MailAccount $mailAccount)
    {
        $this->getMdmAdmin()->removeMailProfile($mailAccount);
    }

    public function updateWebClip(
        \Topalis\WebClip\Entity\WebClipInterface $webClip,
        array $removedGroups,
        array $addedGroups,
        \Doctrine\ORM\UnitOfWork $uow
    )
    {
        foreach ($removedGroups as $group) {
            $this->getMdmAdmin()->installWebClipProfiles($group);
        }
        if ($this->isWebClipProfileInstallRequired($webClip, $uow)) {
            $this->getMdmAdmin()->installWebClipProfilesForWebClip($webClip);

            return;
        }
        if ($webClip->isVisible()) {
            foreach ($addedGroups as $group) {
                $this->getMdmAdmin()->installWebClipProfiles($group);
            }
        }
    }

    public function addWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig)
    {
        $mdmAdmin = $this->getMdmAdmin();
        foreach ($wifiConfig->getConfigGroups() as $configGroup) {
            $mdmAdmin->installWifiProfiles($configGroup);
        }
    }

    public function updateWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig, array $removedGroups,
        array $addedGroups, array $changeSet)
    {
        $addedGroups; // unused
        $changeSet; // unused

        $mdmAdmin = $this->getMdmAdmin();
        foreach ($removedGroups as $configGroup) {
            $mdmAdmin->installWifiProfiles($configGroup);
        }
        foreach ($wifiConfig->getConfigGroups() as $configGroup) {
            $mdmAdmin->installWifiProfiles($configGroup);
        }
    }

    public function removeWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig)
    {
        $mdmAdmin = $this->getMdmAdmin();
        foreach ($wifiConfig->getConfigGroups() as $configGroup) {
            $mdmAdmin->installWifiProfiles($configGroup);
        }
    }

    private function isWebClipProfileInstallRequired(
        \Topalis\WebClip\Entity\AbstractWebClip $webClip,
        \Doctrine\ORM\UnitOfWork $uow
    )
    {
        // install web clips profile if visible status changed,
        // or if it's visible and some other property that effects the profile
        // has changed.
        $changeSet = $uow->getEntityChangeSet($webClip);
        if (isset($changeSet['visible'])) {
            return true;
        }

        if ($webClip->isVisible()) {
            $requiresProfileInstall = array(
                "externalPath", "icon", "name", "precomposed", "fullscreen", "removable", "url",
            );
            foreach ($requiresProfileInstall as $key) {
                if (isset($changeSet[$key])) {
                    return true;
                }
            }
        }

        return false;
    }

    public function addWebClip(\Topalis\WebClip\Entity\WebClipInterface $webClip)
    {
        if ($webClip->isVisible()) {
            $this->getMdmAdmin()->installWebClipProfilesForWebClip($webClip);
        }
    }

    public function removeWebClip(\Topalis\WebClip\Entity\WebClipInterface $webClip)
    {
        $this->getMdmAdmin()->installWebClipProfilesForWebClip($webClip);
    }
}
