<?php
namespace Topalis\MDM;

use Topalis\Kernel\AbstractAuralisService;
use Topalis\Util\ArrayUtil;
use CFPropertyList\CFPropertyList;
use Symfony\Component\HttpFoundation\Response;
use Topalis\Device\Entity\Device;
use Symfony\Component\HttpFoundation\Request;
use Topalis\MDM\Entity\MdmCommand;

/**
 * The MdmService handles MDM requests from devices.
 * Note: the filter for deleted devices is disabled for the whole request
 * processing.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class MdmService extends AbstractAuralisService
{
    /**
     * @return \Topalis\Mail\Mailer
     */
    private function getMailer()
    {
        return $this->container->get("mailer");
    }

    /**
     * @return \Topalis\MDM\MdmQueue
     */
    private function getMdmQueue()
    {
        return $this->container->get("mdm.queue");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\Certificate\Entity\ClientCertificateEntityManager
     */
    private function getClientCertificateEntityManager()
    {
        return $this->container->get("certificate.entity.manager.certificate");
    }

    /**
     * @return \Topalis\MDM\Entity\MdmCommandEntityManager
     */
    private function getMdmCommandEntityManager()
    {
        return $this->container->get("mdm.entity.manager.mdm_command");
    }

    /**
     * @return \Symfony\Component\EventDispatcher\EventDispatcherInterface
     */
    private function getEventDispatcher()
    {
        return $this->container->get("event_dispatcher");
    }

    /**
     * Handles the specified request and returns a fitting response.
     *
     * @param Request $request The request to handle.
     * @return \Symfony\Component\HttpFoundation\Response The response.
     */
    public function handleRequest(Request $request)
    {
        try {
            $this->getDeviceEntityManager()->disableDeleteFilter();
            $context = $this->createMdmContext($request);

            if (!$context->hasDevice()) {
                $mdmRequest = $context->getMdmRequest();
                $this->getLogger()->warn("MdmService: unknown device (UDID: '" . $mdmRequest->getUdid()
                    . "', certificate serial: " . $context->getCertSerial() . "); return HTTP 401.");
                return new Response(Response::$statusTexts[401], 401);
            }

            if (strlen($request->getContent()) == 0) {
                $device = $context->getDevice();
                $this->getLogger()->warn("MdmService: empty request from device " . $device->getId()
                    . " (certificate serial: "
                    . $context->getCertSerial() . "); send additional push message");
                // try to send an additional push message
                $this->getMdmQueue()->setWaiting($device, false);
                return new Response(Response::$statusTexts[400], 400);
            }

            return $this->handleMdmRequest($context);
        } catch (\Exception $e) {
            $this->getLogger()->error("MdmService: error handling request", array("exception" => $e));
            return new Response(Response::$statusTexts[500], 500);
        }
    }

    /**
     * Handles the specified MdmRequest and returns a fitting response.
     *
     * @param MdmServiceContext $context The context containing the request to
     *     handle.
     * @return \Symfony\Component\HttpFoundation\Response The response.
     */
    public function handleMdmRequest(MdmServiceContext $context)
    {
        $mdmRequest = $context->getMdmRequest();
        $device = $context->getDevice();
        $this->getLogger()->debug("MdmService: handle request from device '" . $device->getId()
            . "' (certificate serial: " . $context->getCertSerial() . ")");

        $wipeRequired = false;
        if (($device->isDisabled() || $device->isDeleted()
            || $device->getWipeStatus() != Device::WIPE_NONE)
            && !$this->getMdmQueue()->hasCommand($device)) {
            $wipeRequired = true;
        }
        $wipePending = $device->getWipeStatus() == Device::WIPE_PENDING;

        try {
            $mdmRequest = $context->getMdmRequest();
            if (!$this->isRequestValid($context)) {
                $this->getLogger()->warn("MdmService: invalid request from device '" . $device->getId() . "'");
                $response = new Response(Response::$statusTexts[400], 400);
            } else if (!$wipeRequired || $wipePending) {
                // if wipe is pending we handle one last device request because it
                // could be the OK to the profile removal
                if ($mdmRequest->isMessage()) {
                    $response = $this->handleMdmMessage($context);
                } else {
                    $response = $this->handleMdmResult($context);
                }
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("Error handling request of device '" . $device->getId() . "'",
                array("exception" => $e));
            // ensure that we wipe the device even if an error occurred
            if ($wipePending) {
                $wipeRequired = true;
            }
            if (!$wipeRequired) {
                $response = new Response("", 500);
            }
        }

        if ($wipeRequired) {
            $wipeExecuted = $device->getWipeStatus() == Device::WIPE_EXECUTED;
            if (!$wipePending || !$wipeExecuted) {
                // device should already be wiped but isn't (!$wipePending)
                // the wipe was not executed through some MDM command (!$wipeExecuted)
                $this->getLogger()->warn(
                    "MdmService: device '" . $mdmRequest->getUdid()
                    . "' is disabled, deleted, but not wiped, or the wipe status is set to pending; return HTTP 401."
                );
                $this->removeDeviceFromMdm($device);
                $response = new Response("", 401);
            }
        }

        return $response;
    }

    /**
     *
     * @param MdmServiceContext $context
     * @return \Symfony\Component\HttpFoundation\Response
     */
    protected function handleMdmMessage(MdmServiceContext $context)
    {
        $mdmRequest = $context->getMdmRequest();
        // known message types Authenticate, TokenUpdate, CheckOut
        $methodName = lcfirst($mdmRequest->getMessageType());
        if (!method_exists($this, $methodName)) {
            $logMsg = "MdmService: unknown MessageType received: " . $mdmRequest->getMessageType()
                . " (UDID: " . $mdmRequest->getUdid() . ")";
            $this->getLogger()->warn($logMsg);
            return new Response("Unknown message type", 400);
        }

        $msg = "MdmService: " . $mdmRequest->getMessageType() . " (UDID: " . $mdmRequest->getUdid() . ")";
        $this->getLogger()->debug($msg);
        return call_user_func(array($this, $methodName), $context);
    }

    /**
     *
     * @param MdmServiceContext $context
     * @return \Symfony\Component\HttpFoundation\Response
     */
    protected function authenticate(MdmServiceContext $context)
    {
        $context; // unused
        // already authenticated through certificate and valid UDID.
        return new Response();
    }

    /**
     *
     * @param MdmServiceContext $context
     * @return \Symfony\Component\HttpFoundation\Response
     */
    protected function tokenUpdate(MdmServiceContext $context)
    {
        $mdmRequest = $context->getMdmRequest();
        $device = $context->getDevice();
        try {
            $rePush = false;
            $token = bin2hex($mdmRequest->get("Token"));
            $pushMagic = $mdmRequest->get("PushMagic");
            if ($device->getMdmToken() != $token || $device->getMdmPushMagic() != $pushMagic) {
                $device->setMdmToken($mdmRequest->get("Token"));
                $device->setMdmPushMagic($pushMagic);
                $rePush = true;
            }
            $unlockToken = $mdmRequest->get("UnlockToken");
            if ($unlockToken !== null) {
                $device->setMdmUnlockToken($unlockToken);
            }
            $this->getDeviceEntityManager()->save($device);
            if ($rePush) {
                // ensure that we push again on token change or push magic change
                $this->getMdmQueue()->setWaiting($device, false);
            }
        } catch (\Exception $e) {
            $msg = "MdmService: error updating MDM tokens for device with id " . $device->getId();
            $this->getLogger()->error($msg, array("exception" => $e));
            return new Response("", 500);
        }

        return new Response();
    }

    /**
     *
     * @param MdmServiceContext $context
     * @return \Symfony\Component\HttpFoundation\Response
     */
    protected function checkOut(MdmServiceContext $context)
    {
        $device = $context->getDevice();
        $this->getLogger()->info("Device with id " . $device->getId() . " checks out");
        try {
            $subject = $this->tr("admin.mdm.checkoutSubject");
            // user might be deleted
            $user = $device->getUser() ? $device->getUser()->getLogin() : null;
            $deviceName = $device->getName();
            if (strlen($device->getComment() > 0)) {
                $deviceName .= " (" . $device->getComment() . ")";
            }
            if ($user) {
                $text = $this->tr("admin.mdm.checkoutText.user", array(
                    "%user%" => $user,
                    "%device%" => $deviceName
                ));
            } else {
                $text = $this->tr("admin.mdm.checkoutText.woUser", array(
                    "%device%" => $deviceName
                ));
            }
            $mailer = $this->getMailer();
            if ($mailer->isConfigured()) {
                $mailer->sendAdminMessage($subject, $text);
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("MdmService: sending check out message failed", array("exception" => $e));
        }
        try {
            $device->setWipeStatus(Device::WIPE_EXECUTED);
            $this->removeDeviceFromMdm($device);
        } catch (\Exception $e) {
            $msg = "MdmService: error resetting MDM tokens for device with id " . $device->getId();
            $this->getLogger()->error($msg, array("exception" => $e));
        }
        return new Response();
    }

    protected function handleMdmResult(MdmServiceContext $context)
    {
        $mdmRequest = $context->getMdmRequest();
        $device = $context->getDevice();
        $mdmCommand = $context->getMdmCommand();

        $logMsg = "MdmService: " . $mdmRequest->getStatus() . " (UDID: " . $mdmRequest->getUdid()
            . ", CommandUUID: " . $mdmRequest->getCommandUuid() . ")";
        $this->getLogger()->debug($logMsg);

        switch ($mdmRequest->getStatus()) {
            case MdmCommand::STATUS_ACKNOWLEDGED:
                // handle the response. For now just dump it into SQL
                $mdmCommand->setStatus(MdmCommand::STATUS_ACKNOWLEDGED);
                $mdmCommand->setResponse($mdmRequest->getPayload());
                $mdmCommand->setResponseTime($this->getTimeProvider()->getTime());
                $this->getMdmCommandEntityManager()->save($mdmCommand);
                break;
            case MdmCommand::STATUS_IDLE:
                $device->setBlocked(false);
                $this->getDeviceEntityManager()->save($device);
                break;
            case MdmCommand::STATUS_NOT_NOW:
                $device->setBlocked(true);
                $this->getDeviceEntityManager()->save($device);
                break;
            case MdmCommand::STATUS_COMMAND_FORMAT_ERROR:
                /* fall through */
            case MdmCommand::STATUS_ERROR:
                /* fall through */
            default:
                $errorChain = $mdmRequest->getErrorChain();
                if ($errorChain === null) {
                    $errorChain = "error not defined";
                } else {
                    // @codingStandardsIgnoreStart
                    $errorChain = print_r($mdmRequest->getErrorChain(), true);
                    // @codingStandardsIgnoreEnd
                }
                $msg = "Error in MDM request (Status: " . $mdmRequest->getStatus() . "):" . $errorChain;
                $this->getLogger()->error($msg);
                $mdmCommand->setStatus($mdmRequest->getStatus());
                $mdmCommand->setResponse($mdmRequest->getPayload());
                $mdmCommand->setResponseTime($this->getTimeProvider()->getTime());
                $this->getMdmCommandEntityManager()->save($mdmCommand);
                $this->handleMdmResultError($context);
                break;
        }

        if ($mdmCommand !== null && $mdmRequest->getStatus() != MdmCommand::STATUS_NOT_NOW) {
            $this->getMdmQueue()->removeCommand($mdmCommand);
        }

        /*
         * Catch errors during handling of mdm result. Returning HTTP status 500
         * makes the iOS device sending the request again which could lead to an
         * infinite loop if the request handling fails every time. The entity
         * manager might be closed in case of an error.
         */
        try {
            switch ($mdmRequest->getStatus()) {
                case MdmCommand::STATUS_ACKNOWLEDGED:
                    $methodName = "ack" . $mdmCommand->getRequestType();
                    if (method_exists($this, $methodName)) {
                        call_user_func(array($this, $methodName), $context);
                    }
                    break;
                case MdmCommand::STATUS_IDLE:
                    // nothing to do
                    break;
                case MdmCommand::STATUS_NOT_NOW:
                    // nothing to do
                    break;
                case MdmCommand::STATUS_COMMAND_FORMAT_ERROR:
                    /* fall through */
                case MdmCommand::STATUS_ERROR:
                    /* fall through */
                default:
                    $methodName = "err" . $mdmCommand->getRequestType();
                    if (method_exists($this, $methodName)) {
                        call_user_func(array($this, $methodName), $context);
                    }
                    break;
            }
            if ($mdmCommand !== null) {
                $this->getEventDispatcher()->dispatch("mdm." . $mdmCommand->getRequestType(), new MdmEvent($context));
            }
        } catch (\Exception $e) {
            $commandText = $mdmCommand->getRequestType();
            if ($mdmCommand->getInfo()) {
                $commandText .= " (" . $mdmCommand->getInfo() . ")";
            }
            $this->getLogger()->error("Failed to handle MDM result for command " .
                $commandText . ":" . $e->getMessage(),
                array(
                    "command"   => $mdmCommand->getId(),
                    "exception" => $e,
                )
            );
            try {
                $mailer = $this->getMailer();
                if ($mailer->isConfigured()) {
                    // user might be deleted
                    $user = $device->getUser() ? $device->getUser()->getLogin() : null;
                    $deviceName = $device->getName();
                    if (strlen($device->getComment()) > 0) {
                        $deviceName .= " (" . $device->getComment() . ")";
                    }
                    $subject = $this->tr("admin.mdm.resultFailedSubject");
                    $text = $this->tr("admin.mdm.resultFailedText", array(
                        "%command%"   => $commandText,
                        "%status%"    => $mdmRequest->getStatus(),
                        "%user%"      => $user,
                        "%device%"    => $deviceName,
                        "%exception%" => $e->getMessage(),
                    ));
                    $mailer->sendAdminMessage($subject, $text);
                }
            } catch (\Exception $e) {
                $this->getLogger()->error("sending message failed", array("exception" => $e));
            }

            return new Response();
        }

        if ($mdmCommand !== null && !$mdmCommand->isBlockable()) {
            // for some reason returning the next command here does not work...
            // at least not for "DeviceInformation" to make sure we won't get
            // into trouble just do this for all non blockable commands
            $this->getMdmQueue()->setWaiting($device, false);
            return new Response();
        }

        return $this->getNextCommandResponse($context);
    }

    protected function handleMdmResultError(MdmServiceContext $context)
    {
        try {
            // send error mail
            $mdmRequest = $context->getMdmRequest();
            $device = $context->getDevice();
            $mdmCommand = $context->getMdmCommand();

            $commandText = $mdmCommand->getRequestType();
            if ($mdmCommand->getInfo()) {
                $commandText .= " (" . $mdmCommand->getInfo() . ")";
            }
            $errorChain = $mdmRequest->getErrorChain();
            $subject = $this->tr("admin.mdm.commandFailedSubject");
            // user might be deleted
            $user = $device->getUser() ? $device->getUser()->getLogin() : null;
            $deviceName = $device->getName();
            if (strlen($device->getComment()) > 0) {
                $deviceName .= " (" . $device->getComment() . ")";
            }

            $errorChainTexts = array();
            $count = count($errorChain);
            for ($i = 0; $i < $count; $i++) {
                $error = $errorChain[$i];
                $errorDesc = isset($error['LocalizedDescription']) ? $error['LocalizedDescription'] : "unknown";
                $errorDomain = isset($error['ErrorDomain']) ? $error['ErrorDomain'] : "unknown";
                $errorCode = isset($error['ErrorCode']) ? $error['ErrorCode'] : "unknown";
                $errorChainTexts[] = $this->tr("admin.mdm.commandError", array(
                    "%i%" => $i + 1,
                    "%desc%" => $errorDesc,
                    "%domain%" => $errorDomain,
                    "%code%" => $errorCode
                ));
            }
            $text = $this->tr("admin.mdm.commandFailedText", array(
                "%command%" => $commandText,
                "%status%" => $mdmRequest->getStatus(),
                "%user%" => $user,
                "%device%" => $deviceName,
                "%errorChain%" => implode("\n", $errorChainTexts)
            ));
            $mailer = $this->getMailer();
            if ($mailer->isConfigured()) {
                $mailer->sendAdminMessage($subject, $text);
            } else {
                $this->getLogger()->info("MdmServce: $text");
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("MdmService: sending MDM result error message failed", array("exception" => $e));
            if (isset($text)) {
                $this->getLogger()->info("MdmServce: $text");
            }
        }
    }

    protected function ackDeviceInformation(MdmServiceContext $context)
    {
        $mdmRequest = $context->getMdmRequest();
        $device = $context->getDevice();

        $this->getLogger()->debug("MdmService: Update device information of device " . $device->getId());
        $queryResponses = $mdmRequest->get("QueryResponses");
        $deviceName = ArrayUtil::get($queryResponses, "DeviceName");
        if ($deviceName !== null) {
            $device->setName($deviceName);
        }
        $osVersion = ArrayUtil::get($queryResponses, "OSVersion");
        if ($osVersion !== null) {
            // overwrite initial version with right version
            $device->setVersion($osVersion);
        }

        $device->setDeviceInformation($queryResponses);
        $device->setLastDeviceInformationUpdate($this->getTimeProvider()->getTime());
        $this->getDeviceEntityManager()->save($device);
    }

    protected function ackEraseDevice(MdmServiceContext $context)
    {
        $device = $context->getDevice();
        $this->getLogger()->info("Erase of device " . $device->getId() . " successful");
        $device->setWipeStatus(Device::WIPE_EXECUTED);
        $this->removeDeviceFromMdm($device);
    }

    protected function errEraseDevice(MdmServiceContext $context)
    {
        $device = $context->getDevice();
        $this->getLogger()->error("MdmService: Erase of device " . $device->getId() . " failed");
        $device->setWipeStatus(Device::WIPE_ERROR);
        $this->removeDeviceFromMdm($device);
    }

    /**
     * @param MdmServiceContext $context
     * @return \Symfony\Component\HttpFoundation\Response
     */
    protected function getNextCommandResponse(MdmServiceContext $context)
    {
        $device = $context->getDevice();
        $mdmCommand = $this->getMdmQueue()->getNextCommand($device);
        if ($mdmCommand === null) {
            return new Response();
        }

        $plist = $this->mdmCommandToPlist($mdmCommand);
        $response = new Response($plist->toXML());
        $response->headers->set("Content-Type", "application/xml");
        $response->setCharset("UTF-8");
        return $response;
    }

    /**
     * @param \Symfony\Component\HttpFoundation\Request $request
     * @return \Topalis\MDM\MdmServiceContext
     */
    protected function createMdmContext(Request $request)
    {
        $mdmRequest = new MdmRequest($request->getContent());
        $context = new MdmServiceContext($mdmRequest);
        $certSerial = hexdec($request->server->get("SSL_CLIENT_M_SERIAL"));
        $context->setCertSerial($certSerial);
        if (!$mdmRequest->isMessage() && $mdmRequest->getCommandUuid()) {
            // load last command for resent commands (same UUID) by ordering
            // by id descending
            /** @var $mdmCommand MdmCommand */
            $mdmCommand = $this->getMdmCommandEntityManager()->findOneBy(
                array("uuid" => $mdmRequest->getCommandUuid()), array("id" => "DESC"));
            if ($mdmCommand) {
                $context->setMdmCommand($mdmCommand);
                $context->setDevice($mdmCommand->getDevice());
                return $context;
            }
        }

        $cert = $this->getClientCertificateEntityManager()->find($certSerial);
        $device = $cert ? $cert->getDevice() : null;
        if ($device !== null && ($device->getDeviceid() == $mdmRequest->getUdid() || $mdmRequest->isEmpty())) {
            $context->setDevice($device);
        }
        return $context;
    }

    /**
     * @param MdmServiceContext $context
     * @return boolean
     */
    protected function isRequestValid(MdmServiceContext $context)
    {
        $mdmRequest = $context->getMdmRequest();
        if ($mdmRequest->isMessage() || $mdmRequest->getStatus() == MdmCommand::STATUS_IDLE) {
            return $context->hasDevice();
        }
        return $context->hasCommand() && $context->hasDevice();
    }

    /**
     * @param MdmCommand $mdmCommand
     * @return \CFPropertyList\CFPropertyList
     */
    protected function mdmCommandToPlist(MdmCommand $mdmCommand)
    {
        $commandRequest = $mdmCommand->getRequest();
        $plist = new CFPropertyList();
        $plist->parse($commandRequest, CFPropertyList::FORMAT_XML);
        return $plist;
    }

    protected function cancelRemainingCommands(Device $device)
    {
        $this->getMdmQueue()->cancelCommandsForDevice($device);
    }

    protected function removeDeviceFromMdm(Device $device)
    {
        $device->setDisabled(true);
        $device->setMdmPushMagic(null);
        $device->setMdmToken(null);
        $device->setMdmUnlockToken(null);
        $this->getDeviceEntityManager()->save($device);
        $this->cancelRemainingCommands($device);
        $this->setDeviceCertsUnused($device);
    }

    protected function setDeviceCertsUnused(Device $device)
    {
        $this->getCertificateManager()->setDeviceCertsUnused($device);
    }
}
