<?php
namespace Topalis\Maintenance\Controller;

use Symfony\Component\HttpFoundation\Response;
use Topalis\Util\Process;
use Topalis\Kernel\Controller;
use Topalis\Maintenance\Form\MaintenanceForm;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Topalis\Admin\Entity\Admin;

/**
 * Controller for maintenance. It can be used to activate the maintenance tunnel,
 * or to reboot or shutdown the system.
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/admin/support", service="maintenance.controller")
 */
class MaintenanceController extends Controller
{
    private $rebootCommand = "/usr/sbin/aurreboot";

    private $shutdownCommand = "/usr/sbin/aurhalt";

    /**
     * @return \Topalis\Admin\AdminManager
     */
    private function getAdminManager()
    {
        return $this->container->get("admin.manager");
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\Maintenance\Tunnel
     */
    private function getMaintenanceTunnel()
    {
        return $this->container->get("maintenance.tunnel");
    }

    /**
     * @Route("/disable", name="admin_maintenance_disable", methods="GET")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function disableAction()
    {
        try {
            $this->getAdminManager()->deleteAdminByName("auradmin");
            if ($this->getMaintenanceTunnel()->isSupportedSystem()) {
                $this->getMaintenanceTunnel()->disable();
            }
            $this->addFlash("success", $this->tr("admin.maintenance.disableSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error disabling maintenance tunnel", array("exception" => $e));
            $msg = $this->tr("admin.maintenance.error.disable", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }

        return $this->redirect($this->generateUrl("admin_maintenance"));
    }

    /**
     * @Route("/enable", name="admin_maintenance_enable", methods="POST")
     * @Template("TopalisMaintenanceBundle:Maintenance:index.html.twig")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function enableAction()
    {
        $request = $this->getRequest();
        $form = $this->createMaintenanceForm();
        $form->bind($request);

        if (!$form->isValid()) {
            return $this->indexAction();
        }

        try {
            $password = $form->get("password")->getData();
            if ($this->getMaintenanceTunnel()->isSupportedSystem()) {
                $this->getMaintenanceTunnel()->enable($password);
            }
            $admin = $this->getAdminManager()->getAdminByName("auradmin");
            if ($admin === null) {
                $admin = new Admin();
                $admin->setLogin("auradmin");
            }
            $admin->setLevel(Admin::LEVEL_ADMIN);
            $admin->setPlainPassword($password);
            $admin->setRealname("Maintenance user");
            $this->getAdminManager()->saveAdmin($admin);
            $this->addFlash("success", $this->tr("admin.maintenance.enableSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error enabling maintenance tunnel", array("exception" => $e));
            $msg = $this->tr("admin.maintenance.error.enable", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }
        return $this->redirect($this->generateUrl("admin_maintenance"));
    }

    /**
     * @Route("", name="admin_maintenance", methods="GET")
     * @Template()
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function indexAction()
    {
        $this->getSession()->set("permitReboot", true);
        $form = $this->createMaintenanceForm();

        try {
            $tunnelState = $this->getMaintenanceTunnel()->status();
        } catch (\Exception $e) {
            $this->getLogger()->error("Error fetching maintenance status", array("exception" => $e));
            $msg = $this->tr("admin.maintenance.error.status", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
            $tunnelState = null;
        }

        // indicate whether this is an appliance installation, i.e. it is possible
        // to reboot and shutdown the machine through the admin GUI
        $appliance = Process::commandExists($this->rebootCommand) && Process::commandExists($this->shutdownCommand);
        return array(
            "appliance" => $appliance,
            "tunnelState" => $tunnelState,
            "tunnelSupported" => $this->getMaintenanceTunnel()->isSupportedSystem(),
            "form" => $form->createView(),
        );
    }

    /**
     * @Route("/reboot", name="admin_maintenance_reboot", methods="GET")
     * @Template()
     *
     * @return array|\Symfony\Component\HttpFoundation\Response
     */
    public function rebootAction()
    {
        if ($this->getSession()->get("permitReboot")) {
            $this->getSession()->remove("permitReboot");
            return array();
        }

        return $this->redirect($this->generateUrl("admin_maintenance"));
    }

    /**
     * @Route("/reboot/perform", name="admin_maintenance_reboot_perform", methods="GET", options={"expose" = true})
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function performRebootAction()
    {
        register_shutdown_function(array($this, "reboot"));
        return new Response();
    }

    /**
     * @Route("/shutdown", name="admin_maintenance_shutdown", methods="GET")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function shutdownAction()
    {
        register_shutdown_function(array($this, "shutdown"));
        return new Response($this->tr("admin.maintenance.shuttingdown"));
    }

    public function shutdown()
    {
        try {
            Process::execThrow($this->shutdownCommand);
        } catch (\Exception $e) {
            $this->getLogger()->error("Error initiating shutdown", array("exception" => $e));
            $msg = $this->tr("admin.maintenance.error.shutdown", array("%errmsg%" => $e->getMessage()));
            echo "<br/>" . $msg;
        }
    }

    public function reboot()
    {
        try {
            Process::execThrow($this->rebootCommand);
        } catch (\Exception $e) {
            $this->getLogger()->error("Error initiating reboot", array("exception" => $e));
            $msg = $this->tr("admin.maintenance.error.reboot", array("%errmsg%" => $e->getMessage()));
            echo $msg;
        }
    }

    /**
     * @return \Symfony\Component\Form\FormInterface
     */
    private function createMaintenanceForm()
    {
        static $form = null;
        if ($form === null) {
            $form = $this->createForm(new MaintenanceForm());
        }
        return $form;
    }
}
