<?php
namespace Topalis\Maintenance;

use Topalis\Util\Process;
use Topalis\Configuration\AuralisConfig;
use Topalis\Configuration\AuralisSysConfig;
use Topalis\Apache\Apache;

/**
 * Helper class for managing the maintenance tunnel.
 *
 * @author Andreas Hügel <andreas.huegel@topalis.com>
 * @author Frederick Roth <frederick.roth@topalis.com>
 * @author Fritz Elfert <fritz.elfert@topalis.com>
 * @author Holger Widmann <holger.widmann@topalis.com>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class Tunnel
{
    /**
     * @var \Topalis\Apache\Apache
     */
    private $apache;

    /**
     * @var \Topalis\Configuration\AuralisConfig
     */
    private $config;

    private $helperCommand = "/usr/sbin/auradmin";

    /**
     * @var \Topalis\Configuration\AuralisSysConfig
     */
    private $sysConfig;

    public function __construct(AuralisSysConfig $sysConfig, AuralisConfig $config, Apache $apache)
    {
        $this->sysConfig = $sysConfig;
        $this->config = $config;
        $this->apache = $apache;
    }
    /**
     * Disables the maintenance tunnel.
     *
     * @throws \Topalis\Util\Exception\Process\ExecException If executing auradmin fails.
     */
    public function disable()
    {
        // reset Apache "Allow from" directives
        $this->config->setAllowFromAdditional(null);
        $cmd = $this->helperCommand . " disable";
        Process::execThrow($cmd);
    }
    /**
     * Enables the maintenance tunnel.
     *
     * @param string $pass The password to be set for the auradmin user.
     * @throws \Topalis\Util\Exception\Process\ExecException If executing auradmin fails.
     */
    public function enable($pass)
    {
        $out = "";
        $cmd = $this->helperCommand . " enable --port " . $this->apache->getDefine("auralis_primary_port");
        $sources = $this->sysConfig->getMaintenanceSources();
        if ($sources) {
            // add Apache "Allow from" directive for maintenance sources
            $this->config->setAllowFromAdditional($sources);
            $cmd .= " --sources " . $sources;
        }
        Process::execThrow($cmd, $out, $pass);
    }

    /**
     * @return boolean true if the maintenance tunnel is supported, false
     *     otherwise. It checks if the required command exists.
     */
    public function isSupportedSystem()
    {
        return Process::commandExists($this->helperCommand);
    }

    /**
     * Return the current status of the maintenance tunnel.
     *
     * @return boolean true if enabled, false otherwise.
     * @throws \Topalis\Util\Exception\Process\ExecException If executing auradmin fails.
     */
    public function status()
    {
        if (!$this->isSupportedSystem()) {
            return false;
        }

        $out = "";
        $cmd = $this->helperCommand . " status";
        Process::execThrow($cmd, $out);
        return trim($out) == "ENABLED";
    }

    /**
     * Sets the command used to control the maintenance tunnel.
     *
     * @param string $helperCommand The command to set.
     */
    public function setHelperCommand($helperCommand)
    {
        $this->helperCommand = $helperCommand;
    }
}
