<?php
namespace Topalis\ReverseProxy\Controller;

use Symfony\Component\HttpFoundation\Request;
use Topalis\Kernel\Exception\Http\NotFoundException;
use Topalis\Kernel\Controller;
use Topalis\ReverseProxy\Entity\ReverseProxyUrl;
use Topalis\ReverseProxy\Form\ReverseProxyUrlForm;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;

/**
 * Controller for reverse proxy
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/admin/secure-intranet", service="reverse_proxy.controller")
 */
class ReverseProxyController extends Controller
{
    /**
     * @var string
     */
    private $uploadDir;

    /**
     * @param mixed $uploadDir
     *
     * @return self
     */
    public function setUploadDir($uploadDir)
    {
        $this->uploadDir = $uploadDir;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getUploadDir()
    {
        return $this->uploadDir;
    }

    /**
     * @return \Topalis\ReverseProxy\Entity\ReverseProxyUrlEntityManager
     */
    private function getReverseProxyUrlEntityManager()
    {
        return $this->container->get("reverse_proxy.entity.manager.url");
    }

    /**
     * @Route("", name="admin_reverse_proxy")
     * @Template()
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function indexAction()
    {
        $urls = $this->getReverseProxyUrlEntityManager()->findAll();

        return array(
            "urls" => $urls,
            "hostname" => $this->getConfig()->getHostname(),
            "allowed" => $this->getLicense()->isReverseProxyAllowed()
        );
    }

    /**
     * @Route("/add", name="admin_reverse_proxy_add")
     * @Template()
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
     */
    public function addAction()
    {
        $url = new ReverseProxyUrl();

        return $this->forward("reverse_proxy.controller:editAction", array("url" => $url));
    }

    /**
     * @Route("/{id}/edit", name="admin_reverse_proxy_edit")
     * @Template
     *
     * @param Request         $request
     * @param ReverseProxyUrl $url
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
     */
    public function editAction(Request $request, ReverseProxyUrl $url)
    {
        $formBuilder = $this->getFormFactory()->createBuilder(
            new ReverseProxyUrlForm($this->getConfig()->getHostname()),
            $url
        );

        if (!$this->getLicense()->isReverseProxyAllowed()) {
            $formBuilder->setDisabled(true);
        }
        $form = $formBuilder->getForm();
        try {
            $form->handleRequest($request);

            if ($form->isValid() && $this->getLicense()->isReverseProxyAllowed()) {
                $this->getReverseProxyUrlEntityManager()->save($url);
                $this->addFlash("success", $this->tr("admin.reverseProxy.saveSuccess"));

                return $this->redirect($this->generateUrl("admin_reverse_proxy"));
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("Error saving reverse proxy URL", array("exception" => $e));
            $msg = $this->tr("admin.reverseProxy.error.save", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
            return $this->redirect($this->generateUrl("admin_reverse_proxy"));
        }

        return array(
            "form" => $form->createView(),
            "url" => $url,
            "allowed" => $this->getLicense()->isReverseProxyAllowed()
        );
    }

    /**
     * @Route("/{id}/delete", name="admin_reverse_proxy_delete")
     *
     * @param ReverseProxyUrl $url
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function deleteAction(ReverseProxyUrl $url)
    {
        if (!$this->getLicense()->isReverseProxyAllowed()) {
            return $this->redirect();
        }
        try {
            $this->getReverseProxyUrlEntityManager()->delete($url);
            $this->addSuccess($this->tr("admin.reverseProxy.deleteSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error deleting reverse proxy URL with id " . $url->getId(),
                array("exception" => $e));
            $msg = $this->tr(
                "admin.reverseProxy.error.delete",
                array("%name%" => $url->getName(), "%errmsg%" => $e->getMessage())
            );
            $this->addError($msg);
        }

        return $this->redirect($this->generateUrl("admin_reverse_proxy"));
    }

}
