<?php
namespace Topalis\Update\Command;

use Topalis\Certificate\Certificate;

/**
 * This class contains methods to perform the update to version 2.2.0
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateHelper020200
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    public function __construct(\Symfony\Component\DependencyInjection\ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     *
     * @return \Topalis\Aggregate\Command\AggregateAccessStatYearMonthCommand
     */
    private function getAccessStatYearMonthCommand()
    {
        return $this->container->get("aggregate.command.access_stat_year_month");
    }

    /**
     *
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\Certificate\Entity\ClientCertificateEntityManager
     */
    private function getClientCertificateEntityManager()
    {
        return $this->container->get("certificate.entity.manager.certificate");
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    private function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Symfony\Bridge\Doctrine\RegistryInterface
     */
    private function getDoctrine()
    {
        return $this->container->get("doctrine");
    }

    /**
     * Fill the new aggregation table.
     */
    public function aggregateYearMonth()
    {
        $aggregateYearMonth = $this->getAccessStatYearMonthCommand();
        $aggregateYearMonth->setDoctrine($this->getDoctrine());
        $aggregateYearMonth->setTimeProvider($this->container->get("kernel.time_provider"));
        $aggregateYearMonth->aggregate();
    }

    /**
     * Clear passwords of rolled out devices.
     */
    public function clearDevicePasswords()
    {
        $deviceEm = $this->getDeviceEntityManager();
        foreach ($deviceEm->findAll() as $device) {
            if ($device->getDeviceId()
                || ($device->getType()->getName() == "other" && $device->getCert() !== null)) {
                $device->setPassword(null);
                $deviceEm->save($device, false);
            }
        }
        $deviceEm->flush();
    }

    /**
     * Regnerate the server certificate such that TLS Web Server Authentication
     * is set in Extended Key Usage. But do it only if that usage is not set and
     * it has been signed by the auralis CA.
     *
     * @return boolean true if a new server certificate has been created, false
     *     otherwise.
     */
    public function regenerateServerCert()
    {
        $serverCert = $this->getCertificateManager()->getServerCert();
        if (strpos($serverCert->getExtension("extendedKeyUsage"), "TLS Web Server Authentication") !== false) {
            // TLS Web Server Authentication is already set in extendedKeyUsage
            return false;
        }

        $caCert = $this->getCertificateManager()->getCACertificate();
        // check if server cert is signed by auralis CA
        if ($serverCert->getSignerKeyId() != $caCert->getKeyId()) {
            return false;
        }

        $config = $this->getConfig();
        $serial = $config->getCertNextSerial();
        $cert = $serverCert->export($serial);
        $config->setServerCert($cert);

        $ca = $this->getCertificateManager()->loadCACert();
        $caCert = null;
        openssl_x509_export($ca[0], $caCert);
        array_unshift($ca[2], $caCert);
        $config->setServerChain(serialize($ca[2]));
        $this->getCertificateManager()->exportServerCert();
        // decrease serial such that we won't use the same serial again
        $config->setCertNextSerial($serial - 1);

        return true;
    }

    /**
     * Sets the issuer field of client certificates depending on the certificate
     * issuer.
     */
    public function setClientCertIssuers()
    {
        $clientCertEm = $this->getClientCertificateEntityManager();
        foreach ($clientCertEm->findAll() as $cert) {
            if (strlen($cert->getCert()) == 0) {
                continue;
            }

            $certificate = new Certificate($cert->getCert());
            $x509 = $certificate->getParsed();
            if ($x509['issuer']['CN'] == "auralis sub CA") {
                $issuer = "revproxy";
            } else {
                $issuer = "default";
            }
            $cert->setIssuer($issuer);
            $clientCertEm->save($cert, false);
        }
        $clientCertEm->flush();
    }
}
