<?php
namespace Topalis\Update\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Topalis\Kernel\Controller;
use Topalis\Kernel\Exception\Http\NotFoundException;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Topalis\Update\UpdateManager;

/**
 * list available updates and start installation process
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 *
 * @Route("/admin/update", service="update.controller")
 */
class UpdateController extends Controller
{
    /**
     * @return \Topalis\Update\UpdateManager
     */
    private function getUpdateManager()
    {
        return $this->container->get("update.manager");
    }

    /**
     * @Route("", name="admin_update", methods="GET")
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function indexAction()
    {
        $updatesAvailable = null;
        $releaseNotes = null;
        $lastUpdateCheck = null;

        $license = $this->getLicense();
        if (!$license->isExpired() && (!$license->isTestLicense() || $license->existsApnsCertificate())) {
            $config = $this->getConfig();
            $updatesAvailable = json_decode($config->getUpdatesAvailable(), true);
            $releaseNotes = $config->getUpdateReleaseNotes();
            $lastUpdateCheck = $config->getUpdateLastCheck();
            if ($updatesAvailable === null) {
                $updatesAvailable = array();
            } else {
                // remove entries from previous check if required
                $count = count($updatesAvailable);
                for ($i = 0; $i < $count; $i++) {
                    $versionInfo = $updatesAvailable[$i];
                    if (version_compare($versionInfo['version'], $this->getAuralisVersion(), '<=')) {
                        unset($updatesAvailable[$i]);
                    }
                }
            }
        }

        return $this->render("TopalisUpdateBundle::index.html.twig", array(
            "updates" => $updatesAvailable,
            "releaseNotes" => $releaseNotes,
            "lastUpdateCheck" => $lastUpdateCheck,
        ));
    }

    /**
     * @Route("/check", name="admin_update_check", methods="GET")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function checkAction()
    {
        $updatesAvailable = null;
        $releaseNotes = null;
        $lastUpdateCheck = null;

        $license = $this->getLicense();
        if (!$license->isExpired() && (!$license->isTestLicense() || $license->existsApnsCertificate())) {
            try {
                $updatesAvailable = $this->getUpdateManager()->checkUpdates();
            } catch (\Exception $e) {
                $this->getLogger()->error("Error checking for updates", array("exception" => $e));
                $updatesAvailable = false;
            }

            $lastUpdateCheck = $this->getConfig()->getUpdateLastCheck();

            try {
                $releaseNotes = $this->getUpdateManager()->getReleaseNotes();
                if ($releaseNotes === false) {
                    $releaseNotes = $this->getConfig()->getUpdateReleaseNotes();
                }
            } catch (\Exception $e) {
                $this->getLogger()->error("Error loading release notes", array("exception" => $e));
                $releaseNotes = false;
            }
        }

        return $this->render("TopalisUpdateBundle::index.html.twig", array(
            "updates" => $updatesAvailable,
            "releaseNotes" => $releaseNotes,
            "lastUpdateCheck" => $lastUpdateCheck,
        ));
    }

    /**
     * @Route("/{version}/confirm", name="admin_update_confirm", methods="GET")
     *
     * @param string $version
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function confirmAction($version)
    {
        try {
            if ($this->getLicense()->isExpired()) {
                $msg = $this->tr("admin.update.error.licenseExpired");
                throw new \Exception($msg);
            }
            if (!$this->existsVersion($version)) {
                throw new NotFoundException();
            }
            $this->getUpdateManager()->prepareUpdate($version);
        } catch (\Exception $e) {
            $this->getLogger()->error("Error preparing update to version $version", array("exception" => $e));
            $msg = $this->tr(
                "admin.update.error.prepare",
                array("%errmsg%" => $e->getMessage(), "%targetVersion%" => $version)
            );
            $this->addError($msg);
        }

        return $this->render("TopalisUpdateBundle::confirm.html.twig", array(
            "update_version" => $version
        ));
    }

    /**
     * @Route("/cancel", name="admin_update_cancel", methods="GET")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function cancelAction()
    {
        try {
            $this->getUpdateManager()->cancelUpdate();
        } catch (\Exception $e) {
            $this->getLogger()->error("Error canceling update: ", array("exception" => $e));
            $msg = $this->tr("admin.update.error.cancel", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }

        return $this->redirect($this->generateUrl("admin_update"));
    }

    /**
     * @Route("/{version}/install", name="admin_update_install", methods="GET")
     *
     * @param string $version
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function installAction($version)
    {
        if (!$this->existsVersion($version)) {
            $this->addError($this->tr("admin.update.error.notFound", array("%version%" => $version)));
            return $this->indexAction();
        }

        try {
            if ($this->getUpdateManager()->getUpdateStatus() == UpdateManager::STATUS_UPDATE_PREPARE) {
                $this->getUpdateManager()->startUpdate($version);
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("Error starting update to version $version", array("exception" => $e));
            $msg = $this->tr(
                "admin.update.error.start",
                array("%errmsg%" => $e->getMessage(), "%targetVersion%" => $version)
            );
            $this->addError($msg);
            return $this->indexAction();
        }

        return $this->render("TopalisUpdateBundle::install.html.twig", array(
            "update_version" => $version
        ));
    }

    /**
     * @Route("/status", name="admin_update_status", methods="GET", options={"expose" = true})
     *
     * @return \Symfony\Component\HttpFoundation\JsonResponse
     */
    public function statusAction()
    {
        $status = $this->getUpdateManager()->getUpdateStatus();
        return new JsonResponse($status);
    }

    /**
     * @Route(".php", name="admin_update_status_old", methods="GET", options={"expose" = true})
     * @deprecated This method is used to provide the status after the reboot
     *     for versions prior to 2.1
     */
    public function oldStatusAction()
    {
        $request = $this->getRequest();
        $action = $request->query->get("action");
        if ($action != "status") {
            return $this->redirect($this->generateUrl("admin_update"));
        }

        return $this->statusAction();
    }

    /**
     * @param string $version The version to check.
     * @return boolean true if the specified version exists, false otherwise.
     */
    protected function existsVersion($version)
    {
        $updatesAvailable = json_decode($this->getConfig()->getUpdatesAvailable(), true);
        if ($updatesAvailable === null) {
            return false;
        }

        foreach ($updatesAvailable as $versionInfo) {
            if ($versionInfo['version'] == $version) {
                return true;
            }
        }

        return false;
    }
}
