<?php
namespace Topalis\Update;

use Topalis\Kernel\AbstractAuralisService;
use Topalis\Kernel\Exception\Update\UpdateException;
use Topalis\Kernel\Exception\Download\DownloadException;
use Topalis\Util\Process;
use Topalis\Kernel\Exception\Curl\CurlException;
use Topalis\Util\Downloader;

/**
 * Class containing some update specific functions.
 *
 * @author Andreas Hügel <andreas.huegel@topalis.com>
 * @author Frederick Roth <frederick.roth@topalis.com>
 * @author Fritz Elfert <fritz.elfert@topalis.com>
 * @author Holger Widmann <holger.widmann@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateManager extends AbstractAuralisService
{
    const UPDATE_COMMAND = "/usr/sbin/aurupdate";
    const UPDATE_LOG = "/var/log/auralis/update.log";

    /**
     * @var int An error occured during the update process.
     */
    const STATUS_UPDATE_ERROR = -1;
    /**
     * @var int Default status.
     */
    const STATUS_UPDATE_NO = 0;
    /**
     * @var int User has selected version, but not started the update, yet.
     */
    const STATUS_UPDATE_PREPARE = 1;
    /**
     * @var int User has started the update.
     */
    const STATUS_UPDATE_TRIGGERED = 2;
    /**
     * @var int The update script downloads the update packages.
     */
    const STATUS_UPDATE_DOWNLOAD = 3;
    /**
     * @var int The update script installs the update packages.
     */
    const STATUS_UPDATE_INSTALL = 4;
    /**
     * @var int The update script has finished update, reboot will follow, and the
     *     status will be set back to 0.
     */
    const STATUS_UPDATE_FINISHED = 5;

    /**
     * @return boolean true if update is running, false otherwise.
     */
    public function isUpdateRunning()
    {
        return ($this->getUpdateStatus() >= self::STATUS_UPDATE_TRIGGERED);
    }

    /**
     * @return boolean true if update is prepared, false otherwise.
     */
    public function isUpdatePrepared()
    {
        return ($this->getUpdateStatus() == self::STATUS_UPDATE_PREPARE);
    }

    /**
     * Cancels prepared update.
     *
     * @throws \UpdateException If update status is not prepared.
     */
    public function cancelUpdate()
    {
        if ($this->isUpdateRunning()) {
            throw new UpdateException("Canceling a running update is not possible");
        }
        $this->setUpdateStatus(self::STATUS_UPDATE_NO);
        $this->getLogger()->debug('Update canceled.');
    }

    /**
     * @param string|NULL $message The error message if connection fails.
     * @return bool true if a connection to the update server is possible, false otherwise
     */
    public function checkConnection(&$message = null)
    {
        try {
            $updateUrl = $this->getConfig()->getUpdateUrl();
            $this->getDownloader()->loadFile($updateUrl);
            return true;
        } catch (CurlException $e) {
            if ($message !== null) {
                $message = $e->getCurlErrorMessage();
            }
        } catch (DownloadException $e) {
            if ($message !== null) {
                $message = "HTTP status code " . $e->getHttpStatus();
            }
        } catch (\Exception $e) {
            if ($message !== null) {
                $message = $e->getMessage();
            }
        }

        return false;
    }

    /**
     * Check auralis repo for availbale updates.
     *
     * @return mixed Versioninfo of the available updates or false in case of error.
     */
    public function checkUpdates()
    {
        $config = $this->getConfig();

        try {
            $lastUpdateCheck = $this->getTimeProvider()->getTime();
            $updateMetaData = $this->loadUpdateMetaData();
        } catch (CurlException $e) {
            $this->getLogger()->error(sprintf("Receiving data from repository failed: %s", $e->getCurlErrorMessage()));
            return false;
        } catch (DownloadException $e) {
            $this->getLogger()->warn(
                sprintf("Receiving data from repository failed (HTTP status code %d)", $e->getHttpStatus())
            );
            return false;
        }

        try {
            $updateList = $this->buildUpdateList($updateMetaData);
            $updateList = array_reverse($updateList);
        } catch (\Exception $e) {
            $this->getLogger()->error("Error building update list", array("exception" => $e));
            return false;
        }
        $this->getLogger()->debug("Found " . count($updateList) . " releases");

        $config->setUpdatesAvailable(json_encode($updateList));
        $config->setUpdateLastCheck($lastUpdateCheck);

        return $updateList;
    }

    /**
     * Prepare update to version $version.
     *
     * @param string $version The target version.
     *
     * @throws \Topalis\Kernel\Exception\Update\UpdateException
     */
    public function prepareUpdate($version)
    {
        if ($this->isUpdateRunning()) {
            throw new UpdateException("An update is already running");
        }
        $this->getLogger()->debug('Preparing update to version ' . $version . '.');
        $this->setUpdateStatus(self::STATUS_UPDATE_PREPARE);
    }

    /**
     * Update auralis appliance to a newer version.
     * @param string $targetVersion The version to update to.
     * @throws UpdateException If update fails.
     */
    public function startUpdate($targetVersion)
    {
        if (!$this->isUpdatePrepared()) {
            $msg = sprintf(
                "update status must be %d (prepared), but is %d",
                self::STATUS_UPDATE_PREPARE,
                $this->getUpdateStatus()
            );
            throw new UpdateException($msg);
        }
        $this->setUpdateStatus(self::STATUS_UPDATE_TRIGGERED);
        try {
            $this->getLogger()->debug('Updating to version ' . $targetVersion . '.');
            $versions = $this->checkUpdates();
            // Download all archives where CURRENT_VERSION < RELEASE_VERSION <= TARGET_VERSION.
            // Get .sha1-Files only, real download happens in auralis-update.sh
            exec("rm -rf /tmp/auralis-update");
            exec("mkdir -p /tmp/auralis-update");
            foreach ($versions as $releaseVersion) {
                $this->getLogger()->debug("Check if " . $releaseVersion['version'] . " <= " . $targetVersion);
                if (version_compare($releaseVersion['version'], $targetVersion, "<=")) {
                    $downloadUrl = $releaseVersion['url'] . ".sha1";
                    $filename = "/tmp/auralis-update/auralis-" . $releaseVersion['version'] . ".tgz.sha1";
                    $this->getDownloader()->downloadFile($downloadUrl, $filename, true, 15);
                }
            }
            // Initiate update process
            $auralisId = $this->getLicense()->getAuralisId();
            if (Process::exec(self::UPDATE_COMMAND . " $auralisId start") != 0) {
                throw new \Exception(sprintf("Executing %s failed", self::UPDATE_COMMAND));
            }
        } catch (\Exception $e) {
            $this->setUpdateStatus(self::STATUS_UPDATE_ERROR);
            throw new UpdateException($e->getMessage(), $e->getCode(), $e);
        }
    }

    /**
     * @return string The content of the update log file.
     */
    public function getUpdateLog()
    {
        if (!file_exists(self::UPDATE_LOG)) {
            return "";
        }
        return file_get_contents(self::UPDATE_LOG);
    }

    /**
     * @return int The update status.
     */
    public function getUpdateStatus()
    {
        return intval($this->getConfig()->getUpdateStatus());
    }

    private function setUpdateStatus($updateStatus)
    {
        $this->getConfig()->setUpdateStatus($updateStatus);
    }

    /**
     * Retrieve release notes from auralis release repository.
     * @return string The release notes.
     */
    public function getReleaseNotes()
    {
        $auralisUrl = $this->getConfig()->getUpdateUrl();
        $auralisRelnotesUrl = $auralisUrl . "/Release-Notes.txt";
        $httpcode = 0;
        try {
            $releaseNotes = $this->getDownloader()->loadFile($auralisRelnotesUrl, $httpcode);
            $this->getConfig()->setUpdateReleaseNotes($releaseNotes);
        } catch (CurlException $e) {
            $this->getLogger()->error(sprintf("No release notes received: %s", $e->getCurlErrorMessage()));
            return false;
        } catch (DownloadException $e) {
            $this->getLogger()->warn(sprintf("No release notes received (HTTP status code: %d", $e->getHttpStatus()));
            return false;
        }
        return $releaseNotes;
    }

    public function loadUpdateMetaData()
    {
        $updateUrl = $this->getConfig()->getUpdateUrl();
        $this->getLogger()->debug("Retrieving releases from '" . $updateUrl . "'.");
        return $this->getDownloader()->loadFile($updateUrl);
    }

    /**
     * Builds a list of available updates newer then the specified version and
     * returns it.
     *
     * @param string $data Data for building the list.
     * @param string $currentVersion Versions in the list are newer than this.
     * @return array
     */
    private function buildUpdateList($data, $currentVersion = null)
    {
        static $versionPreg = '/auralis-([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)\.tgz$/';

        if ($currentVersion === null) {
            $currentVersion = $this->getAuralisVersion();
        }

        $updateUrl = $this->getConfig()->getUpdateUrl();

        $dom = new \DOMDocument();
        $dom->loadHTML($data);
        $xpath = new \DOMXPath($dom);
        $nodelist = $xpath->query("//tr/td/a");

        $updateList = array();
        foreach ($nodelist as $node) {
            $val = $node->nodeValue;
            if (preg_match($versionPreg, $val, $matches) && count($matches) > 1) {
                $version = str_replace("-", ".", $matches[1]);
                if (!version_compare($version, $currentVersion, '>')) {
                    continue;
                }

                $versionInfo = array();
                $dataNode = $this->getNextElement($node->parentNode);
                $sizeNode = $this->getNextElement($dataNode);
                $link = $updateUrl . "/" . $node->getAttribute('href');
                $versionInfo['version'] = $version;
                $date = strtotime($dataNode->nodeValue);
                if ($date === false) {
                    $this->getLogger()->error("Invalid date '" . $dataNode->nodeValue . "'");
                    break;
                }
                $versionInfo['date'] = $date;
                $versionInfo['size'] = $sizeNode->nodeValue;
                $versionInfo['url'] = $link;
                $versionInfo['changelog'] = $this->loadChangeLog($link);

                $updateList[] = $versionInfo;
            }
        }

        return $updateList;
    }

    private function getNextElement(\DOMNode $node)
    {
        while (($node = $node->nextSibling) !== null) {
            if ($node instanceof \DOMElement) {
                break;
            }
        }
        return $node;
    }

    private function loadChangeLog($url)
    {
        // Get changelog for each release.
        try {
            return $this->getDownloader()->loadFile($url . ".CHANGELOG");
        } catch (CurlException $e) {
            $this->getLogger()->error(
                sprintf("Downloading changelog failed: %s", $e->getCurlErrorMessage())
            );
            return false;
        } catch (DownloadException $e) {
            $msg = sprintf("Downloading changelog failed (HTTP status code %d)", $e->getHttpStatus());
            $this->getLogger()->warning($msg);
            return false;
        }
    }

    protected function getDownloader()
    {
        static $downloader = null;
        if ($downloader === null) {
            $downloader = new Downloader();
            $downloader->setUsername($this->getLicense()->getAuralisId());
            $downloader->setPassword("us(ohPh3hahh2ege");
            $downloader->setProxy($this->getConfig()->getHttpProxy());
            $downloader->setProxyUsername($this->getConfig()->getHttpProxyUser());
            $downloader->setProxyPassword($this->getConfig()->getHttpProxyPassword());
        }

        return $downloader;
    }
}
