<?php
namespace Topalis\User\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\Device\Entity\Device;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Mapping\ClassMetadata;

/**
 * User
 *
 * @ORM\Table(name="users")
 * @ORM\Entity
 * @UniqueEntity(fields={"login"}, message="users.loginTaken")
 */
class User
{
    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    protected $id;

    /**
     * @var string
     *
     * @ORM\Column(name="login", type="string", length=50, nullable=false, unique=true,
     *     options={"comment"="The unique login name"})
     */
    protected $login;

    /**
     * @var string
     *
     * @ORM\Column(name="realname", type="string", length=50, nullable=true,
     *     options={"comment"="The real name"})
     */
    protected $realname;

    /**
     * @var string
     *
     * @ORM\Column(name="email", type="string", length=50, nullable=false,
     *     options={"comment"="The email address"})
     */
    protected $email;

    /**
     * @var string
     *
     * @ORM\Column(name="comment", type="string", length=255, nullable=true,
     *     options={"comment"="A comment without functional meaning"})
     */
    protected $comment;

    /**
     * @var \Topalis\ConfigGroup\Entity\ConfigGroup
     *
     * @ORM\ManyToOne(targetEntity="Topalis\ConfigGroup\Entity\ConfigGroup", inversedBy="users", cascade={"persist"})
     * @ORM\JoinColumn(nullable=false)
     */
    protected $configGroup;

    /**
     * @var ArrayCollection
     *
     * @ORM\OneToMany(targetEntity="Topalis\Device\Entity\Device", mappedBy="user", cascade={"persist"})
     */
    protected $devices;

    public static function loadValidatorMetadata(ClassMetadata $metadata)
    {
        $metadata->addPropertyConstraint("login", new Assert\NotBlank());
        $metadata->addPropertyConstraint("realname", new Assert\NotBlank());
        $metadata->addPropertyConstraint("email", new Assert\NotBlank());
        $metadata->addPropertyConstraint("email", new Assert\Email());
    }

    public function __construct()
    {
        $this->devices = new ArrayCollection();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set login
     *
     * @param string $login
     * @return User
     */
    public function setLogin($login)
    {
        $this->login = $login;

        return $this;
    }

    /**
     * Get login
     *
     * @return string
     */
    public function getLogin()
    {
        return $this->login;
    }

    /**
     * Set realname
     *
     * @param string $realname
     * @return User
     */
    public function setRealname($realname)
    {
        $this->realname = $realname;

        return $this;
    }

    /**
     * Get realname
     *
     * @return string
     */
    public function getRealname()
    {
        return $this->realname;
    }

    /**
     * Set email
     *
     * @param string $email
     * @return User
     */
    public function setEmail($email)
    {
        $this->email = $email;

        return $this;
    }

    /**
     * Get email
     *
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * Set comment
     *
     * @param string $comment
     * @return User
     */
    public function setComment($comment)
    {
        $this->comment = $comment;

        return $this;
    }

    /**
     * Get comment
     *
     * @return string
     */
    public function getComment()
    {
        return $this->comment;
    }

    /**
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup
     * @return \Topalis\User\Entity\User $this
     */
    public function setConfigGroup(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup)
    {
        if ($this->configGroup === $configGroup) {
            return $this;
        }

        if ($this->configGroup !== null) {
            $this->configGroup->removeUser($this);
        }

        $this->configGroup = $configGroup;
        $configGroup->addUser($this);

        return $this;
    }

    /**
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup
     */
    public function getConfigGroup()
    {
        return $this->configGroup;
    }

    /**
     * @return Device[]
     */
    public function getDevices()
    {
        return $this->devices;
    }

    public function setDevices(array $devices)
    {
        $this->devices->clear();
        foreach ($devices as $device) {
            $this->addDevice($device);
        }

        return $this;
    }

    public function addDevice(Device $device)
    {
        if ($this->devices->contains($device)) {
            return $this;
        }

        $this->devices->add($device);
        $device->setUser($this);

        return $this;
    }

    public function removeDevice(Device $device)
    {
        $this->devices->removeElement($device);

        return $this;
    }

    public function __toString()
    {
        return $this->getLogin();
    }
}
