<?php
namespace Topalis\User\Form\Handler;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\ConstraintViolationInterface;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Symfony\Component\Form\FormError;
use Topalis\Device\Entity\Device;
use Topalis\User\Entity\User;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\Request;

class LdapAddFormHandler
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * @return \Topalis\ConfigGroup\Entity\ConfigGroupEntityManager
     */
    private function getConfigGroupEntityManager()
    {
        return $this->container->get("config_group.entity.manager.config_group");
    }

    /**
     * @return \Topalis\User\Ldap\LdapManager
     */
    private function getLdapManager()
    {
        return $this->container->get("user.ldap.manager");
    }

    /**
     * @return \Topalis\License\License
     */
    private function &getLicense()
    {
        return $this->container->get("license.manager")->getLicense();
    }

    /**
     * @return \Topalis\User\Entity\UserEntityManager
     */
    private function getUserEntityManager()
    {
        return $this->container->get("user.entity.manager.user");
    }

    /**
     * @return \Symfony\Component\Validator\ValidatorInterface
     */
    private function getValidator()
    {
        return $this->container->get("validator");
    }

    /**
     * @return \Topalis\Device\EventListener\SendUserMailEventListener
     */
    private function getSendUserMailEventListener()
    {
        return $this->container->get("device.listener.doctrine.send_user_mail");
    }

    /**
     * Processes the specified form.
     *
     * @param FormInterface $form
     * @param Request $request
     * @return boolean|number the number of imported users on success, false
     *     otherwise.
     */
    public function process(FormInterface $form, Request $request)
    {
        if (!$request->isMethod("POST") || $request->request->has("prepareImport")) {
            $limitExceeded = false;
            $limit = $this->container->getParameter("user.ldap_import_limit");
            $dn = $request->request->get("dn");
            $users = $this->getLdapManager()->getUsers($dn, true, $limit + 1);
            if (count($users) > $limit) {
                unset($users[$limit]);
                $limitExceeded = true;
            }

            foreach ($users as &$user) {
                $user['enabled'] = false;
                unset($user);
            }
            $form->setData(array(
                "users" => $users,
                "limitExceeded" => $limitExceeded,
            ));
            $form->get("sendUserMail")->setData($this->getSendUserMailEventListener()->isEnabled());
            return false;
        }

        $form->handleRequest($request);
        if (!$form->isValid()) {
            return false;
        }

        $validator = $this->getValidator();
        $configGroup = $form->get("configGroup")->getData();
        $configGroupFromOu = $configGroup === null;
        $users = $form->get("users")->getData();
        $userList = array();
        $newConfigGroups = array();
        foreach ($users as $key => $userData) {
            if (!$userData['enabled']) {
                continue;
            }

            if ($configGroupFromOu && isset($userData['ou'])) {
                $configGroup = isset ($newConfigGroups[$userData['ou']]) ? $newConfigGroups[$userData['ou']] : null;
                if ($configGroup === null) {
                    $configGroup = $this->getConfigGroupEntityManager()->findOneBy(array("name" => $userData['ou']));
                    if ($configGroup === null) {
                        $configGroup = new ConfigGroup();
                        $configGroup->setName($userData['ou']);
                    }
                    $newConfigGroups[$userData['ou']] = $configGroup;
                }
            }

            $user = new User();
            $user
                ->setLogin($userData['login'])
                ->setRealname($userData['name'])
                ->setEmail($userData['email'])
            ;
            if ($configGroup !== null) {
                $user->setConfigGroup($configGroup);
            } else {
                $notBlank = new NotBlank();
                $formError = new FormError($notBlank->message);
                $form->get("users")->get($key)->get("ou")->addError($formError);
            }
            $violations = $validator->validate($user);
            foreach ($violations as $violation) {
                /** @var $violation ConstraintViolationInterface */
                $formError = new FormError($violation->getMessage(), $violation->getMessageTemplate(),
                    $violation->getMessageParameters(), $violation->getMessagePluralization());
                $form->get("users")->get($key)->get($violation->getPropertyPath())->addError($formError);
            }
            $userList[] = $user;
        }

        // license check
        $curUserCount = $this->getUserEntityManager()->count();
        $allowedUsers = $this->getLicense()->getAllowedUsers();
        $remainingUserCount = $allowedUsers - $curUserCount;
        $userCount = count($userList);
        if ($userCount > $remainingUserCount) {
            $formError = new FormError("user.ldap.tooManyUsers", null, array(
                "%allowed%" => $allowedUsers,
                "%selected%" => $userCount,
                "%remaining%" => $remainingUserCount,
            ));
            $form->addError($formError);
        }

        if ($form->isValid()) {
            $this->getSendUserMailEventListener()->setEnabled($form->get("sendUserMail")->getData());
            if ($form->get("createDevices")->getData()) {
                $os = $form->get("deviceOs")->getData();
                $owner = $form->get("owner")->getData();
                $rolloutPassword = $form->get("rolloutPassword")->getData();
                $this->createDevices($userList, $os, $owner, $rolloutPassword);
            }
            $this->onSuccess($userList);
            return count($userList);
        }

        return false;
    }

    private function createDevices(array $users, $os, $owner, $rolloutPassword)
    {
        /** @var $users User[] */
        foreach ($users as $user) {
            $device = new Device();
            $device->setConfigGroup($user->getConfigGroup());
            $device->setOs($os);
            $device->setOwner($owner);
            $device->setPlainPassword($rolloutPassword);
            $user->addDevice($device);
        }
    }

    private function onSuccess(array $users)
    {
        $em = $this->getUserEntityManager();
        foreach ($users as $user) {
            $em->save($user);
        }
    }
}
