<?php
namespace Topalis\WebClip\EventListener;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Event\PreUpdateEventArgs;
use Symfony\Component\HttpFoundation\File\File;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Topalis\WebClip\Entity\WebClip;
use Topalis\WebClip\Entity\WebClipInterface;

/**
 * Handle adding, changing and deleting of WebClip icons
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class WebClipIconListener
{
    /**
     * @var string directory for storing files
     */
    private $uploadDir;

    /**
     * @var string this icon is used if no other icon is defined
     */
    private $defaultIcon;

    public function __construct($uploadDir, $defaultIcon)
    {
        $this->uploadDir = $uploadDir;
        $this->defaultIcon = $defaultIcon;
    }

    public function prePersist(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof WebClipInterface) {
            return;
        }

        $file = $entity->getIcon();

        if (null === $file) {
            $file = new File($this->defaultIcon);
        }
        $fileName = $this->saveFile($entity, $file);
        $entity->setIcon($fileName);
    }

    public function preUpdate(PreUpdateEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof WebClipInterface) {
            return;
        }
        $this->upload($eventArgs);
    }

    public function preRemove(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof WebClipInterface) {
            return;
        }

        $uploadDir = $this->getUploadDir($entity);
        $iconFilename = $uploadDir . "/" . $entity->getIcon();

        if (file_exists($iconFilename)) {
            unlink($iconFilename);
        }
    }

    /**
     * @param WebClipInterface $entity
     * @param File|string      $file
     *
     * @return string
     */
    protected function saveFile(WebClipInterface $entity, $file)
    {
        $uploadDir = $this->getUploadDir($entity);
        if (!is_dir($uploadDir)) {
            mkdir($uploadDir, 0775, true);
        }

        $fileName = md5($entity->getName() . time() . rand(0, 1000)) . ".png";
        if ($file instanceof UploadedFile) {
            $file->move($uploadDir, $fileName);
        } else {
            if ($file instanceof \SplFileInfo) {
                copy($file->getPathname(), $uploadDir . "/" . $fileName);
            }
        }

        return $fileName;
    }

    protected function upload(PreUpdateEventArgs $eventArgs)
    {
        /** @var $entity WebClip */
        $entity = $eventArgs->getEntity();

        if (!$eventArgs->hasChangedField("icon")) {
            return;
        }

        $file = $entity->getIcon();

        if (null === $file && $eventArgs->getOldValue("icon") === null) {
            $file = new File($this->defaultIcon);
        }

        if (!$file instanceof File) {
            $old = $eventArgs->getOldValue("icon");
            $eventArgs->setNewValue("icon", $old);
            $entity->setIcon($old);

            return;
        }

        $oldIcon = $this->getUploadDir($entity) . "/" . $eventArgs->getOldValue("icon");
        if (is_file($oldIcon)) {
            unlink($oldIcon);
        }

        $fileName = $this->saveFile($entity, $file);
        $eventArgs->setNewValue("icon", $fileName);
        $entity->setIcon($fileName);
    }

    private function getUploadDir(WebClipInterface $webClip)
    {
        return $this->uploadDir . "/" . $webClip->getUploadDir() . "/";
    }
}
