<?php
namespace Topalis\WinPhone\OMA\Provisioning;

use Doctrine\Common\Collections\ArrayCollection;

/**
 * PHP representation of a characteristic element (not complete; as far as required).
 *
 * @see http://msdn.microsoft.com/en-us/library/bb737266.aspx
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 */
class Characteristic
{
    /**
     * @var \Doctrine\Common\Collections\ArrayCollection
     */
    private $characteristics;

    /**
     * @var \Doctrine\Common\Collections\ArrayCollection
     */
    private $parms;

    /**
     * @var string
     */
    private $type;

    /**
     * @var string
     */
    private $translation;

    /**
     * @var string
     */
    private $mwid;

    public function __construct($type)
    {
        $this->characteristics = new ArrayCollection();
        $this->parms = new ArrayCollection();
        $this->setType($type);
    }

    public function addCharacteristic(Characteristic $characteristic)
    {
        $this->characteristics[] = $characteristic;
    }

    public function addOrReplaceCharacteristic(Characteristic $characteristic)
    {
        $type = $characteristic->getType();
        foreach ($this->characteristics as $key => $characteristic) {
            if ($characteristic->getType() == $type) {
                $this->characteristics->set($key, $characteristic);
                return;
            }
        }

        $this->addCharacteristic($characteristic);
    }

    public function clearCharacteristics($type = null)
    {
        if ($type === null) {
            $this->characteristics->clear();
        } else {
            foreach ($this->characteristics as $key => $characteristic) {
                if ($characteristic->getType() == $type) {
                    $this->characteristics->remove($key);
                }
            }
        }
    }

    public function getCharacteristics($type = null)
    {
        if ($type === null) {
            return $this->characteristics;
        }

        $characteristics = array();
        foreach ($this->characteristics as $characteristic) {
            if ($characteristic->getType() == $type) {
                $characteristics[] = $characteristic;
            }
        }
        return $characteristics;
    }

    /**
     * @param string $type
     * @return Characteristic|NULL
     */
    public function getFirstCharacteristic($type)
    {
        foreach ($this->characteristics as $characteristic) {
            if ($characteristic->getType() == $type) {
                return $characteristic;
            }
        }

        return null;
    }

    public function removeCharacteristic(Characteristic $characteristic)
    {
        return $this->characteristics->removeElement($characteristic);
    }

    protected function setCharacteristic($type, Characteristic $characteristic = null)
    {
        foreach ($this->characteristics as $key => $characteristic) {
            if ($characteristic->getType() == $type) {
                if ($characteristic === null) {
                    $this->characteristics->remove($key);
                } else {
                    $this->characteristics->set($key, $characteristic);
                }
                return;
            }
        }

        if ($characteristic !== null) {
            $this->addCharacteristic($characteristic);
        }
    }

    public function addParm(Parm $parm)
    {
        $this->parms[$parm->getName()] = $parm;
    }

    /**
     * @param string $name
     * @return \Topalis\WinPhone\OMA\Provisioning\Parm|NULL
     */
    public function getParm($name)
    {
        return $this->parms[$name];
    }

    public function getParms()
    {
        return $this->parms;
    }

    public function getParmValue($name)
    {
        $parm = $this->getParm($name);
        return $parm ? $parm->getValue() : null;
    }

    public function removeParm($name)
    {
        $this->parms->remove($name);
    }

    public function setParmValue($name, $value, $datatype = null)
    {
        if ($value === null) {
            $this->removeParm($name);
        } else {
            $this->addParm(new Parm($name, $value, $datatype));
        }
    }

    public function getType()
    {
        return $this->type;
    }

    public function setType($type)
    {
        $this->type = $type;
    }

    /**
     * @return string
     */
    public function getTranslation()
    {
        return $this->translation;
    }

    /**
     * @param string $translation
     */
    public function setTranslation($translation)
    {
        $this->translation = $translation;
    }

    /**
     * @return string
     */
    public function getMwid()
    {
        return $this->mwid;
    }

    /**
     * @param string $mwid
     */
    public function setMwid($mwid)
    {
        $this->mwid = $mwid;
    }

    public function appendToDOM(\DOMNode $node)
    {
        $doc = $node->ownerDocument;
        $elem = $doc->createElement("characteristic");
        $node->appendChild($elem);
        $elem->setAttribute("type", $this->getType());

        if ($this->getTranslation() !== null) {
            $elem->setAttribute("translation", $this->getTranslation());
        }

        if ($this->getMwid() !== null) {
            $elem->setAttribute("mwid", $this->getMwid());
        }

        foreach ($this->parms as $parm) {
            $parm->appendToDOM($elem);
        }

        foreach ($this->characteristics as $characteristic) {
            $characteristic->appendToDOM($elem);
        }
    }
}
