<?php
namespace Topalis\WinPhone\OMA\SyncML\Entity;

use Topalis\Util\TypeCheck;
use Topalis\Util\StringUtil;

abstract class AbstractSyncMLElement
{
    public function appendToDOM(\DOMNode $node)
    {
        if ($node->nodeType == XML_DOCUMENT_NODE) {
            $doc = $node;
        } else {
            $doc = $node->ownerDocument;
        }
        $elem = $doc->createElement($this->getElementName());
        $node->appendChild($elem);

        foreach ($this->getChildren() as $name => $value) {
            $this->appendValueToDOM($name, $value, $elem);
        }
    }

    protected function appendValueToDOM($name, $value, \DOMNode $node)
    {
        if (is_object($value)) {
            $value->appendToDOM($node);
        } else if (TypeCheck::isAssocArray($value)) {
            foreach ($value as $key => $val) {
                $this->appendValueToDOM($key, $val, $node);
            }
        } else if (is_array($value)) {
            foreach ($value as $val) {
                $this->appendValueToDOM($name, $val, $node);
            }
        } else if (is_scalar($value)) {
            if ($name) {
                if ($value === false) {
                    continue;
                }

                $doc = $node->ownerDocument;
                $elem = $doc->createElement($name);
                $node->appendChild($elem);
                if ($value !== true) {
                    $value = (string) $value;
                    $matches = array();
                    if (preg_match("/^<!\[CDATA\[(.*)\]\]>$/", $value, $matches)) {
                        $value = $matches[1];
                        $cdataSection = $doc->createCDATASection($value);
                        $elem->appendChild($cdataSection);
                    } else {
                        $elem->nodeValue = $value;
                    }
                }
            } else {
                $node->nodeValue = (string) $value;
            }
        } else {
            throw new \InvalidArgumentException("The specified value has an invalid type: ". gettype($value));
        }
    }

    public function getChildren()
    {
        $reflection = new \ReflectionClass($this);
        return $this->getChildrenForClass($reflection);
    }

    public function getElementName()
    {
        static $elementName = null;
        if ($elementName === null) {
            $className = get_class($this);
            $pos = strrpos($className, "\\");
            $elementName = substr($className, $pos + 1);
        }
        return $elementName;
    }

    protected function getChildrenForClass(\ReflectionClass $reflection)
    {
        $parent = $reflection->getParentClass();
        if ($parent) {
            $children = $this->getChildrenForClass($parent);
        } else {
            $children = array();
        }

        $properties = $reflection->getProperties(\ReflectionProperty::IS_PUBLIC | \ReflectionProperty::IS_PROTECTED);
        foreach ($properties as $property) {
            if ($property->isStatic()) {
                continue;
            }

            $property->setAccessible(true);
            $name = ucfirst($property->getName());
            $value = $property->getValue($this);

            if ($value === null || (is_array($value) && count($value) == 0)) {
                continue;
            }

            if (is_array($value)) {
                if (StringUtil::endsWith($name, "List")) {
                    $name = substr($name, 0, -4);
                } else if (StringUtil::endsWith($name, "s")) {
                    $name = substr($name, 0, -1);
                }
            }

            $children[$name] = $value;
        }

        return $children;
    }

    public function toXml($formatted = false)
    {
        $doc = new \DOMDocument();
        $this->appendToDOM($doc);

        if ($formatted) {
            $doc->formatOutput = true;
        } else {
            $doc->preserveWhiteSpace = false;
        }

        return $doc->saveXML($doc->documentElement);
    }
}
