<?php
namespace Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider;

use Symfony\Component\DependencyInjection\ContainerAware;
use Topalis\Device\Entity\MailAccount;
use Topalis\Device\Entity\Device;
use Topalis\Profile\UUIDGenerator;
use Topalis\WinPhone\WinPhoneConstants;
use Topalis\Wifi\Entity\WifiConfig;
use Topalis\WinPhone\MS\MSWlanProfile;
use Symfony\Component\PropertyAccess\PropertyAccess;
use Topalis\Certificate\Certificate;

class ConfigurationServiceProviderFactory extends ContainerAware
{
    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    private function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     *
     * @param MailAccount $mailAccount
     * @return \Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider\ActiveSyncCSP
     */
    public function createActiveSyncCSP(MailAccount $mailAccount)
    {
        $config = $this->getConfig();

        if ($mailAccount->getUuid() !== null) {
            $uuid = $mailAccount->getUuid();
        } else {
            $uuidGenerator = new UUIDGenerator();
            foreach ($mailAccount->getDevice()->getMailAccounts() as $mailAccount) {
                if ($mailAccount->getUuid() !== null) {
                    $uuidGenerator->addUUID($mailAccount->getUuid());
                }
            }
            $uuid = $uuidGenerator->generateUUID();
        }

        $activeSyncCSP = new ActiveSyncCSP(null, $uuid);
        $activeSyncCSP->setAccountName("auralis " . $mailAccount->getEmail());
        $activeSyncCSP->setDomain($config->getExchangeDomain());
        $activeSyncCSP->setEmailAddress($mailAccount->getEmail());
        $activeSyncCSP->setServerName($config->getHostname() . ":" . $config->getPrimaryPort());
        $activeSyncCSP->setUserName($mailAccount->getUser());

        $configGroup = $mailAccount->getDevice()->getConfigGroup();
        if ($configGroup->getWinPhoneExchangeConfig() !== null) {
            foreach ($configGroup->getWinPhoneExchangeConfig() as $key => $value) {
                switch ($key) {
                    case "mailAgeFilter":
                        $activeSyncCSP->setMailAgeFilter($value);
                        break;
                    case "schedule":
                        $activeSyncCSP->setSchedule($value);
                        break;
                }
            }
        }

        return $activeSyncCSP;
    }

    /**
     * @param \Topalis\Device\Entity\Device $device
     * @return \Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider\DeviceLockCSP
     */
    public function createDeviceLockCSP(Device $device)
    {
        $group = $device->getConfigGroup();
        $passCode = $group->getWinPhonePassCode();
        if ($passCode === null) {
            return null;
        }

        $deviceLockCSP = new DeviceLockCSP();
        $deviceLockCSP->setProviderId(WinPhoneConstants::PROVIDER_ID);
        $deviceLockCSP->setDevicePasswordEnabled(true);
        foreach ($passCode as $key => $value) {
            if (strlen($value) == 0) {
                continue;
            }

            switch ($key) {
                case "allowSimpleDevicePassword":
                    $deviceLockCSP->setAllowSimplePassword($value);
                    break;
                case "alphanumericDevicePasswordRequired":
                    $deviceLockCSP->setAlphanumericDevicePasswordRequired($value);
                    break;
                case "minDevicePasswordLength":
                    $deviceLockCSP->setMinDevicePasswordLength($value);
                    break;
                case "minDevicePasswordComplexCharacters":
                    $deviceLockCSP->setMinDevicePasswordComplexCharacters($value);
                    break;
                case "devicePasswordExpiration":
                    $deviceLockCSP->setDevicePasswordExpiration($value);
                    break;
                case "maxInactivityTimeDeviceLock":
                    $deviceLockCSP->setMaxInactivityTimeDeviceLock($value);
                    break;
                case "devicePasswordHistory":
                    $deviceLockCSP->setDevicePasswordHistory($value);
                    break;
                case "maxDevicePasswordFailedAttempts":
                    $deviceLockCSP->setMaxDevicePasswordFailedAttempts($value);
                    break;
            }
        }

        return $deviceLockCSP;
    }

    public function createPolicyManagerCSP(Device $device)
    {
        $group = $device->getConfigGroup();
        $passCode = $group->getWinPhonePassCode();
        $restrictions = $group->getWinPhoneRestrictions();
        $appPolicy = $group->getWinPhoneAppPolicy();

        $policyManagerCSP = new PolicyManagerCSP();
        $accessor = PropertyAccess::createPropertyAccessor();

        if ($passCode !== null) {
            $policyManagerCSP->setDevicePasswordEnabled(true);
            foreach ($passCode as $key => $value) {
                if (strlen($value) == 0) {
                    continue;
                }

                $accessor->setValue($policyManagerCSP, $key, $value);
            }
        }

        if ($appPolicy !== null) {
            $policyManagerCSP->setApplicationRestrictions($appPolicy->getPolicy());
        }

        // enable encryption if not explicitly disabled
        if (!isset($restrictions['requireDeviceEncryption'])) {
            $restrictions['requireDeviceEncryption'] = true;
        }

        foreach ($restrictions as $key => $value) {
            if (strlen($value) == 0) {
                continue;
            }

            $accessor->setValue($policyManagerCSP, $key, $value);
        }

        return $policyManagerCSP;
    }

    /**
     * @param string $purpose Is used as CN.
     * @param \Topalis\Certificate\Certificate $caCert The CA certificate.
     * @param string $challenge The challenge.
     * @return \Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider\ScepInstallCSP
     */
    public function createScepCSP($purpose, Certificate $caCert, $challenge = null)
    {
        $csp = new ScepInstallCSP();
        $csp->setKeyUsage(160); // 0xA0 digital signature (0x80) | key encipherment (0x20)
        $csp->setKeyLength(2048);
        $csp->setHashAlgorithm("SHA-1");
        $csp->setCAThumbprint($caCert->getFingerprint());
        $csp->setEKUMapping("1.3.6.1.5.5.7.3.2"); // clientAuth

        $subjectName = "C=" . $caCert->getCountry()
        . ",ST=" . $caCert->getState()
        . ",L=" . $caCert->getLocation()
        . ",O=" . $caCert->getOrganization()
        . ",OU=" . $caCert->getOrganizationalUnit()
        . ",CN=" . $purpose
        ;
        $csp->setSubjectName($subjectName);

        if ($challenge) {
            $csp->setChallenge($challenge);
        }

        $port = $this->getConfig()->getSecondaryPort();
        $serverURL = "https://" . $this->getConfig()->getHostname() . ":" . $port . "/scep";
        $csp->setServerURL($serverURL);

        return $csp;
    }

    /**
     *
     * @param Device $device
     * @return NULL|\Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider\StorageCSP
     */
    public function createStorageCSP(Device $device)
    {
        $restrictions = $device->getConfigGroup()->getWinPhoneRestrictions();
        if ($restrictions === null) {
            return null;
        }

        $storageCSP = new StorageCSP();
        if (isset($restrictions['allowStorageCard'])) {
            $storageCSP->setDisabled(!$restrictions['allowStorageCard']);
        }

        return $storageCSP;
    }

    /**
     *
     * @param \Topalis\Device\Entity\Device $device
     * @param string $ssid
     * @return \Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider\WiFiCSP
     */
    public function createWiFiCSP(Device $device, $ssid)
    {
        foreach ($device->getConfigGroup()->getMSWlanConfigs() as $wlanConfig) {
            if ($wlanConfig->getSsid() != $ssid) {
                continue;
            }

            $wifiCSP = new WiFiCSP(null, $ssid);
            $wifiCSP->setWlanXml($wlanConfig->getProfile());
            return $wifiCSP;
        }

        return null;
    }

    /**
     * @param \Topalis\Device\Entity\Device $device
     * @param \Topalis\Wifi\Entity\WifiConfig $wifiConfig
     * @return \Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider\WiFiCSP
     */
    public function createWiFiCSPFromConfig(Device $device, WifiConfig $wifiConfig)
    {
        $wlanProfile = new MSWlanProfile();
        if ($wifiConfig->isAutoJoin() && version_compare($device->getVersion(), "8.1", ">=")) {
            $wlanProfile->setConnectionMode("auto");
        } else {
            // for some reason connectionMode MUST be set to "manual" on WP 8,
            // otherwise the installation is rejected.
            $wlanProfile->setConnectionMode("manual");
        }
        $wlanProfile->setSsid($wifiConfig->getSsid());
        // name must match SSID
        $wlanProfile->setName($wifiConfig->getSsid());
        $wlanProfile->setNonBroadcast($wifiConfig->isHidden());

        switch ($wifiConfig->getEncryptionType()) {
            case WifiConfig::ENCRYPTION_TYPE_WPAPSK:
                $wlanProfile->setAuthentication("WPAPSK");
                $wlanProfile->setEncryption("TKIP");
                $wlanProfile->setKeyMaterial($wifiConfig->getEncryptionKey());
                $wlanProfile->setKeyType("passPhrase");
                $wlanProfile->setProtected(false);
                break;
            case WifiConfig::ENCRYPTION_TYPE_WPA2PSK:
                $wlanProfile->setAuthentication("WPA2PSK");
                $wlanProfile->setEncryption("AES");
                $wlanProfile->setKeyMaterial($wifiConfig->getEncryptionKey());
                $wlanProfile->setKeyType("passPhrase");
                $wlanProfile->setProtected(false);
                break;
            case WifiConfig::ENCRYPTION_TYPE_NONE:
                $wlanProfile->setAuthentication("open");
                $wlanProfile->setEncryption("none");
                break;
        }
        $wifiCSP = new WiFiCSP(null, $wifiConfig->getSsid());
        if ($wifiConfig->getProxyType() == WifiConfig::PROXY_TYPE_MANUAL) {
            $wifiCSP->setProxy($wifiConfig->getProxyHost() . ":" . $wifiConfig->getProxyPort());
        }
        $wifiCSP->setWlanXml($wlanProfile->toXml());
        return $wifiCSP;
    }

    /**
     *
     * @param \Topalis\Device\Entity\Device $device
     * @return array Array containing WiFiCSP objects.
     */
    public function createWiFiCSPs(Device $device)
    {
        $wifiCSPs = array();

        foreach ($device->getConfigGroup()->getMSWlanConfigs() as $wlanConfig) {
            $wifiCSP = new WiFiCSP(null, $wlanConfig->getSsid());
            $wifiCSP->setWlanXml($wlanConfig->getProfile());
            $wifiCSPs[] = $wifiCSP;
        }

        foreach ($device->getConfigGroup()->getWifiConfigs() as $wifiConfig) {
            $wifiCSPs[] = $this->createWiFiCSPFromConfig($device, $wifiConfig);
        }

        return $wifiCSPs;
    }
}
