<?php
namespace Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider;

/**
 * PolicyManager configuration service provider.
 *
 * @see Windows Phone 8.1 Enterprise Device Management Protocol
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 */
class PolicyManagerCSP extends AbstractConfigurationServiceProvider
{
    /**
     * @param string $cmdID
     * @param string $root "My" to retrieve, modify, or delete the provisioned
     *     policies. "Device" to retrieve the evaluated policies of all
     *     providers. (Default: "My")
     */
    public function __construct($cmdID = null, $root = "My")
    {
        parent::__construct($cmdID);
        $this->setLocURIPrefix("./Vendor/MSFT/PolicyManager/" . $root. "/");
        if ($root == "Device") {
            $this->setDefaultCommand("Get", true);
        }

        foreach ($this->getDefaults() as $locURI => $data) {
            $this->setData($locURI, $data);
        }
    }

    /**
     * @return boolean true if device lock is enabled, false otherwise.
     *     (Default: true)
     */
    public function isDevicePasswordEnabled()
    {
        // value is inverted
        return $this->isData("DeviceLock/DevicePasswordEnabled") ? false : true;
    }

    /**
     * @param boolean $devicePasswordEnabled true to enable device lock, false
     *     to disable. (Default: true)
     */
    public function setDevicePasswordEnabled($devicePasswordEnabled)
    {
        // value is inverted
        $this->setBoolAsInt("DeviceLock/DevicePasswordEnabled", !$devicePasswordEnabled);
    }

    /**
     * @return boolean true if simple passwords, such as "1111" or "1234" are
     *    allowed, false otherwise. (Default: true)
     */
    public function isAllowSimpleDevicePassword()
    {
        return $this->isData("DeviceLock/AllowSimpleDevicePassword");
    }

    /**
     *
     * @param boolean $allowSimpleDevicePassword true to allow simple passwords,
     *     such as "1111" or "1234", false to disallow. (Default: true)
     */
    public function setAllowSimpleDevicePassword($allowSimpleDevicePassword)
    {
        $this->setBoolAsInt("DeviceLock/AllowSimpleDevicePassword", $allowSimpleDevicePassword);
    }

    /**
     *
     * @return int The minimum number of characters required in the PIN. Valid
     *     values are 4 to 18. (Default: 4)
     */
    public function getMinDevicePasswordLength()
    {
        return $this->getData("DeviceLock/MinDevicePasswordLength");
    }

    /**
     *
     * @param int $minDevicePasswordLength The minimum number of characters
     *     required in the PIN. Valid values are 4 to 16. (Default: 4)
     */
    public function setMinDevicePasswordLength($minDevicePasswordLength)
    {
        if ($minDevicePasswordLength < 4 || $minDevicePasswordLength > 16) {
            throw new \InvalidArgumentException("MinDevicePasswordLength must be between 4 and 16");
        }
        $this->setData("DeviceLock/MinDevicePasswordLength",
            array("data" => $minDevicePasswordLength, "format" => "int"));
    }

    /**
     * @return int 0 if alphanumeric password required, 1 if numeric password
     *     required, 2 if user can choose a numeric password or alphanumeric
     *     password. (Default: 1)
     */
    public function getAlphanumericDevicePasswordRequired()
    {
        return $this->getData("DeviceLock/AlphanumericDevicePasswordRequired");
    }

    /**
     *
     * @param int $alphanumericDevicePasswordRequired 0 if alphanumeric
     *     password required, 1 if numeric password required, 2 if user can
     *     choose a numeric password or alphanumeric password. (Default: 1)
     */
    public function setAlphanumericDevicePasswordRequired($alphanumericDevicePasswordRequired)
    {
        if ($alphanumericDevicePasswordRequired < 0 || $alphanumericDevicePasswordRequired > 2) {
            throw new \InvalidArgumentException("AlphanumericDevicePasswordRequired must be between 0 and 2");
        }
        $this->setData("DeviceLock/AlphanumericDevicePasswordRequired",
            array("data" => $alphanumericDevicePasswordRequired, "format" => "int"));
    }

    /**
     * @return int The number of days before password expiration. Valid values
     *    are 0 to 730. 0 indicates that the password does not expire. (Default: 0)
     */
    public function getDevicePasswordExpiration()
    {
        return $this->getData("DeviceLock/DevicePasswordExpiration");
    }

    /**
     *
     * @param int $devicePasswordExpiration The number of days before password
     *    expiration. Valid values are 0 to 730. 0 indicates that the password
     *    does not expire. (Default: 0)
     */
    public function setDevicePasswordExpiration($devicePasswordExpiration)
    {
        if ($devicePasswordExpiration < 0 || $devicePasswordExpiration > 730) {
            throw new \InvalidArgumentException("DevicePasswordExpiration must be between 0 and 730");
        }
        $this->setData("DeviceLock/DevicePasswordExpiration",
            array("data" => $devicePasswordExpiration, "format" => "int"));
    }

    /**
     * @return int The number of passwords that can be stored in the history
     *    (can't be reused). Valid values are 0 to 50. (Default: 0)
     */
    public function getDevicePasswordHistory()
    {
        return $this->getData("DeviceLock/DevicePasswordHistory");
    }

    /**
     *
     * @param int $devicePasswordHistory The number of passwords that can be
     *    stored in the history (can't be reused). Valid values are 0 to 50.
     *    (Default: 0)
     */
    public function setDevicePasswordHistory($devicePasswordHistory)
    {
        if ($devicePasswordHistory < 0 || $devicePasswordHistory > 50) {
            throw new \InvalidArgumentException("DevicePasswordHistory must be between 0 and 50");
        }
        $this->setData("DeviceLock/DevicePasswordHistory", array("data" => $devicePasswordHistory, "format" => "int"));
    }

    /**
     * @return int The number of authentication failures allowed before the
     *     phone will be wiped. Valid values are 0 to 999. 0 indicates the
     *     phone will not be wiped regrdless of the number of authentication
     *     failures. (Default: 0)
     */
    public function getMaxDevicePasswordFailedAttempts()
    {
        return $this->getData("DeviceLock/MaxDevicePasswordFailedAttempts");
    }

    /**
     *
     * @param int $maxDevicePasswordFailedAttempts The number of authentication
     *     failures allowed before the phone will be wiped. Valid values are 0
     *     to 999. 0 indicates the phone will not be wiped regardless of the
     *     number of authentication failures. (Default: 0)
     */
    public function setMaxDevicePasswordFailedAttempts($maxDevicePasswordFailedAttempts)
    {
        if ($maxDevicePasswordFailedAttempts < 0 || $maxDevicePasswordFailedAttempts > 999) {
            throw new \InvalidArgumentException("MaxDevicePasswordFailedAttempts must be between 0 and 999");
        }
        $this->setData("DeviceLock/MaxDevicePasswordFailedAttempts",
            array("data" => $maxDevicePasswordFailedAttempts, "format" => "int"));
    }

    /**
     * @return int The amount of times in minutes that the phone can remain
     *    idle before it is password locked. Valid values are 0 to 999.
     *    0 indicates no time-out is specified. In this case, the maximum
     *    screen time-out allowed by the UI applies. (Default: 0)
     */
    public function getMaxInactivityTimeDeviceLock()
    {
        return $this->getData("DeviceLock/MaxInactivityTimeDeviceLock");
    }

    /**
     *
     * @param int $maxInactivityTimeDeviceLock The amount of times in minutes
     *    that the phone can remain idle before it is password locked. Valid
     *    values are 0 to 999. 0 indicates no time-out is specified. In this
     *    case, the maximum screen time-out allowed by the UI applies.
     *    (Default: 0)
     */
    public function setMaxInactivityTimeDeviceLock($maxInactivityTimeDeviceLock)
    {
        if ($maxInactivityTimeDeviceLock < 0 || $maxInactivityTimeDeviceLock > 999) {
            throw new \InvalidArgumentException("MaxInactivityTimeDeviceLock must be between 0 and 999");
        }
        $this->setData("DeviceLock/MaxInactivityTimeDeviceLock",
            array("data" => $maxInactivityTimeDeviceLock, "format" => "int"));
    }

    /**
     * @return int The number of complex element types (uppercase and lowercase
     *     letters, numbers, and punctuation) required for a string password.
     *     Valid values are 1 to 4. (Default: 1)
     */
    public function getMinDevicePasswordComplexCharacters()
    {
        return $this->getData("DeviceLock/MinDevicePasswordComplexCharacters");
    }

    /**
     *
     * @param int $minDevicePasswordComplexCharacters The number of complex
     *     element types (uppercase and lowercase letters, numbers, and
     *     punctuation) required for a string password. Valid values are 1 to 4.
     *     (Default: 1)
     * @throws \InvalidArgumentException
     */
    public function setMinDevicePasswordComplexCharacters($minDevicePasswordComplexCharacters)
    {
        if ($minDevicePasswordComplexCharacters < 1 || $minDevicePasswordComplexCharacters > 4) {
            throw new \InvalidArgumentException("MinDevicePasswordComplexCharacters must be between 1 and 4");
        }
        $this->setData("DeviceLock/MinDevicePasswordComplexCharacters",
            array("data" => $minDevicePasswordComplexCharacters, "format" => "int"));
    }

    /**
     * @return boolean true if WiFi connections are allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowWiFi()
    {
        return $this->isData("WiFi/AllowWiFi");
    }

    /**
     * @param boolean $allowWiFi true to allow WiFi connections, false to
     *     disallow. (Default: true)
     */
    public function setAllowWiFi($allowWiFi)
    {
        $this->setBoolAsInt("WiFi/AllowWiFi", $allowWiFi);
    }

    /**
     * @return boolean true if internet sharing is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowInternetSharing()
    {
        return $this->isData("WiFi/AllowInternetSharing");
    }

    /**
     * @param boolean $allowInternetSharing true to allow internet sharing,
     *     false to disallow. (Default: true)
     */
    public function setAllowInternetSharing($allowInternetSharing)
    {
        $this->setBoolAsInt("WiFi/AllowInternetSharing", $allowInternetSharing);
    }

    /**
     * @return boolean true if automatic connect to WiFi hotspots and friend
     *     social networks is allowed, false otherwise. (Default: true)
     */
    public function isAllowAutoConnectToWiFiSenseHotspots()
    {
        return $this->isData("WiFi/AllowAutoConnectToWiFiSenseHotspots");
    }

    /**
     * @param boolean $allowAutoConnectToWifiSenseHotspots true to allow
     *     automatic connect to WiFi hotspots and friend social networks, false
     *     to disallow. (Default: true)
     */
    public function setAllowAutoConnectToWiFiSenseHotspots($allowAutoConnectToWifiSenseHotspots)
    {
        return $this->setBoolAsInt("WiFi/AllowAutoConnectToWiFiSenseHotspots", $allowAutoConnectToWifiSenseHotspots);
    }

    /**
     * @return boolean true if WiFi hotspot information reporting to Microsoft
     *     is allowed, false otherwise. (Default: true)
     */
    public function isAllowWiFiHotspotReporting()
    {
        // Microsoft writes HotSpot with capital S here indeed
        return $this->isData("WiFi/AllowWiFiHotSpotReporting");
    }

    /**
     * @param boolean $allowWiFiHotspotReporting true to allow WiFi hotspot
     *     information reporting to Microsoft, false to disallow.
     *     (Default: true)
     */
    public function setAllowWiFiHotspotReporting($allowWiFiHotspotReporting)
    {
        // Microsoft writes HotSpot with capital S here indeed
        return $this->setBoolAsInt("WiFi/AllowWiFiHotSpotReporting", $allowWiFiHotspotReporting);
    }

    /**
     * @@return boolean $allowManualWiFiConfiguration true if connections to
     *     WiFis outside of MDM server-installed networks are allowed, false
     *     otherwise. (Default: true)
     */
    public function isAllowManualWiFiConfiguration()
    {
        return $this->isData("WiFi/AllowManualWiFiConfiguration");
    }

    /**
     * @param boolean $allowManualWiFiConfiguration true to allow connections to
     *     WiFis outside of MDM server-installed networks, false to disallow.
     *     (Default: true)
     */
    public function setAllowManualWiFiConfiguration($allowManualWiFiConfiguration)
    {
        $this->setBoolAsInt("WiFi/AllowManualWiFiConfiguration", $allowManualWiFiConfiguration);
    }

    /**
     * @return boolean true if NFC is allowed, false otherwise. (Default: true)
     */
    public function isAllowNFC()
    {
        return $this->isData("Connectivity/AllowNFC");
    }

    /**
     * @param boolean $allowNFC true to allow NFC, false to disallow.
     *     (Default: true)
     */
    public function setAllowNFC($allowNFC)
    {
        $this->setBoolAsInt("Connectivity/AllowNFC", $allowNFC);
    }

    /**
     * @return boolean true if bluetooth is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowBluetooth()
    {
        return $this->isData("Connectivity/AllowBluetooth");
    }

    /**
     * @param boolean $allowBluetooth true to allow bluetooth, false to disallow.
     *     (Default: true)
     */
    public function setAllowBluetooth($allowBluetooth)
    {
        $data = $allowBluetooth ? 2 : 0;
        $this->setData("Connectivity/AllowBluetooth", array("data" => $data, "format" => "int"));
    }

    /**
     * @return boolean true if VPN connections are allowed when the device
     *     roams over cellular networks, false otherwise. (Default: true)
     */
    public function isAllowVPNRoamingOverCellular()
    {
        return $this->isData("Connectivity/AllowVPNRoamingOverCellular");
    }

    /**
     * @param boolean $allowVPNRoamingOverCellular true to allow VPN connections
     *     when the device roams over cellular networks, false to disallow.
     *     (Default: true)
     */
    public function setAllowVPNRoamingOverCellular($allowVPNRoamingOverCellular)
    {
        $this->setBoolAsInt("Connectivity/AllowVPNRoamingOverCellular", $allowVPNRoamingOverCellular);
    }

    /**
     * @return boolean true if VPN connections over cellular networks are
     *     allowed, false otherwise. (Default: true)
     */
    public function isAllowVPNOverCellular()
    {
        return $this->isData("Connectivity/AllowVPNOverCellular");
    }

    /**
     * @param boolean $allowVPNOverCellular true to allow VPN connections over
     *     cellular networks, false to disallow. (Default: true)
     */
    public function setAllowVPNOverCellular($allowVPNOverCellular)
    {
        $this->setBoolAsInt("Connectivity/AllowVPNOverCellular", $allowVPNOverCellular);
    }

    /**
     * @return boolean true if SD card use is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowStorageCard()
    {
        return $this->isData("System/AllowStorageCard");
    }

    /**
     * @param boolean $allowStorageCard true to allow SD card use, false to
     *     disallow. (Default: true)
     */
    public function setAllowStorageCard($allowStorageCard)
    {
        $this->setBoolAsInt("System/AllowStorageCard", $allowStorageCard);
    }

    /**
     * @return int 0 if sending telemetry data is not allowed, 1 if sending
     *     telemetry data is allowed except for Secondary Data Requests, 2 if
     *     sending telemetry data is allowed. (Default: 2)
     */
    public function getAllowTelemetry()
    {
        return $this->getData("System/AllowTelemetry");
    }

    /**
     * @param int $allowTelemetry 0 to disallow sending telemetry data, 1 to
     *     allow sending telemetry data except for Secondary Data Requests, 2 to
     *     allow sending telemetry data. (Default: 2)
     */
    public function setAllowTelemetry($allowTelemetry)
    {
        if ($allowTelemetry < 0 || $allowTelemetry > 2) {
            throw new \InvalidArgumentException("AllowTelemetry must be between 0 and 2");
        }
        $this->setData("System/AllowTelemetry", array("data" => $allowTelemetry, "format" => "int"));
    }

    /**
     * @return boolean true if copy and paste is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowCopyPaste()
    {
        return $this->isData("Experience/AllowCopyPaste");
    }

    /**
     * @param boolean $allowCopyPaste true to allow copy and paste, false to
     *     disallow. (Default: true)
     */
    public function setAllowCopyPaste($allowCopyPaste)
    {
        $this->setBoolAsInt("Experience/AllowCopyPaste", $allowCopyPaste);
    }

    /**
     * @return boolean true if using MSA account for non email related
     *     connection authentication and services is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowMicrosoftAccountConnection()
    {
        return $this->isData("Accounts/AllowMicrosoftAccountConnection");
    }

    /**
     * @param boolean $allowMicrosoftAccountConnection true to allow using MSA
     *     account for non email related connection authentication and services,
     *     false to disallow. (Default: true)
     */
    public function setAllowMicrosoftAccountConnection($allowMicrosoftAccountConnection)
    {
        $this->setBoolAsInt("Accounts/AllowMicrosoftAccountConnection", $allowMicrosoftAccountConnection);
    }

    /**
     * @return boolean true if adding non MSA email accounts is allowed, false
     *     otherwise. (Default: true)
     */
    public function isAllowAddingNonMicrosoftAccountsManually()
    {
        return $this->isData("Accounts/AllowAddingNonMicrosoftAccountsManually");
    }

    /**
     * @param boolean $allowAddingNonMicrosoftAccountsManually true to allow
     *     adding non MSA email accounts, false to disallow. (Default: true)
     */
    public function setAllowAddingNonMicrosoftAccountsManually($allowAddingNonMicrosoftAccountsManually)
    {
        $this->setBoolAsInt("Accounts/AllowAddingNonMicrosoftAccountsManually",
            $allowAddingNonMicrosoftAccountsManually);
    }

    /**
     * @return boolean true if installation of root and intermediate CA
     *     certificates is allowed, false otherwise. (Default: true)
     */
    public function isAllowManualRootCertificateInstallation()
    {
        return $this->isData("Security/AllowManualRootCertificateInstallation");
    }

    /**
     * @param boolean $allowManualRootCertificateInstallation true to allow
     *     installation of root and intermediate CA certificates.
     *     (Default: true)
     */
    public function setAllowManualRootCertificateInstallation($allowManualRootCertificateInstallation)
    {
        $this->setBoolAsInt("Security/AllowManualRootCertificateInstallation",
            $allowManualRootCertificateInstallation);
    }

    /**
     * @return boolean true if device encryption is required, false otherwise.
     *     (Default: false)
     */
    public function isRequireDeviceEncryption()
    {
        return $this->isData("Security/RequireDeviceEncryption");
    }

    /**
     * @param boolean $requireDeviceEncryption true to require device
     *     encryption, false otherwise. Once turned on, it cannot be turned off
     *     via policy. (Default: false)
     */
    public function setRequireDeviceEncryption($requireDeviceEncryption)
    {
        $this->setBoolAsInt("Security/RequireDeviceEncryption", $requireDeviceEncryption);
    }

    /**
     * @return boolean true if app store is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowStore()
    {
        return $this->isData("ApplicationManagement/AllowStore");
    }

    /**
     * @param boolean $allowStore true to allow app store, false to disallow.
     *     (Default: true)
     */
    public function setAllowStore($allowStore)
    {
        $this->setBoolAsInt("ApplicationManagement/AllowStore", $allowStore);
    }

    /**
     * @return string|NULL An XML blob specifying the application restrictions.
     *     (Default: null)
     */
    public function getApplicationRestrictions()
    {
        $matches = array();
        if (preg_match("/^<!\[CDATA\[(.*)\]\]>$/",
            $this->getData("ApplicationManagement/ApplicationRestrictions"), $matches)) {
                return $matches[1];
        }

        return "";
    }

    /**
     * @param string|NULL $applicationRestrictions An XML blob specifying
     *     the application restrictions to set.
     */
    public function setApplicationRestrictions($applicationRestrictions)
    {
        $this->setData("ApplicationManagement/ApplicationRestrictions", array(
            "data" => "<![CDATA[" . $applicationRestrictions . "]]>",
            "format" => "chr"
        ));
    }

    /**
     * @return boolean true of developer unlock is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowDeveloperUnlock()
    {
        return $this->isData("ApplicationManagement/AllowDeveloperUnlock");
    }

    /**
     * @param boolean $allowDeveloperUnlock true to allow developer unlock,
     *     false to disallow. (Default: true)
     */
    public function setAllowDeveloperUnlock($allowDeveloperUnlock)
    {
        $this->setBoolAsInt("ApplicationManagement/AllowDeveloperUnlock", $allowDeveloperUnlock);
    }

    /**
     * @return boolean true if using Internet Explorer is allowed, false
     *     otherwise. (Default: true)
     */
    public function isAllowBrowser()
    {
        return $this->isData("Browser/AllowBrowser");
    }

    /**
     * @param boolean $allowBrowser true to allow using Internet Explorer, false
     *     to disallow. (Default: true)
     */
    public function setAllowBrowser($allowBrowser)
    {
        $this->setBoolAsInt("Browser/AllowBrowser", $allowBrowser);
    }

    /**
     * @return boolean true if screen capture is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowScreenCapture()
    {
        return $this->isData("Experience/AllowScreenCapture");
    }

    /**
     * @param boolean $allowScreenCapture true to allow screen capture, false
     *     to disallow. (Default: true)
     */
    public function setAllowScreenCapture($allowScreenCapture)
    {
        $this->setBoolAsInt("Experience/AllowScreenCapture", $allowScreenCapture);
    }

    /**
     * @return boolean true if location service is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowLocation()
    {
        return $this->isData("System/AllowLocation");
    }

    /**
     * @param boolean $allowLocation true to allow location service, false to
     *     disallow. (Default: true)
     */
    public function setAllowLocation($allowLocation)
    {
        $this->setBoolAsInt("System/AllowLocation", $allowLocation);
    }

    /**
     * @return boolean true if accessing phone storeage via USB is allowed,
     *     false otherwise. (Default: true)
     */
    public function isAllowUSBConnection()
    {
        return $this->isData("Connectivity/AllowUSBConnection");
    }

    /**
     * @param boolean $allowUSBConnection true to allow accessing phone storage
     *     via USB, false to disallow. (Default: true)
     */
    public function setAllowUSBConnection($allowUSBConnection)
    {
        $this->setBoolAsInt("Connectivity/AllowUSBConnection", $allowUSBConnection);
    }

    /**
     * @return boolean true if cellular data roaming is allowed, false
     *     otherwise. (Default: true)
     */
    public function isAllowCellularDataRoaming()
    {
        return $this->isData("Connectivity/AllowCellularDataRoaming");
    }

    /**
     * @param boolean $allowCellularDataRoaming true to allow cellular data
     *     roaming, false to disallow. (Default: true)
     */
    public function setAllowCellularDataRoaming($allowCellularDataRoaming)
    {
        $this->setBoolAsInt("Connectivity/AllowCellularDataRoaming", $allowCellularDataRoaming);
    }

    /**
     * @return boolean true if using camera is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowCamera()
    {
        return $this->isData("Camera/AllowCamera");
    }

    /**
     * @param boolean $allowCamera true to allow using camera, false to
     *     disallow. (Default: true)
     */
    public function setAllowCamera($allowCamera)
    {
        $this->setBoolAsInt("Camera/AllowCamera", $allowCamera);
    }

    /**
     * @return boolean true if search is allowed to use location information,
     *     false otherwise. (Default: true)
     */
    public function isAllowSearchToUseLocation()
    {
        return $this->isData("Search/AllowSearchToUseLocation");
    }

    /**
     * @param boolean $allowSearchToUseLocation true to allow search to use
     *     location information, false to disallow. (Default: true)
     */
    public function setAllowSearchToUseLocation($allowSearchToUseLocation)
    {
        $this->setBoolAsInt("Search/AllowSearchToUseLocation", $allowSearchToUseLocation);
    }

    /**
     * Returns the level of safe search (filtering of adult content).
     *
     * @return int 0 highest filtering, 1 moderate filtering. (Default: 1)
     */
    public function getSafeSearchPermissions()
    {
        return $this->getData("Search/SafeSearchPermissions");
    }

    /**
     * Sets the level of safe search (filtering adult content).
     *
     * @param int $safeSearchPermissions 0 highest filtering, 1 moderate
     *     filtering. (Default: 1)
     */
    public function setSafeSearchPermissions($safeSearchPermissions)
    {
        if ($safeSearchPermissions < 0 || $safeSearchPermissions > 1) {
            throw new \InvalidArgumentException("SafeSearchPermissions must be between 0 and 1");
        }
        $this->setData("Search/SafeSearchPermissions", array("data" => $safeSearchPermissions, "format" => "int"));
    }

    /**
     * @return true if BingVision is allowed to store the contents of the images
     *     captured when performing BingVision search, false otherwise.
     *     (Default: true)
     */
    public function isAllowStoringImagesFromVisionSearch()
    {
        return $this->isData("Search/AllowStoringImagesFromVisionSearch");
    }

    /**
     * @param boolean $allowStoringImagesFromVisionSearch true to allow
     *     BingVision to store the contents of the images captured when
     *     performing BingVision search, false to disallow. (Default: true)
     */
    public function setAllowStoringImagesFromVisionSearch($allowStoringImagesFromVisionSearch)
    {
        $this->setBoolAsInt("Search/AllowStoringImagesFromVisionSearch", $allowStoringImagesFromVisionSearch);
    }

    /**
     * @return boolean true if voice recording is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowVoiceRecording()
    {
        return $this->isData("Experience/AllowVoiceRecording");
    }

    /**
     * @param boolean $allowVoiceRecording true to allow voice recording, false
     *     to disallow. (Default: true)
     */
    public function setAllowVoiceRecording($allowVoiceRecording)
    {
        $this->setBoolAsInt("Experience/AllowVoiceRecording", $allowVoiceRecording);
    }

    /**
     * @return boolean true if saving files is as office files is allowed, false
     *     otherwise. (Default: true)
     */
    public function isAllowSaveAsOfOfficeFiles()
    {
        return $this->isData("Experience/AllowSaveAsOfOfficeFiles");
    }

    /**
     * @param boolean $allowSaveAsOfOfficeFiles true to allow saving files as
     *     office files, false to disallow. (Default: true)
     */
    public function setAllowSaveAsOfOfficeFiles($allowSaveAsOfOfficeFiles)
    {
        $this->setBoolAsInt("Experience/AllowSaveAsOfOfficeFiles", $allowSaveAsOfOfficeFiles);
    }

    /**
     * @return boolean true if the user is allowed to specify the grace period,
     *     false if the user is forced to input password every time the device
     *     is returning from idle state. (Default: true)
     */
    public function isAllowIdleReturnWithoutPassword()
    {
        return $this->isData("DeviceLock/AllowIdleReturnWithoutPassword");
    }

    /**
     * @param boolean $allowIdleReturnWithoutPassword true to allow the user to
     *     specify the grace period, false to force the user to input password
     *     every time the device is returning from idle state. (Default: true)
     */
    public function setAllowIdleReturnWithoutPassword($allowIdleReturnWithoutPassword)
    {
        $this->setBoolAsInt("DeviceLock/AllowIdleReturnWithoutPassword", $allowIdleReturnWithoutPassword);
    }

    /**
     * @return boolean true if Cortana is allowed, false otherwise.
     *     (Default: true)
     */
    public function isAllowCortana()
    {
        return $this->isData("Experience/AllowCortana");
    }

    /**
     * @param boolean $allowCortana true to allow Cortana, false otherwise.
     *     (Default: true)
     */
    public function setAllowCortana($allowCortana)
    {
        $this->setBoolAsInt("Experience/AllowCortana", $allowCortana);
    }

    /**
     * @return boolean true if syncing settings between devices is allowed,
     *     false otherwise. (Default: true)
     */
    public function isAllowSyncMySettings()
    {
        return $this->isData("Experience/AllowSyncMySettings");
    }

    /**
     * @param boolean $allowSyncMySettings true to allow syncing settings
     *     between devices, false to disallow. (Default: true)
     */
    public function setAllowSyncMySettings($allowSyncMySettings)
    {
        $this->setBoolAsInt("Experience/AllowSyncMySettings", $allowSyncMySettings);
    }

    protected function setBoolAsInt($locUri, $data, $command = null)
    {
        if (is_array($data)) {
            $data['data'] = $data['data'] ? 1 : 0;
        } else {
            $data = array("data" => $data ? 1 : 0);
        }
        $data['format'] = "int";
        $this->setData($locUri, $data, $command);
    }

    protected function getDefaults()
    {
        return array(
            "DeviceLock/DevicePasswordEnabled" => array("data" => 1, "format" => "int"),
            "DeviceLock/AllowSimpleDevicePassword" => array("data" => 1, "format" => "int"),
            "DeviceLock/MinDevicePasswordLength" => array("data" => 4, "format" => "int"),
            "DeviceLock/AlphanumericDevicePasswordRequired" => array("data" => 1, "format" => "int"),
            "DeviceLock/DevicePasswordExpiration" => array("data" => 0, "format" => "int"),
            "DeviceLock/DevicePasswordHistory" => array("data" => 0, "format" => "int"),
            "DeviceLock/MaxDevicePasswordFailedAttempts" => array("data" => 0, "format" => "int"),
            "DeviceLock/MaxInactivityTimeDeviceLock" => array("data" => 0, "format" => "int"),
            "DeviceLock/MinDevicePasswordComplexCharacters" => array("data" => 1, "format" => "int"),

            "WiFi/AllowWiFi" => array("data" => 1, "format" => "int"),
            "WiFi/AllowInternetSharing" => array("data" => 1, "format" => "int"),
            "WiFi/AllowAutoConnectToWiFiSenseHotspots" => array("data" => 1, "format" => "int"),
            "WiFi/AllowWiFiHotSpotReporting" => array("data" => 1, "format" => "int"),
            "WiFi/AllowManualWiFiConfiguration" => array("data" => 1, "format" => "int"),

            "Connectivity/AllowNFC" => array("data" => 1, "format" => "int"),
            "Connectivity/AllowBluetooth" => array("data" => 2, "format" => "int"),
            "Connectivity/AllowVPNRoamingOverCellular" => array("data" => 1, "format" => "int"),
            "Connectivity/AllowVPNOverCellular" => array("data" => 1, "format" => "int"),

            "System/AllowStorageCard" => array("data" => 1, "format" => "int"),
            "System/AllowTelemetry" => array("data" => 1, "format" => "int"),

            "Experience/AllowCopyPaste" => array("data" => 1, "format" => "int"),

            "Accounts/AllowMicrosoftAccountConnection" => array("data" => 1, "format" => "int"),
            "Accounts/AllowAddingNonMicrosoftAccountsManually" => array("data" => 1, "format" => "int"),

            "Security/AllowManualRootCertificateInstallation" => array("data" => 1, "format" => "int"),
            "Security/RequireDeviceEncryption" => array("data" => 1, "format" => "int"),

            "ApplicationManagement/AllowStore" => array("data" => 1, "format" => "int"),
            "ApplicationManagement/ApplicationRestrictions" => array("data" => "", "format" => "chr"),
            "ApplicationManagement/AllowDeveloperUnlock" => array("data" => 1, "format" => "int"),

            "Browser/AllowBrowser" => array("data" => 1, "format" => "int"),

            "Experience/AllowScreenCapture" => array("data" => 1, "format" => "int"),

            "System/AllowLocation" => array("data" => 1, "format" => "int"),

            "Connectivity/AllowUSBConnection" => array("data" => 1, "format" => "int"),
            "Connectivity/AllowCellularDataRoaming" => array("data" => 1, "format" => "int"),

            "Camera/AllowCamera" => array("data" => 1, "format" => "int"),

            "Search/AllowSearchToUseLocation" => array("data" => 1, "format" => "int"),
            "Search/SafeSearchPermissions" => array("data" => 1, "format" => "int"),
            "Search/AllowStoringImagesFromVisionSearch" => array("data" => 1, "format" => "int"),

            "Experience/AllowVoiceRecording" => array("data" => 1, "format" => "int"),
            "Experience/AllowSaveAsOfOfficeFiles" => array("data" => 1, "format" => "int"),

            "DeviceLock/AllowIdleReturnWithoutPassword" => array("data" => 1, "format" => "int"),

            "Experience/AllowCortana" => array("data" => 1, "format" => "int"),
            "Experience/AllowSyncMySettings" => array("data" => 1, "format" => "int"),
        );
    }
}
