<?php
namespace Topalis\WinPhone\OMA\SyncML;

use Topalis\Kernel\AbstractAuralisService;
use Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider\ConfigurationServiceProviderFactory;
use Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider\AbstractConfigurationServiceProvider;

use Topalis\Device\Entity\Device;

// @codingStandardsIgnoreStart
// ignore rule Generic.PHP.ForbiddenFunctions.FoundWithAlternative
use Topalis\WinPhone\OMA\SyncML\Entity\Delete;
// @codingStandardsIgnoreStart
use Topalis\WinPhone\OMA\SyncML\Entity\Item;
use Topalis\WinPhone\OMA\SyncML\Entity\Target;
use Topalis\WinPhone\OMA\SyncML\Entity\Exec;
use Topalis\WinPhone\OMA\SyncML\Entity\Meta;
use Topalis\WinPhone\OMA\SyncML\Entity\Get;
use Topalis\WinPhone\Entity\SyncMLCommand;
use Topalis\WinPhone\Entity\SyncMLCSPStatus;
use Topalis\WinPhone\OMA\SyncML\Entity\AbstractCommandGroup;
use Topalis\WinPhone\OMA\SyncML\Entity\Replace;
use Topalis\WinPhone\WinPhoneConstants;
use Topalis\WinPhone\OMA\SyncML\Entity\XmlConverter;
use Topalis\Wifi\Entity\WifiConfig;
use Topalis\Admin\Entity\Admin;

/**
 * Class to manage Windows Phone devices through MDM.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class SyncMLAdmin extends AbstractAuralisService
{
    /**
     * @var \Topalis\Admin\Entity\Admin
     */
    private $admin;

    /**
     *
     * @return \Topalis\Admin\Entity\Admin
     */
    private function getAdmin()
    {
        if ($this->admin === null) {
            $token = $this->getSecurityContext()->getToken();
            if (null !== $token && $token->getUser() instanceof Admin) {
                $this->admin = $token->getUser();
            } else {
                $this->admin = $this->getAdminEntityManager()->find(0);
            }
        } else if (!$this->getAdminEntityManager()->contains($this->admin)) {
            // if the admin is not managed anymore because the enity manager was
            // cleared it has to be reloaded to prevent an exception
            $this->admin = $this->getAdminEntityManager()->find($this->admin->getId());
        }

        return $this->admin;
    }

    /**
     * @return \Topalis\Admin\Entity\AdminEntityManager
     */
    private function getAdminEntityManager()
    {
        return $this->container->get("admin.entity.manager.admin");
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\WinPhone\Entity\SyncMLCommandEntityManager
     */
    private function getCommandEntityManager()
    {
        return $this->container->get("winphone.entity.manager.syncml_command");
    }

    /**
     *
     * @return \Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider\ConfigurationServiceProviderFactory
     */
    private function getCSPFactory()
    {
        $factory = new ConfigurationServiceProviderFactory();
        $factory->setContainer($this->container);
        return $factory;
    }

    /**
     * @return \Topalis\WinPhone\Entity\SyncMLCSPStatusEntityManager
     */
    private function getCSPStatusEntityManager()
    {
        return $this->container->get("winphone.entity.maanger.syncml_cspstatus");
    }

    /**
     * @return \Symfony\Component\Security\Core\SecurityContext
     */
    private function getSecurityContext()
    {
        return $this->container->get("security.context");
    }

    /**
     *
     * @param Device $device
     * @param \Topalis\WinPhone\OMA\SyncML\Entity\AbstractCommand $command
     * @throws \Exception
     * @return \Topalis\WinPhone\Entity\SyncMLCommand
     */
    public function addCommand(Device $device, \Topalis\WinPhone\OMA\SyncML\Entity\AbstractCommand $command, $info = null)
    {
        if (!$device->isManaged() || !$device->isActive()) {
            throw new \Exception("Commands can only be added for active managed devices");
        }

        $this->getLogger()->debug("Add SyncML command for device " . $device->getId());
        $entity = new SyncMLCommand();
        $entity->setAdmin($this->getAdmin());
        $entity->setCommandType($command->getElementName());
        $entity->setDevice($device);
        $entity->setCreateTime($this->getTimeProvider()->getTime());
        $entity->setInfo($info);
        $this->getCommandEntityManager()->save($entity);
        $command->setCmdID($entity->getId());
        $entity->setCommand($command->toXml());

        if ($command instanceof AbstractConfigurationServiceProvider) {
            $entity->setDefaultCommandType($command->getDefaultCommand());
        } else if ($command instanceof AbstractCommandGroup) {
            // detect most often used command
            // use null if there's no unique maximum
            $typesCount = $command->countCommandTypes();
            reset($typesCount);
            $maxType = key($typesCount);
            $defaultType = $maxType;
            while (($count = next($typesCount)) !== false) {
                if ($count > $typesCount[$maxType]) {
                    $defaultType = $maxType = key($typesCount);
                } else if ($count == $typesCount[$maxType]) {
                    $defaultType = null;
                }
            }
            $entity->setDefaultCommandType($defaultType);
        } else {
            $entity->setDefaultCommandType($command->getElementName());
        }

        $this->getCommandEntityManager()->save($entity);
        return $entity;
    }

    public function cancelRemainingCommands(Device $device)
    {
        $commandEm = $this->getCommandEntityManager();
        $changed = $commandEm->createQueryBuilder("c")
            ->update()->set("c.status", ":canceled")
            ->where("c.device = :device AND c.status = :new")
            ->getQuery()->execute(array(
                    "canceled" => SyncMLCommand::STATUS_CANCELED,
                    "device" => $device,
                    "new" => SyncMLCommand::STATUS_NEW
            ));
        if (!$changed) {
            return;
        }

        $qb3 = $commandEm->createQueryBuilder("c2")
            ->select("MAX(c2.id)")
            ->where("c2.cspStatus = s");
        $qb2 = $commandEm->createQueryBuilder("c")
            ->where("c.status = :waiting")
            ->andWhere("c.id = (" . $qb3->getDQL() . ")");
        $qb = $this->getCSPStatusEntityManager()->createQueryBuilder("s")
            ->update()->set("s.pendingOperation", ":null")
            ->where("s.device = :device")
            ->andWhere("NOT EXISTS (" . $qb2->getDQL() . ")")
            ->setParameters(array("null" => null, "device" => $device, "waiting" => SyncMLCommand::STATUS_WAITING));

        $changed = $qb->getQuery()->execute();
        if ($changed) {
            $this->getCSPStatusEntityManager()->refreshForDevice($device);
        }
    }

    public function cancelCommandsForCSP(Device $device, SyncMLCSPStatus $cspStatus)
    {
        $commandEm = $this->getCommandEntityManager();
        $changed = $commandEm->createQueryBuilder("c")
            ->update()->set("c.status", ":canceled")
            ->where("c.device = :device AND c.status = :new")
            ->andWhere("c.cspStatus = :cspStatus")
            ->getQuery()->execute(array(
                "canceled" => SyncMLCommand::STATUS_CANCELED,
                "device" => $device,
                "new" => SyncMLCommand::STATUS_NEW,
                "cspStatus" => $cspStatus
            ));
        if (!$changed) {
            return;
        }

        $lastCommand = $cspStatus->getCommands()->last();
        $commandEm->refresh($lastCommand);
        if ($lastCommand->getStatus() != SyncMLCommand::STATUS_WAITING) {
            $cspStatus->setPendingOperation(null);
            $this->getCSPStatusEntityManager()->save($cspStatus);
        }
    }

    /**
     * Enable encryption of the internal storage of the specified device.
     * Note: cannot be undone.
     *
     * @param Device $device The device to encrypt.
     */
    public function enableEncryption(Device $device)
    {
        $this->getLogger()->info("Enable encryption for device " . $device->getId() . " if not enabled, yet");
        $deviceInfo = $device->getDeviceinformation();
        if (isset($deviceInfo['Encrypted']) && $deviceInfo['Encrypted']) {
            // already encrypted
            return;
        }

        $locURI = "./Vendor/MSFT/Registry/HKLM/Software/Microsoft/Provisioning/DeviceEncryption/Enabled";
        $item = $this->createCommandItem($locURI, 1, array("Format" => "int"));
        $replace = new Replace();
        $replace->addItem($item);
        $this->addCommand($device, $replace, "Enable encryption");
    }

    public function initDevice(Device $device)
    {
        $this->getLogger()->debug("Init device " . $device->getId());
        $this->requestDeviceDetail($device);

        if (version_compare($device->getVersion(), "8.1", ">=")) {
            $this->installPolicyManagerCSP($device);
            $this->enrollScepClientCert($device, "auralis MDM");
        } else {
            $this->installDeviceLockCSP($device);
            $this->installStorageCSP($device);
            $restrictions = $device->getConfigGroup()->getWinPhoneRestrictions();
            // enable encryption if not explicitly disabled
            $encrypt = !isset($restrictions['requireDeviceEncryption']) || $restrictions['requireDeviceEncryption'];
            if ($encrypt) {
                $this->enableEncryption($device);
            }
        }
        $this->installActiveSyncCSPs($device);
        $this->installWiFiCSPs($device);
    }

    /**
     * Installs an ActiveSync CSP for the specified mail account on the device
     * the mail account belongs to.
     *
     * @param \Topalis\Device\Entity\MailAccount $mailAccount The mail account
     *     to install an ActiveSync CSP for.
     * @param boolean $force true to replace existing ActiveSync CSP. Otherwise
     *     only new accounts are added.
     */
    public function installActiveSyncCSP(\Topalis\Device\Entity\MailAccount $mailAccount, $force = true)
    {
        $this->getLogger()->debug("Install ActiveSync CSP '" . $mailAccount->getEmail()
            . "' (id " . $mailAccount->getId() . ") on device " . $mailAccount->getDevice()->getId());
        $csp = $this->getCSPFactory()->createActiveSyncCSP($mailAccount);

        $device = $mailAccount->getDevice();
        $locURIPrefix = rtrim($csp->getLocURIPrefix(), "/");
        if (!$force) {
            $cspStatus = $this->getCSPStatusEntityManager()->getOneOrNullForDevice($device, $locURIPrefix);
            if ($cspStatus !== null && ($cspStatus->getPendingOperation() != "Delete"
                || ($cspStatus->getPendingOperation() == "" && $cspStatus->isInstalled()))) {
                return;
            }
        }

        $mailAccount->setUuid($csp->getGuID());
        $this->installCSP($device, $csp, "Install ActiveSync " . $csp->getEmailAddress());
    }

    /**
     * Installs ActiveSync CSPs for all mail accounts of the specified device
     * that are not installed, yet, i.e. they have no referenced certificate.
     *
     * @param \Topalis\Device\Entity\Device $device The device to install
     *     ActiveSync CSPs for its mail accounts.
     * @param boolean $force If false the profile for each mail account is only
     *     installed if no certificate is related to it.
     */
    public function installActiveSyncCSPs(\Topalis\Device\Entity\Device $device, $force = true)
    {
        foreach ($device->getMailAccounts() as $mailAccount) {
            $this->installActiveSyncCSP($mailAccount, $force);
        }
    }

    public function installCSP(Device $device, AbstractConfigurationServiceProvider $csp, $info = null)
    {
        $locURIPrefix = rtrim($csp->getLocURIPrefix(), "/");
        $cspStatus = $this->getCSPStatusEntityManager()->getOneOrNullForDevice($device, $locURIPrefix);

        if ($cspStatus === null) {
            $cspStatus = new SyncMLCSPStatus();
            $command = "Add";
        } else {
            $this->cancelCommandsForCSP($device, $cspStatus);
            switch ($cspStatus->getPendingOperation()) {
                case "Add":
                    $command = "Replace";
                    break;
                case "Delete":
                    $command = "Add";
                    break;
                case "Replace":
                    $command = "Replace";
                    break;
                default:
                    if ($cspStatus->isInstalled()) {
                        $command = "Replace";
                    } else {
                        $command = "Add";
                    }
                    break;
            }
        }

        $csp->setDefaultCommand($command);
        $cspStatus->setDevice($device);
        $cspStatus->setPendingOperation($command);
        $cspStatus->setLocURIPrefix($locURIPrefix);

        if ($info === null) {
            $info = "Install " . $this->getSimplifiedCSPClass($csp);
        }

        $this->getCommandEntityManager()->beginTransaction();
        $commandEntity = $this->addCommand($device, $csp, $info);
        $cspStatus->addCommand($commandEntity);
        $this->getCSPStatusEntityManager()->save($cspStatus);
        $this->getCommandEntityManager()->commit();
    }

    /**
     * @param Device $device
     */
    public function installDeviceLockCSP(Device $device)
    {
        $csp = $this->getCSPFactory()->createDeviceLockCSP($device);
        if ($csp === null) {
            $this->removeDeviceLockCSP($device);
        } else {
            $this->getLogger()->debug("Install DeviceLock CSP for device " . $device->getId());
            $this->installCSP($device, $csp);
        }
    }

    /**
     * @param Device $device
     * @param string $purpose
     */
    public function enrollScepClientCert(Device $device, $purpose)
    {
        $clientCert = $this->getCertificateManager()->initClientCert($device);
        $caCert = $this->getCertificateManager()->getCACertificate();
        $csp = $csp = $this->getCSPFactory()->createScepCSP($purpose, $caCert,
            $clientCert->getPlainChallenge());
        $this->installCSP($device, $csp, "SCEP");
    }

    /**
     * @param Device $device
     */
    public function installPolicyManagerCSP(Device $device)
    {
        $csp = $this->getCSPFactory()->createPolicyManagerCSP($device);
        $this->getLogger()->debug("Install PolicyManager CSP for device " . $device->getCombinedId());
        $this->installCSP($device, $csp);
    }

    /**
     *
     * @param Device $device
     */
    public function installStorageCSP(Device $device)
    {
        $csp = $this->getCSPFactory()->createStorageCSP($device);
        if ($csp === null) {
            $this->removeStorageCSP($device);
        } else {
            $this->getLogger()->debug("Install Storage CSP for device " . $device->getId());
            $info = $csp->isDisabled() ? "Disallow SD cards" : "Allow SD cards";
            $this->installCSP($device, $csp, $info);
        }
    }

    public function installWiFiCSP(Device $device, $ssid)
    {
        $csp = $this->getCSPFactory()->createWiFiCSP($device, $ssid);
        $this->installCSP($device, $csp, "Install WiFi " . $csp->getSsid());
    }

    public function installWiFiCSPFromConfig(Device $device, WifiConfig $wifiConfig)
    {
        $csp = $this->getCSPFactory()->createWiFiCSPFromConfig($device, $wifiConfig);
        $this->installCSP($device, $csp, "Install WiFi " . $csp->getSsid());
    }

    public function installWiFiCSPs(Device $device)
    {
        $csps = $this->getCSPFactory()->createWifiCSPs($device);
        foreach ($csps as $csp) {
            $this->installCSP($device, $csp, "Install WiFi " . $csp->getSsid());
        }
    }

    /**
     * Locks the specified device.
     *
     * @param Device $device The device to lock.
     * @since Windows Phone 8.1
     */
    public function lockDevice(Device $device)
    {
        $exec = new Exec();
        $exec->addItem($this->createCommandItem("./Vendor/MSFT/RemoteLock/Lock"));
        $this->addCommand($device, $exec, "Lock");
    }

    public function removeActiveSyncCSP(\Topalis\Device\Entity\MailAccount $mailAccount)
    {
        $locURIPrefix = "./Vendor/MSFT/ActiveSync/Accounts/%7B" . $mailAccount->getUuid() . "%7D";
        $this->removeCSP($mailAccount->getDevice(), $locURIPrefix, "Remove ActiveSync " . $mailAccount->getEmail());
    }

    private function removeCertificateCSPs(Device $device)
    {
        $prefixes = array(
            "./Vendor/MSFT/CertificateStore/My/SCEP/",
            "./Vendor/MSFT/CertificateStore/CA/System/"
        );
        foreach ($prefixes as $prefix) {
            $cspStatusList = $this->getCSPStatusEntityManager()->getForDevice($device, $prefix);
            foreach ($cspStatusList as $cspStatus) {
                $this->removeCSP($device, $cspStatus->getLocURIPrefix());
            }
        }
    }

    public function removeCSP(Device $device, $locURIPrefix, $info = null)
    {
        $cspStatus = $this->getCSPStatusEntityManager()->getOneOrNullForDevice($device, $locURIPrefix);
        if ($cspStatus === null) {
            return;
        }

        $this->cancelCommandsForCSP($device, $cspStatus);
        if ((!$cspStatus->isInstalled() && $cspStatus->getPendingOperation() == "")
            || $cspStatus->getPendingOperation() == "Delete") {
            return;
        }

        $cspStatus->setPendingOperation("Delete");
        $commandEntity = $this->removeLocURI($device, $locURIPrefix, $info);
        $cspStatus->addCommand($commandEntity);
        $this->getCSPStatusEntityManager()->save($cspStatus);
    }

    /**
     *
     * @param Device $device
     */
    public function removeDeviceLockCSP(Device $device)
    {
        $this->getLogger()->debug("Remove DeviceLock CSP for device " . $device->getId());
        $locURIPrefix = "./Vendor/MSFT/DeviceLock/Provider/" . WinPhoneConstants::PROVIDER_ID;
        $this->removeCSP($device, $locURIPrefix, "Remove DeviceLock");
    }

    /**
     *
     * @param Device $device
     */
    public function removeStorageCSP(Device $device)
    {
        $this->getLogger()->debug("Remove Storage CSP for device " . $device->getId());
        $locURIPrefix = "./Vendor/MSFT/Storage";
        $this->removeCSP($device, $locURIPrefix);
    }

    /**
     *
     * @param Device $device
     * @param string $locURI
     * @return \Topalis\WinPhone\Entity\SyncMLCommand
     */
    private function removeLocURI(Device $device, $locURI, $info = null)
    {
        // @codingStandardsIgnoreStart
        // ignore rule Generic.PHP.ForbiddenFunctions.FoundWithAlternative
        $command = new Delete();
        $item = $this->createCommandItem($locURI);
        $command->addItem($item);

        if ($info === null) {
            // remove prefix
            $info = "Remove " . str_replace("./Vendor/MSFT/", "", $locURI);
        }

        return $this->addCommand($device, $command, $info);
        // @codingStandardsIgnoreEnd
    }

    private function removeWiFiCSPs(Device $device)
    {
        $cspStatusList = $this->getCSPStatusEntityManager()->getForDevice($device, "./Vendor/MSFT/WiFi/Profile/");
        foreach ($cspStatusList as $cspStatus) {
            $ssid = strstr(str_replace("./Vendor/MSFT/WiFi/Profile/", "", $cspStatus->getLocURIPrefix()), "/", true);
            $this->removeCSP($device, $cspStatus->getLocURIPrefix(), "Remove WiFi " . $ssid);
        }
    }

    public function removeWiFiCSP(Device $device, $ssid)
    {
        $locURIPrefix = "./Vendor/MSFT/WiFi/Profile/" . $ssid;
        $this->removeCSP($device, $locURIPrefix, "Remove WiFi " . $ssid);
    }

    public function requestDeviceDetail(Device $device)
    {
        // see DevDetail configuration service provider
        // @codingStandardsIgnoreStart
        static $locURIs = array(
            "./DevDetail/DevTyp", // phone model name
            "./DevDetail/OEM", // OEM as defined in the specification SyncML Device Information, version 1.1.2
            "./DevDetail/FwV", // firmware version
            "./DevDetail/SwV", // Windows Phone OS software version
            "./DevDetail/HwV", // hardware version
            // "./DevDetail/LrgObj", // flag whether phone uses OMA DM Large Object Handling. Always false.
            // "./DevDetail/URI/MaxDepth", // Maximum depth of the tree (uri segments) that the phone supports. Default "0" indicates unlimited.
            // "./DevDetail/URI/MaxTotLen", // Maximum total length of any URI. The default "0" indicates unlimited.
            // "./DevDetail/URI/MaxSegLen", // Maximum total length of a single URI segment. Default "0" indicates unlimited,
            "./DevDetail/Ext/Microsoft/MobileID", // mobile phone ID associated with the cellular network
            "./DevDetail/Ext/Microsoft/LocalTime", // local time in ISO 8601 format
            "./DevDetail/Ext/Microsoft/OSPlatform", // OS platform
            "./DevDetail/Ext/Microsoft/ProcessorType", // processor type
            "./DevDetail/Ext/Microsoft/RadioSwV", // software version number
            "./DevDetail/Ext/Microsoft/Resolution", // UI screen resolution (e.g. "480x800")
            "./DevDetail/Ext/Microsoft/CommercializationOperator", // mobile operator
            "./DevDetail/Ext/Microsoft/ProcessorArchitecture", // processor archtitecture (e.g. "arm" or "x86")
            "./DevDetail/Ext/Microsoft/DeviceName", // user-specified phone name
            "./DevDetail/Ext/WLANMACAddress", // MAC address of active WLAN connection, as 12-digit hex number,
        );
        // @codingStandardsIgnoreEnd

        // URIs for WP 8.1
        static $locURIs81 = array(
            "./Vendor/MSFT/DeviceInstanceService/Roaming",
            "./Vendor/MSFT/DeviceInstanceService/PhoneNumber",
            "./Vendor/MSFT/DeviceInstanceService/IMEI",
            "./Vendor/MSFT/DeviceInstanceService/IMSI",
            "./Vendor/MSFT/PolicyManager/Device/Security/RequireDeviceEncryption",
        );

        $commands = $this->getCommandEntityManager()->createQueryBuilder("c")
            ->where("c.device = :device")
            ->andWhere("c.commandType = 'Get'")
            ->andWhere("c.status = :new OR c.status = :waiting")
            ->setParameter("device", $device)
            ->setParameter("new", SyncMLCommand::STATUS_NEW)
            ->setParameter("waiting", SyncMLCommand::STATUS_WAITING)
            ->getQuery()->getResult();

        foreach ($commands as $commandEntity) {
            $command = XmlConverter::fromXml($commandEntity->getCommand());
            $item = $command->getItem(0);
            if ($item !== null) {
                if (array_search($item->getTarget()->getLocUri(), $locURIs) !== false) {
                    $this->getLogger()->debug("A request for device information of device " . $device->getId()
                        . " is already in the queue");
                    return;
                }
            }
        }

        $command = new Get();
        foreach ($locURIs as $locURI) {
            $command->addItem($this->createCommandItem($locURI));
        }
        if (version_compare($device->getVersion(), "8.1", "<")) {
            // encryption state of internal storage on WP 8.0
            $command->addItem($this->createCommandItem(
                "./Vendor/MSFT/Registry/HKLM/Software/Microsoft/Provisioning/DeviceEncryption/Enabled"));
        } else if (version_compare($device->getVersion(), "8.1", ">=")) {
            foreach ($locURIs81 as $locURI) {
                $command->addItem($this->createCommandItem($locURI));
            }
        }

        $this->addCommand($device, $command, "Device information");
    }

    public function resendCommand(Device $device, SyncMLCommand $command)
    {
        if ($command->getDevice() != $device) {
            throw new \InvalidArgumentException("The SyncML command with the id " . $command->getId()
                . " is not destined for the specified device");
        }

        $newCommand = clone $command;
        $newCommand->setCreateTime($this->getTimeProvider()->getTime());
        $newCommand->setAdmin($this->getAdmin());
        $newCommand->setStatus(SyncMLCommand::STATUS_NEW);
        $newCommand->setResponseTime(null);
        $newCommand->setStatusCode(null);
        $newCommand->setStatusCodes(array());

        $command->incCopyCount();

        $em = $this->getCommandEntityManager();
        $em->save($command, false);
        $em->save($newCommand);
    }

    public function remoteWipe(Device $device)
    {
        $this->cancelRemainingCommands($device);
        $command = new Exec();
        $item = $this->createCommandItem("./Vendor/MSFT/RemoteWipe/doWipe");
        $command->setItem($item);
        $this->addCommand($device, $command, "Factory reset");
    }

    public function unenroll(Device $device)
    {
        $this->cancelRemainingCommands($device);
        $this->getCommandEntityManager()->beginTransaction();
        foreach ($device->getMailAccounts() as $mailAccount) {
            $this->removeActiveSyncCSP($mailAccount);
        }
        $this->removeWiFiCSPs($device);
        $this->removeDeviceLockCSP($device);
        $this->removeStorageCSP($device);
        $this->removeCertificateCSPs($device);

        $unenrollCommand = new Exec();
        $unenrollCommand->addItem($this->createCommandItem("./Vendor/MSFT/DMClient/Unenroll",
            WinPhoneConstants::PROVIDER_ID, array("Format" => "chr")));
        $this->addCommand($device, $unenrollCommand, "Wipe");
        $this->getCommandEntityManager()->commit();
    }

    private function createCommandItem($locURI, $data = null, array $meta = null)
    {
        $item = new Item();
        $target = new Target();
        $target->setLocURI($locURI);
        $item->setTarget($target);
        $item->setData($data);
        if ($meta !== null) {
            $item->setMeta(new Meta($meta));
        }

        return $item;
    }

    private function getSimplifiedCSPClass(AbstractConfigurationServiceProvider $csp)
    {
        $className = get_class($csp);
        $pos = strrpos($className, "\\");
        // remove namespace and "CSP" at the end of the class name
        return preg_replace("/CSP$/", "", substr($className, $pos + 1));
    }
}
