<?php
namespace Topalis\WinPhone\OMA\SyncML;

use Symfony\Component\DependencyInjection\ContainerAware;
use Topalis\Device\MDM\MdmFacadeInterface;
use Topalis\Device\Entity\Device;

/**
 * MDM facade for Windows Phone devices.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class SyncMLMdmFacade extends ContainerAware implements MdmFacadeInterface
{
    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     *
     * @return integer
     */
    private function getDeviceOs()
    {
        static $deviceOs = null;
        if ($deviceOs === null) {
            $deviceTypeFactory = $this->container->get("device.type.factory");
            $deviceOs = $deviceTypeFactory->getType("winphone")->getId();
        }

        return $deviceOs;
    }

    /**
     * @return SyncMLAdmin
     */
    private function getSyncMLAdmin()
    {
        return $this->container->get("winphone.oma.syncml.admin");
    }

    /**
     *
     * @return array Array of active (i.e. not disabled, not deleted, no
     *     wipe pending) managed WindowsPhone devices.
     */
    public function getActiveManagedDevices()
    {
        $devices = $this->getDeviceEntityManager()->createQueryBuilder("d")
            ->select("d")
            ->where("d.os = :os")
            ->andWhere("d.disabled = 0")
            ->andWhere("d.deleted = 0")
            ->andWhere("d.wipestatus = :wipestatus")
            ->andWhere("d.mdmCert IS NOT NULL")
            ->setParameter("os", $this->getDeviceOs())
            ->setParameter("wipestatus", Device::WIPE_NONE)
            ->getQuery()
            ->getResult()
        ;

        return $devices;
    }

    /**
     * @return \Topalis\WinPhone\Entity\SyncMLCommandEntityManager
     */
    private function getCommandEntityManager()
    {
        return $this->container->get("winphone.entity.manager.syncml_command");
    }

    public function getCommands(\Topalis\Device\Entity\Device $device, $limit = 0, array $orderBy = null)
    {
        $qb = $this->getCommandEntityManager()->createQueryBuilder("c")
            ->where("c.device = :device")
            ->orderBy("c.id", "DESC")
            ->setParameter("device", $device);
        if ($limit > 0) {
            $qb->setMaxResults($limit);
        }
        foreach ($orderBy as $key => $value) {
            if (is_int($key)) {
                $qb->orderBy("c." . $value);
            } else {
                $qb->orderBy("c." . $key, $value);
            }
        }

        return $qb->getQuery()->getResult();
    }

    public function isActiveManagedDevice($device)
    {
        return $device->isActive() && $device->isManaged() && $device->getOs() == $this->getDeviceOs();
    }

    public function installMailAccount(\Topalis\Device\Entity\MailAccount $mailAccount)
    {
        $this->getSyncMLAdmin()->installActiveSyncCSP($mailAccount, true);
    }

    public function removeMailAccount(\Topalis\Device\Entity\MailAccount $mailAccount)
    {
        $this->getSyncMLAdmin()->removeActiveSyncCSP($mailAccount);
    }

    public function requestDeviceInfo(\Topalis\Device\Entity\Device $device)
    {
        $this->getSyncMLAdmin()->requestDeviceDetail($device);
    }

    public function requestDeviceInfos()
    {
        foreach ($this->getActiveManagedDevices() as $device) {
            $this->getSyncMLAdmin()->requestDeviceDetail($device);
        }
    }

    public function resendCommand(\Topalis\Device\Entity\Device $device, $commandId)
    {
        $command = $this->getCommandEntityManager()->find($commandId);
        if ($command === null) {
            throw new \InvalidArgumentException("No SyncML command with the id " . $commandId . " found");
        }

        $this->getSyncMLAdmin()->resendCommand($device, $command);
    }

    public function lockDevice(\Topalis\Device\Entity\Device $device, $message = null, $phoneNumber = null)
    {
        // displaying a message is not supported by WP
        $message; // unused
        $phoneNumber; // unused
        $this->getSyncMLAdmin()->lockDevice($device);
    }

    public function eraseDevice(\Topalis\Device\Entity\Device $device)
    {
        $this->getSyncMLAdmin()->remoteWipe($device);
        $device->setWipestatus(Device::WIPE_PENDING);
    }

    public function wipeDevice(\Topalis\Device\Entity\Device $device)
    {
        $this->getSyncMLAdmin()->unenroll($device);
        $device->setWipestatus(Device::WIPE_PENDING);
    }

    public function updateAppPolicy(\Topalis\App\Entity\WinPhoneAppPolicy $appPolicy,
        array $removedGroups, array $addedGroups, array $changeSet)
    {
        $removedGroups; // unused

        if (!isset($changeSet['policy'])) {
            return;
        }

        $syncMLAdmin = $this->getSyncMLAdmin();
        foreach ($appPolicy->getConfigGroups() as $group) {
            if (in_array($group, $addedGroups)) {
                // update will be done in updateConfigGroup() because group is
                // changed
                continue;
            }

            foreach ($group->getDevices() as $device) {
                if (!$this->isActiveManagedDevice($device)) {
                    continue;
                }

                $syncMLAdmin->installPolicyManagerCSP($device);
            }
        }
    }

    public function updateConfigGroup(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup,
        \Doctrine\ORM\UnitOfWork $uow)
    {
        $changeSet = $uow->getEntityChangeSet($configGroup);
        $updateDeviceLockCSP = isset($changeSet['winPhonePassCode']);
        $updateActiveSyncCSP = isset($changeSet['winPhoneExchangeConfig']);

        $enableEncryption = false;
        $updateStorageCSP = false;
        if (isset($changeSet['winPhoneRestrictions'])) {
            // enable encryption if not explicitly disabled
            $oldEncrypt = !isset($changeSet['winPhoneRestrictions'][0]['requireDeviceEncryption'])
                || $changeSet['winPhoneRestrictions'][0]['requireDeviceEncryption'];
            $newEncrypt = !isset($changeSet['winPhoneRestrictions'][1]['requireDeviceEncryption'])
                || $changeSet['winPhoneRestrictions'][1]['requireDeviceEncryption'];
            if ($newEncrypt && $newEncrypt != $oldEncrypt) {
                $enableEncryption = true;
            }
            $oldAllowStorage = isset($changeSet['winPhoneRestrictions'][0]['allowStorageCard']) ?
                $changeSet['winPhoneRestrictions'][0]['allowStorageCard'] : 1;
            $newAllowStorage = isset($changeSet['winPhoneRestrictions'][0]['allowStorageCard']) ?
                $changeSet['winPhoneRestrictions'][0]['allowStorageCard'] : 1;
            $updateStorageCSP = $oldAllowStorage != $newAllowStorage;
        }

        $updatePolicyManagerCSP = $updateDeviceLockCSP || isset($changeSet['winPhoneRestrictions'])
            || $enableEncryption || isset($changeSet['winPhoneAppPolicy']);

        if (!$enableEncryption && !$updateDeviceLockCSP && !$updateStorageCSP
            && !$updateActiveSyncCSP && !$updatePolicyManagerCSP) {
            return;
        }

        $syncMLAdmin = $this->getSyncMLAdmin();
        foreach ($configGroup->getDevices() as $device) {
            if (!$this->isActiveManagedDevice($device)) {
                continue;
            }

            if (version_compare($device->getVersion(), "8.1", ">=")) {
                if ($updatePolicyManagerCSP) {
                    $syncMLAdmin->installPolicyManagerCSP($device);
                }
            } else {
                if ($enableEncryption) {
                    $syncMLAdmin->enableEncryption($device);
                }

                if ($updateDeviceLockCSP) {
                    $syncMLAdmin->installDeviceLockCSP($device);
                }
                if ($updateStorageCSP) {
                    $syncMLAdmin->installStorageCSP($device);
                }
            }

            if ($updateActiveSyncCSP) {
                $syncMLAdmin->installActiveSyncCSPs($device);
            }
        }
    }

    public function updateDevice(\Topalis\Device\Entity\Device $device, \Doctrine\ORM\UnitOfWork $uow)
    {
        $changeSet = $uow->getEntityChangeSet($device);
        $syncMLAdmin = $this->getSyncMLAdmin();

        if (isset($changeSet['configGroup'])) {
            if (version_compare($changeSet['version'][0], "8.1", "<")) {
                $syncMLAdmin->installDeviceLockCSP($device);
                $syncMLAdmin->installStorageCSP($device);
                $restrictions = $changeSet['configGroup'][1]->getWinPhoneRestrictions();
                if (isset($restrictions['requireDeviceEncryption']) && $restrictions['requireDeviceEncryption']) {
                    $this->getSyncMLAdmin()->enableEncryption($device);
                }
            } else {
                $this->getSyncMLAdmin()->installPolicyManagerCSP($device);
            }

            $newWlanConfigs = $changeSet['configGroup'][1]->getMSWlanConfigs();
            $oldWlanConfigsAssoc = array();
            foreach ($changeSet['configGroup'][0]->getMSWlanConfigs() as $wlanConfig) {
                $oldWlanConfigsAssoc[$wlanConfig->getSsid()] = $wlanConfig;
            }

            foreach ($newWlanConfigs as $wlanConfig) {
                $ssid = $wlanConfig->getSsid();
                if (isset($oldWlanConfigsAssoc[$ssid])) {
                    if ($oldWlanConfigsAssoc[$ssid]->getProfile() != $wlanConfig->getProfile()) {
                        $syncMLAdmin->installWiFiCSP($device, $ssid);
                    }
                    unset($oldWlanConfigsAssoc[$ssid]);
                } else {
                    $syncMLAdmin->installWiFiCSP($device, $ssid);
                }
            }

            foreach (array_keys($oldWlanConfigsAssoc) as $ssid) {
                $syncMLAdmin->removeWiFiCSP($device, $ssid);
            }
        }

        if (isset($changeSet['version']) && $changeSet['version'][0] !== null) {
            $wasWp80 = version_compare($changeSet['version'][0], "8.1", "<");
            $isWp81 = version_compare($changeSet['version'][1], "8.1", ">=");
            if ($wasWp80 && $isWp81) {
                // device updated from WP 8.0 to WP 8.1 => use PolicyManager CSP
                // instead of DeviceLock CSP and Storage CSP
                if (!isset($changeSet['configGroup'])) {
                    // PolicyManager CSP is not installed above
                    $this->getSyncMLAdmin()->installPolicyManagerCSP($device);
                }
                $this->getSyncMLAdmin()->removeDeviceLockCSP($device);
                $this->getSyncMLAdmin()->removeStorageCSP($device);
            }
        }
    }

    public function updateWebClip(
        \Topalis\WebClip\Entity\WebClipInterface $webClip,
        array $removedGroups, array $addedGroups, \Doctrine\ORM\UnitOfWork $uow)
    {
        // nothing to do here
    }

    public function addWebClip(\Topalis\WebClip\Entity\WebClipInterface $webClip)
    {
        // nothing to do here
    }

    public function removeWebClip(\Topalis\WebClip\Entity\WebClipInterface $webClip)
    {
        // nothing to do here
    }

    public function addWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig)
    {
        $syncMLAdmin = $this->getSyncMLAdmin();
        foreach ($wifiConfig->getConfigGroups() as $configGroup) {
            foreach ($configGroup->getDevices() as $device) {
                if ($this->isActiveManagedDevice($device)) {
                    $syncMLAdmin->installWiFiCSPFromConfig($device, $wifiConfig);
                }
            }
        }
    }

    public function updateWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig, array $removedGroups,
        array $addedGroups, array $changeSet)
    {
        $addedGroups; // unused

        if (isset($changeSet['ssid'])) {
            $prevSsid = $changeSet['ssid'][0];
            $this->removeWifiForGroups($wifiConfig->getConfigGroups(), $prevSsid);
        } else {
            $prevSsid = $wifiConfig->getSsid();
        }

        $this->addWifi($wifiConfig);
        $this->removeWifiForGroups($removedGroups, $prevSsid);
    }

    public function removeWifi(\Topalis\Wifi\Entity\WifiConfig $wifiConfig)
    {
        $this->removeWifiForGroups($wifiConfig->getConfigGroups(), $wifiConfig->getSsid());
    }

    private function removeWifiForGroups($configGroups, $ssid)
    {
        $syncMLAdmin = $this->getSyncMLAdmin();
        foreach ($configGroups as $configGroup) {
            foreach ($configGroup->getDevices() as $device) {
                if ($this->isActiveManagedDevice($device)) {
                    $syncMLAdmin->removeWiFiCSP($device, $ssid);
                }
            }
        }
    }
}
