<?php
namespace Topalis\WinPhone\Soap;

use Topalis\Util\PasswordUtil;
use Topalis\Device\Entity\Device;

/**
 * SoapService that checks authentication for requests with UsernameToken in
 * the Security header.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 */
class SoapServiceAuth extends SoapService
{
    private $device = 0;

    private $username;

    private $password;

    // @codingStandardsIgnoreStart
    public function Security($security)
    {
        $token = $security->UsernameToken;
        $this->username = isset($token->Username) ? trim($token->Username) : null;
        $this->password = isset($token->Password) ? trim($token->Password) : null;
    }
    // @codingStandardsIgnoreEnd

    /**
     * Checks if the authentication was successful. Creates a SOAP fault if it
     * has failed.
     *
     * @return boolean true if the authentication was successful, i.e. a device
     *    corresponding to the provided username password combination exists,
     *    false otherwise.
     * @see createFault()
     */
    protected function checkAuth()
    {
        if ($this->getDevice() === null) {
            $this->createFault("env:Receiver", "env:Authentication", "Failed authentication");
            return false;
        }
        return true;
    }

    /**
     * @return \Topalis\Device\Entity\Device|null The device corresponding to
     *     the provided login data, or null if authentication has failed.
     */
    protected function getDevice()
    {
        if ($this->device === 0) {
            $this->device = $this->getNewDevice($this->username, $this->password);
        }
        return $this->device;
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    protected function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     *
     * @return integer
     */
    protected function getDeviceOs()
    {
        static $deviceOs = null;
        if ($deviceOs === null) {
            $deviceTypeFactory = $this->container->get("device.type.factory");
            $deviceOs = $deviceTypeFactory->getType("winphone")->getId();
        }

        return $deviceOs;
    }

    private function getNewDevice($username, $password)
    {
        if (strlen($username) == 0 || strlen($password) == 0) {
            return null;
        }

        $expire = new \DateTime();
        $expire->sub(new \DateInterval("P".$this->getConfig()->getPasswordValidity()."D"));

        $devices = $this->getDeviceEntityManager()->createQueryBuilder("d")
        ->select("d")
        ->join("d.user", "u")
        ->where("u.email = :email")
        ->andWhere("d.os = :os")
        ->andWhere("d.deviceid IS NULL AND d.disabled = 0 AND d.deleted = 0")
        ->andWhere("d.password IS NOT NULL")
        ->andWhere("d.passwordStamp > :max_password_validity")
        ->setParameters(array(
            "email" => $username,
            "os" => $this->getDeviceOs(),
            "max_password_validity" => $expire
        ))
        ->getQuery()
        ->getResult()
        ;

        foreach ($devices as $device) {
            if (PasswordUtil::comparePasswords($device->getPassword(), $password) == 0) {
                return $device;
            }
        }

        return null;
    }
}
