<?php
namespace Topalis\WinPhone\Wstep;

use Topalis\Certificate\Certificate;
use Topalis\Certificate\Util\CertUtil;
use Topalis\WinPhone\Wstep\Entity\RequestSecurityToken;
use Topalis\WinPhone\Wstep\Entity\RequestSecurityTokenResponseCollection;
use Topalis\WinPhone\Wstep\Entity\RequestSecurityTokenResponse;
use Topalis\WinPhone\Wstep\Entity\RequestedSecurityToken;
use Topalis\WinPhone\Wstep\Entity\BinarySecurityToken;
use Topalis\WinPhone\Wstep\Entity\EncodedString;
use Topalis\WinPhone\Wstep\Entity\AttributedString;
use Topalis\WinPhone\OMA\Provisioning\WapProvisioningDoc;
use Topalis\WinPhone\OMA\Provisioning\CertificateStoreProvider;
use Topalis\WinPhone\OMA\Provisioning\CertificateCharacteristic;
use Topalis\WinPhone\OMA\Provisioning\Characteristic;
use Topalis\WinPhone\OMA\Provisioning\ApplicationProvider;
use Topalis\WinPhone\OMA\Provisioning\AppAddrCharacteristic;
use Topalis\WinPhone\OMA\Provisioning\RegistryProvider;
use Topalis\WinPhone\OMA\Provisioning\DMClientProvider;
use Topalis\WinPhone\OMA\Provisioning\AppAuthCharacteristic;
use Topalis\WinPhone\Wstep\Entity\Timestamp;
use Topalis\WinPhone\Wstep\Entity\SecurityHeader;
use Topalis\WinPhone\Soap\SoapServiceAuth;
use Topalis\Util\Random;
use Topalis\Device\Entity\Device;
use Topalis\WinPhone\Entity\SyncMLCSPStatus;
use Topalis\WinPhone\WinPhoneConstants;
use Topalis\WinPhone\OMA\Provisioning\PollCharacteristic;

/**
 * Implements the MS-WSTEP protocol for the rollout of Windows Phone devices.
 *
 * @see http://msdn.microsoft.com/en-us/library/dd340609.aspx
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class WstepService extends SoapServiceAuth
{
    private $messageId;

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\WinPhone\Entity\SyncMLCSPStatusEntityManager
     */
    private function getCSPStatusEntityManager()
    {
        return $this->container->get("winphone.entity.maanger.syncml_cspstatus");
    }

    /**
     * @return \Topalis\WinPhone\OMA\SyncML\SyncMLAdmin
     */
    private function getSyncMLAdmin()
    {
        return $this->container->get("winphone.oma.syncml.admin");
    }

    private function addResponseHeaders()
    {
        $xmlnsA = "http://www.w3.org/2006/05/addressing";
        $action = "http://schemas.microsoft.com/windows/pki/2009/01/enrollment/RSTRC/wstep";
        $this->addSoapHeader(new \SoapHeader($xmlnsA, "Action", $action, true));
        $this->addSoapHeader(new \SoapHeader($xmlnsA, "RelatesTo", $this->messageId));
        $xmlnsS = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd";
        $security = new SecurityHeader();
        $timestamp = new Timestamp();
        $time = time();
        $timestamp->setCreated(date("Y-m-d", $time) . "T" . date("H:i:s.Z", $time) . "Z");
        $time += 5*3600;
        $timestamp->setExpires(date("Y-m-d", $time) . "T" . date("H:i:s.Z", $time) . "Z");
        $security->setTimestamp($timestamp);
        $this->addSoapHeader(new \SoapHeader($xmlnsS, "Security", $security, true));
    }

    /*
     * Header begin
     */

    // @codingStandardsIgnoreStart
    public function Action($action)
    {
        // nothing to do
    }
    // @codingStandardsIgnoreEnd

    // @codingStandardsIgnoreStart
    public function Address($address)
    {
        // noting to do
    }
    // @codingStandardsIgnoreEnd

    // @codingStandardsIgnoreStart
    public function MessageID($messageId)
    {
        $this->messageId = $messageId;
    }
    // @codingStandardsIgnoreEnd

    // @codingStandardsIgnoreStart
    public function ReplyTo($replyTo)
    {
        // noting to do
    }
    // @codingStandardsIgnoreEnd

    // @codingStandardsIgnoreStart
    public function To($to)
    {
        // nothing to do
    }
    // @codingStandardsIgnoreEnd

    /*
     * Header end
     */

    // @codingStandardsIgnoreStart
    public function RequestSecurityToken2(RequestSecurityToken $request)
    {
        $this->getLogger()->debug("RequestSecurityToken2");

        try {
            $this->addResponseHeaders();

            if (!$this->checkAuth()) {
                return null;
            }

            $device = $this->getDevice();
            if ($request->getAdditionalContext() !== null) {
                foreach ($request->getAdditionalContext()->getContextItem() as $contextItem) {
                    if ($contextItem->getName() == "ApplicationVersion") {
                        $device->setVersion($contextItem->getValue());
                        break;
                    }
                }
            }

            $csrDer = base64_decode(trim($request->getBinarySecurityToken()->get_()));
            $caCert = $this->getCertificateManager()->getCACertificate();
            // Override values in subject of CSR
            $subject = array(
                "OU" => $caCert->getOrganizationalUnit(),
                "O"  => $caCert->getOrganization(),
                "L"  => $caCert->getLocation(),
                "ST" => $caCert->getState(),
                "C"  => $caCert->getCountry()
            );
            $csr = CertUtil::der2pem($csrDer, "CERTIFICATE REQUEST");

            $cert = $this->getCertificateManager()->initClientCert($device, false);
            $revProxyCaCert = $this->getCertificateManager()->getReverseProxyCACert();
            $signedCert = $this->getCertificateManager()->signClientCert($cert, $csr, $revProxyCaCert, $subject);
            $device->setMdmCert($cert);
            $device->setPassword(null);

            $tokenResponseCollection = $this->createTokenResponseCollection(new Certificate($signedCert), $device);
            $this->getDeviceEntityManager()->save($device, false);
            $this->getSyncMLAdmin()->initDevice($device);

            return $tokenResponseCollection;
        } catch (\Exception $e) {
            $this->getLogger()->error($e->getMessage(), array("exception" => $e));
            $this->createFault("env:Receiver", "env:InternalServer", "Internal server error");
            return null;
        }
    }
    // @codingStandardsIgnoreEnd

    /**
     *
     * @param \Topalis\Certificate\Certificate $clientCert
     * @param \Topalis\Device\Entity\Device $device
     * @return \Topalis\WinPhone\OMA\Provisioning\WapProvisioningDoc
     */
    private function createProvisioningDoc(Certificate $clientCert, Device $device)
    {
        $doc = new WapProvisioningDoc("1.1");

        $omaDMCertStorePrefix = "./Vendor/MSFT/CertificateStore/";

        $certStore = new CertificateStoreProvider();
        $cert = new CertificateCharacteristic();
        $cert->setCertificate($this->getCertificateManager()->getCACertificate()->getDer());
        $certStore->setCertificate("Root/System", $cert);
        $this->createCSPStatus($device, $omaDMCertStorePrefix . "Root/System/" . $cert->getType());

        $cert = new CertificateCharacteristic();
        $cert->setCertificate($this->getCertificateManager()->getReverseProxyCACert()->getDer());
        $certStore->setCertificate("CA/System", $cert);
        $this->createCSPStatus($device, $omaDMCertStorePrefix . "CA/System/" . $cert->getType());

        $cert = new CertificateCharacteristic();
        $cert->setCertificate($clientCert->getDer());
        $certStore->setCertificate("My/User", $cert, array(new Characteristic("PrivateKeyContainer")));
        $this->createCSPStatus($device, $omaDMCertStorePrefix . "My/User/" . $cert->getType());

        $doc->addConfigurationServiceProvider($certStore);

        $application = new ApplicationProvider();
        $application->setAppId("w7");
        $application->setProviderId(WinPhoneConstants::PROVIDER_ID);
        $application->setName($this->getCertificateManager()->getCAOrg() . " - auralis MDM");
        $appAddr = new AppAddrCharacteristic();
        $appAddr->setAddr("https://" . $this->getConfig()->getHostname() . "/omadm/winphone");
        $appAddr->setPort($this->getConfig()->getSecondaryPort());
        $application->setAppAddr($appAddr);
        $application->setInit(true);
        $application->setConnRetryFreq(6);
        $application->setInitialBackOffTime(5000);
        $application->setMaxBackOffTime(120000);
        $application->setBackCompatRetryDisabled(true);

        $certSearchCriteria = array();
        $parsedCert = $clientCert->getParsed();
        foreach ($parsedCert['subject'] as $key => $value) {
            $certSearchCriteria[] = $key . "=" . $value;
        }
        $certSearchCriteria = rawurlencode(implode(",", $certSearchCriteria));
        $application->setSslClientCertSearchCriteria("Subject=" . $certSearchCriteria . "&Stores=My%5CUser");

        $nonce = base64_encode(Random::getString(20));
        $device->setOmaAuthNonce($nonce);
        $passwordAppSrv = Random::getString(20);
        $device->setOmaAuthAppSrv(sha1($passwordAppSrv));
        $passwordClient = Random::getString(20);
        $device->setOmaAuthClient(base64_encode(md5(WinPhoneConstants::PROVIDER_ID . ":" . $passwordClient, true)));

        $appAuthClient = new AppAuthCharacteristic();
        $appAuthClient->setAAuthLevel("CLIENT");
        $appAuthClient->setAAuthType("DIGEST");
        $appAuthClient->setAAuthSecret($passwordClient);
        $appAuthClient->setAAuthData($nonce);
        $application->addCharacteristic($appAuthClient);
        $appAuthAppSrv = new AppAuthCharacteristic();
        $appAuthAppSrv->setAAuthLevel("APPSRV");
        $appAuthAppSrv->setAAuthType("BASIC");
        $appAuthAppSrv->setAAuthName($device->getId());
        $appAuthAppSrv->setAAuthSecret($passwordAppSrv);
        $application->addCharacteristic($appAuthAppSrv);
        $doc->addConfigurationServiceProvider($application);

        $dmClient = new DMClientProvider();
        $dmClient->getProvider()->setId(WinPhoneConstants::PROVIDER_ID);
        $dmClient->getProvider()->setEntDeviceName("TestPhone");

        if (version_compare($device->getVersion(), "8.1", ">=")) {
            $poll = new PollCharacteristic();
            $poll->setIntervalForFirstSetOfRetries(
                $this->container->getParameter("winphone.interval_for_first_set_of_retries"));
            $poll->setNumberOfFirstRetries(
                $this->container->getParameter("winphone.number_of_first_retries"));
            $poll->setIntervalForSecondSetOfRetries(
                $this->container->getParameter("winphone.interval_for_second_set_of_retries"));
            $poll->setNumberOfSecondRetries(
                $this->container->getParameter("winphone.number_of_second_retries"));
            $poll->setIntervalForRemainingScheduledRetries(
                $this->container->getParameter("winphone.interval_for_remaining_scheduled_retries"));
            $poll->setNumberOfRemainingScheduledRetries(
                $this->container->getParameter("winphone.number_of_remaining_scheduled_retries"));
            $dmClient->getProvider()->setPoll($poll);
        } else {
            $registry = new RegistryProvider();
            $registry->setValue("HKLM\Software\Microsoft\Enrollment\OmaDmRetry\AuxRetryInterval",
                $this->container->getParameter("winphone.auxretryinterval"), "integer");
            $registry->setValue("HKLM\Software\Microsoft\Enrollment\OmaDmRetry\Aux2RetryInterval",
                $this->container->getParameter("winphone.aux2retryinterval"), "integer");
            $doc->addConfigurationServiceProvider($registry);
        }
        $doc->addConfigurationServiceProvider($dmClient);

        return $doc;
    }

    private function createCSPStatus(Device $device, $certificateCSPPrefix)
    {
        $cspStatus = new SyncMLCSPStatus();
        $cspStatus->setLocURIPrefix($certificateCSPPrefix);
        $cspStatus->setInstalled(true);
        $cspStatus->setDevice($device);
        $this->getCSPStatusEntityManager()->save($cspStatus, false);
    }

    private function createTokenResponseCollection(Certificate $clientCert, Device $device)
    {
        $tokenResponseCollection = new RequestSecurityTokenResponseCollection();
        $tokenResponse = new RequestSecurityTokenResponse();
        $tokenResponse->setTokenType(
            "http://schemas.microsoft.com/5.0.0.0/ConfigurationManager/Enrollment/DeviceEnrollmentToken");
        $tokenResponse->setRequestID(0);
        $requestedToken = new RequestedSecurityToken();
        $binaryToken = new BinarySecurityToken();
        $binaryToken->setValueType(
            "http://schemas.microsoft.com/5.0.0.0/ConfigurationManager/Enrollment/DeviceEnrollmentProvisionDoc");

        $wapProvisioningDoc = $this->createProvisioningDoc($clientCert, $device);
        $encStr = new EncodedString(new AttributedString(base64_encode($wapProvisioningDoc->toXml(true, false))));
        $encStr->setEncodingType(
            "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd#base64binary");
        $binaryToken->set_($encStr);
        $requestedToken->setBinarySecurityToken($binaryToken);
        $tokenResponse->setRequestedSecurityToken($requestedToken);
        $tokenResponseCollection->setRequestSecurityTokenResponse($tokenResponse);

        return $tokenResponseCollection;
    }
}
