<?php
namespace Topalis\Certificate;

use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\Validator\Constraints\Email;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\Mapping\ClassMetadata;
use Topalis\Certificate\Util\CertUtil;

/**
 * Certificate to be used in forms
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class Certificate
{
    /**
     * @var string
     */
    protected $country;
    /**
     * @var string
     */
    protected $state;
    /**
     * @var string
     */
    protected $location;
    /**
     * @var string
     */
    protected $organization;
    /**
     * @var string
     */
    protected $organizationalUnit;
    /**
     * @var string
     */
    protected $commonName;
    /**
     * @var string
     */
    protected $email;

    /**
     * @var resource
     */
    protected $certificate;

    /**
     * @var resource
     */
    protected $privateKey;

    /**
     * @var string
     */
    private $privateKeyPem;

    /**
     * @var \Symfony\Component\HttpFoundation\ParameterBag
     */
    protected $configArgs;

    /**
     * @var string
     */
    private $pem;

    /**
     * @var string
     */
    private $keyId;

    /**
     * @var integer
     */
    private $serial;

    /**
     * @var string
     */
    private $signerKeyId;

    /**
     * @param resource|string|null $certificate
     * @param resource|string|null $privateKey
     * @param array $configArgs
     */
    public function __construct($certificate = null, $privateKey = null, array $configArgs = array())
    {
        $this->configArgs = new ParameterBag($configArgs);

        $subject = array();
        if ($certificate !== null) {
            if (is_string($certificate)) {
                if (is_string($certificate) && CertUtil::isPem($certificate)) {
                    $this->pem = $certificate;
                }
                $certificate = openssl_x509_read($certificate);
            }
            $x509 = openssl_x509_parse($certificate);
            $subject = $x509['subject'];
            $this->setCertificate($certificate);
        }

        if ($privateKey !== null && is_string($privateKey)) {
            if (CertUtil::isPem($privateKey)) {
                $this->privateKeyPem = $privateKey;
            }
            $this->privateKey = openssl_pkey_get_private($privateKey);
        } else {
            $this->privateKey = $privateKey;
        }

        $subject += array(
            "C" => "DE", "ST" => "", "L" => "", "O" => "", "OU" => "",
            "CN" => "auralis CA", "emailAddress" => ""
        );

        $this->country = $subject['C'];
        $this->state = $subject['ST'];
        $this->location = $subject['L'];
        $this->organization = $subject['O'];
        $this->organizationalUnit = $subject['OU'];
        $this->commonName = $subject['CN'];
        $this->email = $subject['emailAddress'];
    }

    /**
     * Validation Vodoo
     * @param \Symfony\Component\Validator\Mapping\ClassMetadata $metadata
     */
    public static function loadValidatorMetadata(ClassMetadata $metadata)
    {
        $metadata
            ->addPropertyConstraint("country", new NotBlank())
            ->addPropertyConstraint("state", new NotBlank())
            ->addPropertyConstraint("location", new NotBlank())
            ->addPropertyConstraint("organization", new NotBlank())
            ->addPropertyConstraint("commonName", new NotBlank())
            ->addPropertyConstraint("email", new NotBlank())
            ->addPropertyConstraint("email", new Email())
        ;
    }

    /**
     *
     * @param integer $serial The certificate serial. If null the serial will
     *     be 0.
     * @param Certificate $caCert The CA certificate to sign the certificate.
     *     If null the certificate will be self-signed.
     * @param integer $days specifies the length of time for which the
     *     generated certificate will be valid, in days.
     * @throws Exception\CreateCertificateException
     * @return resource The signed certificate resource.
     */
    public function sign($serial = null, Certificate $caCert = null, $days = 3650)
    {
        $csr = $this->createCsr();

        try {
            if ($caCert !== null) {
                $cert = $caCert->getCertificate();
                $privateKey = $caCert->getPrivateKey();
            } else {
                $cert = null;
                $privateKey = $this->getPrivateKey();
            }
            $sscert = openssl_csr_sign($csr, $cert, $privateKey, $days, $this->configArgs->all(), $serial);
        } catch (\Exception $e) {
            $msg = "Could not create certificate: " . $e->getMessage();
            throw new Exception\CreateCertificateException($msg, $e->getCode(), $e);
        }

        $this->internalClear();
        $this->serial = $serial;
        $this->setCertificate($sscert);
        return $sscert;
    }

    /**
     * Create certificate signing request
     */
    public function createCsr()
    {
        $data = $this->getCertData();

        foreach ($data as $key => $value) {
            if (empty($value)) {
                unset($data[$key]);
            }
        }

        $privateKey = $this->getPrivateKey();

        return openssl_csr_new($data, $privateKey, $this->configArgs->all());
    }

    public function exportCsr()
    {
        $csr = $this->createCsr();
        $out = null;
        openssl_csr_export($csr, $out);

        return $out;
    }

    /**
     * Signs the certificate and returns it PEM encoded.
     *
     * @param integer $serial The certificate serial. If null the serial will
     *     be 0.
     * @param Certificate $caCert The CA certificate to sign the certificate.
     *     If null the certificate will be self-signed.
     * @param integer $days specifies the length of time for which the
     *     generated certificate will be valid, in days.
     * @throws Exception\CreateCertificateException
     * @return string PEM encoded certificate.
     */
    public function export($serial, Certificate $caCert = null, $days = 3650)
    {
        $this->sign($serial, $caCert, $days);
        return $this->getPem();
    }

    /**
     * @return array
     */
    protected function getCertData()
    {
        $data = array(
            "C"            => $this->getCountry(),
            "ST"           => $this->getState(),
            "L"            => $this->getLocation(),
            "O"            => $this->getOrganization(),
            "OU"           => $this->getOrganizationalUnit(),
            "CN"           => $this->getCommonName(),
            "emailAddress" => $this->getEmail()
        );

        return $data;
    }

    /**
     * @throws Exception\CreateCertificateException
     * @return string The PEM encoded private key. If there is none, a new one
     *     will be created.
     */
    public function exportPrivateKey()
    {
        if ($this->privateKeyPem === null) {
            try {
                openssl_pkey_export($this->getPrivateKey(), $this->privateKeyPem, "",
                    $this->configArgs->all() + array("encrypt_key" => false));
            } catch (\Exception $e) {
                $msg = "Could not export private key: " . $e->getMessage();
                throw new Exception\CreateCertificateException($msg, $e->getCode(), $e);
            }
        }

        return $this->privateKeyPem;
    }

    /**
     * Convenience method for exportPrivateKey().
     * @throws Exception\CreateCertificateException
     * @return string The PEM encoded private key. If there is none, a new one
     *     will be created.
     */
    public function getPrivateKeyPem()
    {
        return $this->exportPrivateKey();
    }

    /**
     * @param string $key The key of the extension to return.
     * @return string|NULL The value of the extension, null if it is not set
     *     or the certificate has not been signed.
     */
    public function getExtension($key)
    {
        $extensions = $this->getExtensions();
        return isset($extensions[$key]) ? $extensions[$key] : null;
    }

    /**
     * @return array|NULL The extensions of the certificate, or null if it has
     *    not been signed.
     */
    public function getExtensions()
    {
        $parsed = $this->getParsed();
        return isset($parsed['extensions']) ? $parsed['extensions'] : null;
    }

    /**
     * @return string|NULL The certificate fingerprint (SHA1 hash of DER encoded
     *     certificate), or null if it has not been signed.
     */
    public function getFingerprint()
    {
        $der = $this->getDer();
        return $der === null ? null : sha1($der);
    }

    /**
     *
     * @return string|NULL The certificate hash, or null if it has not been
     *     signed.
     */
    public function getHash()
    {
        $parsed = $this->getParsed();
        return isset($parsed['hash']) ? $parsed['hash'] : null;
    }

    /**
     * @param resource $certificate
     * @return \Topalis\Certificate\Certificate $this
     */
    protected function setCertificate($certificate)
    {
        $this->certificate = $certificate;
        if ($certificate !== null) {
            $parsed = openssl_x509_parse($certificate);
            $this->serial = intval($parsed['serialNumber']);
            if (isset($parsed['extensions'])) {
                $extensions = $parsed['extensions'];
                $this->keyId  = isset($extensions['subjectKeyIdentifier'])
                    ? $extensions['subjectKeyIdentifier'] : null;
                // remove "keyid:" at the beginning and remove line break
                $this->signerKeyId = isset($extensions['authorityKeyIdentifier'])
                    ? rtrim(substr($extensions['authorityKeyIdentifier'], 6)) : null;
            }
        }

        return $this;
    }

    /**
     * @return resource The certificate's resource, or null if it has not been
     *     signed.
     */
    public function getCertificate()
    {
        return $this->certificate;
    }

    /**
     * @param resource $privateKey
     * @return \Topalis\Certificate\Certificate
     */
    protected function setPrivateKey($privateKey)
    {
        if ($this->privateKey != $privateKey) {
            $this->privateKey = $privateKey;
            $this->privateKeyPem = null;
            if ($this->isSelfSigned()) {
                $this->sign($this->getSerial());
            }
        }

        return $this;
    }

    /**
     * @return resource The private key resource. If there is none, a new one
     *     will be created.
     */
    public function getPrivateKey()
    {
        if (null === $this->privateKey) {
            $this->privateKey = openssl_pkey_new($this->configArgs->all());
        }

        return $this->privateKey;
    }

    /**
     * @return string
     */
    public function getCountry()
    {
        return $this->country;
    }

    /**
     * @return string
     */
    public function getState()
    {
        return $this->state;
    }

    /**
     * @param string $state
     *
     * @return $this
     */
    public function setState($state)
    {
        if ($this->state != $state) {
            $this->internalClear();
            $this->state = $state;
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getLocation()
    {
        return $this->location;
    }

    /**
     * @return string
     */
    public function getOrganization()
    {
        return $this->organization;
    }

    /**
     * @return string
     */
    public function getOrganizationalUnit()
    {
        return $this->organizationalUnit;
    }

    /**
     * @param string $organizationalUnit
     *
     * @return $this
     */
    public function setOrganizationalUnit($organizationalUnit)
    {
        if ($this->organizationalUnit != $organizationalUnit) {
            $this->internalClear();
            $this->organizationalUnit = $organizationalUnit;
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getCommonName()
    {
        return $this->commonName;
    }

    /**
     * @param string $commonName
     *
     * @return $this
     */
    public function setCommonName($commonName)
    {
        if ($this->commonName != $commonName) {
            $this->internalClear();
            $this->commonName = $commonName;
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * @param string $email
     *
     * @return $this
     */
    public function setEmail($email)
    {
        if ($this->email != $email) {
            $this->internalClear();
            $this->email = $email;
        }

        return $this;
    }

    /**
     * @param string $organization
     *
     * @return $this
     */
    public function setOrganization($organization)
    {
        if ($this->organization != $organization) {
            $this->internalClear();
            $this->organization = $organization;
        }

        return $this;
    }

    /**
     * @param string $location
     *
     * @return $this
     */
    public function setLocation($location)
    {
        if ($this->location != $location) {
            $this->internalClear();
            $this->location = $location;
        }

        return $this;
    }

    /**
     * @param string $country
     *
     * @return $this
     */
    public function setCountry($country)
    {
        if ($this->country != $country) {
            $this->internalClear();
            $this->country = $country;
        }

        return $this;
    }

    /**
     * @return null|string The certificate's key id, or null if it has not been
     *     signed.
     */
    public function getKeyId()
    {
        return $this->keyId;
    }

    /**
     * @return string The DER encoded certificate, or null if it has not been
     *     signed.
     * @throws Exception\CreateCertificateException
     */
    public function getDer()
    {
        return CertUtil::pem2der($this->getPem());
    }

    /**
     *
     * @return null|string The certificate as array, or null if it has not been
     *     signed.
     * @param boolean $shortNames controls how the data is indexed in the array
     *     - if shortnames is true (the default) then fields will be indexed
     *     with the short name form, otherwise, the long name form will be used
     *     - e.g.: CN is the shortname form of commonName.
     * @see openssl_x509_parse()
     */
    public function getParsed($shortNames = true)
    {
        $resource = $this->getCertificate();
        if ($resource === null) {
            return null;
        }

        return openssl_x509_parse($resource, $shortNames);
    }

    /**
     * @return null|string The PEM encoded certificate, or null if it has not
     *     been signed.
     * @throws Exception\CreateCertificateException
     */
    public function getPem()
    {
        if ($this->pem === null) {
            $resource = $this->getCertificate();
            if ($resource !== null) {
                try {
                    openssl_x509_export($resource, $this->pem);
                } catch (\Exception $e) {
                    $msg = "Could not export certificate: " . $e->getMessage();
                    throw new Exception\CreateCertificateException($msg, $e->getCode(), $e);
                }
            }
        }

        return $this->pem;
    }

    public function getResource()
    {
        return $this->certificate;
    }

    /**
     * @return int The certificate's serial number, or null if it has not been
     *     signed.
     */
    public function getSerial()
    {
        return $this->serial;
    }

    /**
     * @return string The key id of the signing certificate, or null if it has
     *     not been signed.
     */
    public function getSignerKeyId()
    {
        return $this->signerKeyId;
    }

    /**
     * @return NULL|boolean true if the certificate is a CA certificate, false
     *     otherwise, or null if it has not been signed.
     */
    public function isCA()
    {
        $basicConstraints = $this->getExtension("basicConstraints");
        if ($basicConstraints === null) {
            return null;
        }

        return $basicConstraints == "CA:TRUE";
    }

    /**
     * @return boolean true if the certificate is self-signed, false otherwise.
     */
    public function isSelfSigned()
    {
        if (!$this->isSigned()) {
            return false;
        }

        return $this->getKeyId() == $this->getSignerKeyId();
    }

    /**
     * @return boolean true if the certifcate is signed, false otherwise.
     */
    public function isSigned()
    {
        return $this->getCertificate() !== null;
    }

    protected function internalClear()
    {
        $this->certificate = null;
        $this->pem = null;
        $this->serial = null;
        $this->keyId = null;
        $this->signerKeyId = null;
    }
}
