<?php
namespace Topalis\Certificate\Util;

use Topalis\Util\Process;
use Topalis\Util\FileSystemUtil;
use Topalis\Util\StringUtil;
use Topalis\Certificate\Certificate;

class CertUtil
{
    /**
     * Checks if the specified string is PEM encoded.
     *
     * @param string $string The string to check.
     * @return boolean true if the specified string is PEM encoded, false
     *     otherwise.
     */
    public static function isPem($string)
    {
        return StringUtil::startsWith($string, "-----BEGIN ");
    }

    /**
     * Converts PEM encoded data to DER.
     *
     * @param string $pem The PEM encoded data.
     * @return string The data converted to DER.
     */
    public static function pem2der($pem)
    {
        if ($pem === null) {
            return null;
        }

        $data = trim($pem);
        $data = substr(strstr($data, "\n"), 1);
        $data = substr($data, 0, strrpos($data, "\n"));
        $der = base64_decode($data);
        return $der;
    }

    /**
     * Converts the specified DER data to PEM.
     *
     * @param string $der The DER encoded data.
     * @param string $type The PEM type.
     * @return string The data converted to PEM.
     */
    public static function der2pem($der, $type = "CERTIFICATE")
    {
        $pem = "-----BEGIN " . $type . "-----\n";
        $pem .= chunk_split(base64_encode($der), 64, "\n");
        $pem .= "-----END " . $type . "-----\n";
        return $pem;
    }

    /**
     * Converts the specified certificate chain into a PEM encoded string. The
     * specified chain may be one of the following:
     * <ul>
     * <li>PEM encoded certificate chain</li>
     * <li>X509 resource</li>
     * <li>file name (in this case the file will be just copied to the specified
     * file name)</li>
     * <li>Object of type Certificate</li>
     * <li>One dimensional array with entries of the previous types (can be mixed).</li>
     * </ul>
     *
     * @param mixed   $chain       The certificate chain.
     * @param boolean $createDummy If true and $chain is null, a dummy
     *                             certificate will be created.
     *
     * @throws \InvalidArgumentException
     * @return string PEM encoded certificate chain.
     */
    public static function convertChainToPem($chain, $createDummy = false)
    {
        $rv = '';
        if ($chain === null && $createDummy) {
            // Create dummy cert
            $pkey = openssl_pkey_new();
            $chain = openssl_csr_sign(openssl_csr_new(array(), $pkey), null, $pkey, 0);
            openssl_x509_export($chain, $rv);
        } else {
            if (!is_array($chain)) {
                $chain = array($chain);
            }
            $chainlen = count($chain);
            for ($i = 0; $i < $chainlen; $i++) {
                if (is_resource($chain[$i])) {
                    $output = "";
                    openssl_x509_export($chain[$i], $output);
                    $chain[$i] = $output;
                } else if (is_string($chain[$i])) {
                    if (is_file($chain[$i])) {
                        $chain[$i] = file_get_contents($chain[$i]);
                    }
                    if (!StringUtil::endsWith($chain[$i], "\n")) {
                        $chain[$i] .= "\n";
                    }
                } else if ($chain[$i] instanceof Certificate) {
                    $chain[$i] = $chain[$i]->getPem();
                } else {
                    throw new \InvalidArgumentException("Invalid certificate chain");
                }
            }
            $rv = implode("", $chain);
        }

        return $rv;
    }

    /**
     * Decodes a PKCS7-signed container.
     *
     * @param string $content The binary content to be decoded.
     * @param int $flags Can be used to affect how the signature is verified -
     *     see PKCS7 constants for more information.
     * @param mixed $x509CA The CA certificate (chain) to do the verification.
     *     If not defined the signers certificate will not be verified.
     *     (PKCS7_NOVERIFY is added to the flags.) See exportChainToFile() for
     *     possible values.
     * @return array With the following keys:<ul>
     *     <li>data: the signed data</li>
     *     <li>signerCert: The certificate of the signer as array</li>
     *     <li>verified: true, if CA certificate is specified and verification
     *     was successful, false otherwise.</li>
     * @throws \Exception On error.
     */
    public static function decodePKCS7($content, $flags = null, $x509CA = null)
    {
        if ($flags === null) {
            $flags = PKCS7_NOSIGS | PKCS7_BINARY;
        }
        if ($x509CA === null) {
            $flags |= PKCS7_NOVERIFY;
        }

        try {
            $tmpPKCS7File = tempnam(sys_get_temp_dir(), 'auralisI');
            $tmpDataFile = tempnam(sys_get_temp_dir(), 'auralisO');
            $tmpCACertFile = tempnam(sys_get_temp_dir(), 'auralisC');
            $tmpSignerCertFile = tempnam(sys_get_temp_dir(), 'auralis-signer');

            /*
             * PHP's pkcs7_verify cannot read DER-encoded input. So we have to create
             * a temporary base64-encoded mime message. Furthermore, it requires all
             * data to be presented in files. Also, due to unfortunate arrangement of
             * optional parameters, we *require* a (temporary) cert file, even if it is
             * not used at all.
             */
            $b64 = base64_encode($content);
            $f = fopen($tmpPKCS7File, 'w');
            fwrite($f, "MIME-Version: 1.0\n");
            fwrite($f, "Content-Disposition: attachment; filename=\"smime.p7m\"\n");
            fwrite($f, "Content-Type: application/x-pkcs7-mime; smime-type=signed-data; name=\"smime.p7m\"\n");
            fwrite($f, "Content-Transfer-Encoding: base64\n");
            fwrite($f, "\n");
            while (strlen($b64) > 76) {
                fwrite($f, substr($b64, 0, 76) . "\n");
                $b64 = substr($b64, 76);
            }
            fwrite($f, $b64."\n");
            fwrite($f, "\n");
            fclose($f);

            self::exportChainToFile($x509CA, $tmpCACertFile, true);

            $verified = openssl_pkcs7_verify($tmpPKCS7File, $flags, $tmpSignerCertFile, array($tmpCACertFile),
                $tmpCACertFile, $tmpDataFile
            );

            if ($verified === false) {
                $flags |= PKCS7_NOVERIFY;
                openssl_pkcs7_verify($tmpPKCS7File, $flags, $tmpSignerCertFile, array($tmpCACertFile),
                    $tmpCACertFile, $tmpDataFile);
            }

            if ($x509CA === null) {
                $verified = false;
            }
            $signerCert = file_get_contents($tmpSignerCertFile);
            $data = file_get_contents($tmpDataFile);
            unlink($tmpSignerCertFile);
            unlink($tmpCACertFile);
            unlink($tmpPKCS7File);
            unlink($tmpDataFile);

            $ret = array('data' => $data, 'signerCert' => $signerCert, 'verified' => $verified);
            return $ret;
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpPKCS7File);
            FileSystemUtil::tryUnlinkFile($tmpDataFile);
            FileSystemUtil::tryUnlinkFile($tmpCACertFile);
            FileSystemUtil::tryUnlinkFile($tmpSignerCertFile);
            throw $e;
        }
    }

    /**
     * Exports the specified certificate chain to the specified file in PEM
     * fomrat. The specified chain may be one of the following:
     * <ul>
     * <li>PEM encoded certificate chain</li>
     * <li>X509 resource</li>
     * <li>file name (in this case the file will be just copied to the specified
     * file name)</li>
     * <li>One dimensional array with entries of the previous types (can be mixed).</li>
     * </ul>
     *
     * @param mixed $chain The certificate chain.
     * @param string $filename The file to write the PEM encoded certificate chain into.
     * @param boolean $createDummy If true and $chain is null, a dummy
     *     certificate will be created.
     * @throws \InvalidArgumentException
     * @see convertChainToPem()
     */
    public static function exportChainToFile($chain, $filename, $createDummy = false)
    {
        file_put_contents($filename, self::convertChainToPem($chain, $createDummy));
    }

    /**
     * Converts the specified certificate into an PCKS7 container and encrypts
     * it using the specified receipcert
     *
     * @param string $cert      The PEM encoded certificate to encrypt.
     * @param string $recipcert The PEM encoded certificate used for encryption.
     *
     * @throws \Exception On error.
     * @return string The DER encoded encrypted certificate.
     */
    public static function encryptCert($cert, $recipcert)
    {
        $tmpfi = tempnam(sys_get_temp_dir(), 'auralisI');
        $tmpfio = tempnam(sys_get_temp_dir(), 'auralisIO');
        $tmpfo = tempnam(sys_get_temp_dir(), 'auralisO');
        $tmpfr = tempnam(sys_get_temp_dir(), 'auralisR');

        try {
            file_put_contents($tmpfi, $cert);
            $cmd = "openssl crl2pkcs7 -nocrl -certfile $tmpfi -outform der -out $tmpfio";
            $out = "";
            Process::execThrow($cmd, $out);

            // XXX Doing it the openssl way
            file_put_contents($tmpfr, $recipcert);
            $cmd = "openssl cms -encrypt -binary -nosmimecap -des3 -rand /tmp/.rnd -in " . $tmpfio
            . " -outform der -out $tmpfo $tmpfr";
            $out = "";
            Process::execThrow($cmd, $out);
            $ret = file_get_contents($tmpfo);

            unlink($tmpfi);
            unlink($tmpfio);
            unlink($tmpfo);
            unlink($tmpfr);

            return $ret;
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpfi);
            FileSystemUtil::tryUnlinkFile($tmpfo);
            FileSystemUtil::tryUnlinkFile($tmpfio);
            FileSystemUtil::tryUnlinkFile($tmpfr);
            throw $e;
        }
    }

    /**
     * Get croot entry from certificate chain.
     *
     * @param mixed $chn The certificate chain.
     * @return string The croot entry.
     * @throws \Exception On error.
     */
    public static function getCroot($chn)
    {
        $cr = null;
        foreach (unserialize($chn) as $c) {
            $x509 = openssl_x509_parse($c);
            if ($x509['issuer'] == $x509['subject']) {
                $cr = $c;
                break;
            }
        }
        if ($cr === null) {
            throw new \Exception("Root certificate not found");
        }

        return $cr;
    }

    /**
     * Checks if the specified certificate is expired.
     *
     * @param mixed $x509cert Certificate as array or in a form that can be
     *     parsed by OpenSSL.
     * @param int $time The time to compare with. If null the current system time
     *     is used.
     * @return boolean true if the certificate is expired, false otherwise.
     * @throws \Exception
     */
    public static function isCertExpired($x509cert, $time = null)
    {
        if ($time === null) {
            $time = time();
        }
        return self::getValidTo($x509cert) < $time;
    }

    /**
     * Checks if the specified certificate is valid an can be used for the
     * specified purposes.
     *
     * @param mixed $cert The x509 certificate.
     * @param int[] $purposes Array of purposes.
     * @param mixed $x509CA The CA certificate (chain) to do the verification.
     * @param int $time The time to check if the certificate is expired. If
     *     null the current system time is used. If a purpose is specified, the
     *     time is ignored because the openssl_x509_checkpurpose function does
     *     not allow to specify a time PHP#67500.
     * @throws \Exception
     * @return boolean true if the specified certificate is valid an can be used
     *     for the specified purposes.
     */
    public static function isCertValid($cert, array $purposes, $x509CA, $time = null)
    {
        $tmpCACertFile = tempnam(sys_get_temp_dir(), 'auralisC');
        try {
            if (self::isCertExpired($cert, $time)) {
                return false;
            }
            self::exportChainToFile($x509CA, $tmpCACertFile);
            $valid = true;
            foreach ($purposes as $purpose) {
                if (!openssl_x509_checkpurpose($cert, $purpose, array($tmpCACertFile))) {
                    $valid = false;
                    break;
                }
            }
            unlink($tmpCACertFile);
            return $valid;
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpCACertFile);
            throw $e;
        }
    }

    /**
     * Checks if the specified private key corresponds to the specified
     *     certificate.
     *
     * @param mixed $cert The certificate the private key corresponds to.
     * @param mixed $key The private key that corresponds to the certificate.
     * @return boolean true if the key is valid, false otherwise.
     */
    public static function isPrivateKeyValid($cert, $key)
    {
        return openssl_x509_check_private_key($cert, $key);
    }

    /**
     * Returns the public key belonging to the specified private key.
     *
     * @param string|resource $privateKey The private key. $privateKey can be
     *     one of the following: <ol>
     *     <li>A string having the format file://path/to/file.pem. The named
     *     file must contain a PEM encoded certificate/private key
     *     (it may contain both).</li>
     *     <li>A PEM formatted private key.</li>
     *     <li>A Resource holding the key.</li>
     *     </ol>
     * @param string $passphrase The optional parameter passphrase must be used
     *     if the specified key is encrypted (protected by a passphrase).
     * @return string The PEM encoded public key.
     */
    public static function getPublicKeyFromPrivateKey($privateKey, $passphrase = null)
    {
        $privateKey = is_resource($privateKey) ? $privateKey : openssl_pkey_get_private($privateKey, $passphrase);
        $privateKeyDetails = openssl_pkey_get_details($privateKey);
        return isset($privateKeyDetails['key']) ? $privateKeyDetails['key'] : null;
    }

    /**
     * return subject-cn
     *
     * @param mixed $x509cert Certificate as array or in a form that can be
     *     parsed by OpenSSL.
     * @return string cn, null when missing
     * @throws \Exception On error.
     */
    public static function getSubjectCN($x509cert)
    {
        if (!is_array($x509cert)) {
            $x509cert = openssl_x509_parse($x509cert);
        }
        if (isset($x509cert['subject']) && is_array($x509cert['subject'])
            && isset($x509cert['subject']['CN'])) {
            $cn = $x509cert['subject']['CN'];
            return $cn;
        }
        return null;
    }

    /**
     * return validFrom
     *
     * @param mixed $x509cert Certificate as array or in a form that can be
     *     parsed by OpenSSL.
     * @return int validFrom as UNIX timestamp, null if missing.
     * @throws \Exception On error.
     */
    public static function getValidFrom($x509cert)
    {
        if (!is_array($x509cert)) {
            $x509cert = openssl_x509_parse($x509cert);
        }
        if (isset($x509cert['validFrom_time_t'])) {
            return $x509cert['validFrom_time_t'];
        }
        return null;
    }

    /**
     * @param mixed $x509cert Certificate as array or in a form that can be
     *     parsed by OpenSSL.
     * @return int validTo as UNIX timestamp, null if missing.
     * @throws \Exception On error.
     */
    public static function getValidTo($x509cert)
    {
        if (!is_array($x509cert)) {
            $x509cert = openssl_x509_parse($x509cert);
        }
        if (isset($x509cert['validTo_time_t'])) {
            return $x509cert['validTo_time_t'];
        }
        return null;
    }

    /**
     * return subjectKeyIdentifier
     *
     * @param mixed $x509cert Certificate as array or in a form that can be
     *     parsed by OpenSSL.
     * @return string subjectKeyIdentifier
     * @throws \Exception On error.
     */
    public static function getSubjectKeyIdentifier($x509cert)
    {
        if (!is_array($x509cert)) {
            $x509cert = openssl_x509_parse($x509cert);
        }
        if (isset($x509cert['extensions']) && is_array($x509cert['extensions'])
            && isset($x509cert['extensions']['subjectKeyIdentifier'])) {
            $cn = $x509cert['extensions']['subjectKeyIdentifier'];
            return $cn;
        }
        return null;
    }

    /**
     * Returns the subject key identifer belonging to the specified private
     * key. This is done by generating a self-signed certificate using the
     * specified private key.
     *
     * @param string|resource $privateKey The private key. $privateKey can be
     *     one of the following: <ol>
     *     <li>A string having the format file://path/to/file.pem. The named
     *     file must contain a PEM encoded certificate/private key
     *     (it may contain both).</li>
     *     <li>A PEM formatted private key.</li>
     *     <li>A Resource holding the key.</li>
     *     </ol>
     * @param string $passphrase The optional parameter passphrase must be used
     *     if the specified key is encrypted (protected by a passphrase).
     * @return string The subject key identifer belonging to the specified
     *     private key.
     */
    public static function getSubjectKeyIdFromPrivateKey($privateKey, $passphrase = null)
    {
        $privateKey = is_resource($privateKey) ? $privateKey : openssl_pkey_get_private($privateKey, $passphrase);
        $cert = new Certificate(null, $privateKey);
        $cert->sign();
        return $cert->getKeyId();
    }

    /**
     * Generate a PKCS7 container from the specified content.
     *
     * @param string     $content The content to be PKCS7-signed.
     * @param mixed      $cert    The certificate to sign with.
     * @param mixed      $key     The private key that corresponds to the certificate.
     * @param null|array $chain
     *
     * @throws \Exception On error.
     * @return string The signed content.
     */
    public static function signPKCS7($content, $cert, $key, $chain = null)
    {
        $flags = PKCS7_BINARY;
        try {
            $tmpfi = tempnam(sys_get_temp_dir(), 'auralisSI');
            $tmpfo = tempnam(sys_get_temp_dir(), 'auralisSO');

            $f = fopen($tmpfi, 'w');
            fwrite($f, $content);
            fclose($f);
            $tmpChain = null;
            if (null !== $chain) {
                $tmpChain = tempnam(sys_get_temp_dir(), 'auralisChain');
                file_put_contents($tmpChain, implode("\n", $chain));
                openssl_pkcs7_sign($tmpfi, $tmpfo, $cert, array($key, ''), array(), $flags, $tmpChain);
            } else {
                openssl_pkcs7_sign($tmpfi, $tmpfo, $cert, array($key, ''), array(), $flags);
            }
            $signed = file_get_contents($tmpfo);
            $content = base64_decode(preg_replace('/(.+\n)+\n/', '', $signed, 1));
            unlink($tmpfi);
            unlink($tmpfo);
            if (null !== $chain) {
                unlink($tmpChain);
            }
            return $content;
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpfi);
            FileSystemUtil::tryUnlinkFile($tmpfo);
            throw $e;
        }
    }

    /**
     * Creates a degenerate PKCS#7 containing the specified certificates.
     * @param mixed $chain The certificates. May be any value supported by
     *     self::convertChainToPem()
     * @param string $outform PEM or DER. Defaults to PEM
     * @see http://tools.ietf.org/html/rfc2315
     */
    public static function createDegneratePKCS7($chain, $outform = "PEM")
    {
        $pem = self::convertChainToPem($chain);
        try {
            $tmpCertFile = tempnam(sys_get_temp_dir(), "auralisI");
            $tmpPKCS7File = tempnam(sys_get_temp_dir(), "auralisR");
            file_put_contents($tmpCertFile, $pem);
            Process::execThrow("openssl crl2pkcs7 -nocrl -outform " . $outform
                . " -certfile " . $tmpCertFile . " -out " . $tmpPKCS7File);
            $rv = file_get_contents($tmpPKCS7File);
            unlink($tmpCertFile);
            unlink($tmpPKCS7File);
            return $rv;
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpCertFile);
            FileSystemUtil::tryUnlinkFile($tmpPKCS7File);
            throw $e;
        }
    }
}
