<?php
namespace Topalis\Scep\Util;

use Topalis\Util\Process;
use Topalis\Util\FileSystemUtil;
use Topalis\Certificate\Certificate;
use Topalis\Scep\ScepConstants;
use Topalis\Util\StringUtil;
use Topalis\Util\Exception\Process\ExecException;

/**
 * Utility methods for SCEP
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 * @see http://tools.ietf.org/html/draft-nourse-scep-23
 */
class ScepUtil
{
    /**
     * @param string $csr PEM encoded certificate signing request.
     * @throws ExecException
     * @throws \Exception
     * @return string The challenge of the specified certificate signing request.
     */
    public static function getChallenge($csr)
    {
        $tmpFile = tempnam(sys_get_temp_dir(), 'auralisI');
        try {
            file_put_contents($tmpFile, $csr);
            $cmd = "openssl req -inform PEM -in $tmpFile -noout -text | grep challengePassword | cut -d: -f2";
            $challenge = "";
            $rc = Process::exec($cmd, $challenge);
            if ($rc !== 0) {
                throw new ExecException($cmd, $challenge, $rc, "Failed to get challenge password");
            }
            unlink($tmpFile);
            return $challenge;
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpFile);
            throw $e;
        }
    }

    public static function getSignedAttributes($content, array $oids)
    {
        try {
            $tmpfi = tempnam(sys_get_temp_dir(), "auralisI");
            file_put_contents($tmpfi, $content, LOCK_EX);

            $cmd = "pkcs7attributes -inform DER -in " . $tmpfi . " -- " . implode(" ", $oids);
            $out = "";
            Process::execThrow($cmd, $out);

            $attrs = array();
            $lines = explode("\n", $out);
            foreach ($lines as $line) {
                $split = explode(": ", $line, 2);
                if (!isset($split[1])) {
                    continue;
                }

                $oid = $split[0];
                $value = $split[1];
                if ($value == "not found") {
                    $value = false;
                } else if (StringUtil::startsWith($value, "0x")) {
                    // convert hex string to binary
                    $value = pack("H*", substr($value, 2));
                }
                $attrs[$oid] = $value;
            }

            foreach ($oids as $oid) {
                if (!isset($attrs[$oid])) {
                    throw new \Exception("pkcs7attributes returned no information about OID " . $oid);
                }
            }

            unlink($tmpfi);
            return $attrs;
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpfi);
            throw $e;
        }
    }

    /**
     * Creates a SCEP pkiMessage.
     * @param string $content The content to encrypt and sign.
     * @param array $signedAttributes The attributes to add to the signed
     *     attributes in the PKCS#7
     * @param Certificate $signCert The certificate to sign the PKCS#7.
     * @param string|NULL $encCert The certificate to encrypt the content with,
     *     or null to not encrypt the content.
     * @param integer $failInfo The failInfo in case of errors. If specified the
     *     pkiStatus will be set to FAILURE, otherwise to SUCCESS.
     * @return string The pkiMessage.
     * @see http://tools.ietf.org/html/draft-nourse-scep-23#section-3
     */
    public static function createPkiMessage($content, array $signedAttributes, Certificate $signCert,
        $encCert = null, $failInfo = null)
    {
        if ($failInfo === null) {
            $signedAttributes[ScepConstants::$oids['pkiStatus']] = ScepConstants::PKISTATUS_SUCCESS;
        } else {
            $signedAttributes[ScepConstants::$oids['pkiStatus']] = ScepConstants::PKISTATUS_FAILURE;
            $signedAttributes[ScepConstants::$oids['failInfo']] = $failInfo;
        }

        try {
            $tmpfi = tempnam(sys_get_temp_dir(), 'auralisI');
            $tmpfo = tempnam(sys_get_temp_dir(), 'auralisO');
            $tmpCert = tempnam(sys_get_temp_dir(), "auralisC");
            $tmpKey = tempnam(sys_get_temp_dir(), "auralisK");
            $tmpSignedAttributes = tempnam(sys_get_temp_dir(), "auralisSA");

            if ($encCert) {
                $tmpEncCert = tempnam(sys_get_temp_dir(), "auralisEC");
                file_put_contents($tmpEncCert, $encCert);
            } else {
                $tmpEncCert = null;
            }
            file_put_contents($tmpfi, $content);
            file_put_contents($tmpCert, $signCert->getPem());
            file_put_contents($tmpKey, $signCert->getPrivateKeyPem());

            $file = fopen($tmpSignedAttributes, "w");
            foreach ($signedAttributes as $oid => $value) {
                fwrite($file, $oid . " " . ScepConstants::$oidTypes[$oid] . " " . base64_encode($value) . "\n");
            }
            fclose($file);

            $cmd = "pkcs7sign -sign -binary -des3 -nodetach -nosmimecap -in " . $tmpfi . " -out "
                . $tmpfo . " -inkey " . $tmpKey . " -signer " . $tmpCert
                . " -outform DER -signed-attributes " . $tmpSignedAttributes . " " . $tmpEncCert;
            Process::execThrow($cmd);
            $pkiMessage = file_get_contents($tmpfo);

            unlink($tmpfi);
            unlink($tmpfo);
            unlink($tmpCert);
            unlink($tmpKey);
            unlink($tmpSignedAttributes);
            if ($tmpEncCert !== null) {
                unlink($tmpEncCert);
            }

            return $pkiMessage;
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpfi);
            FileSystemUtil::tryUnlinkFile($tmpfo);
            FileSystemUtil::tryUnlinkFile($tmpCert);
            FileSystemUtil::tryUnlinkFile($tmpKey);
            FileSystemUtil::tryUnlinkFile($tmpSignedAttributes);
            FileSystemUtil::tryUnlinkFile($tmpEncCert);
            throw $e;
        }
    }
}
