<?php
namespace Topalis\System;

use Topalis\Util\FileSystemUtil;
use Topalis\Util\Process;

class MTAConfig
{
    private $aliases = array();
    private $clientTLS;
    private $genericMaps = array();
    private $relayHost;
    private $relayPort;

    /**
     * @var string
     */
    private $aliasesFilename = "/etc/aliases";

    /**
     * @var string
     */
    private $postfixMainConfigFilename = "/etc/postfix/main.cf";

    /**
     * @var string
     */
    private $genericMapsFilename = "/etc/postfix/generic";

    /**
     * @var string
     */
    private $helperCommand = "/usr/sbin/aurpostfix";

    /**
     * @return string The name of the file containing the aliases.
     */
    protected function getAliasesFilename()
    {
        return $this->aliasesFilename;
    }

    /**
     * Sets the name of the file containing the aliases.
     *
     * @param string $aliasesFilename The file name to set.
     */
    public function setAliasesFilename($aliasesFilename)
    {
        $this->aliasesFilename = $aliasesFilename;
    }

    /**
     * @return string The name of the file containing the generic mappings.
     */
    public function getGenericMapsFilename()
    {
        return $this->genericMapsFilename;
    }

    /**
     * Sets the name of the file containing the generic mappings.
     *
     * @param string $genericMapsFilename The file name to set.
     */
    public function setGenericMapsFilename($genericMapsFilename)
    {
        $this->genericMapsFilename = $genericMapsFilename;
    }

    /**
     * @return string The name of the Postfix main configuration file.
     */
    protected function getPostfixMainConfigFilename()
    {
        return $this->postfixMainConfigFilename;
    }

    /**
     * Sets the name of the Postfix main configuration file.
     *
     * @param string $postfixMainConfigFilename The file name to set.
     */
    public function setPostfixMainConfigFilename($postfixMainConfigFilename)
    {
        $this->postfixMainConfigFilename = $postfixMainConfigFilename;
    }

    /**
     * Sets the command used to store the MTA configuration.
     *
     * @param string $helperCommand The command to set.
     */
    public function setHelperCommand($helperCommand)
    {
        $this->helperCommand = $helperCommand;
    }

    /**
     * @return boolean true if MTA configuration is supported, false otherwise.
     *     It checks if the related files exist.
     */
    public function isSupportedSystem()
    {
        return is_file($this->aliasesFilename) && is_file($this->genericMapsFilename)
            && is_file($this->postfixMainConfigFilename) && Process::commandExists($this->helperCommand);
    }

    /**
     * @param string $user user to get the alias for.
     * @return string|null The alias for the specified user, or null if not
     *     defined.
     */
    public function getAlias($user)
    {
        return isset($this->aliases[$user]) ? $this->aliases[$user] : null;
    }

    /**
     * Sets the alias for the specified user.
     *
     * @param string $user The user to set the alias for.
     * @param string|null $alias The alias to set. If empty the alias will be
     *     removed on store, if null the alias stays unchanged on store.
     */
    public function setAlias($user, $alias)
    {
        if ($alias === null) {
            if (array_key_exists($user, $this->aliases)) {
                unset($this->aliases[$user]);
            }
        } else {
            $this->aliases[$user] = $alias;
        }
    }

    /**
     * @return string[string] Associative array of aliases with key as user and
     *     value as alias.
     */
    public function getAliases()
    {
        return $this->aliases;
    }

    /**
     * Sets the aliases.
     *
     * @param string[string] $aliases Associative array of aliases to set.
     * @see setAlias()
     */
    public function setAliases(array $aliases)
    {
        $this->aliases = $aliases;
    }

    public function getGenericMapping($pattern)
    {
        return isset($this->genericMaps[$pattern]) ? $this->genericMaps[$pattern] : null;
    }

    public function setGenericMapping($pattern, $result)
    {
        $this->genericMaps[$pattern] = $result;
    }

    public function getGenericMappings()
    {
        return $this->genericMaps;
    }

    public function setGenericMappings(array $genericMappings)
    {
        $this->genericMaps = $genericMappings;
    }

    /**
     * @return boolean true if TLS is enabled for connections to another MTA.
     */
    public function isClientTLS()
    {
        return $this->clientTLS;
    }

    /**
     * Enables/disables TLS for connections to another MTA.
     *
     * @param boolean $enabled true to enable TLS, false otherwise.
     */
    public function setClientTLSEnabled($enabled)
    {
        $this->clientTLS = $enabled;
    }

    /**
     * @return string The relay host.
     */
    public function getRelayHost()
    {
        return $this->relayHost;
    }

    /**
     * Sets the relay host.
     *
     * @param string $relayHost The relay host to set.
     */
    public function setRelayHost($relayHost)
    {
        $this->relayHost = $relayHost;
    }

    /**
     * @return int The port of the relay host.
     */
    public function getRelayPort()
    {
        return $this->relayPort;
    }

    /**
     * Sets the port of the relay host.
     *
     * @param int $relayPort The port to set.
     */
    public function setRelayPort($relayPort)
    {
        $this->relayPort = $relayPort;
    }

    /**
     * Stores the MTA configuration.
     *
     * @throws \Exception
     */
    public function store()
    {
        $aliasesLines = $this->aliasesToLines();
        $genericMapsLines = $this->genericMapsToLines();
        $configLines = $this->toConfigLines();

        try {
            $tmpAliasesFilename = tempnam(sys_get_temp_dir(), "auralisAliases");
            $tmpGenericMapsFilename = tempnam(sys_get_temp_dir(), "auralisPostfixGeneric");
            $tmpConfigFilename = tempnam(sys_get_temp_dir(), "auralisPostfixMain");
            file_put_contents($tmpAliasesFilename, $aliasesLines);
            file_put_contents($tmpGenericMapsFilename, $genericMapsLines);
            file_put_contents($tmpConfigFilename, $configLines);

            $cmd = $this->helperCommand . " " . $tmpAliasesFilename . " "
                . $tmpGenericMapsFilename ." " . $tmpConfigFilename;
            Process::execThrow($cmd);
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpAliasesFilename);
            FileSystemUtil::tryUnlinkFile($tmpGenericMapsFilename);
            FileSystemUtil::tryUnlinkFile($tmpConfigFilename);
            throw $e;
        }
    }

    /**
     * @return array containing the lines for the configuration file.
     */
    private function aliasesToLines()
    {
        $aliasesWritten = array();
        $aliasesLines = file($this->aliasesFilename);
        $count = count($aliasesLines);
        for ($i = 0; $i < $count; $i++) {
            $user = trim(strstr($aliasesLines[$i], ":", true));
            $alias = $this->getAlias($user);
            if ($alias !== null) {
                if (strlen($alias) > 0) {
                    $aliasesLines[$i] = "$user: $alias\n";
                } else {
                    // remove alias
                    unset($aliasesLines[$i]);
                }
                $aliasesWritten[] = $user;
            }
        }
        $aliases = $this->getAliases();
        foreach ($aliases as $user => $alias) {
            if (!in_array($user, $aliasesWritten)) {
                if (strlen($alias) > 0) {
                    $aliasesLines[] = "$user: $alias\n";
                }
            }
        }

        return $aliasesLines;
    }

    /**
     * @return array containing the lines for the configuration file.
     */
    private function genericMapsToLines()
    {
        $genericMapsWritten = array();
        $genericMapsLines = file($this->getGenericMapsFilename());
        $count = count($genericMapsLines);
        for ($i = 0; $i < $count; $i++) {
            $pattern = trim(strstr($genericMapsLines[$i], " ", true));
            $result = $this->getGenericMapping($pattern);
            if ($result !== null) {
                if (strlen($result) > 0) {
                    $genericMapsLines[$i] = "$pattern $result\n";
                } else {
                    // remove mapping
                    unset($genericMapsLines[$i]);
                }
                $genericMapsWritten[] = $pattern;
            }
        }
        $maps = $this->getGenericMappings();
        foreach ($maps as $pattern => $result) {
            if (!in_array($pattern, $genericMapsWritten)) {
                if (strlen($pattern) > 0) {
                    $genericMapsLines[] = "$pattern $result\n";
                }
            }
        }

        return $genericMapsLines;
    }

    /**
     * @return array containing the lines for the postfix main configuration
     *     file.
     */
    private function toConfigLines()
    {
        $relayHostLine = "relayhost = [" . $this->getRelayHost() . "]";
        $relayPort = $this->getRelayPort();
        if (strlen($relayPort) > 0) {
            $relayHostLine .= ":$relayPort";
        }
        $relayHostLine .= "\n";
        if ($this->isClientTLS()) {
            $tlsLine = "smtp_use_tls = yes\n";
        } else {
            $tlsLine = "smtp_use_tls = no\n";
        }
        $configLines = file($this->postfixMainConfigFilename);
        $count = count($configLines);
        $tlsSet = false;
        $relayHostSet = false;
        for ($i = 0; $i < $count; $i++) {
            $line = trim($configLines[$i]);
            if (!$relayHostSet) {
                if (preg_match("/relayhost\s*=\s*/", $line)) {
                    $configLines[$i] = $relayHostLine;
                    $relayHostSet = true;
                }
            }
            if (!$tlsSet) {
                if (preg_match("/smtp_use_tls\s*=\s*/", $line)) {
                    $configLines[$i] = $tlsLine;
                    $tlsSet = true;
                }
            }
        }
        if (!$relayHostSet) {
            $configLines[] = $relayHostLine;
        }
        if (!$tlsSet) {
            $configLines[] = $tlsLine;
        }

        return $configLines;
    }
}
