<?php
namespace Topalis\Util;

use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Process\ProcessBuilder;

/**
 * Class with helper functions for the configuration.
 *
 * @author Andreas Hügel <andreas.huegel@topalis.com>
 * @author Frederick Roth <frederick.roth@topalis.com>
 * @author Fritz Elfert <fritz.elfert@topalis.com>
 * @author Holger Widmann <holger.widmann@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class CryptUtil
{
    /**
     * Decrypts an S/MIME encrypted message.
     *
     * @param string $content The encrypted content,
     * @param mixed $cert
     * @param mixed $key
     * @return string The decrypted content.
     * @throws \Exception On error.
     */
    public static function decryptPKCS7($content, $cert, $key)
    {
        $tmpfi = tempnam(sys_get_temp_dir(), 'auralisEI');
        $tmpfo = tempnam(sys_get_temp_dir(), 'auralisEO');
        try {
            /*
             * PHP's pkcs7_decrypt cannot read DER-encoded input. So we have to create
             * a temporary base64-encoded mime message. Furthermore, it requires all
             * data to be presented in files.
             */
            $b64 = base64_encode($content);
            $f = fopen($tmpfi, 'w');
            fwrite($f, "MIME-Version: 1.0\n");
            fwrite($f, "Content-Disposition: attachment; filename=\"smime.p7m\"\n");
            fwrite($f, "Content-Type: application/x-pkcs7-mime; smime-type=signed-data; name=\"smime.p7m\"\n");
            fwrite($f, "Content-Transfer-Encoding: base64\n");
            fwrite($f, "\n");
            while (strlen($b64) > 76) {
                fwrite($f, substr($b64, 0, 76) . "\n");
                $b64 = substr($b64, 76);
            }
            fwrite($f, $b64."\n");
            fwrite($f, "\n");
            fclose($f);

            if (openssl_pkcs7_decrypt($tmpfi, $tmpfo, $cert, $key) === false) {
                throw new \Exception(openssl_error_string());
            }

            $ret = file_get_contents($tmpfo);
            unlink($tmpfi);
            unlink($tmpfo);

            return $ret;
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpfi);
            FileSystemUtil::tryUnlinkFile($tmpfo);
            throw $e;
        }
    }

    /**
     * Encrypts an S/MIME message using 3DES.
     * @param string $content The content to encrypt.
     * @param mixed $recipcerts Either a lone X.509 certificate, or an array of
     *     X.509 certificates.
     * @return string The encrypted content.
     * @throws \Exception On error.
     */
    public static function encryptPKCS7($content, $recipcerts)
    {
        $flags = PKCS7_BINARY;
        $tmpfi = tempnam(sys_get_temp_dir(), 'auralisI');
        $tmpfo = tempnam(sys_get_temp_dir(), 'auralisO');
        $ret = false;

        try {
            file_put_contents($tmpfi, $content);

            if (openssl_pkcs7_encrypt($tmpfi, $tmpfo, $recipcerts, null, $flags, OPENSSL_CIPHER_3DES) === false) {
                throw new \Exception(openssl_error_string());
            }

            $signed = file_get_contents($tmpfo);
            $ret = base64_decode(preg_replace('/(.+\n)+\n/', '', $signed, 1));
            unlink($tmpfi);
            unlink($tmpfo);

            return $ret;
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpfi);
            FileSystemUtil::tryUnlinkFile($tmpfo);
            throw $e;
        }
    }

    /**
     * Encrypt the specified data using the specified password and a randomly
     * created salt. The used cipher method is AES-256-CBC.
     *
     * @param string $data The data to encrypt.
     * @param string $pass The password to be used for encryption.
     *
     * @throws \Exception On error.
     * @return string The encrypted data.
     */
    public static function encrypt($data, $pass)
    {
        static $cipherMethod = 'aes-256-cbc';
        $salt = Random::getString(16);
        $key = $pass.$salt;
        $sha = sha1($key, true);
        $tmp = unpack('H*', $sha);
        $edata = base64_encode($salt . "\n" . $tmp[1]) . "\n";
        $iv = substr($sha, 0, 16);
        $tmp = openssl_encrypt($data, $cipherMethod, $key, false, $iv);
        if ($tmp === false) {
            throw new \Exception(openssl_error_string());
        }
        return $edata . $tmp;
    }

    /**
     * Decrypt the specified data using the specified password.
     *
     * @param string $cdata The data to decrypt.
     * @param string $pass The password to be used for decryption.
     * @return string The decrypted data.
     * @throws \InvalidArgumentException If specified data has wrong format.
     * @throws \Exception On error.
     */
    public static function decrypt($cdata, $pass)
    {
        $a = explode("\n", $cdata);
        if (count($a) != 2) {
            throw new \InvalidArgumentException("Encrypted config data has wrong format");
        }
        $a2 = explode("\n", base64_decode($a[0]));
        if (count($a2) != 2) {
            throw new \InvalidArgumentException("Encrypted config data has wrong format");
        }
        $salt = $a2[0];
        $sha = pack('H*', $a2[1]);
        $edata = $a[1];
        $key = $pass.$salt;
        $iv = substr($sha, 0, 16);
        $ret = openssl_decrypt($edata, 'aes-256-cbc', $key, false, $iv);
        if ($ret === false) {
            throw new \Exception(openssl_error_string());
        }
        return $ret;
    }

    /**
     * Encrypts a file at filesytem level with openssl.
     * This method is a lot faster and less memory consuming than {@see CryptUtil::encrypt()}
     *
     * @param string $file path for plain file
     * @param string $password The password to be used for encryption
     *
     * @return string path of encrypted file
     *
     * @throws ProcessFailedException
     */
    public static function encryptInFilesystem($file, $password)
    {
        $tmpEncrypted = tempnam(sys_get_temp_dir(), "auralisBackupEncrypted");
        $processBuilder = new ProcessBuilder(array(
            "openssl",
            "enc",
            "-aes-256-cbc",
            "-salt",
            "-base64",
            "-in",
            $file,
            "-out",
            $tmpEncrypted,
            // use stdin to pass the password to openssl
            "-pass",
            "stdin"
        ));
        $process = $processBuilder->getProcess();
        $process
            ->setStdin($password)
            ->run()
        ;
        if (!$process->isSuccessful()) {
            throw new ProcessFailedException($process);
        }

        return $tmpEncrypted;
    }

    /**
     * Decrypts a file at filesytem level with openssl.
     * This method is a lot faster and less memory consuming than {@see CryptUtil::decrypt()}
     *
     * @param string $file path for encrypted file
     * @param string $password The password to be used for decryption
     *
     * @return string path of decrypted file
     *
     * @throws ProcessFailedException
     */
    public static function decryptInFilsystem($file, $password)
    {
        $tmpDecrypted = tempnam(sys_get_temp_dir(), "auralisBackupDecrypted");
        $processBuilder = new ProcessBuilder(array(
            "openssl",
            "enc",
            "-d", // decrypt
            "-aes-256-cbc",
            "-salt",
            "-base64",
            "-in",
            $file,
            "-out",
            $tmpDecrypted,
            // use stdin to pass the password to openssl
            "-pass",
            "stdin"
        ));
        $process = $processBuilder->getProcess();
        $process
            ->setStdin($password)
            ->run()
        ;
        if (!$process->isSuccessful()) {
            throw new ProcessFailedException($process);
        }

        return $tmpDecrypted;
    }
}
