<?php
namespace Topalis\Util;

use Topalis\Form\Validator\Constraint\PasswordValidator;

/**
 * This class contains functions to convert error codes to human readable
 * messages.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class ErrorCodeTranslator
{
    /**
     * Translate PHP upload error code into human readable message.
     * @param int $errorCode PHP error code from file upload
     * @return string The corresponding error message.
     */
    public static function uploadError($errorCode)
    {
        switch ($errorCode) {
            case UPLOAD_ERR_OK:
                return "No error";
            case UPLOAD_ERR_INI_SIZE:
                return "Maximum upload file size exceeded";
            case UPLOAD_ERR_FORM_SIZE:
                return "Maximum file size (specified in form) exceeded";
            case UPLOAD_ERR_PARTIAL:
                return "Upload incomplete";
            case UPLOAD_ERR_NO_FILE:
                return "No file uploaded";
            case UPLOAD_ERR_NO_TMP_DIR:
                return "Upload temporary directory missing";
            case UPLOAD_ERR_CANT_WRITE:
                return "Could not write uploaded file to disk";
            case UPLOAD_ERR_EXTENSION:
                return "Uploaded blocked by PHP extension";
            default:
                return "Unknown error";
        }
    }

    /**
     * @param integer $errorCode the ZipArchive error code to get an error message for.
     * @return string Error message for a ZipArchive error code.
     */
    public static function zipError($errorCode)
    {
        // using constant name as a string to make this function PHP4 compatible
        $zipFileFunctionsErrors = array(
                'ZIPARCHIVE::ER_MULTIDISK' => 'Multi-disk zip archives not supported.',
                'ZIPARCHIVE::ER_RENAME' => 'Renaming temporary file failed.',
                'ZIPARCHIVE::ER_CLOSE' => 'Closing zip archive failed',
                'ZIPARCHIVE::ER_SEEK' => 'Seek error',
                'ZIPARCHIVE::ER_READ' => 'Read error',
                'ZIPARCHIVE::ER_WRITE' => 'Write error',
                'ZIPARCHIVE::ER_CRC' => 'CRC error',
                'ZIPARCHIVE::ER_ZIPCLOSED' => 'Containing zip archive was closed',
                'ZIPARCHIVE::ER_NOENT' => 'No such file.',
                'ZIPARCHIVE::ER_EXISTS' => 'File already exists',
                'ZIPARCHIVE::ER_OPEN' => 'Can\'t open file',
                'ZIPARCHIVE::ER_TMPOPEN' => 'Failure to create temporary file.',
                'ZIPARCHIVE::ER_ZLIB' => 'Zlib error',
                'ZIPARCHIVE::ER_MEMORY' => 'Memory allocation failure',
                'ZIPARCHIVE::ER_CHANGED' => 'Entry has been changed',
                'ZIPARCHIVE::ER_COMPNOTSUPP' => 'Compression method not supported.',
                'ZIPARCHIVE::ER_EOF' => 'Premature EOF',
                'ZIPARCHIVE::ER_INVAL' => 'Invalid argument',
                'ZIPARCHIVE::ER_NOZIP' => 'Not a zip archive',
                'ZIPARCHIVE::ER_INTERNAL' => 'Internal error',
                'ZIPARCHIVE::ER_INCONS' => 'Zip archive inconsistent',
                'ZIPARCHIVE::ER_REMOVE' => 'Can\'t remove file',
                'ZIPARCHIVE::ER_DELETED' => 'Entry has been deleted',
        );

        $errorMessage = 'unknown';
        foreach ($zipFileFunctionsErrors as $constName => $em) {
            if (defined($constName) && constant($constName) === $errorCode) {
                $errorMessage = $em;
                break;
            }
        }
        $message = 'Zip File Function error: '.$errorMessage;
        return $message;
    }

    public static function passwordValidationError($errorCode, $minLength, $kernel)
    {
        switch ($errorCode) {
            case PasswordValidator::E_DIGIT:
                return $kernel->tr("admin.error.password.digit");
            case PasswordValidator::E_LOWER_CASE:
                return $kernel->tr("admin.error.password.lowercase");
            case PasswordValidator::E_MATCH:
                return $kernel->tr("admin.error.password.doNotMatch");
            case PasswordValidator::E_MIN_LENGTH:
                return $kernel->tr("admin.error.password.minLength", array("%minLength%" => $minLength));
            case PasswordValidator::E_SPECIAL_CHAR:
                return $kernel->tr("admin.error.password.specialChar");
            case PasswordValidator::E_UPPER_CASE:
                return $kernel->tr("admin.error.password.capital");
            case PasswordValidator::E_WHITESPACE:
                return $kernel->tr("admin.error.password.whitespace");
        }
    }
}
