<?php
namespace Topalis\Util;

use Topalis\Util\Exception\Process\ExecException;

/**
 * This class contains a method to execute external commands.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class Process
{
    /**
     * Determines if a command exists on the current environment.
     *
     * @param string $command The command to check
     * @return bool true if the command has been found, false otherwise.
     */
    public static function commandExists($command)
    {
        if (PHP_OS === "WINNT") {
            return self::exec(sprintf("where /q %s", $command)) == 0;
        } else {
            return self::exec(sprintf("command -v %s >/dev/null 2>&1", $command)) == 0;
        }

        return false;
    }

    /**
     * Executes the specified command. Throws ExecException if the return code
     * is not 0 Do not use output redirection when using this method.
     *
     * @param string $cmd The command to execute.
     * @param string $out Reference to output buffer. If set, STDOUT and STDERR are returned in this var.
     * @param string $input Input data. If set, the content is fed into STDIN of the invoked program.
     * @param int $bytesWritten Set to the number of written bytes of the input data, if input data is given.
     * @throws ExecException if return code is not 0.
     */
    public static function execThrow($cmd, &$out = null, $input = null, &$bytesWritten = null)
    {
        if ($out === null) {
            $out = '';
        }
        $rc = self::exec($cmd, $out, $input, $bytesWritten);
        if ($rc !== 0) {
            throw new ExecException($cmd, $out, $rc);
        }
    }

    /**
     * Executes the specified command. Do not use output redirection when using
     * this method.
     *
     * @param string $cmd The command to execute.
     * @param string $out Reference to output buffer. If set, STDOUT and STDERR are returned in this var.
     * @param string $input Input data. If set, the content is fed into STDIN of the invoked program.
     * @param int $bytesWritten Set to the number of written bytes of the input data, if input data is given.
     * @return int|boolean exit status of the executed command, or false if the process couldn't be started.
     */
    public static function exec($cmd, &$out = null, $input = null, &$bytesWritten = null)
    {
        $pipes = array();
        $proc = proc_open($cmd . " 2>&1", array(0 => array('pipe', 'r'), 1 => array('pipe', 'w')), $pipes);
        if ($proc === false) {
            return false;
        }
        if ($input !== null) {
            $written = fwrite($pipes[0], $input);
            if ($bytesWritten !== null) {
                $bytesWritten = $written;
            }
        }
        fclose($pipes[0]);
        if ($out !== null) {
            $out = trim(stream_get_contents($pipes[1]));
        }
        fclose($pipes[1]);
        $r = proc_close($proc);

        return $r;
    }

    /**
     * Executes the specified command. In contrast to exec() stdout and stderr
     * are written into different references, so it's possible to use output
     * redirection.
     *
     * @param string $cmd The command to execute.
     * @param string $stdout Reference to output buffer. If set, STDOUT is returned in this var.
     * @param string $stderr Reference to output buffer. If set, STDERR is returned in this var.
     * @param string $input Input data. If set, the content is fed into STDIN of the invoked program.
     * @param int $bytesWritten Set to the number of written bytes of the input data, if input data is given.
     * @return int|boolean exit status of the executed command, or false if the process couldn't be started.
     */
    public static function execErr($cmd, &$stdout = null, &$stderr = null, $input = null, &$bytesWritten = null)
    {
        $descriptorspec = array(0 => array('pipe', 'r'), 1 => array('pipe', 'w'), 2 => array('pipe', 'w'));
        $pipes = array();
        $proc = proc_open($cmd, $descriptorspec, $pipes);
        if ($proc === false) {
            return false;
        }
        if ($input !== null) {
            $written = fwrite($pipes[0], $input);
            if ($bytesWritten !== null) {
                $bytesWritten = $written;
            }
        }
        fclose($pipes[0]);
        if ($stdout !== null) {
            $stdout = trim(stream_get_contents($pipes[1]));
        }
        fclose($pipes[1]);
        if ($stderr !== null) {
            $stderr = stream_get_contents($pipes[2]);
        }
        fclose($pipes[2]);
        $r = proc_close($proc);

        return $r;
    }
}
