// Globals
var graphs = {};

/**
 * Extend Ember.js ArrayController to add setFromJSON functionallity
 */
App.ArrayController = Ember.ArrayController.extend({
    idProperty: null, // property used to identify objects
    sort: false, // sort array by sortValue
    sortOrder: 'DESC',

    /**
     * Set array value by json data. If item doesn't exist in array this.createFromJSON will be called
     *
     * @param items JSON array with items
     */
    setFromJSON: function (items) {
        var self = this;
        _.each(items, function(item) {
            var oldItem = this.find(function(item2) {
                if (item[self.idProperty] == item2.get(self.idProperty)) {
                    return true;
                }
            });

            if (oldItem === undefined) {
                this.add(this.createFromJSON(item));

            } else {
                oldItem.setProperties(item);
            }
        }, this);
    },
    /**
     * add object at correct position if sort = true
     * @param object
     */
    add: function(object) {
        if (this.sort) {
            var length = this.get('length'), idx;

            idx = this.binarySearch(object.get('sortValue'), 0, length);

            this.insertAt(idx, object);

            object.addObserver('sortValue', this, 'sortValueDidChange');
        } else {
            this.addObject(object);
        }
    },
    /**
     * Binary search implementation that finds the index
     * where a item should be inserted.
     * @param value
     * @param low
     * @param high
     * @return {*}
     */
    binarySearch: function(value, low, high) {
        var mid, midValue;

        if (low === high) {
            return low;
        }

        mid = low + Math.floor((high - low) / 2);
        midValue = this.objectAt(mid).get('sortValue');

        if ('DESC' == this.sortOrder) {
            if (value > midValue) {
                return this.binarySearch(value, mid+1, high);
            } else if (value < midValue) {
                return this.binarySearch(value, low, mid);
            }
        } else {
            if (value < midValue) {
                return this.binarySearch(value, mid+1, high);
            } else if (value > midValue) {
                return this.binarySearch(value, low, mid);
            }
        }


        return mid;
    },
    remove: function(object) {
        this.removeObject(object);
        object.removeObserver('sortValue', this, 'sortValueDidChange');
    },
    sortValueDidChange: function(object) {
        this.remove(object);
        this.add(object);
    }
});


// ----------------------

// Devices

App.Devices = App.ArrayController.extend({
    content: [],
    idProperty: 'deviceid',
    sort: true,
    sortOrder: 'ASC',
    createFromJSON: function(device) {
        return App.Device.create(device);
    }

});

App.Device = Ember.Object.extend({
    deviceid: "",
    date: null,
    comment: null,
    timestamp: 0,
    os: 0,
    sortValue: function() {
        return this.timestamp;
    }.property('timestamp')
});

App.DevicesView = Ember.View.extend({
    tagName: "tbody"
});

// ----------------------

// Dashboard

App.DashboardController = Ember.Controller.create({
    init: function() {
        var self = this;
        this.refresh();
        setInterval(_.bind(this.refresh, this), 5000);
    },

    devices: App.Devices.create({}),
    data: null,

    refresh: function() {
        var self = this;
        $.ajax("/admin/dashboard/data", {
            dataType: 'json',
            success: function(data) {
                self.setData(data);
            }
        });
    },
    setData: function(data) {
        this.get('devices').setFromJSON(data.devices);
        this.set('data', data);
        if (data.cpu) {
            graphs.cpu.addUsage(data.cpu);
        }
        if (data.memory) {
            graphs.memory.addUsage(data.memory);
            graphs.memory.graph.window.xMax = data.memory.total;
        }
        if (data.interfaces) {
            for (var i = 0; i < data.interfaces.length; i++) {
                var iface = data.interfaces[i];
                if (typeof graphs.network[iface.name] !== 'undefined') {
                    graphs.network[iface.name].addUsage({
                        rx: iface.received * -1, // show rx values opposed to tx in graph
                        tx: iface.send
                    });
                }
            }
        }
        if (data.disks) {
            graphs.filesystem.setDisks(data.disks);
        }
    }
});


App.DashboardTrafficView = Ember.View.extend({
    templateName: "dashboard-devices"
});
