<?php
namespace Topalis\Profile\Payload;

use CFPropertyList\CFArray;
use CFPropertyList\CFDictionary;
use CFPropertyList\CFPropertyList;
use CFPropertyList\CFString;
use CFPropertyList\CFNumber;

// @codingStandardsIgnoreStart
/**
 * Represents an APN payload.
 * This payload configures a APN connection on the device.
 * @see        https://developer.apple.com/library/ios/featuredarticles/iPhoneConfigurationProfileRef/iPhoneConfigurationProfileRef.pdf#//apple_ref/doc/uid/TP40010206-CH1-SW24
 * @deprecated since iOS 7 in favor of the Cellular payload.
 * @author     Dominic Tubach <dominic.tubach@topalis.com>
 */
// @codingStandardsIgnoreEnd
class ApnPayload extends AbstractPayload
{
    const TYPE = "com.apple.apn.managed";

    /**
     * @var \CFPropertyList\CFArray
     */
    protected $content;

    public function __construct(CFDictionary $dictionary = null)
    {
        parent::__construct($dictionary);
        $content = $this->get("PayloadContent");
        if ($content === null) {
            $content = new CFArray();

            $defaultData = new CFDictionary();
            $defaultData->add("DefaultsData", new CFDictionary(array(
                "apns" => new CFArray(array(
                    0 => new CFDictionary(array(
                        "apn"       => new CFString(),
                        "username"  => new CFString(),
                        "password"  => new CFString(),
                        "proxy"     => new CFString(),
                        "proxyPort" => new CFNumber(),
                    ))
                ))
            )));
            $defaultData->add("DefaultsDomainName", new CFString("com.apple.managedCarrier"));
            $content->add($defaultData);

            $this->set("PayloadContent", $content);
        }
        $this->content = $content;
    }

    protected function getContent()
    {
        return $this->content;
    }

    /**
     * @return CFDictionary
     */
    public function getDefaultsData()
    {
        return $this->getContent()->get(0)->get("DefaultsData");
    }

    public function setDefaultsData(array $defaultsData = null)
    {
        $this->getContent()->get(0)->setValue(new CFArray($defaultsData));
    }

    public function getDefaultsDomainName()
    {
        return $this->getContent()->get(0)->get("DefaultsDomainName")->getValue();
    }

    public function setDefaultsDomainName($defaultsDomainName)
    {
        $this->getContent()->get(0)->get("DefaultsDomainName")->setValue($defaultsDomainName);
    }

    /**
     * @return CFArray
     */
    public function getApns()
    {
        $defaultData = $this->getDefaultsData();

        return $defaultData->get("apns");
    }

    public function setApns(array $apns = null)
    {
        $this->getDefaultsData()->set("apns", new CFArray($apns));
    }

    /**
     * @return CFDictionary
     */
    protected function getFirstApn()
    {
        return $this->getApns()->get(0);
    }

    public function getApn()
    {
        return $this->getFirstApn()->get("apn")->getValue();
    }

    public function setApn($apn)
    {
        $this->getFirstApn()->add("apn", new CFString($apn));
    }

    public function getUsername()
    {
        return $this->getFirstApn()->get("username")->getValue();
    }

    public function setUsername($username)
    {
        $this->getFirstApn()->add("username", new CFString($username));
    }

    public function getPassword()
    {
        return $this->getFirstApn()->get("password")->getValue();
    }

    public function setPassword($password)
    {
        $this->getFirstApn()->add("password", new CFString($password));
    }

    public function getProxy()
    {
        return $this->getFirstApn()->get("proxy")->getValue();
    }

    public function setProxy($proxy)
    {
        $this->getFirstApn()->add("proxy", new CFString($proxy));
    }

    public function getProxyPort()
    {
        return $this->getFirstApn()->get("proxyPort")->getValue();
    }

    public function setProxyPort($proxyPort)
    {
        $this->getFirstApn()->add("proxyPort", new CFNumber($proxyPort));
    }
}
