<?php
namespace Topalis\Profile\Payload;

use Topalis\Util\StringUtil;
use CFPropertyList\CFTypeDetector;
use CFPropertyList\CFPropertyList;
use CFPropertyList\CFDictionary;
use Topalis\Profile\AbstractProfile;
use Topalis\Wifi\Entity\WifiConfig;

/**
 * Factory to create payloads.
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class PayloadFactory
{
    /**
     * @var \Topalis\Configuration\AuralisConfig
     */
    private $config;

    /**
     * @var \Topalis\Profile\AbstractProfile
     */
    protected $profile;

    public static function fromArray(array $payload)
    {
        $td = new CFTypeDetector();
        $dictionary = $td->toCFType($payload);
        return self::fromDictionary($dictionary);
    }

    public static function fromDictionary(CFDictionary $dictionary)
    {
        $type = $dictionary->get("PayloadType");
        if ($type !== null) {
            $type = $type->getValue();
        }
        switch ($type) {
            case ExchangePayload::TYPE:
                $payload = new ExchangePayload($dictionary);
                break;
            case WebClipPayload::TYPE:
                $payload = new WebClipPayload($dictionary);
                break;
            case MdmPayload::TYPE:
                $payload = new MdmPayload($dictionary);
                break;
            case ScepPayload::TYPE:
                $payload = new ScepPayload($dictionary);
                break;
            case PasscodePayload::TYPE:
                $payload = new PasscodePayload($dictionary);
                break;
            case RestrictionsPayload::TYPE:
                $payload = new RestrictionsPayload($dictionary);
                break;
            case WifiPayload::TYPE:
                $payload = new WifiPayload($dictionary);
                break;
            case ApnPayload::TYPE:
                $payload = new ApnPayload($dictionary);
                break;
            case WebContentFilterPayload::TYPE:
                $payload = new WebContentFilterPayload($dictionary);
                break;
            case AirPlayPayload::TYPE:
                $payload = new AirPlayPayload($dictionary);
                break;
            case AirPrintPayload::TYPE:
                $payload = new AirPrintPayload($dictionary);
                break;
            case \Topalis\Profile\Payload\Android\PasscodePayload::TYPE:
                $payload = new \Topalis\Profile\Payload\Android\PasscodePayload($dictionary);
                break;
            case \Topalis\Profile\Payload\Android\RestrictionsPayload::TYPE:
                $payload = new \Topalis\Profile\Payload\Android\RestrictionsPayload($dictionary);
                break;
            case \Topalis\Profile\Payload\Android\WifiPayload::TYPE:
                $payload = new \Topalis\Profile\Payload\Android\WifiPayload($dictionary);
                break;
            case null:
                $payload = new MinimalPayload($dictionary);
                break;
            default:
                $payload = new GeneralPayload($dictionary);
                break;
        }
        return $payload;
    }

    public static function fromXML($xml)
    {
        $plist = new CFPropertyList();
        $plist->parse($xml);
        return self::fromDictionary($plist->getValue());
    }

    public function __construct(AbstractProfile $profile, \Topalis\Configuration\AuralisConfig $config)
    {
        $this->profile = $profile;
        $this->config = $config;
    }

    /**
     * @param string $emailAddress User email address
     * @param string $username
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup The config
     *     group of the device the profile will be installed on.
     * @return ExchangePayload
     */
    public function createExchangePayload($emailAddress, $username,
        \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup)
    {
        $exchangePayload = new ExchangePayload();
        $this->initPayload($exchangePayload, "eas.account");
        $exchangePayload->setEmailAddress($emailAddress);
        $exchangePayload->setDescription("Exchange via auralis");
        $exchangePayload->setDisplayName($emailAddress . " via auralis");

        $config = $this->getConfig();
        $exchangeDomain = $config->getExchangeDomain();
        if (strlen($exchangeDomain) == 0) {
            $exchangePayload->setUserName($username);
        } else if ($config->isExchangeAltLoginType()) {
            $exchangePayload->setUserName($username . "@" . $exchangeDomain);
        } else {
            $exchangePayload->setUserName($exchangeDomain . "\\" . $username);
        }
        $exchangePayload->setHost($config->getHostname() .":" . $config->getPrimaryPort());
        $exchangePayload->setSSL(true);

        $exchangePayload->setMailNumberOfPastDaysToSync($configGroup->getSyncDays());
        $exchangePayload->setPreventMove(!$configGroup->isAllowMove());
        $exchangePayload->setPreventAppSheet(!$configGroup->isFromAnyApp());

        return $exchangePayload;
    }

    public function createMdmPayload($mdmTopic, $scepPayloadUUID, $useDevelopmentAPNS = false)
    {
        $mdmPayload = new MdmPayload();
        $this->initPayload($mdmPayload, "mdm");
        $mdmPayload->setDescription("Configures TOsecurity auralis MDM");
        $mdmPayload->setDisplayName("auralis MDM");
        $mdmPayload->setServerUrl($this->getMdmUrl());
        $mdmPayload->setTopic($mdmTopic);
        $mdmPayload->setIdentityCertificateUUID($scepPayloadUUID);
        $mdmPayload->setUseDevelopmentAPNS($useDevelopmentAPNS);
        $mdmPayload->setAccessRights(8191); // all rights
        $mdmPayload->setCheckOutWhenRemoved(true);

        return $mdmPayload;
    }

    public function createProfileServicePayload($challenge)
    {
        $payload = new ProfileServicePayload();
        $payload->setUrl($this->getProfileServiceUrl());
        // Avoid warning about commented code
        // @codingStandardsIgnoreStart
        $payload->setDeviceAttributes(
            array(
                    'UDID',
                    'VERSION',
                    'PRODUCT',              // ie. iPhone1,1 or iPod2,1
                    //'MAC_ADDRESS_EN0',      // WiFi MAC address
                    'DEVICE_NAME',          // given device name "iPhone"
                    // Items below are only available on iPhones
                    'IMEI',
                    'ICCID',
                    //'COMPROMISED'
            )
        );
        // @codingStandardsIgnoreEnd
        if (strlen($challenge) > 0) {
            $payload->setChallenge($challenge);
        }

        return $payload;
    }

    /**
     * @param string $certificate The DER encoded certificate.
     * @return \Topalis\Profile\Payload\RootCertificatePayload
     */
    public function createRootCertificatePayload($certificate)
    {
        $x509 = openssl_x509_parse($certificate);
        $certificatePayload = new RootCertificatePayload();
        $this->initPayload($certificatePayload, "certificate.ca");
        $certificatePayload->setCertificate($certificate);
        $certificatePayload->setDisplayName($x509['issuer']['CN']);
        return $certificatePayload;
    }

    /**
     * @param string $purpose The purpose will be used as payload name, payload
     *    display name, and certificate common name.
     * @param \Topalis\Certificate\Certificate $caCert This certificate is used
     *     to fill the different certificate subject fields.
     * @param string $challenge
     * @param string $email The email address is added to the certificate
     *    subject if provided.
     * @return \Topalis\Profile\Payload\ScepPayload
     */
    public function createScepPayload($purpose, \Topalis\Certificate\Certificate $caCert,
        $challenge = null, $email = null)
    {
        $scepPayload = new ScepPayload();
        $this->initPayload($scepPayload, "scep");
        $scepPayload->setDisplayName($purpose);
        $scepPayload->setUrl($this->getScepUrl());
        $scepPayload->setName($purpose);
        $uuid = $this->getProfile()->getUUIDGenerator()->generateUUID();
        $scepPayload->setUUID($uuid);

        $subject = array(
                array(array("C", $caCert->getCountry())),
                array(array("ST", $caCert->getState())),
                array(array("L", $caCert->getLocation())),
                array(array("O", $caCert->getOrganization())),
                array(array("OU", $caCert->getOrganizationalUnit())),
                array(array("CN", $purpose))
        );
        if ($email !== null) {
            $emailSubject = array(array(array("0.2.262.1.10.7.28", $email)));
            $subject = array_merge($subject, $emailSubject);
        }

        $scepPayload->setSubject($subject);
        if (strlen($challenge) > 0) {
            $scepPayload->setChallenge($challenge);
        }
        $scepPayload->setKeysize(2048);
        $scepPayload->setKeyType("RSA");
        $scepPayload->setKeyUsage(5); // digital signature (1) | key encipherment (4)

        return $scepPayload;
    }

    /**
     * @param string $url The URL of the WebClip.
     * @param string $label The label of the WebClip.
     * @return WebClipPayload
     */
    public function createWebClipPayload($url, $label)
    {
         $webClipPayload = new WebClipPayload();
         $this->initPayload($webClipPayload, "webclip");
         $webClipPayload->setUrl($url);
         $webClipPayload->setLabel($label);
         $webClipPayload->setDisplayName($label);
         $webClipPayload->setDescription("Reverse Proxy for ". $label);

         return $webClipPayload;
    }

    public function createWifiPayload(WifiConfig $wifiConfig)
    {
        $wifiPayload = new WifiPayload();
        $this->initPayload($wifiPayload, $wifiConfig->getSsid());
        $wifiPayload->setDisplayName($wifiConfig->getName());
        $wifiPayload->setSsid($wifiConfig->getSsid());
        $wifiPayload->setHiddenNetwork($wifiConfig->isHidden());
        $wifiPayload->setAutoJoin($wifiConfig->isAutoJoin());
        $wifiPayload->setHotspot($wifiConfig->isHotspot());

        switch ($wifiConfig->getEncryptionType()) {
            case WifiConfig::ENCRYPTION_TYPE_NONE:
                $wifiPayload->setEncryptionType("None");
                break;
            case WifiConfig::ENCRYPTION_TYPE_WPAPSK:
                /* fall through */
            case WifiConfig::ENCRYPTION_TYPE_WPA2PSK:
                $wifiPayload->setEncryptionType("WPA");
                break;
        }
        $wifiPayload->setPassword($wifiConfig->getEncryptionKey());

        switch ($wifiConfig->getProxyType()) {
            case WifiConfig::PROXY_TYPE_NONE:
                $wifiPayload->setProxyType("None");
                break;
            case WifiConfig::PROXY_TYPE_MANUAL:
                $wifiPayload->setProxyType("Manual");
                $wifiPayload->setProxyServer($wifiConfig->getProxyHost());
                $wifiPayload->setProxyServerPort($wifiConfig->getProxyPort());
                $wifiPayload->setProxyUsername($wifiConfig->getProxyUsername());
                $wifiPayload->setProxyPassword($wifiConfig->getProxyPassword());
                break;
            case WifiConfig::PROXY_TYPE_AUTO:
                $wifiPayload->setProxyType("Auto");
                $wifiPayload->setProxyPacUrl($wifiConfig->getProxyPacUrl());
                $wifiPayload->setProxyPacFallbackAllowed($wifiConfig->isProxyPacFallbackAllowed());
        }

        return $wifiPayload;
    }

    /**
     * @return VpnPayload
     */
    public function createVpnPayload()
    {
        $payload = new VpnPayload();
        $this->initPayload($payload, "vpn");

        return $payload;
    }

    public function createPkcs12Payload($pkcs12, $password)
    {
        $pkcs12Payload = new Pkcs12Payload();
        $this->initPayload($pkcs12Payload, "credential");
        $pkcs12Payload->setPayloadContent($pkcs12);
        $pkcs12Payload->setPassword($password);

        return $pkcs12Payload;
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    public function getConfig()
    {
        return $this->config;
    }

    public function getOrganization()
    {
        return $this->getProfile()->getOrganization();
    }

    public function getMdmUrl()
    {
        $port = $this->getConfig()->getPrimaryPort();
        return "https://" . $this->getConfig()->getHostname() . ":" . $port . "/mdm";
    }

    public function getProfileServiceUrl()
    {
        $port = $this->getConfig()->getSecondaryPort();
        return "https://" . $this->getConfig()->getHostname() . ":" . $port . "/profile/ios";
    }

    public function getScepUrl()
    {
        $port = $this->getConfig()->getSecondaryPort();
        return "https://" . $this->getConfig()->getHostname() . ":" . $port . "/scep";
    }

    /**
     * @return AbstractProfile
     */
    public function getProfile()
    {
        return $this->profile;
    }

    public function initPayload(AbstractPayload $payload, $identifierPostfix)
    {
        $identifier = $this->getProfile()->getIdentifier();
        if (!StringUtil::startsWith($identifierPostfix, ".")) {
            $identifier .= ".";
        }
        $identifier .= $identifierPostfix;
        $payload->setIdentifier($identifier);
        $payload->setOrganization($this->getOrganization());
    }
}
