<?php
namespace Topalis\Profile\Payload;

use CFPropertyList\CFBoolean;
use CFPropertyList\CFData;
use CFPropertyList\CFNumber;
use CFPropertyList\CFString;
use Topalis\Profile\Payload\Data\Ipv4;
use Topalis\Profile\Payload\Data\OpenVpnConfig;

// @codingStandardsIgnoreStart
/**
 * Represents an VPN payload.
 * This payload configures a VPN connection on the device.
 * Note: this class doesn't contain methods for entries marked as deprecated.
 * @see https://developer.apple.com/library/ios/featuredarticles/iPhoneConfigurationProfileRef/iPhoneConfigurationProfileRef.pdf#//apple_ref/doc/uid/TP40010206-CH1-SW27
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
// @codingStandardsIgnoreEnd
class VpnPayload extends AbstractPayload
{
    const TYPE = "com.apple.vpn.managed";

    public function getUserDefinedName()
    {
        return $this->getValue("UserDefinedName");
    }

    public function setUserDefinedName($userDefinedName)
    {
        if ($userDefinedName === null) {
            $this->del("UserDefinedName");
        } else {
            $this->set("UserDefinedName", new CFString($userDefinedName));
        }
    }

    public function getIpv4()
    {
        return new Ipv4($this->getValue("IPv4"));
    }

    public function setIpv4(Ipv4 $ipv4)
    {
        $this->setValue("IPv4", $ipv4->getDictionary(), self::TYPE_DICTIONARY);
    }

    /**
     * @return string|null The type of VPN connection. Possible values are L2TP,
     *     PPTP, IPSec, and VPN. Default: null.
     */
    public function getVpnType()
    {
        return $this->getValue("VPNType");
    }

    /**
     * Sets the type of the VPN connection.
     *
     * @param string $vpnType The connection type to set. Possible values are
     *     L2TP, PPTP, IPSec, and VPN. Use null to delete this entry.
     */
    public function setVpnType($vpnType)
    {
        $this->setValue("VPNType", $vpnType, self::TYPE_STRING);
    }

    public function getVpnSubType()
    {
        return $this->getValue("VPNSubType");
    }

    public function setVpnSubType($subType)
    {
        $this->setValue("VPNSubType", $subType, self::TYPE_STRING);
    }

    /**
     * @return int|null Interestingly this is an integer: 1 if enabled, 0
     *     if disabled. Default: null
     */
    public function isOnDemandEnabled()
    {
        return $this->getValue("OnDemandEnabled");
    }

    public function getVendorConfig()
    {
        return $this->getValue("VendorConfig");
    }

    public function setVendorConfig(array $vendorConfig = null)
    {
        $this->setValue("VendorConfig", $vendorConfig, self::TYPE_DICTIONARY);
    }

    public function getPpp()
    {
        return $this->getValue("PPP");
    }

    public function setPpp(array $ppp = null)
    {
        $this->setValue("PPP", $ppp, self::TYPE_DICTIONARY);
    }

    public function setOpenVpn(OpenVpnConfig $config)
    {
        $this->setVpnType("VPN");
        $this->setVpnSubType("net.openvpn.OpenVPN-Connect.vpnplugin");
        $this->setValue("VPN", $config->getDictionary(), self::TYPE_DICTIONARY);
    }

    public function getOpenVpn()
    {
        return new OpenVpnConfig($this->getValue("VPN"));
    }

    public function getIpSec()
    {
        return $this->getValue("IPSec");
    }

    public function setIpSec($ipSec)
    {
        $this->setValue("IPSec", $ipSec, self::TYPE_DICTIONARY);
    }

    /*
     * The following code describes the PPP dictionary and the IPSec dictionary.
     * May be we can use it to create special dictionary classess...
     */
    // @codingStandardsIgnoreStart
//     // Payloads with VPN connection type PPP begin
//     public function getAuthName()
//     {
//         return $this->getValue("AuthName");
//     }

//     public function setAuthName($authName)
//     {
//         if ($authName === null) {
//             $this->del("AuthName");
//         } else {
//             $this->set("AuthName", new CFString($authName));
//         }
//     }

//     public function getAuthPassword()
//     {
//         return $this->getValue("AuthPassword");
//     }

//     public function setAuthPassword($authPassword)
//     {
//         if ($authPassword === null) {
//             $this->del("AuthPassword");
//         } else {
//             $this->set("AuthPassword", new CFString($authPassword));
//         }
//     }

//     public function isUseTokenCard()
//     {
//         return $this->getValue("TokenCard");
//     }

//     public function setUseTokenCard($useTokenCard)
//     {
//         if ($useTokenCard === null) {
//             $this->del("TokenCard");
//         } else {
//             $this->set("TokenCard", new CFBoolean($useTokenCard));
//         }
//     }

//     public function getCommRemoteAddress()
//     {
//         return $this->getValue("CommRemoteAddress");
//     }

//     public function setCommRemoteAddress($commRemoteAddress)
//     {
//         if ($commRemoteAddress === null) {
//             $this->del("CommRemoteAddress");
//         } else {
//             $this->set("CommRemoteAddress", new CFString($commRemoteAddress));
//         }
//     }

//     public function getAuthEapPlugins()
//     {
//         return $this->getValue("AuthEAPPlugins");
//     }

//     public function setAuthEapPlugins(array $authEapPlugins = null)
//     {
//         $this->setValue("AuthEAPPlugins", $authEapPlugins, self::TYPE_ARRAY);
//     }

//     public function getAuthProtocol()
//     {
//         return $this->getValue("AuthProtocol");
//     }

//     public function setAuthProtocol(array $authProtocol = null)
//     {
//         $this->setValue("AuthProtocol", $authProtocol, self::TYPE_ARRAY);
//     }

//     public function isCcpMppE40Enabled()
//     {
//         return $this->getValue("CCPMPPE40Enabled");
//     }

//     public function setCcpMppE40Enabled($enabled)
//     {
//         if ($enabled === null) {
//             $this->del("CCPMPPE40Enabled");
//         } else {
//             $this->set("CCPMPPE40Enabled", new CFBoolean($enabled));
//         }
//     }

//     public function isCcpMppE128Enabled()
//     {
//         return $this->getValue("CCPMPPE128Enabled");
//     }

//     public function setCcpMppE128Enabled($enabled)
//     {
//         if ($enabled === null) {
//             $this->del("CCPMPPE128Enabled");
//         } else {
//             $this->set("CCPMPPE128Enabled", new CFBoolean($enabled));
//         }
//     }

//     public function isCcpEnabled()
//     {
//         return $this->getValue("CCPEnabled");
//     }

//     public function setCcpEnabled($ccpEnabled)
//     {
//         if ($ccpEnabled === null) {
//             $this->del("CCPEnabled");
//         } else {
//             $this->set("CCPEnabled", new CFBoolean($ccpEnabled));
//         }
//     }
//     // Payloads with VPN connection type PPP end

//     // Payloads with VPN connection type IPSec begin
//     public function getRemoteAddress()
//     {
//         return $this->getValue("RemoteAddress");
//     }

//     public function setRemoteAddress($remoteAddress)
//     {
//         if ($remoteAddress === null) {
//             $this->del("RemoteAddress");
//         } else {
//             $this->set("RemoteAddress", new CFString($remoteAddress));
//         }
//     }

//     /**
//      * @return string|null The authentication method. Possible values are
//      *     SharedSecret and Certificate. Default: null.
//      */
//     public function getAuthenticationMethod()
//     {
//         return $this->getValue("AuthenticationMethod");
//     }

//     /**
//      * Sets the authentication method.
//      *
//      * @param string $authenticationMethod The autentication method. Possible
//      *     values SharedSecret and Certificate. Use null to delete this entry.
//      */
//     public function setAuthenticationMethod($authenticationMethod)
//     {
//         if ($authenticationMethod === null) {
//             $this->del("AuthenticationMethod");
//         } else {
//             $this->set("AuthenticationMethod", new CFString($authenticationMethod));
//         }
//     }

//     public function getXAuthName()
//     {
//         return $this->getValue("XAuthName");
//     }

//     public function setXAuthName($xAuthName)
//     {
//         if ($xAuthName === null) {
//             $this->del("XAuthName");
//         } else {
//             $this->set("XAuthName", new CFString($xAuthName));
//         }
//     }

//     /**
//      * @return int|null Interestingly this is an integer: 1 if enabled, 0
//      *     if disabled. Default: null
//      */
//     public function isXAuthEnabled()
//     {
//         return $this->getValue("XAuthEnabled");
//     }

//     public function setXAuthEnabled($xAuthEnabled)
//     {
//         if ($xAuthEnabled === null) {
//             $this->del("XAuthEnabled");
//         } else {
//             $this->set("XAuthEnabled", new CFNumber($xAuthEnabled));
//         }
//     }

//     public function getLocalIdentifier()
//     {
//         return $this->getValue("LocalIdentifier");
//     }

//     public function setLocalIdentifier($localIdentifier)
//     {
//         if ($localIdentifier === null) {
//             $this->del("LocalIdentifier");
//         } else {
//             $this->set("LocalIdentifier", new CFString($localIdentifier));
//         }
//     }

//     public function getLocalIdentifierType()
//     {
//         return $this->getValue("LocalIdentifierType");
//     }

//     public function setLocalIdentifierType($localIdentifierType)
//     {
//         if ($localIdentifierType === null) {
//             $this->del("LocalIdentifierType");
//         } else {
//             $this->set("LocalIdentifierType", $localIdentifierType);
//         }
//     }

//     public function getSharedSecret()
//     {
//         return $this->getValue("SharedSecret");
//     }

//     public function setSharedSecret($sharedSecret)
//     {
//         if ($sharedSecret === null) {
//             $this->del("SharedSecret");
//         } else {
//             $this->set("SharedSecret", new CFData($sharedSecret));
//         }
//     }

//     public function getCertificateUUID()
//     {
//         return $this->getValue("PayloadCertificateUUID");
//     }

//     public function setCertificateUUID($certificateUUID)
//     {
//         if ($certificateUUID === null) {
//             $this->del("PayloadCertificateUUID");
//         } else {
//             $this->set("PayloadCertificateUUID", new CFString($certificateUUID));
//         }
//     }

//     public function isPromptForVpnPin()
//     {
//         return $this->getValue("PromptForVPNPIN");
//     }

//     public function setPromptForVpnPin($promptForVpnPin)
//     {
//         if ($promptForVpnPin === null) {
//             $this->del("PromptForVPNPIN");
//         } else {
//             $this->set("PromptForVPNPIN", new CFBoolean($promptForVpnPin));
//         }
//     }
//     // Payloads with VPN connection type IPSec end
    // @codingStandardsIgnoreEnd
}
