<?php
namespace Topalis\Profile\Payload;

use CFPropertyList\CFString;
use CFPropertyList\CFBoolean;
use CFPropertyList\CFData;

// @codingStandardsIgnoreStart
/**
 * This class represents a Web Clip payload.
 * A Web Clip payload provides a web clipping on the user’s home screen as
 * though the user had saved a bookmark to the home screen.
 * @see https://developer.apple.com/library/ios/#featuredarticles/iPhoneConfigurationProfileRef/Introduction/Introduction.html#//apple_ref/doc/uid/TP40010206-CH1-SW12
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
// @codingStandardsIgnoreEnd
class WebClipPayload extends AbstractPayload
{
    const TYPE = "com.apple.webClip.managed";

    /**
     * Get data array for payload
     *
     * @param string $idPrefix
     *
     * @return array
     */
    protected function getData($idPrefix)
    {
        $icon = $this->getIcon();
        if ($icon instanceof \SplFileInfo) {
            $icon = file_get_contents($icon->getPathname());
        }

        return array(
            "Icon"                => new CFData($icon),
            "Label"               => $this->getLabel(),
            "URL"                 => $this->getUrl(),
            "Precomposed"         => $this->isPreComposed(),
            "IsRemovable"         => $this->isRemovable(),
            "FullScreen"          => $this->isFullScreen(),
            "PayloadDescription"  => $this->getDescription(),
            "PayloadDisplayName"  => $this->getDisplayName(),
            "PayloadIdentifier"   => $idPrefix . ".WebClip",
            "PayloadOrganization" => "",
            "PayloadType"         => "com.apple.webClip.managed",
            "PayloadUUID"         => $this->getUUID(),
            "PayloadVersion"      => 1,
        );
    }

    /**
     * If true, the browser will be opened in fullscreen mode. (Optional)
     *
     * @param boolean $fullScreen
     * @return $this
     */
    public function setFullScreen($fullScreen)
    {
        if ($fullScreen === null) {
            $this->del("FullScreen");
        }
        $this->set("FullScreen", new CFBoolean($fullScreen));
        return $this;
    }

    /**
     * @return boolean true if the browser will be opened in fullscreen mode
     */
    public function isFullScreen()
    {
        return $this->get("FullScreen", false);
    }

    /**
     * Sets a PNG icon to be shown on the Home screen. Should be 59x60 pixels in
     * size. If not specified, a white square will be shown.
     *
     * @param string $icon The icon to set.
     * @return $this
     */
    public function setIcon($icon)
    {
        if ($icon === null) {
            $this->del("Icon");
        } else {
            $this->set("Icon", new CFData($icon));
        }
        return $this;
    }

    /**
     * @return string The PNG icon to be shown on the Home screen. If not
     *     specified, a white square will be shown.
     */
    public function getIcon()
    {
        return $this->getValue("Icon");
    }

    /**
     * Sets the name of the Web Clip as displayed on the Home screen.
     *
     * @param string $label The label to set.
     * @return $this
     */
    public function setLabel($label)
    {
        $this->set("Label", new CFString($label));
        return $this;
    }

    /**
     * @return string The name of the Web Clip as displayed on the Home screen.
     */
    public function getLabel()
    {
        return $this->getValue("Label");
    }

    /**
     * If true icon is pre composed, no effects will be applied. (Optional)
     *
     * @param boolean $preComposed
     * @return $this
     */
    public function setPreComposed($preComposed)
    {
        if ($preComposed === null) {
            $this->del("Precomposed");
        } else {
            $this->set("Precomposed", new CFBoolean($preComposed));
        }
        return $this;
    }

    /**
     * @return boolean true if icon is pre composed, no effects will be applied.
     */
    public function isPreComposed()
    {
        return $this->getValue("Precomposed", false);
    }

    /**
     * If false, the user cannot remove the Web Clip, but it will be
     * removed if the profile is deleted. (Optional, default true)
     *
     * @param boolean $removable
     * @return $this
     */
    public function setRemovable($removable)
    {
        if ($removable === null) {
            $this->del("IsRemovable");
        } else {
            $this->set("IsRemovable", new CFBoolean($removable));
        }
        return $this;
    }

    /**
     * @return boolean If false, the user cannot remove the Web Clip, but it
     *     will be removed if the profile is deleted. (Optional, default true)
     */
    public function isRemovable()
    {
        return $this->getValue("IsRemovable", true);
    }

    /**
     * Sets the URL that the Web Clip should open when clicked. The URL must
     * begin with HTTP or HTTPS or it won't work.
     *
     * @param string $url The URL to set.
     * @return $this
     */
    public function setUrl($url)
    {
        $this->set("URL", new CFString($url));
        return $this;
    }

    /**
     * @return string The URL that the Web Clip should open when clicked.
     */
    public function getUrl()
    {
        return $this->getValue("URL");
    }
}
