<?php
namespace Topalis\Profile\Payload;

use CFPropertyList\CFNumber;
use CFPropertyList\CFBoolean;
use CFPropertyList\CFString;

// @codingStandardsIgnoreStart
/**
 * Represents a WiFi payload.
 * This payload configures WiFi on the device.
 * @see https://developer.apple.com/library/ios/featuredarticles/iPhoneConfigurationProfileRef/iPhoneConfigurationProfileRef.pdf#//apple_ref/doc/uid/TP40010206-CH1-SW30
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
// @codingStandardsIgnoreEnd
class WifiPayload extends AbstractPayload
{
    const TYPE = "com.apple.wifi.managed";

    public function getSsid()
    {
        return $this->getValue("SSID_STR");
    }

    public function setSsid($ssid)
    {
        if ($ssid === null) {
            $this->del("SSID_STR");
        } else {
            $this->set("SSID_STR", new CFString($ssid));
        }
    }

    public function isHiddenNetwork()
    {
        return $this->getValue("HIDDEN_NETWORK", false);
    }

    public function setHiddenNetwork($hiddenNetwork)
    {
        if ($hiddenNetwork === null) {
            $this->del("HIDDEN_NETWORK");
        } else {
            $this->set("HIDDEN_NETWORK", new CFBoolean($hiddenNetwork));
        }
    }

    public function isAutoJoin()
    {
        return $this->getValue("AutoJoin", true);
    }

    public function setAutoJoin($autoJoin)
    {
        if ($autoJoin === null) {
            $this->del("AutoJoin");
        } else {
            $this->set("AutoJoin", new CFBoolean($autoJoin));
        }
    }

    /**
     *
     * @return string|null The encryption type. Possible values are WEP, WPA,
     *     Any, and None. Default: null.
     */
    public function getEncryptionType()
    {
        return $this->getValue("EncryptionType");
    }

    /**
     * Sets the encryption type.
     *
     * @param string $encryptionType The encryption type to set. Possible values
     *     are WEP, WPA, Any, and None. Use null to delete this entry.
     */
    public function setEncryptionType($encryptionType)
    {
        if ($encryptionType === null) {
            $this->del("EncryptionType");
        } else {
            $this->set("EncryptionType", new CFString($encryptionType));
        }
    }

    public function getPassword()
    {
        return $this->getValue("Password");
    }

    public function setPassword($password)
    {
        if ($password === null) {
            $this->del("Password");
        } else {
            $this->set("Password", new CFString($password));
        }
    }

    public function getEapClientConfiguration()
    {
        return $this->getValue("EAPClientConfiguration");
    }

    public function setEapClientConfiguration(array $eapClientConfiguration = null)
    {
        $this->setValue("EAPClientConfiguration", $eapClientConfiguration, self::TYPE_DICTIONARY);
    }

    public function getCertificateUUID()
    {
        return $this->getValue("PayloadCertificateUUID");
    }

    public function setCertificateUUID($certificateUUID)
    {
        if ($certificateUUID === null) {
            $this->del("PayloadCertificateUUID");
        } else {
            $this->set("PayloadCertificateUUID", new CFString($certificateUUID));
        }
    }

    public function isHotspot()
    {
        return $this->getValue("IsHotspot", false);
    }

    public function setHotspot($hotspot)
    {
        if ($hotspot === null) {
            $this->del("IsHotspot");
        } else {
            $this->set("IsHotspot", new CFBoolean($hotspot));
        }
    }

    public function getPriority()
    {
        $this->getValue("Priority", 0);
    }

    public function setPriority($priority)
    {
        if ($priority === null) {
            $this->del("Priority");
        } else {
            $this->set("Priority", new CFNumber($priority));
        }
    }

    public function getDomainName()
    {
        return $this->getValue("DomainName");
    }

    public function setDomainName($domainName)
    {
        if ($domainName === null) {
            $this->del("DomainName");
        } else {
            $this->set("DomainName", new CFString($domainName));
        }
    }

    public function getHessid()
    {
        return $this->getValue("HESSID");
    }

    public function setHessid($hessid)
    {
        if ($hessid === null) {
            $this->del("HESSID");
        } else {
            $this->set("HESSID", new CFString($hessid));
        }
    }

    public function isServiceProviderRoamingEnabled()
    {
        return $this->getValue("ServiceProviderRoamingEnabled");
    }

    public function setServiceProviderEnabled($enabled)
    {
        if ($enabled === null) {
            $this->del("ServiceProviderRoamingEnabled");
        } else {
            $this->set("ServiceProviderRoamingEnabled", new CFBoolean($enabled));
        }
    }

    public function getRoamingConsortiumOIs()
    {
        return $this->getValue("RoamingConsortiumOIs");
    }

    public function setRoamingConsortiumOIs(array $roamingConsortiumOIs = null)
    {
        $this->setValue("RoamingConsortiumOIs", $roamingConsortiumOIs, self::TYPE_ARRAY);
    }

    public function getNaiRealmNames()
    {
        return $this->getValue("NAIRealmNames");
    }

    public function setNaiRealmNames(array $naiRealmNames = null)
    {
        $this->setValue("NAIRealmNames", $naiRealmNames, self::TYPE_ARRAY);
    }

    public function getMccAndMncs()
    {
        return $this->getValue("MCCAndMNCs");
    }

    public function setMccAndMncs(array $mccAndMncs = null)
    {
        $this->setValue("MCCAncMNCs", $mccAndMncs, self::TYPE_ARRAY);
    }

    public function getDisplayedOperatorName()
    {
        return $this->getValue("DisplayedOperatorName");
    }

    public function setDisplayedOperatorName($displayedOperatorName)
    {
        $this->setValue("DisplayedOperatorName", $displayedOperatorName, self::TYPE_STRING);
    }

    /**
     * @return string|null The proxy type. Valid values are None, Manual, and
     *     Auto. Default: null.
     */
    public function getProxyType()
    {
        return $this->getValue("ProxyType");
    }

    /**
     * Sets the proxy type.
     *
     * @param string $proxyType The proxy type to set. Valid values are None,
     *     Manual, and Auto. Use null to delete this entry.
     */
    public function setProxyType($proxyType)
    {
        if ($proxyType === null) {
            $this->del("ProxyType");
        } else {
            $this->set("ProxyType", new CFString($proxyType));
        }
    }

    public function getProxyServer()
    {
        return $this->getValue("ProxyServer");
    }

    public function setProxyServer($proxyServer)
    {
        if ($proxyServer === null) {
            $this->del("ProxyServer");
        } else {
            $this->set("ProxyServer", new CFString($proxyServer));
        }
    }

    public function getProxyServerPort()
    {
        return $this->getValue("ProxyServerPort");
    }

    public function setProxyServerPort($proxyServerPort)
    {
        if ($proxyServerPort === null) {
            $this->del("ProxyServerPort");
        } else {
            $this->set("ProxyServerPort", new CFNumber($proxyServerPort));
        }
    }

    public function getProxyUsername()
    {
        return $this->getValue("ProxyUsername");
    }

    public function setProxyUsername($proxyUsername)
    {
        if ($proxyUsername === null) {
            $this->del("ProxyUsername");
        } else {
            $this->set("ProxyUsername", new CFString($proxyUsername));
        }
    }

    public function getProxyPassword()
    {
        return $this->getValue("ProxyPassword");
    }

    public function setProxyPassword($proxyPassword)
    {
        if ($proxyPassword === null) {
            $this->del("ProxyPassword");
        } else {
            $this->set("ProxyPassword", new CFString($proxyPassword));
        }
    }

    public function getProxyPacUrl()
    {
        return $this->getValue("ProxyPACURL");
    }

    public function setProxyPacUrl($proxyPacUrl)
    {
        if ($proxyPacUrl === null) {
            $this->del("ProxyPACURL");
        } else {
            $this->set("ProxyPACURL", new CFString($proxyPacUrl));
        }
    }

    public function isProxyPacFallbackAllowed()
    {
        return $this->getValue("ProxyPACFallbackAllowed");
    }

    public function setProxyPacFallbackAllowed($proxyPacFallbackAllowed)
    {
        if ($proxyPacFallbackAllowed === null) {
            $this->del("ProxyPACFallbackAllowed");
        } else {
            $this->set("ProxyPACFallbackAllowed", new CFBoolean($proxyPacFallbackAllowed));
        }
    }
}
